/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Loading and showing bitmaps with the Applet support.
 * Example code how to load and show bitmaps with GameFrame for Java.
 * Extends the BasicExample that sets up the GameFrame for Java library.
 *
 *@author Pasi Kernen 30. Jan 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletExample1 extends BasicAppletExample
{
    /** The graphics engine we are using. */
    protected GraphicsEngine m_grfxEngine;
    /** The background bitmap. */
    protected DrawableBitmap m_backgroundBitmap;
    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        AppletExample1 instance = new AppletExample1( args );

        // When running as application we need to call the init() method
        // by ourselves followed by start().
        instance.init();
        instance.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public AppletExample1( String[] args )
    {
        super( args );
    }

    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Applet Example 1" );
    }

    /**
     * Initializes the example by loading two tile images and then constructing a background
     * image using those two tile images.
     */
    public void init()
    {
        // Setup the GameFrame for Java library
        super.init();
        
        // You can allways get the existing instance of the GameFrame
        // with the static getInstance() method. If you haven't created
        // the GameFrame object with new GameFrame() or new GameFrame( GameFrameSettings )
        // the method will return null.
        
        // We need the graphics engine, so we just get it from the GameFrame
        m_grfxEngine = GameFrame.getGraphicsEngine();
        printToConsole( m_grfxEngine.getDescription() );

        // We set the backgound color to black
        m_grfxEngine.setBackgroundColor( java.awt.Color.black );

        // We paint the backbuffer with the current background color
        m_grfxEngine.clearBackbuffer();

        // We flip the current backbuffer and frontbuffer
        m_grfxEngine.flip();

        // We paint the new backbuffer with the current background color
        m_grfxEngine.clearBackbuffer();

        try
        {
            // Lets set the directory wherefrom we wish to load the bitmaps
            m_grfxEngine.setDataDirectory( "resources" );

            // And then we load some non-transparent bitmaps to be used as the background tiles
            Bitmap tile1 = m_grfxEngine.loadBitmap( "tile1" );
            Bitmap tile2 = m_grfxEngine.loadBitmap( "tile2" );

            // We query the first tile the dimensions and assume the second tile has the same size
            int tileWidth  = tile1.getWidth();
            int tileHeight = tile1.getHeight();

            // Then we create a drawable bitmap that we draw the background upon
            m_backgroundBitmap = m_grfxEngine.getDrawableBitmap( WIDTH, HEIGHT );
            for ( int row = 0; row <= ( HEIGHT / tileHeight ); row++ )
            {
                for ( int col = 0; col < ( WIDTH / tileWidth ); col++ )
                {
                    if ( ( col + row ) % 2 == 0 )
                    {
                        tile1.blitTo( m_backgroundBitmap, col * tileWidth, row * tileHeight );
                    }
                    else
                    {
                        tile2.blitTo( m_backgroundBitmap, col * tileWidth, row * tileHeight );
                    }
                }
            }
        }
        catch( FileNotFoundException noSuchFile )
        {
            printToConsole("FileNotFound:\n"+noSuchFile.getMessage() );
            kill();
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            kill();
        }
    }
    
    /**
     * The method where the game should be executed.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        // And then we paint the background buffer with the background bitmap
        m_backgroundBitmap.blitTo( 0, 0 );
        
        // We flip the background buffer with the current front buffer thus making the
        // background buffer we just draw into the visible front buffer
        m_grfxEngine.flip();
        
        // Then we wait for three seconds...
        super.executeCodeThatUsesGameFrame();
    }
}
