/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Showing text on the screen.
 * Example code how to load font bitmaps and show text on the screen.
 *
 *@author Pasi Kernen 30. Jan 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletExample3 extends AppletExample2
{
    /** The number of ghosts */
    public final int NUM_GHOSTS = 20;
    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        AppletExample3 instance = new AppletExample3( args );
        
        instance.init();
        instance.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public AppletExample3( String[] args )
    {
        super( args );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Applet Example 3" );
    }

    /**
     * Initializes the example.
     */
    public void init()
    {
        // Load and setup the background image. Also sets up the data directory.
        super.init();
        
        try
        {
            // First we load the font bitmap (a bitmap that has 16x16 2D array of letters)
            m_grfxEngine.loadFontBitmap( "fadedFonts" );
        }
        catch( FileNotFoundException noSuchFile )
        {
            printToConsole("FileNotFound:\n"+noSuchFile.getMessage() );
            loadDefaultFontBitmap();
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            loadDefaultFontBitmap();
        }
    }

	/**
	 * Tries to load the default font bitmap, if fails, exits.
	 */
	private void loadDefaultFontBitmap()
	{
        try
        {
            // First we load the font bitmap (a bitmap that has 16x16 2D array of letters)
            m_grfxEngine.loadDefaultFontBitmap();
        }
        catch( FileNotFoundException noSuchFile )
        {
            printToConsole("FileNotFound:\n"+noSuchFile.getMessage() );
            kill();
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            kill();
        }
	}
	
    /**
     * The method where the game should be executed. Animates the two ghosts moving
     * around against the tile background.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        DrawableBitmap helloWorld = null;
        try
        {
            // After loading the font bitmap in init() we can now transform text to bitmaps
            helloWorld = m_grfxEngine.getAsBitmap( "HELLO WORLD!" );
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            kill();
        }

        // Get the current engine implementations description text.
        // We use byte array as it is more efficient to use than a String.
        byte[] descriptionText = GameFrame.getDescription().getBytes();
        
        // Get the width and height of the description text
        int descriptionWidth = m_grfxEngine.getTextWidth ( descriptionText );
        int descriptionHeight = m_grfxEngine.getFontHeight();
        
        // Calculate the halfway down the screen
        int halfScreen = HEIGHT / 2;
        
        // Calculate a suitable offset for the HELLO WORLD text
        int helloWorldOffsetX = ( descriptionWidth - helloWorld.getWidth() ) / 2;

        // Setup ghost related variables
        int ghostX[] = new int[NUM_GHOSTS];
        int ghostY[] = new int[NUM_GHOSTS];
        int ghostDestX[] = new int[NUM_GHOSTS];
        int ghostDestY[] = new int[NUM_GHOSTS];

        // Randomly initialize all the current and destination coordinates
        for ( int i = 0; i < NUM_GHOSTS; i++ )
        {
            ghostX[i]  = (int) ( Math.random() * WIDTH );
            ghostY[i]  = (int) ( Math.random() * HEIGHT );
            ghostDestX[i]  = (int) ( Math.random() * WIDTH );
            ghostDestY[i]  = (int) ( Math.random() * HEIGHT );
        }
        
        for ( int x = WIDTH ; x > -descriptionWidth; x-- )
        {
            // First we draw the background image to the backbuffer
            m_backgroundBitmap.blitTo( 0, 0 );

            for ( int i = 0; i < NUM_GHOSTS; i++ )
            {
                // If ghost reaches destination we come up with a new destination
                if ( ( ghostX[i] == ghostDestX[i] ) && ( ghostY[i] == ghostDestY[i]) )
                {
                    ghostDestX[i]  = (int) ( Math.random() * WIDTH );
                    ghostDestY[i]  = (int) ( Math.random() * HEIGHT );
                }
            
                // We move our towards its destination
                if ( ghostX[i] < ghostDestX[i] )
                {
                    ghostX[i]++;
                }
                if ( ghostX[i] > ghostDestX[i] )
                {
                    ghostX[i]--;
                }
                if ( ghostY[i] < ghostDestY[i] )
                {
                    ghostY[i]++;
                }
                if ( ghostY[i] > ghostDestY[i] )
                {
                    ghostY[i]--;
                }
                
                // Then we draw our ghosts
                m_pinkGhost.blitTo( ghostX[i], ghostY[i] );
            }

            
            try
            {
                // Then we draw the description text on the screen on the fly.
                // This is because the description text is too large to
                // fit inside a single bitmap (as bitmaps are limited to the size
                // of the current resolution).
                m_grfxEngine.drawTextAt( descriptionText, x, halfScreen + ( descriptionHeight << 2 ) );
            }
            catch( GameFrameException gameFrameError )
            {
                printToConsole( "GameFrameException:\n"+gameFrameError.getMessage() );
                kill();
            }

            // Then we draw the "Hello World" to the screen
            helloWorld.blitTo( x + helloWorldOffsetX, halfScreen );

            // Then we just flip the backbuffer with the frontbuffer
            // making the just drawn graphics visible on the screen
            m_grfxEngine.flip();
        }
    }
}
