/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Using subbitmaps.
 * Example code how to use the getSubBitmap() and getSubBitmapClone() methods of a bitmap.
 * Uses one tiled animation bitmap that contains 2x2 cells of animation of a ghost (again).
 *
 *@author Pasi Kernen 19. Mar 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletExample5 extends AppletExample3
{
    /** Rotated copies of the ghost. */
    protected Bitmap[] m_ghostAnimationBitmaps;
    /** The current input engine. */
    protected InputEngine m_inputEngine;
    /** The default keyboard device. */
    protected KeyboardDevice m_keyboard;
    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        AppletExample5 instance = new AppletExample5( args );
        instance.init();
        instance.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public AppletExample5( String[] args )
    {
        super( args );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Applet Example 5" );
    }

    /**
     * Initializes the example. Uses the m_pinkGhost variable that gets
     * loaded in the GraphicsExample2's init() method.
     */
    public void init()
    {
        super.init();
		
        try
        {
            // We get the input engine
            m_inputEngine = GameFrame.getInputEngine();
            printToConsole( m_inputEngine.getDescription() );

            // Then we get the default keyboard device
            m_keyboard    = m_inputEngine.getDefaultKeyboardDevice();

            // We load the bitmap containing the tiled image
            Bitmap tiledBitmap = m_grfxEngine.loadBitmap( "tiledImage" );
            
            // First we divide the bitmap into upper and lower half.
            Bitmap upperHalf = tiledBitmap.getSubBitmap(
                0, 0,
                tiledBitmap.getWidth(), tiledBitmap.getHeight() / 2 );
            Bitmap lowerHalf = tiledBitmap.getSubBitmap(
                0, tiledBitmap.getHeight() / 2,
                tiledBitmap.getWidth(), tiledBitmap.getHeight() / 2 );

            // Then we divide the upper half into two
            Bitmap upperLeft = upperHalf.getSubBitmap(
                0, 0,
                upperHalf.getWidth() / 2, upperHalf.getHeight() );
            Bitmap upperRight = upperHalf.getSubBitmap(
                upperHalf.getWidth() / 2, 0,
                upperHalf.getWidth() / 2, upperHalf.getHeight() );
            
            // and then we divide the lower half into two
            Bitmap lowerLeft = lowerHalf.getSubBitmap(
                0, 0,
                lowerHalf.getWidth() / 2, lowerHalf.getHeight() );
            Bitmap lowerRight = lowerHalf.getSubBitmap(
                lowerHalf.getWidth() / 2, 0,
                lowerHalf.getWidth() / 2, lowerHalf.getHeight() );

            // Then we just create an array of bitmaps that make
            // up the animation we are going to show.
            m_ghostAnimationBitmaps = new Bitmap[4];
            m_ghostAnimationBitmaps[0] = upperLeft;
            m_ghostAnimationBitmaps[1] = upperRight;
            m_ghostAnimationBitmaps[2] = lowerLeft;
            m_ghostAnimationBitmaps[3] = lowerRight;
        }
        catch( FileNotFoundException fnfe )
        {
            printToConsole( "FileNotFoundException:\n"+fnfe.getMessage() );
            kill();
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            kill();
        }
    }

    /**
     * The method where the game should be executed. Animates the two ghosts moving
     * around against the tile background.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        while( !m_keyboard.wasKeyDown( java.awt.event.KeyEvent.VK_ESCAPE ) && m_fRunning )
        {
            for ( int idx = 0; idx < m_ghostAnimationBitmaps.length; idx++ )
            {

                // First we draw the background image to the backbuffer
                m_backgroundBitmap.blitTo( 0, 0 );
                drawFrame( idx );
            
                // Pause for awhile
                try
                {
                    Thread.currentThread().sleep( 30 );
                }
                catch( InterruptedException ie )
                {
                }
            
                m_grfxEngine.flip();
            }
            
            for ( int idx = m_ghostAnimationBitmaps.length - 1;
                  idx >= 0; idx-- )
            {

                // First we draw the background image to the backbuffer
                m_backgroundBitmap.blitTo( 0, 0 );
                drawFrame( idx );

                // Pause for awhile
                try
                {
                    Thread.currentThread().sleep( 50 );
                }
                catch( InterruptedException ie )
                {
                }

                m_grfxEngine.flip();
            }
        }
    }
    
    /**
     * Draws given animation frame evenly spaced to fill the screen.
     *@param idx The animation frame index to use.
     */
    private void drawFrame( int idx )
    {
        Bitmap animationFrame = m_ghostAnimationBitmaps[idx];
        int xStep = animationFrame.getWidth() * 2;
        int yStep = animationFrame.getHeight() * 2;

        for ( int x = 0;
              x < WIDTH;
              x = x + xStep )
        {
            for ( int y = 0;
                  y < WIDTH;
                  y = y + yStep )
            {
                animationFrame.blitTo( x, y );
            }
        }
    }
}
