/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Using ready made bitmap effects.
 * Example code how to use the getClone( BitmapEffect ) method of a bitmap with the selfmade
 * bitmap effect class.
 *
 *@author Pasi Kernen 01. Apr 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletExample6 extends AppletExample3
{
    /** Rotated copies of the ghost. */
    protected Bitmap m_effectedGhost;
    /** The current input engine. */
    protected InputEngine m_inputEngine;
    /** The default keyboard device. */
    protected KeyboardDevice m_keyboard;
    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        AppletExample6 instance = new AppletExample6( args );
        instance.init();
        instance.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public AppletExample6( String[] args )
    {
        super( args );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Applet Example 6" );
    }

    /**
     * Initializes the example. Uses the m_pinkGhost variable that gets
     * loaded in the GraphicsExample2's init() method.
     */
    public void init()
    {
        super.init();
		
        try
        {
            // We get the input engine
            m_inputEngine = GameFrame.getInputEngine();
            printToConsole( m_inputEngine.getDescription() );

            // Then we get the default keyboard device
            m_keyboard    = m_inputEngine.getDefaultKeyboardDevice();

            // We create an instance of our own effect
            // (see below)
            VideoScreenEffect videoEffect =
				new VideoScreenEffect();
            
            // And use that to make an effect processed bitmap
            m_effectedGhost =
                m_pinkGhost.getClone( videoEffect );
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            kill();
        }        
    }

    /**
     * The method where the game should be executed. Animates the two ghosts moving
     * around against the tile background.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        int xStep = m_effectedGhost.getWidth() * 2;
        int yStep = m_effectedGhost.getHeight() * 2;
        
        while( !m_keyboard.wasKeyDown( java.awt.event.KeyEvent.VK_ESCAPE ) && m_fRunning )
        {
            // First we draw the background image to the backbuffer
            m_backgroundBitmap.blitTo( 0, 0 );

            for ( int x = 0;
                  x < WIDTH;
                  x = x + xStep )
            {
                for ( int y = 0;
                      y < WIDTH;
                      y = y + yStep )
                {
                    m_effectedGhost.blitTo( x, y );
                }
            }
            m_grfxEngine.flip();
        }
    }

    /**
     * This effect makes every second line the color of the background. If the
     * background is transparent the lines are made transparent. This
     * causes a kind of a semitransparency effect. If the background has some
     * color, the effect looks a bit different.
     */
    public class VideoScreenEffect extends BitmapEffect
    {
	    /**
	     * Processes the given bitmap data in some way.
	     *@param data The raw bitmap data that is to be processed.
	     *@see gameframe.graphics.BitmapData
	     */
	    public BitmapData processData( BitmapData data )
        {
            // We get the pixeldata from the bitmap data object
            int[] pixelData = data.getPixels();
            int height = data.getHeight();
            int width  = data.getWidth();
            
            // We go through every second line
            // starting from the second line
            for ( int y = 1; y < height; y = y + 2 )
            {
                for ( int x = 0; x < width; x++ )
                {
                    pixelData[y * width + x] = pixelData[0];
                }
            }
            
            // And since we were modifying the pixel data instance
            // that the bitmap data uses, we are now done.
            return data;
        }

	    /**
         * Resets the effects settings.
         */
        public void reset()
        {
            // There is nothing to reset...
        }
    }
}
