/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Using animated bitmaps.
 * Example code how to use the AnimBitmap class.
 * Uses one tiled animation bitmap that contains
 * 2x2 cells of animation of a ghost (again).
 *
 *@author Pasi Kernen 04. Apr 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletExample7 extends AppletExample3
{
    /** Animation of the ghost. */
    protected AnimBitmap m_ghostAnimationBitmap;
    /** The current input engine. */
    protected InputEngine m_inputEngine;
    /** The default keyboard device. */
    protected KeyboardDevice m_keyboard;
    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        AppletExample7 instance = new AppletExample7( args );
        instance.init();
        instance.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public AppletExample7( String[] args )
    {
        super( args );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Applet Example 7" );
    }

    /**
     * Initializes the example. Uses the m_pinkGhost variable that gets
     * loaded in the GraphicsExample2's init() method.
     */
    public void init()
    {
        super.init();
		
        try
        {
            // We get the input engine
            m_inputEngine = GameFrame.getInputEngine();
            printToConsole( m_inputEngine.getDescription() );

            // Then we get the default keyboard device
            m_keyboard    = m_inputEngine.getDefaultKeyboardDevice();

            // We load the bitmap containing the tiled image
            Bitmap tiledBitmap = m_grfxEngine.loadBitmap( "tiledImage" );
            
            // and we create the animation from the loaded bitmap
            // by giving the bitmap and the size of one animation cell
            m_ghostAnimationBitmap = new AnimBitmap( tiledBitmap, 45, 45 );
            
            // Finalize the bitmap as the anim bitmap uses copies
            tiledBitmap.finalize();

            // and set the animatio mode to ping-pong
            m_ghostAnimationBitmap.setMode( AnimBitmap.MODE_PING_PONG );
        }
        catch( FileNotFoundException fnfe )
        {
            printToConsole( "FileNotFoundException:\n"+fnfe.getMessage() );
            kill();
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            kill();
        }
    }

    /**
     * The method where the game should be executed. Animates the two ghosts moving
     * around against the tile background.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        String strText = "Press SPACE to change animation mode.";
        int middleX = ( WIDTH - m_grfxEngine.getTextWidth( strText ) ) / 2;
        int middleY = ( HEIGHT - m_grfxEngine.getFontHeight() ) / 2;

        String strText2 = "Press ESC to exit.";
        int middleX2 = ( WIDTH - m_grfxEngine.getTextWidth( strText2 ) ) / 2;
        int middleY2 = middleY + m_grfxEngine.getFontHeight();
        
        while( !m_keyboard.wasKeyDown( java.awt.event.KeyEvent.VK_ESCAPE ) )
        {
            // Chekc if user pressed space
            if( m_keyboard.wasKeyDown( java.awt.event.KeyEvent.VK_SPACE ) )
            {
                switch( m_ghostAnimationBitmap.getMode() )
                {
                    case AnimBitmap.MODE_LOOP:
                        m_ghostAnimationBitmap.setMode( AnimBitmap.MODE_PING_PONG );
                        break;
                    case AnimBitmap.MODE_PING_PONG:
                        m_ghostAnimationBitmap.setMode( AnimBitmap.MODE_LOOP );
                        break;
                }
            }

            // First we draw the background image to the backbuffer
            m_backgroundBitmap.blitTo( 0, 0 );


            // Draw the animated bitmap
            m_ghostAnimationBitmap.nextFrame();
            int xStep = m_ghostAnimationBitmap.getWidth() * 2;
            int yStep = m_ghostAnimationBitmap.getHeight() * 2;

            for ( int x = 0;
                  x < WIDTH;
                  x = x + xStep )
            {
                for ( int y = 0;
                      y < WIDTH;
                      y = y + yStep )
                {
                    m_ghostAnimationBitmap.blitTo( x, y );
                }
            }

            // Draw the text to backbuffer
            try
            {
                m_grfxEngine.drawTextAt( strText, middleX, middleY );
                m_grfxEngine.drawTextAt( strText2, middleX2, middleY2 );
            }
            catch( GameFrameException error )
            {
                printToConsole( "Exception:\n"+error.getMessage() );
                kill();
            }
            
            try
            {
                Thread.currentThread().sleep( 100 );
            }
            catch( InterruptedException ie )
            {
            }
            
            // And make the drawn backbuffer visible
            m_grfxEngine.flip();
        }
    }
}
