/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Loading and playing a sampled sound.
 * Example code how to load and play a sample.
 * Extends the BasicExample that sets up the
 * GameFrame for Java library.
 *
 *@author Pasi Kernen 19. Mar 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletSoundExample1 extends BasicAppletExample
{
    /** The graphics engine we are using. */
    protected GraphicsEngine m_grfxEngine;
    /** The sound engine we are using. */
    protected SoundEngine m_soundEngine;
    /** The input engine we are using. */
    protected InputEngine m_inputEngine;
    /** The keyboard device we are using. */
    protected KeyboardDevice m_keyboard;
    /** The pointer (mouse) device we are using. */
    protected PointerDevice m_pointer;
    /** The explosion sound. */
    protected Sample m_explosion;
    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        AppletSoundExample1 instance = new AppletSoundExample1( args );

        // When running as application we need to call the init() method
        // by ourselves followed by start().
        instance.init();
        instance.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public AppletSoundExample1( String[] args )
    {
        super( args );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Applet Sound Example 1" );
    }

    /**
     * Initializes the example by loading two tile images and then constructing a background
     * image using those two tile images.
     */
    public void init()
    {
        // Setup the GameFrame for Java library
        super.init();

        // We need the graphics engine, so we get it from the GameFrame
        m_grfxEngine = GameFrame.getGraphicsEngine();
        printToConsole( m_grfxEngine.getDescription() );

        // We need the input engine, so we get it from the GameFrame
        m_inputEngine = GameFrame.getInputEngine();
        printToConsole( m_inputEngine.getDescription() );

        // We need the sound engine, so we get it from the GameFrame
        m_soundEngine = GameFrame.getSoundEngine();
        printToConsole( m_soundEngine.getDescription() );
        
        // We set the backgound color to black
        m_grfxEngine.setBackgroundColor( java.awt.Color.black );

        // We paint the backbuffer with the current background color
        m_grfxEngine.clearBackbuffer();

        // We flip the current backbuffer and frontbuffer
        m_grfxEngine.flip();

        // We paint the new backbuffer with the current background color
        m_grfxEngine.clearBackbuffer();

        try
        {
            // We get the pointer and keyboard devices from the input engine
            m_keyboard = m_inputEngine.getDefaultKeyboardDevice();
            m_pointer  = m_inputEngine.getDefaultPointerDevice();

            // Set the directory wherefrom we wish to load any bitmaps
            m_grfxEngine.setDataDirectory( "resources" );

            // Set the directory wherefrom we wish to load any bitmaps
            m_soundEngine.setDataDirectory( "resources" );

            // Then we load a font bitmap, so we can display some text
            m_grfxEngine.loadDefaultFontBitmap();
            
            // And then we just load any samples we want to use
            m_explosion = m_soundEngine.loadSample( "explosion" );
        }
        catch( FileNotFoundException noSuchFile )
        {
            printToConsole("FileNotFound:\n"+noSuchFile.getMessage() );
            GameFrame.exit( 0 );
        }
        catch( GameFrameException gameFrameError )
        {
            printToConsole("GameFrameException:\n"+gameFrameError.getMessage() );
            GameFrame.exit( 0 );
        }
    }
    
    /**
     * The method where the game should be executed.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        String strText = "Playing Explosion Sound Once";
        int middleX = ( WIDTH - m_grfxEngine.getTextWidth( strText ) ) / 2;
        int middleY = ( HEIGHT - m_grfxEngine.getFontHeight() ) / 2;

        // We draw the text to backbuffer
        try
        {
            m_grfxEngine.drawTextAt( strText, middleX, middleY );
        }
        catch( GameFrameException error )
        {
            printToConsole( "Exception:\n"+error.getMessage() );
            kill();
        }
        
        // We flip the background buffer with the current front buffer thus making the
        // background buffer we just draw into the visible front buffer
        m_grfxEngine.flip();

        // We play the explosion sound once...
        m_explosion.playOnce();
        
        // Then we wait for three seconds...
        super.executeCodeThatUsesGameFrame();
    }
}
