/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Using the timer class.
 * Example code how to use the Timer class in the GameFrame for Java library.
 * Uses the same rotated pink ghost bitmaps that were used in GraphcisExample4
 * Uses one pink ghost bitmap that is processed to 36x3 = 108 bitmaps by rotating
 * the bitmap in 10 degrees steps and changing its colour twice (original pink,
 * green and yellow). Then while the ghosts are being drawn, copies the backbuffer
 * after each draw one pixel downwards without clearing it. Causes a very chaotic
 * "tail" effect.
 * The difference to GraphicsExample5 is that in this case time the main loop
 * running time with the Timer class and move the ghosts and the background
 * according to the time. This should take care of the different speeds of
 * different platforms.
 *
 *@author Pasi Kernen 18. March 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletTimerExample extends AppletExample4
{
    public static final int NUM_TIMES_FOR_AVERAGE = 5;
    /** The timer used to time the loop. */
    Timer m_loopTimer;

    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        AppletTimerExample instance = new AppletTimerExample( args );
        instance.run();
    }

    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public AppletTimerExample( String[] args )
    {
        super( args );
    }

    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Applet Timer Example" );
    }

    /**
     * Initializes the example.
     */
    public void init()
    {
        super.init();
        
        try
        {
            m_loopTimer = GameFrame.createTimer();
        }
        catch( GameFrameException error )
        {
            printToConsole( "Couldn't create timer:\n"+error );
            kill();
        }
    }

    /**
     * The method where the game should be executed. Animates the two ghosts moving
     * around against the tile background.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        float[] averagedTimes = new float[NUM_TIMES_FOR_AVERAGE];
        int averagedTimesIdx  = 0;

        // Setup ghost related variables
        int ghostX[]          = new int[NUM_GHOSTS];
        int ghostY[]          = new int[NUM_GHOSTS];
        int ghostDestX[]      = new int[NUM_GHOSTS];
        int ghostDestY[]      = new int[NUM_GHOSTS];
        float ghostRotation[] = new float[NUM_GHOSTS];
        boolean ghostRotDir[] = new boolean[NUM_GHOSTS];
        int ghostColor[]      = new int[NUM_GHOSTS];

        float flTime;
        float flAverageTime;
        for ( int i = 0; i < NUM_GHOSTS; i++ )
        {
            ghostX[i]        = (int)   ( Math.random() * WIDTH );
            ghostY[i]        = (int)   ( Math.random() * HEIGHT );
            ghostDestX[i]    = (int)   ( Math.random() * WIDTH );
            ghostDestY[i]    = (int)   ( Math.random() * HEIGHT );
            ghostRotation[i] = (float) ( Math.random() * NUM_ROTATIONS );
            ghostColor[i]    = (int)   ( Math.random() * NUM_COLORS );
            ghostRotDir[i]   = (Math.random() < 0.5D );
        }

        m_loopTimer.calculateTime();

        // Continue until user presses ESC key
        while( !m_keyboard.wasKeyDown( java.awt.event.KeyEvent.VK_ESCAPE ) )
        {
            m_grfxEngine.clearBackbuffer();

            flTime = m_loopTimer.calculateTime();
			
            averagedTimes[averagedTimesIdx] = flTime;
            averagedTimesIdx++;
            if ( averagedTimesIdx >= NUM_TIMES_FOR_AVERAGE )
            {
                averagedTimesIdx = 0;
            }

            flAverageTime = 0.0F;
            for ( int i = 0; i < NUM_TIMES_FOR_AVERAGE; i++ )
            {
                flAverageTime += averagedTimes[i];
            }
            flAverageTime = flAverageTime/(float)NUM_TIMES_FOR_AVERAGE;
            try
            {
                m_grfxEngine.drawTextAt( ""+ (int) ( 1000.0F / flAverageTime ),0 ,0 );
            }
            catch( GameFrameException error )
            {
            }

            // Move, spin, recolour and draw each ghost
            for ( int i = 0; i < NUM_GHOSTS; i++ )
            {
                // If ghost reaches destination we come up with a new destination
                if ( ( ghostX[i] == ghostDestX[i] ) && ( ghostY[i] == ghostDestY[i]) )
                {
                    ghostDestX[i]  = (int) ( Math.random() * WIDTH );
                    ghostDestY[i]  = (int) ( Math.random() * HEIGHT );
                }

                // We move our towards its destination
                if ( ghostX[i] < ghostDestX[i] )
                {
                    ghostX[i]++;
                }
                if ( ghostX[i] > ghostDestX[i] )
                {
                    ghostX[i]--;
                }
                if ( ghostY[i] < ghostDestY[i] )
                {
                    ghostY[i]++;
                }
                if ( ghostY[i] > ghostDestY[i] )
                {
                    ghostY[i]--;
                }

                // Then we draw our ghosts
                m_ghostBitmaps[ (int) ghostRotation[i] ][ ghostColor[i] ].blitTo(
                    ghostX[i], ghostY[i] );

                if ( ghostRotDir[i] )
                {
                    ghostRotation[i] += 0.2F;
                    if ( ghostRotation[i] >= NUM_ROTATIONS )
                    {
                        ghostRotation[i] = 0.0F;
                        ghostColor[i]    = (int) ( Math.random() * NUM_COLORS );
                    }
                }
                else
                {
                    ghostRotation[i] -= 0.2F;
                    if ( ghostRotation[i] < 0.0F )
                    {
                        ghostRotation[i] = (float) ( NUM_ROTATIONS - 1 );
                        ghostColor[i]    = (int) ( Math.random() * NUM_COLORS );
                    }
                }
            }


            // Then we just flip the backbuffer with the frontbuffer
            // making the just drawn graphics visible on the screen
            m_grfxEngine.flip();
        }
    }
}
