/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Setting up GameFrame for Java.
 * Example code how to startup an application that uses GameFrame for Java.
 *
 *@author Pasi Kernen 30. Jan 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class BasicExample implements Runnable
{
    /** The used screen width of the example. */
    public static final int WIDTH  = 640;
    /** The used screen height of the example. */
    public static final int HEIGHT = 480;

    /** The arguments given to the program. */
    protected String[] m_args;

    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        BasicExample instance = new BasicExample( args );
        instance.run();
    }

    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public BasicExample( String[] args )
    {
        m_args = args;
    }
    
    /**
     * Initializes the GameFrame for Java library.
     */
    protected void init()
    {
        // Build the message that is shown if the user doesn't have the
        // GameFrame for Java library installed
        String strNoLibraryMessage;
        strNoLibraryMessage  = "You need to have the GameFrame for\n";
        strNoLibraryMessage += "Java library installed in order to use\n";
        strNoLibraryMessage += "this program. The GameFrame for\n";
        strNoLibraryMessage += "Java library can be gotten from:\n";
        strNoLibraryMessage += "http://www.gamedev.net/hosted/javanerd";

        // Check if the user has the GameFrame for Java library
        try
        {
            // This is just a harmless access to a static method
            // in the GameFrame object. If this fails we don't have
            // the library installed.
            gameframe.GameFrame.getBuild();
        }
        catch( Error anyError )
        {
            System.out.println( strNoLibraryMessage );
            System.exit(0);
        }        
        catch(Exception anyException)
        {
            System.out.println( strNoLibraryMessage );
            System.exit(0);
        }
        
        // Setup the settings
        GameFrameSettings gameFrameSettings = new GameFrameSettings();
        setGameFrameSettings( gameFrameSettings );

        // Allow command line disabling of features
        gameFrameSettings.parseCommandLineArgs( m_args );

        // Create the GameFrame instance
        try
        {
            new GameFrame( gameFrameSettings );
        }
        catch( SettingsException error )
        {
            // We allready had one existing GameFrame instance
            System.out.println( "Error in GameFrame creation:\n"+error.getMessage() );
        }
        catch( GameFrameException frameError )
        {
            // Something went wrong during the library creation
            System.out.println( ""+frameError.getMessage() );
            System.exit( 0 );
        }
        
        // Print out the used GameFrame implementation description
        System.out.println( "Using:\n"+GameFrame.getDescription() );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        gameFrameSettings.setRequiredResolution( ""+WIDTH+"x"+HEIGHT );
        gameFrameSettings.setScreenMode( GameFrameSettings.SCREENMODE_FULLSCREEN );
        gameFrameSettings.setAllowMsJava( true );
        gameFrameSettings.setTitle( "Basic Example" );
    }

    /**
     * Deinitializes the example application and exits.
     */
    protected void deInit()
    {
        // For the last thing we cleanup after ourselves
        // using the GameFrame's own exit method.
        GameFrame.exit( 0 );
    }
    
    /**
     * Executes the example application
     */
    public void run()
    {
        // Initialize the example application
        init();
        
        // We now have the graphics and sound engines and there should
        // be a black screen visible.
        executeCodeThatUsesGameFrame();

        // Do some cleanup
        deInit();
    }
    
    /**
     * The method where the game should be executed.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        // Your game should be run here, but
        // we just wait for 3 seconds before continuing
        try
        {
            Thread.currentThread().sleep( 3000 );
        }
        catch(InterruptedException ie)
        {
        }
    }
}
