/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.awt.*;
import java.awt.event.*;

import java.io.FileNotFoundException;

/**
 * Usage of component mode.
 * Example code how to use the GraphicsEngine's component mode.
 *
 *@author Pasi Kernen 04. Apr 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class ComponentExample1 extends BasicExample
{
    /* The frame that contains the component mode graphics engine. */
    protected Frame m_frame;
    
    /** The input engine. */
    protected InputEngine m_inputEngine;
    /** The keyboard device. */
    protected KeyboardDevice m_keyboard;    
    /** The graphics engine we are using. */
    protected GraphicsEngine m_grfxEngine;
    /** The background bitmap. */
    protected DrawableBitmap m_backgroundBitmap;
    /** The pink ghost bitmap. */
    protected Bitmap m_pinkGhost;
    /** The wireframe ghost bitmap. */
    protected Bitmap m_wireGhost;
    /** The currently drawn bitmap. */
    protected Bitmap m_currentBitmap;
    /** Flag that indicates if we are running or not. */
    protected boolean m_fRunning = true;

    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        ComponentExample1 instance = new ComponentExample1( args );
        Thread instanceThread = new Thread( instance );
        instanceThread.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public ComponentExample1( String[] args )
    {
        super( args );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setScreenMode( GameFrameSettings.SCREENMODE_COMPONENT );
    }

    /**
     * Initializes the example.
     */
    protected void init()
    {
        super.init();
        // We need the graphics engine, so we just get it from the GameFrame.
        // Because we have specified the SCREENMODE_COMPONENT we know that
        // this graphics engine can be casted to java.awt.Component without
        // problems.
        m_grfxEngine = GameFrame.getGraphicsEngine();
        System.out.println( m_grfxEngine.getDescription() );

        // Before we can do anything with the component graphics engine
        // we have to add it to some container and make it visible
        
        // so we create a frame
        m_frame = new Frame( "Component Example 1" );
        m_frame.setMenuBar( getMenuBarForApp() );
        
        // we listen to the frames closing
        m_frame.addWindowListener( new WindowAdapter()
        {
            public void windowClosing( WindowEvent event )
            {
                m_fRunning = false;
            }
        } );
        
        // We add the graphics engine to the frame
        m_frame.setLayout( new BorderLayout() );
        m_frame.add( (Component) m_grfxEngine, BorderLayout.CENTER );

        // We add the panel containing buttons to the frame
        m_frame.add( getButtonPanel(), BorderLayout.SOUTH );

        // thighten up the size
        m_frame.pack();
        
        // and NOW we show the frame which makes the graphics engine
        // visible and thus we can use the graphics engine after this
        // point on normally
        m_frame.show();

        try
        {
            // Lets set the directory wherefrom we wish to load the bitmaps
            m_grfxEngine.setDataDirectory( "resources" );

            // First we load the font bitmap (a bitmap that has 16x16 2D array of letters)
            m_grfxEngine.loadFontBitmap( "fadedFonts" );

            // Then we load the pink ghost bitmap
            m_pinkGhost = m_grfxEngine.loadAlphaBitmap( "pinkGhost" );
            m_wireGhost = m_grfxEngine.loadAlphaBitmap( "wireGhost" );

            // And then we load some non-transparent bitmaps to be used as the background tiles
            Bitmap tile = m_grfxEngine.loadBitmap( "tile1" );

            // We query the first tile the dimensions and assume the second tile has the same size
            int tileWidth  = tile.getWidth();
            int tileHeight = tile.getHeight();

            // Then we create a drawable bitmap that we draw the background upon
            m_backgroundBitmap = m_grfxEngine.getDrawableBitmap( WIDTH, HEIGHT );
            for ( int row = 0; row <= ( HEIGHT / tileHeight ); row++ )
            {
                for ( int col = 0; col < ( WIDTH / tileWidth ); col++ )
                {
                    tile.blitTo( m_backgroundBitmap, col * tileWidth, row * tileHeight );
                }
            }
            
            // We get the input engine implementation from GameFrame
            m_inputEngine = GameFrame.getInputEngine();
            
            // then we can get the default keyboard (just in case
            // there are several keyboard like devices attached).
            m_keyboard = m_inputEngine.getDefaultKeyboardDevice();
        }
        catch( FileNotFoundException noSuchFile )
        {
            System.out.println("FileNotFound:\n"+noSuchFile.getMessage() );
            GameFrame.exit( 0 );
        }
        catch( GameFrameException gameFrameError )
        {
            System.out.println("GameFrameException:\n"+gameFrameError.getMessage() );
            GameFrame.exit( 0 );
        }
    }
    
    /**
     * Builds and returns the applications menu.
     *@return The menubar for the application.
     */
    private MenuBar getMenuBarForApp()
    {
        MenuBar menuBar = new MenuBar();
        Menu menu       = new Menu( "File" );
        
        MenuItem item   = new MenuItem( "Exit" );
        item.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent event )
            {
                m_fRunning = false;
            }
        } );
        menu.add( item );
        
        menuBar.add( menu );
        
        return menuBar;
    }
    
    /**
     * Builds and returns the panel containing buttons.
     *@return Button panel.
     */
    private Panel getButtonPanel()
    {
        Panel panel = new Panel( new GridLayout( 1, 2 ) );
        Button button = new Button( "Pink" );
        button.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent event )
            {
                m_currentBitmap = m_pinkGhost;
            }
        } );
        panel.add( button );
        button = new Button( "Wire" );
        button.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent event )
            {
                m_currentBitmap = m_wireGhost;
            }
        } );
        panel.add( button );

        return panel;
    }
	
    /**
     * The method where the game should be executed. Animates the two ghosts moving
     * around against the tile background.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        // Define the start coordinates for the ghosts
        int currentX = WIDTH/2;
        int currentY = HEIGHT/2;
        m_currentBitmap = m_pinkGhost;
        
        // We check if user has pressed the ESC key by using
        // the wasPressed() method as it doesn't require the
        // key to be pressed at the moment we read it
        while( m_fRunning && !m_keyboard.wasKeyDown( KeyEvent.VK_ESCAPE ) )
        {
            if( m_keyboard.isKeyDown( KeyEvent.VK_W ) )
            {
                currentY--;
                if ( currentY < 0 )
                {
                    currentY = 0;
                }
            }
            if( m_keyboard.isKeyDown( KeyEvent.VK_S ) )
            {
                currentY++;
                if ( currentY > HEIGHT )
                {
                    currentY = HEIGHT;
                }
            }
            if( m_keyboard.isKeyDown( KeyEvent.VK_A ) )
            {
                currentX--;
                if ( currentX < 0 )
                {
                    currentX = 0;
                }
            }
            if( m_keyboard.isKeyDown( KeyEvent.VK_D ) )
            {
                currentX++;
                if ( currentX > WIDTH )
                {
                    currentX = WIDTH;
                }
            }
            
            m_backgroundBitmap.blitTo( 0, 0 );

            m_currentBitmap.blitTo( currentX, currentY );
            
            // Then we just flip the backbuffer with the frontbuffer
            // making the just drawn graphics visible on the screen
            m_grfxEngine.flip();
            //m_frame.repaint();

            // We rest for a while
            try
            {
                Thread.currentThread().sleep( 50 );
            }
            catch( InterruptedException ie )
            {
            }
        }
    }
}
