/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;
import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Using bitmap effects.
 * Example code how to use the getClone( EffectSettings ) method of a bitmap with the
 * ready made bitmap effect classes in the library.
 * Uses one pink ghost bitmap that is processed to 36x3 = 108 bitmaps by rotating
 * the bitmap in 10 degrees steps and changing its colour twice (original pink,
 * green and yellow).
 * Sidestep: Also shows how to setup input engine reading for keyboard.
 *
 *@author Pasi Kernen 30. Jan 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class GraphicsExample4 extends GraphicsExample3
{
    /** The number of rotations we take from 360 degrees turn. */
    public final int NUM_ROTATIONS = 36;
    /** The number of different colours we have. */
    public final int NUM_COLORS = 3;
    
    /** Rotated copies of the ghost. */
    protected Bitmap[][] m_ghostBitmaps;
    /** The current input engine. */
    protected InputEngine m_inputEngine;
    /** The default keyboard device. */
    protected KeyboardDevice m_keyboard;
    
    /**
     * Starts up the example application (creates an
     * instance and calls the run() method in the instance).
     */
    public static void main( String[] args )
    {
        GraphicsExample4 instance = new GraphicsExample4( args );
        instance.run();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given.
     *@param args The command line arguments.
     */
    public GraphicsExample4( String[] args )
    {
        super( args );
    }
    
    /**
     * Sets the GameFrame settings to the given settings object.
     * Overridden to change the title.
     *@param gameFrameSettings The settings to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
        super.setGameFrameSettings( gameFrameSettings );
        gameFrameSettings.setTitle( "Graphics Example 4" );
    }

    /**
     * Initializes the example. Uses the m_pinkGhost variable that gets
     * loaded in the GraphicsExample2's init() method.
     */
    protected void init()
    {
        super.init();
		
        m_ghostBitmaps = new Bitmap[NUM_ROTATIONS][NUM_COLORS];
        
        try
        {
            // We get the input engine
            m_inputEngine = GameFrame.getInputEngine();
            System.out.println( m_inputEngine.getDescription() );

            // Then we get the default keyboard device
            m_keyboard    = m_inputEngine.getDefaultKeyboardDevice();
            
            // We take our little ghost and spin it around a bit
            // and after each spin we give him another colour
            // thus building the two dimensional bitmap array...
            ColorReplacementEffect colourReplace =
				new ColorReplacementEffect();
			RotationEffect rotation =
				new RotationEffect();
			EffectGroup effectGroup =
				new EffectGroup();
			effectGroup.add( colourReplace );
			effectGroup.add( rotation );
            for ( int colorCounter = 0; colorCounter < NUM_COLORS; colorCounter++ )
            {
                colourReplace.reset();
                
                // Setup the colour mapping that changes the colour of the ghost
                switch( colorCounter )
                {
                    case 0:
                        // Leave the pink colour (ALPHA = 255, RED = 255, GREEN = 0, BLUE = 255)
                        break;
                    case 1:
                        // Replace pink with green (ALPHA = 255, RED = 0, GREEN = 255, BLUE = 0)
                        colourReplace.addColourMapping( 0xFFFF00FF, 0xFF00FF00 );
                        break;
                    case 2:
                        // Replace pink with yellow (ALPHA = 255, RED = 255, GREEN = 255, BLUE = 0)
                        colourReplace.addColourMapping( 0xFFFF00FF, 0xFFFFFF00 );
                        break;
                    default:
                        // Replace pink with white (ALPHA = 255, RED = 255, GREEN = 255, BLUE = 255)
                        colourReplace.addColourMapping( 0xFFFF00FF, 0xFFFFFFFF );
                        break;
                }
                
                // Rotate the coloured ghost
                for ( int i = 0; i < NUM_ROTATIONS; i++ )
                {
                    rotation.setRotationDegrees( i * ( 360 / NUM_ROTATIONS ) );
                    
                    // Here we actually apply the effects we have set up and get the resulting
                    // bitmap
                    m_ghostBitmaps[i][colorCounter] = m_pinkGhost.getClone( effectGroup );
                }
            }    
        }
        catch( GameFrameException gameFrameError )
        {
            System.out.println("GameFrameException:\n"+gameFrameError.getMessage() );
            GameFrame.exit( 0 );
        }
    }

    /**
     * The method where the game should be executed. Animates the two ghosts moving
     * around against the tile background.
     */
    protected void executeCodeThatUsesGameFrame()
    {
        // Setup ghost related variables
        int ghostX[]          = new int[NUM_GHOSTS];
        int ghostY[]          = new int[NUM_GHOSTS];
        int ghostDestX[]      = new int[NUM_GHOSTS];
        int ghostDestY[]      = new int[NUM_GHOSTS];
        float ghostRotation[] = new float[NUM_GHOSTS];
        boolean ghostRotDir[] = new boolean[NUM_GHOSTS];
        int ghostColor[]      = new int[NUM_GHOSTS];
        
        for ( int i = 0; i < NUM_GHOSTS; i++ )
        {
            ghostX[i]        = (int)   ( Math.random() * WIDTH );
            ghostY[i]        = (int)   ( Math.random() * HEIGHT );
            ghostDestX[i]    = (int)   ( Math.random() * WIDTH );
            ghostDestY[i]    = (int)   ( Math.random() * HEIGHT );
            ghostRotation[i] = (float) ( Math.random() * NUM_ROTATIONS );
            ghostColor[i]    = (int)   ( Math.random() * NUM_COLORS );
            ghostRotDir[i]   = (Math.random() < 0.5D );
        }
        
        // Continue until user presses ESC key
        while( !m_keyboard.wasKeyDown( java.awt.event.KeyEvent.VK_ESCAPE ) )
        {
            // First we draw the background image to the backbuffer
            m_backgroundBitmap.blitTo( 0, 0 );

            // Move, spin, recolour and draw each ghost
            for ( int i = 0; i < NUM_GHOSTS; i++ )
            {
                // If ghost reaches destination we come up with a new destination
                if ( ( ghostX[i] == ghostDestX[i] ) && ( ghostY[i] == ghostDestY[i]) )
                {
                    ghostDestX[i]  = (int) ( Math.random() * WIDTH );
                    ghostDestY[i]  = (int) ( Math.random() * HEIGHT );
                }
            
                // We move our towards its destination
                if ( ghostX[i] < ghostDestX[i] )
                {
                    ghostX[i]++;
                }
                if ( ghostX[i] > ghostDestX[i] )
                {
                    ghostX[i]--;
                }
                if ( ghostY[i] < ghostDestY[i] )
                {
                    ghostY[i]++;
                }
                if ( ghostY[i] > ghostDestY[i] )
                {
                    ghostY[i]--;
                }
                
                // Then we draw our ghosts
                m_ghostBitmaps[ (int) ghostRotation[i] ][ ghostColor[i] ].blitTo( ghostX[i], ghostY[i] );
            
                if ( ghostRotDir[i] )
                {
                    ghostRotation[i] += 0.2F;
                    if ( ghostRotation[i] >= NUM_ROTATIONS )
                    {
                        ghostRotation[i] = 0.0F;
                        ghostColor[i]    = (int) ( Math.random() * NUM_COLORS );     
                    }
                }
                else
                {
                    ghostRotation[i] -= 0.2F;
                    if ( ghostRotation[i] < 0.0F )
                    {
                        ghostRotation[i] = (float) ( NUM_ROTATIONS - 1 );
                        ghostColor[i]    = (int) ( Math.random() * NUM_COLORS );     
                    }
                }
            }

            
            // Then we just flip the backbuffer with the frontbuffer
            // making the just drawn graphics visible on the screen
            m_grfxEngine.flip();
        }
    }
}
