/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe;

import gameframe.util.ObjectPreserver;
import gameframe.graphics.GraphicsEngine;
import gameframe.sound.SoundEngine;
import gameframe.input.InputEngine;
import gameframe.util.Timer;
import gameframe.engines.EngineFactory;

import java.applet.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * The GameFrame librarys main entry point class.
 * 
 * The methods in this class are mainly used when the game
 * is starting, after that this class is normally not used
 * at all. For this reason many of the methods in this class aren't
 * optimized for speed, but for lesser memory usage. Only
 * exception is the getGameFramePath() method that might
 * be used by implementations to find out e.g. the location
 * of the default font bitmap. This one method is made to be as
 * fast as possible.
 *
 *@author Pasi Kernen 07.Apr 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AppletGameFrame extends GameFrame
{
    /** The Applet that is using the GameFrame.  */
    protected Applet m_applet              = null;

    /**
     * Constructs the GameFrame using default settings. Also initializes the
     * recommended engines for the current platform, so that when any of the
     * getXXXXEngine() methods is called no exceptions are thrown.
     *@param applet The Applet that is the applet using gameframe.
     *@exception SettingsException Thrown if a GameFrame instance allready exists.
     *           GameFrame instance can be destroyed by calling the finalize() method.
     *@exception GameFrameException Thrown if some engine fails to initialize.
     *@see GameFrameSettings
     *@author Pasi Keranen, based on suggestion by Donald Strong.
     *@since GameFrame for Java 0.9.1
     */
    public AppletGameFrame( Applet applet )
        throws SettingsException, GameFrameException
    {
        this( applet, null );
    }

    /**
     * Constructs the GameFrame using the given settings. Also initializes the
     * recommended engines for the current platform, so that when any of the
     * getXXXXEngine() methods is called no exceptions are thrown.
     *@param settings The settings to use inside the GameFrame library. If null is provided uses
     *                default settings.
     *@exception SettingsException Thrown if a GameFrame instance allready exists.
     *           GameFrame instance can be destroyed by calling the finalize() method.
     *@exception GameFrameException Thrown if some engine fails to initialize.
     *@see GameFrameSettings
     *@author Pasi Keranen, based on suggestion by Donald Strong.
     *@since GameFrame for Java 0.9.1
     */
    public AppletGameFrame( Applet applet, GameFrameSettings settings )
        throws SettingsException, GameFrameException
    {
        super( false );
        m_applet = applet;
        
        initFromSettings( settings );
        System.err.println("Applet GameFrame created and initialized" );
    }

    /**
     * Returns the absolute path to the main directory of the library that contains
     * the GameFrame entry point. The directory is found by getting the Applet document
     * base and adding "/" plus the engine package location to the URL.
     * and addig the string "gameframe" as subdirectory to each entry.
     *@return The directory that is the main directory of the GameFrame library.
     *@since GameFrame for Java 0.9.4
     */
    public String getGameFramePath()
    {
        return ""+m_applet.getCodeBase()+GAME_FRAME_PACKAGE_NAME.replace( '.', '/' );
    }
    
    /**
     * Returns the Applet that is using GF4J.
     *@return The Applet that is using GF4J.
     *@since GameFrame for Java 0.9.4
     */
    public Applet getApplet()
    {
        return m_applet;
    }

    /**
     * Returns the Image that is set as the default image icon for all graphics engine windows.
     *@return Default icon for engine windows.
     */
    public java.awt.Image getDefaultIcon()
    {
        return null;
    }
    
    /**
     * Filename that can be used to load the default font bitmap.
     *@return The default fontbitmap filename.
     */
    public String getDefaultFontBitmapFilename()
    {
        return "gameframe/images/DefaultFontBitmap";
    }
    
    /**
     * <p>Does the core job of adding all engine factories to the list of
     * runnable factories (engine factories that can run on the current
     * platform). Later added engine factories that have the same suitability
     * value will override engine factories that were added earlier.</p>
     *
     * <p>Note to engine implementors: Engine implementors that add new engines
     * for Applet use must add their engine factories in this method. Also this
     * is the method to override if you want to make your own custom GameFrame
     * subclass.</p>
     *
     * Overridden to use Applet friendly query tactics.
     *@since GameFrame for Java 0.9.4
     */
    protected void searchEngineFactories()
    {
        if ( gameframe.engines.jdk11x.CEngineFactory.isRunnableOnPlatform() )
        {
            addFactory( gameframe.appletengines.jdk11x.CEngineFactory.getInstance( m_settings ) );
        }
    }
}