/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe;

import gameframe.graphics.*;
import gameframe.sound.*;
import gameframe.input.*;
import gameframe.util.*;

import java.io.FileNotFoundException;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Base class for a game that can act both as application
 * and as an applet.
 *
 *@author Pasi Kernen 07. Apr 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class ApplicationAppletBase extends Applet implements Runnable
{
    /** The thread use to run the applet/application. */
    protected Thread m_thread;
    /** Whether the program is running as an applet (true) or application (false). */
    protected boolean m_fApplet = true;
    /** Whether the program is properly initialized or not. */
    protected boolean m_fInitialized = false;
    /** True when applet/application should be running, false when it should exit. */
    protected boolean m_fRunning = true;
    /** The arguments given to the program. */
    protected String[] m_args;

    /**
     * Starts up the example application (creates an
     * instance and calls the init() and start() methods of
     * the instance).
     */
    public static void main( String[] args )
    {
        ApplicationAppletBase instance = new ApplicationAppletBase( args );

        // When we are executed as an application we need to call
        // the init() method by ourselves followed by start().
        instance.init();
        instance.start();
    }
    
    /**
     * Constructs the example application and stores the command line
     * arguments given. This method is called when the program is ran
     * as a stand alone application.
     *@param args The command line arguments given.
     */
    public ApplicationAppletBase( String[] args )
    {
        m_args = args;

        // Setup the flag that indicates that we
        // are running as an application
        m_fApplet = false;
    }
    
    /**
     * Constructs the example Applet. This is used when running the
     * program as an Applet.
     */
    public ApplicationAppletBase()
    {
    }

    /**
     * Initializes the GameFrame for Java library.
     * Subclasses can override this to initialize themselves (but remember
     * to say "super.init()" as the first statement in the overriding init()
     * method).
     */
    public void init()
    {
        // Initially we are not initialized
        if ( m_fInitialized )
        {
            return;
        }
        
        // Check that we have the GF4J library installed
        if ( !checkLibrary() )
        {
            return;
        }

        // Create and setup the GF4J settings
        GameFrameSettings gameFrameSettings = new GameFrameSettings();
        setGameFrameSettings( gameFrameSettings );

        if ( m_args != null )
        {
            // Allow command line disabling of features when running as application
            gameFrameSettings.parseCommandLineArgs( m_args );
        }
        else
        {
            // Allow parameter disabling of features when running as Applet
            gameFrameSettings.setFromApplet( this );
        }

        try
        {
            if ( m_fApplet )
            {
                // Create the Applet GameFrame instance
                new AppletGameFrame( this, gameFrameSettings );
                
                // If the graphics engine is a component automatically add it to the Applet
                if ( gameFrameSettings.getScreenMode() == GameFrameSettings.SCREENMODE_COMPONENT )
                {
                    printToConsole("Screen mode is component, adding component to applet.");
                    setLayout( new BorderLayout() );
                    add( (Component) GameFrame.getGraphicsEngine(), BorderLayout.CENTER );
                }
            }
            else
            {
                // If the graphics mode is component force it to be windowed
                if ( gameFrameSettings.getScreenMode() == GameFrameSettings.SCREENMODE_COMPONENT )
                {
                    gameFrameSettings.setScreenMode( GameFrameSettings.SCREENMODE_WINDOWED );
                    printToConsole("Screen mode is component, forcing windowed.");
                }

                // Find the toplevel component and add the close listener
                if ( gameFrameSettings.getScreenMode() == GameFrameSettings.SCREENMODE_WINDOWED )
                {
                    
                }


                // Create the application GameFrame instance
                new GameFrame( gameFrameSettings );
            }
        }
        catch( SettingsException error )
        {
            // We allready had one existing GameFrame instance
            printToConsole( "Error in GameFrame creation:\n"+error.getMessage() );
            return;
        }
        catch( GameFrameException frameError )
        {
            // Something went wrong during the library creation
            printToConsole( ""+frameError.getMessage() );
            return;
        }
        
        // Print out the used GameFrame implementation description
        printToConsole( "Using:\n"+GameFrame.getDescription() );
        
        // Setup the flag to indicate that we are initialized
        m_fInitialized = true;
    }
    
    /**
     * Sets the GameFrame settings to the given settings object. Subclasses
     * must implement this method and set the wanted settings to the
     * settings object.
     *@param gameFrameSettings The settings object to be configured.
     */
    protected void setGameFrameSettings( GameFrameSettings gameFrameSettings )
    {
    }

    /**
     * Starts the applet/application running.
     */
    public void start()
    {
        if ( ( m_thread == null ) && ( m_fInitialized ) )
        {
            m_fRunning = true;
            m_thread = new Thread( this );
            m_thread.start();
        }
    }
    
    /**
     * Executes the example applet/application
     */
    public void run()
    {
        // We now have the graphics and sound engines and there should
        // be a black screen visible.
        mainLoop();
        
        // Then we close and cleanup after the program
        destroy();
    }
    
    /**
     * Overridden to display a text message when the applet has stopped execution
     * (user presses ESC etc.)
     *@param grfx The graphics context to draw to.
     */
    public void paint( Graphics grfx )
    {
        if ( m_fInitialized )
        {
            // Applet is still in initialized state, do the normal painting
            super.paint( grfx );
        }
        else
        {
            // Here we just pring a plain message that notifies the user that the applet has
            // finished running. This will not be seen in applications as the application window
            // closes automatically upon application exit.
            grfx.setColor( Color.green );
            grfx.drawString( "Finished Running", 100, 240 );
        }
    }
    
    /**
     * Stops the applets/applications running.
     */
    public void stop()
    {
        m_fRunning = false;
        
        // Wait until the thread finishes running
        while( m_thread.isAlive() )
        {
            // Give other threads change to run (in order to finish off their running)
            Thread.yield();
        }

        // Release the thread reference
        m_thread = null;
    }

    /**
     * Closes and cleans up after applet/application.
     */
    public void destroy()
    {
        if ( m_fApplet )
        {
            // We are running as applet so we just finalize the library
            if ( GameFrame.isInitialized() )
            {
                GameFrame.getInstance().finalize();
                m_fInitialized = false;
                
                // We make the background black after the GF4J application has finished
                setBackground( Color.black );
                
                // And we force a repaint that will now show whatever necessary
                repaint();
            }
        }
        else
        {
            // We are running as application, so we use the GameFrame's
            // exit method to finalize the library and stop the program
            GameFrame.exit( 0 );
        }
    }
    
    /**
     * The method where the game should be executed while flag m_fRunning
     * is true.
     * When this method exits, the applet/application
     * will stop/exit.
     */
    protected void mainLoop()
    {
    }

    /**
     * Checks if the library is installed or not. If not shows an error.
     *@return True if the library is installed, false if not.
     */
    private boolean checkLibrary()
    {
        // Build the message that is shown if the user doesn't have the
        // GameFrame for Java library installed
        String strNoLibraryMessage;
        strNoLibraryMessage  = "You need to have the GameFrame for\n";
        strNoLibraryMessage += "Java library installed in order to use\n";
        strNoLibraryMessage += "this program. The GameFrame for\n";
        strNoLibraryMessage += "Java library can be gotten from:\n";
        strNoLibraryMessage += "http://www.gamedev.net/hosted/javanerd";

        // Check if the user has the GameFrame for Java library
        try
        {
            // This is just a harmless access to a static method
            // in the GameFrame object. If this fails we don't have
            // the library installed.
            gameframe.GameFrame.getBuild();
        }
        catch( Error anyError )
        {
            printToConsole( strNoLibraryMessage );
            return false;
        }        
        catch(Exception anyException)
        {
            printToConsole( strNoLibraryMessage );
            return false;
        }
        
        // We have the library and it is correct version
        return true;
    }

    /**
     * Prints the given message to the console (even on browsers).
     *@param strText The message to be shown.
     */
    protected void printToConsole( String strText )
    {
        System.err.println( strText );
    }
}
