/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe;

import gameframe.util.ObjectPreserver;
import gameframe.graphics.GraphicsEngine;
import gameframe.sound.SoundEngine;
import gameframe.input.InputEngine;
import gameframe.util.Timer;
import gameframe.engines.EngineFactory;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * The GameFrame librarys main entry point class.
 * 
 * The methods in this class are mainly used when the game
 * is starting, after that this class is normally not used
 * at all. For this reason many of the methods in this class aren't
 * optimized for speed, but for lesser memory usage. Only
 * exception is the getGameFramePath() method that might
 * be used by implementations to find out e.g. the location
 * of the default font bitmap. This one method is made to be as
 * fast as possible.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class GameFrame
{
    /**
     * The major version number of this library.
     *@since GameFrame for Java 0.9
     */
    public static final int MAJOR_VERSION   = 0;
    
    /**
     * The minor version number of this library.
     *@since GameFrame for Java 0.9
     */
    public static final int MINOR_VERSION   = 9;
    
    /**
     * The build number of this library.
     *@since GameFrame for Java 0.9.1
     */
    public static final int BUILD_NUMBER    = 4;
    
    /**
     * Is this release a stable release.
     *@since GameFrame for Java 0.9.1
     */
    public static final boolean IS_STABLE_RELEASE = false;
    
    /** The package name of the main package for engines. */
    public static final String GAME_FRAME_PACKAGE_NAME = "gameframe";
    /** The package name of the main package for engines. */
    public static final String ENGINE_MAIN_PACKAGE_NAME = "gameframe.engines";
    /** The subdirectory name of the directory for engines. */
    public static final String ENGINE_SUBDIR_NAME = "engines";
    /** The engine factory class (used when finding engine factories). */
    public static final Class ENGINE_FACTORY_CLASS = EngineFactory.class;
    /** The engine factory's get instance method name. */
    public static final String ENGINE_FACTORY_GET_INSTANCE_METHOD_NAME = "getInstance";
    /** The engine factory's is runnable on current platform method name. */
    public static final String ENGINE_FACTORY_IS_RUNNABLE_METHOD_NAME = "isRunnableOnPlatform";
    /** The engine factory's get instance methods parameter classes. */
    public static final Class[] ENGINE_FACTORY_GET_INSTANCE_METHOD_PARAM_CLASSES = { GameFrameSettings.class };
    
    /**
     * The singleton instance of GameFrame class.
     *@since GameFrame for Java 0.9.1
     */
    protected static GameFrame mStatic_singletonInstance = null;
    
    /**
     * The main directory of GameFrame library. Defaults to null,
     * initialized upon first invocation of method getGameFramePath().
     *@since GameFrame for Java 0.9
     */
    protected String m_strGameFrameMainDirectory = null;
    
    /**
     * Flag that indicates if the path to the GameFrame classes has been
     * searched at least once. Defaults to false (classes not searched).
     *@since GameFrame for Java 0.9
     */
    protected boolean m_fGameFrameMainDirectorySearched = false;

    /**
     * The settings that are used to find the engine factory.
     *@since GameFrame for Java 0.9
     */    
    protected GameFrameSettings m_settings = null;

    /**
     * The current engine factory or null if not searched.
     *@since GameFrame for Java 0.9.1
     */
    protected EngineFactory m_currentEngineFactory = null;

    /**
     * Vector of the engine factories that are found.
     * Sorted so that the best implementation is found
     * at the index 0.
     *@since GameFrame for Java 0.9
     */
    protected Vector m_engineFactories = null;

    /** The recommended graphics engine. */
    protected static GraphicsEngine mStatic_graphicsEngine;

    /** The recommended sound engine. */
    protected static SoundEngine    mStatic_soundEngine;

    /** The recommended input engine. */
    protected static InputEngine    mStatic_inputEngine;

    /**
     * Returns whether GF4J library is initialized at the moment or not.
     *@return The true if the library is initialized, false if not.
     *@since GameFrame for Java 0.9.4
     */
    public static boolean isInitialized()
    {
        return ( mStatic_singletonInstance != null );
    }

    /**
     * Returns the singleton instance of the GameFrame class.
     *@return The singleton instance of the library main class if one has been created.
     *@exception NullPointerException Thrown if GameFrame has not been created.
     *@author Pasi Keranen, based on suggestion by Donald Strong.
     *@since GameFrame for Java 0.9.1
     */
    public static GameFrame getInstance()
    {
        if ( mStatic_singletonInstance == null )
        {
            throw new NullPointerException ("GameFrame must be created before calling getInstance");
        }

        return mStatic_singletonInstance;
    }

    /**
     * Constructs the GameFrame using default settings. Also initializes the
     * recommended engines for the current platform, so that when any of the
     * getXXXXEngine() methods is called no exceptions are thrown.
     *@exception SettingsException Thrown if a GameFrame instance allready exists.
     *           GameFrame instance can be destroyed by calling the finalize() method.
     *@exception GameFrameException Thrown if some engine fails to initialize.
     *@see GameFrameSettings
     *@author Pasi Keranen, based on suggestion by Donald Strong.
     *@since GameFrame for Java 0.9.1
     */
    public GameFrame()
        throws SettingsException, GameFrameException
    {
        this( null );
    }

    /**
     * Constructs the GameFrame using the given settings. Also initializes the
     * recommended engines for the current platform, so that when any of the
     * getXXXXEngine() methods is called no exceptions are thrown.
     *@param settings The settings to use inside the GameFrame library. If null is provided uses
     *                default settings.
     *@exception SettingsException Thrown if a GameFrame instance allready exists.
     *           GameFrame instance can be destroyed by calling the finalize() method.
     *@exception GameFrameException Thrown if some engine fails to initialize.
     *@see GameFrameSettings
     *@since GameFrame for Java 0.9.1
     */
    public GameFrame( GameFrameSettings settings )
        throws SettingsException, GameFrameException
    {
        initFromSettings( settings );
    }
    
    /**
     * For use by subclasses that don't want to initialize the GameFrame
     * in this classes constructors, but want to call the initFromSettings()
     * by themselves when they are ready.
     *@param fDoNotInitialize Ignored...
     */
    protected GameFrame( boolean fDoNotInitialize )
    {
    }
    
    /**
     * Initializes the library instance from the given settings object.
     *@param settings The settings to use inside the GameFrame library. If null is provided uses
     *                default settings.
     *@exception SettingsException Thrown if a GameFrame instance allready exists.
     *           GameFrame instance can be destroyed by calling the finalize() method.
     *@exception GameFrameException Thrown if some engine fails to initialize.
     *@see GameFrameSettings
     *@since GameFrame for Java 0.9.4
     */
    protected void initFromSettings( GameFrameSettings settings )
        throws SettingsException, GameFrameException
    {
        if (settings == null)
        {
            settings = new GameFrameSettings();
        }

        // Allow only one instance
        if ( mStatic_singletonInstance != null )
        {
            throw new SettingsException("Only single GameFrame instance is allowed."+
                "If you need instance with different settings finalize() the current instance first.");
        }

        // Make a copy so that user can't change settings with his reference
        m_settings                 = settings.getCopy();
        mStatic_singletonInstance  = this;
        ObjectPreserver.preserveObject( this );

        initEngines();
    }

    /**
     * Initializes the recommended engines for the current platform, so
     * that when any of the static getXXXXEngine() methods are called no
     * exceptions are thrown.
     *@see GameFrameSettings
     *@exception GameFrameException Thrown if some engine fails to initialize.
     *@author Pasi Keranen, based on suggestion by Donald Strong.
     *@since GameFrame for Java 0.9.2
     */
    protected void initEngines()
        throws GameFrameException
    {
        EngineFactory engineFactory = getRecommendedEngineFactory();
        mStatic_graphicsEngine  = engineFactory.getGraphicsEngine();
        mStatic_soundEngine     = engineFactory.getSoundEngine();
        mStatic_inputEngine     = engineFactory.getInputEngine();
    }



    /**
     * Upon invocation decides which of the engine factory implementations
     * meets the set requirements and suits the current platform the best
     * and returns it. Also registers the found engine factory and returns
     * that engine factory upon every later invocation.
     *@return The best engine factory for this platform.
     *@since GameFrame for Java 0.9
     */
    protected EngineFactory getRecommendedEngineFactory()
    {
        if ( m_currentEngineFactory == null )
        {
            // Enumerate all engine factories
            m_engineFactories = new Vector(5,5);
            searchEngineFactories();

            m_currentEngineFactory = (EngineFactory) m_engineFactories.firstElement();

            // Try to do some cleanup of memory
            m_engineFactories.removeAllElements();
            m_engineFactories = null;
            System.gc();
        }
        
        return m_currentEngineFactory;
    }

    /**
     * Returns the absolute path to the main directory of the library that contains
     * the GameFrame public classes. Uses the singleton instance reference and internal
     * overridable version of the method.
     *@return The directory that is the main directory of the GameFrame library or null if not found.
     *@since GameFrame for Java 0.9
     */
    public String getGameFramePath()
    {
        // Check if we allready know the main directory of GameFrame
        if ( m_fGameFrameMainDirectorySearched )
        {
            return m_strGameFrameMainDirectory;
        }

        String strFoundPath = "";
        // Find the location of GameFrame classes from the classpath
        String classPath = (String)System.getProperty("java.class.path");
        StringTokenizer tokenizer = new StringTokenizer( classPath, ";", false );
        while(true)
        {
            try
            {
                String strPathEntry = tokenizer.nextToken();
                if ( strPathEntry.endsWith( File.separator ) )
                {
                    strPathEntry = strPathEntry.substring( 0, strPathEntry.length() - 1 );
                }

                File pathEntry = new File( strPathEntry+File.separator+GAME_FRAME_PACKAGE_NAME);

                if ( (pathEntry.exists()) || (pathEntry.isDirectory()))
                {
                    strFoundPath = pathEntry.toString();
                    
                    if ( strFoundPath.endsWith(File.separator) )
                    {
                        strFoundPath =
                            strFoundPath.substring(
                                0, strFoundPath.length() - 1 );
                    }
                    break;
                }
            }
            catch(NoSuchElementException nsee)
            {
                break;
            }
        }

        m_strGameFrameMainDirectory = strFoundPath;
        m_fGameFrameMainDirectorySearched = true;
        
        return m_strGameFrameMainDirectory;
    }

    /**
     * Returns the Image that is set as the default image icon for all graphics engine windows.
     *@return Default icon for engine windows.
     */
    public java.awt.Image getDefaultIcon()
    {
        return java.awt.Toolkit.getDefaultToolkit().getImage(
        getGameFramePath()+java.io.File.separator+
        "Images"+java.io.File.separator+"GameFrameIcon.gif" );
    }
    
    /**
     * Filename that can be used to load the default font bitmap.
     *@return The default fontbitmap filename.
     */
    public String getDefaultFontBitmapFilename()
    {
        return getGameFramePath()+java.io.File.separator+"images"+java.io.File.separator+"DefaultFontBitmap";
    }
        
    /**
     * <p>Does the core job of adding all engine factories to the list of
     * runnable factories (engine factories that can run on the current
     * platform). Later added engine factories that have the same suitability
     * value will override engine factories that were added earlier.</p>
     *
     * <p>Note to engine implementors: Engine implementors that add new engines
     * must add their engine factories in this method. Also this is the method
     * to override if you want to make your own custom GameFrame subclass.</p>
     *@since GameFrame for Java 0.9
     */
    protected void searchEngineFactories()
    {
        // Find subdirectories under the engines main package
        Vector subDirectories = findSubdirectories(
			getGameFramePath()+File.separator+ENGINE_SUBDIR_NAME );

        // Try to find class files in the subdirs that implement EngineFactory interface
        Vector engineFactoryClasses = findClassesInSubdirectoriesThatImplementInterface(
            ENGINE_MAIN_PACKAGE_NAME, subDirectories, ENGINE_FACTORY_CLASS);

        // Try to instantiate the found engine factory classes by calling their static getInstance(settings) method.
        addEngineFactoryClasses( engineFactoryClasses);
    }

    /**
     * Finds the subdirectories under the given directory name.
     *@param The directory whose subdirectories are searched.
     *@return The subdirectories (as java.io.File) of the given directory.
     *@since GameFrame for Java 0.9.1
     */
    protected Vector findSubdirectories( String strDirectory )
    {
        File mainDir = new File( strDirectory );
        String[] astrFilesInDir = mainDir.list();

        // Find subdirs
        Vector subDirectories = new Vector(5, 5);
        for (int i = 0; i < astrFilesInDir.length; i++)
        {
            File item = new File( strDirectory+File.separator+astrFilesInDir[i] );
            if ( item.isDirectory() )
            {
                subDirectories.addElement(item);
            }
        }

        return subDirectories;
    }

    /**
     * Finds the classes that implement the given interface and that reside in one of
     * the subdirectories in the given list of directories.
     *@param strParentDirectoryName The name of the parent directory (used as the main package name).
     *@param subDirectories The subdirectories that are searched through.
     *@param targetInterface The interface whose implementors are to be searched.
     *@return Class objects that meet the aforementioned criteria.
     *@since GameFrame for Java 0.9.1
     */
    protected Vector findClassesInSubdirectoriesThatImplementInterface(
        String strParentDirectoryName, Vector subDirectories, Class targetInterface )
    {
        Vector foundClasses = new Vector(5,5);

        // Go through all the given subdirectories
        for ( Enumeration enum = subDirectories.elements(); enum.hasMoreElements(); )
        {
            File subDir = (File) enum.nextElement();

            // Go through the files in the subdirectory
            String[] astrItemsInSubDir = subDir.list();
            for (int i = 0; i < astrItemsInSubDir.length; i++)
            {
                // Try to find "*.class" ending files
                if ( astrItemsInSubDir[i].endsWith(".class") )
                {
                    // Leave out the ".class" from the end of the name
                    int length = astrItemsInSubDir[i].length();
                    length -= 6;
                    String strClassName = astrItemsInSubDir[i].substring(0, length);

                    // Build the class name to try out
                    strClassName = strParentDirectoryName+"."+subDir.getName()+"."+strClassName;
                    try
                    {
                        // Get the class for the name
                        Class factoryClass = Class.forName(strClassName);

                        // Check if the class implements the wanted interface
                        Class[] aImplementedInterfaces = factoryClass.getInterfaces();
                        for ( int j = 0; j < aImplementedInterfaces.length; j++)
                        {
                            if ( aImplementedInterfaces[j] == targetInterface )
                            {
                                // Add the class to found classes
                                foundClasses.addElement( factoryClass );
                            }
                        }
                    }
                    catch(ClassNotFoundException noClass)
                    {
                    }
                    catch(NoClassDefFoundError noClassDef)
                    {
                    }
                    catch(IllegalAccessError noAccess)
                    {
                    }
                }
            }
        }

        return foundClasses;
    }

    /**
     * Tries to instantiate the given EngineFactory classes by invoking their
     * static "getInstance(GameFrameSettings)" methods. After successfull
     * instantiation the EngineFactory class is added to the sorted list of
     * engine factories.
     *@param engineFactoryClasses The EngineFactory classes to be instantiated and added.
     *@since GameFrame for Java 0.9.1
     */
    protected void addEngineFactoryClasses( Vector engineFactoryClasses )
    {
        // Setup the GameFrameSettings parameter for the getInstance methods
        Object[] aParams = new Object[1];
        aParams[0] = m_settings;

        // Go through all of the engine factory classes
        for (Enumeration enum = engineFactoryClasses.elements(); enum.hasMoreElements();)
        {
            Class factoryClass = (Class) enum.nextElement();

            try
            {
                // Check that the engine factory is indeed runnable on current platform
                Method isRunnableOnPlatformMethod = factoryClass.getMethod( ENGINE_FACTORY_IS_RUNNABLE_METHOD_NAME, null );
                Boolean isRunnable = (Boolean) isRunnableOnPlatformMethod.invoke( isRunnableOnPlatformMethod.getClass(), null );

                // Only try to instantiate if runnable on current platform
                if ( ( isRunnable == null ) || ( isRunnable.booleanValue() ) )
                {
                    // Try to invoke the static "getInstance" method
                    Method getInstanceMethod = factoryClass.getMethod(ENGINE_FACTORY_GET_INSTANCE_METHOD_NAME, ENGINE_FACTORY_GET_INSTANCE_METHOD_PARAM_CLASSES);
                    EngineFactory factory = (EngineFactory) getInstanceMethod.invoke(getInstanceMethod.getClass(), aParams);

                    // Add the factory to the sorted list of factories
                    addFactory(factory);
                }
            }
            catch(NoSuchMethodException noSuchMethod)
            {
            }
            catch(IllegalAccessException illegalAccess)
            {
            }
            catch(IllegalArgumentException illegalArgument)
            {
            }
            catch(InvocationTargetException invokationTarget)
            {
            }
            catch(ClassCastException returnValueWrong)
            {
            }
        }
    }

    /**
     * Adds the given factory instance to the list of factories if
     * the given factory is runnable on this platform.
     *@param factory The engine factory to be added to list of factories.
     *@since GameFrame for Java 0.9
     */
    protected void addFactory( EngineFactory factory )
    {
        // If this engine is not runnable on this platform, don't add it
        if (
            factory.getSuitabilityForPlatform() ==
            EngineFactory.SUITABILITY_NOT_SUITED )
        {
            return;
        }

        // This is the first engine added, just add it to the list
        if (m_engineFactories.size() == 0)
        {
            m_engineFactories.addElement(factory);
            return;
        }

        // We have engines in the list, find place in the list for this new one
        for (int i = 0; i < m_engineFactories.size(); i++)
        {
            EngineFactory existingFactory = (EngineFactory)
                m_engineFactories.elementAt(i);
            if (
                factory.getSuitabilityForPlatform() >=
                existingFactory.getSuitabilityForPlatform() )
            {
                m_engineFactories.insertElementAt(factory, i);
                return;
            }
        }

        // This engine is the worst one found, add it to the end of the list.
        m_engineFactories.addElement(factory);
    }

    /**
     * Returns true if this version is a stable release (not prerelease, beta or
     * alpha or anything like that) meant for anyones use. This of course can't
     * quarantee bugfree implementation, but it should notify that the library
     * has been tested and found working on most platforms. This means that any
     * needed pre-release testing has been done.
     *@return True if this is a stable library version, false if this is a
     *        experimental library version.
     *@since GameFrame for Java 0.9.1
     */
    public static boolean isStableVersion()
    {
        return IS_STABLE_RELEASE;
    }

    /**
     * Returns the major version of the GameFrame library. This will be
     * incremented by one upon each new public stable release that will change
     * existing functions in a major way or adds major new functionality. Major
     * versions might break downwards compatibility, but are don't allways do
     * that. Breaking downwards compatibility must allways be done in a major
     * version update and must be well documented. Backwards compatibility
     * should be preserved for as long as possible.
     *@return GameFrame library's major version.
     *@since GameFrame for Java 0.9
     */
    public static int getMajorVersion()
    {
        return MAJOR_VERSION;
    }

    /**
     * Returns the minor version of the GameFrame library. This will be
     * incremented by one upon each new public release that adds (or changes in
     * a minor way) functionality to the library's interfaces. This means
     * versions that are bugfix versions or include new engine implementations
     * for the existing interfaces. Upon new major version release this version
     * number will be reset to 0. The game shouldn't need to check for minor
     * version, as minor versions are compatible between eachother as far as
     * the interfaces go. But if you know that you use a feature in the library
     * that was fully functional only after a certain minor release, you can
     * check that the library's minor version meets your requirements. The game
     * should allways check for "at least version" as the game library should
     * be downwards compatible inside the same major version.
     *@return GameFrame library's minor version.
     *@since GameFrame for Java 0.9
     */
    public static int getMinorVersion()
    {
        return MINOR_VERSION;
    }

    /**
     * Returns the build number of the GameFrame library.
     * This is incremented by each build, one minor version can have
     * (theoretically) up to 2147483647 builds (that is the limit of the used
     * integer number). A build is considered being a compiling of edited source
     * code that is given to person/persons outside the development team. This
     * means that there is no need to count the number of compiles inside the
     * development team.
     *@return The build number of the library.
     *@since GameFrame for Java 0.9.1
     */
    public static int getBuild()
    {
       return BUILD_NUMBER;
    }

// Methods for handling the recommended engine factory
//////////////////////////////////////////////////////

    /**
     * Returns the short name (max 8 characters) of the currently used
     * implementation e.g. "Java 1.1" or "MSDX".
     *@return The short name of the engine factory.
     *@since GameFrame for Java 0.9
     */
    public static String getName()
    {
        return mStatic_singletonInstance.m_currentEngineFactory.getName();
    }

    /**
     * Returns the (possibly multiline) description of the currently used
     * implementation.
     *@return A description of the engine factory.
     *@since GameFrame for Java 0.9
     */
    public static String getDescription()
    {
        return mStatic_singletonInstance.m_currentEngineFactory.getDescription();
    }

    /**
     * Returns the suitability of the currently used implementation for
     * the current platform as one of the enumerated
     * EngineFactory.SUITABILITY_xxx values.
     *@return Suitability of the implementation for this platform.
     *@since GameFrame for Java 0.9
     */
    public static int getSuitabilityForPlatform()
    {
        return mStatic_singletonInstance.m_currentEngineFactory.getSuitabilityForPlatform();
    }

    /**
     * Returns the recommended graphics engine for the current platform or null
     * if graphics engine was for some reason not created. If no exception is
     * thrown when the GameFrame instance is created this method is quaranteed
     * to return a non-null value.
     *@return Recommended graphics engine for the current platform or null
     *        if not initialized.
     *@since GameFrame for Java 0.9.1
     */
    public static GraphicsEngine getGraphicsEngine()
    {
        return mStatic_graphicsEngine;
    }

    /**
     * Returns the recommended input engine for the current platform or null
     * if input engine was for some reason not created. If no exception is
     * thrown when the GameFrame instance is created this method is quaranteed
     * to return a non-null value.
     *@return Recommended input engine for the current platform or null
     *        if not initialized.
     *@since GameFrame for Java 0.9.1
     */
    static public InputEngine getInputEngine()
    {
        return mStatic_inputEngine;
    }

    /**
     * Returns the recommended sound engine for the current platform or null
     * if sound engine was for some reason not created. If no exception is
     * thrown when the GameFrame instance is created this method is quaranteed
     * to return a non-null value.
     *@return Recommended sound engine for the current platform or null
     *        if not initialized.
     *@since GameFrame for Java 0.9.1
     */
    public static SoundEngine getSoundEngine()
    {
        return mStatic_soundEngine;
    }

    /**
     * Returns all the fullscreen graphics modes supported by recommended
     * graphics engine. If the recommended graphics engine supports only
     * windowed modes, this method returns null.
     *@return Supported fullscreen modes or null.
     *@since GameFrame for Java 0.9.1
     */
    public static String[] getSupportedResolutions()
    {
        return mStatic_singletonInstance.m_currentEngineFactory.getSupportedResolutions();
    }

    /**
     * Returns a new timer instance. Several timers can be created with this
     * method. If a native timer can't be for some reason created, this method
     * returns the basic implementation CTimer from the GameFrame.jdk11x
     * package. If even this doesn't succeed throws GameFrameException
     * exception.
     *@return A new timer instance.
     *@exception GameFrameException Thrown if timer creation fails.
     *@since GameFrame for Java 0.9
     */
    public static Timer createTimer()
        throws GameFrameException
    {
        return mStatic_singletonInstance.m_currentEngineFactory.createTimer();
    }

    /**
     * Exits cleanly from application that has used GameFrame by first
     * finalizing the whole library (that ensures all native resources are
     * freed) and then calling carbage collector and waiting for a while
     * and THEN exiting the application just like System.exit() method.
     *@param iValue The integer value to be returned by the exit method
     *       to the native system.
     */
    public static void exit( int iValue )
    {
        // Try to finalize the GameFrame library.
        if ( mStatic_singletonInstance != null )
        {
            mStatic_singletonInstance.finalize();

            System.gc();

            try
            {
                Thread.currentThread().sleep( 100 );
            }
            catch( InterruptedException ie )
            {
            }

        }

        System.gc();
        System.exit( iValue );
    }

    /**
     * Finalizes this GameFrame instance and finalizes the whole GameFrame
     * library. In short resets the GameFrame for Java library.
     *@since GameFrame for Java 0.9
     */
    public void finalize()
    {
        // Try to finalize the currently used factory
        if ( m_currentEngineFactory != null )
        {
            m_currentEngineFactory.finalize();
        }

        // Reset all static variables
        m_currentEngineFactory              = null;
        mStatic_singletonInstance               = null;
        m_strGameFrameMainDirectory       = null;
        m_fGameFrameMainDirectorySearched = false;
        ObjectPreserver.unpreserveObject( this );

        System.gc();
    }
}