/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe;

import gameframe.graphics.GraphicsEngine;

/**
 * This class offers the methods for a game developer to define what kind
 * of features he wants to be enabled (fullscreen or non-fullscreen graphics
 * modes, proprietary engines or plain Java engines etc.) the resolution of
 * the created graphics engine.<br>
 * <br>
 * Default values are:<br><ul>
 * <li>Fullscreen modes allowed.</li>
 * <li>Microsoft Java allowed.</li>
 * <li>Java Media Framework 1.1 allowed.</li>
 * <li>Native libraries allowed.</li>
 * <li>Required resolution "640x480".</li>
 * <li>16 bits per pixel.</li>
 * <li>No preferred refresh rate.</li></ul>
 *
 *@author Pasi Kernen 29. Oct 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class GameFrameSettings implements Cloneable
{
    /**
     * Enumerated mode for the graphics engine, means fullscreen graphics mode.
     * In fullscreen mode the GF4J takes over the whole screenspace for itself.
     * 
     * If the implementation doesn't support fullscreen mode, the fullscreen
     * mode is emulated (with a background window that fills the screen and a
     * centered smalled window that is of the required size).
     * 
     * If the implementation supports fullscreen mode, but the user asks for a
     * mode that is not supported, the fullscreen emulation should be used.
     * 
     *@since GameFrame for Java 0.9.4
     */
    public static final int SCREENMODE_FULLSCREEN = 0;

    /**
     * Enumerated mode for the graphics engine, means windowed graphics mode.
     * In windowed mode the GF4J displays a window with title frames and all.
     * 
     * If the implementation doesn't support windowed mode when such is required
     * the implementation shouldn't be used.
     *@since GameFrame for Java 0.9.4
     */
    public static final int SCREENMODE_WINDOWED = 1;

    /**
     * Enumerated mode for the graphics engine, means component graphics mode.
     * Component mode means that the GF4J doesn't show any window by default
     * but the user must cast the returned GraphicsEngine to a java.awt.Component
     * and add it to a container of his own.
     * 
     * If the implementation doesn't support component mode when such is required
     * the implementation shouldn't be used.
     *@since GameFrame for Java 0.9.4
     */
    public static final int SCREENMODE_COMPONENT = 2;

    /**
     * Graphics mode for the graphics engine. One of the SCREENMODE_xxx values.
     */
    private int m_eScreenMode = SCREENMODE_FULLSCREEN;

    /**
     * Flag that indicates if the Microsoft Java VM specific
     * implementations of engines are allowed. Defaults to true (Microsoft
     * Java VM specific engines allowed).
     *
     *@since GameFrame for Java 0.9
     */
    private boolean m_fAllowMSJava = true;

    /**
     * Flag that indicates if the Java Media Framework 1.1 specific
     * implementations of engines are allowed. Defaults to true (JMF 1.1
     * engines allowed).
     *
     *@since GameFrame for Java 0.9
     */
    private boolean m_fAllowJmf11 = true;

    /**
     * Flag that indicates if the native code implementations
     * of engines are allowed. Defaults to true (native engines
     * allowed).
     * 
     *@since GameFrame for Java 0.9
     */
    private boolean m_fAllowNative = true;

    /**
     * The resolution that is wished from the graphics engine implementation.
     * Must be set before engine factory enumeration occurs (before first invocation
     * of getRecommendedEngineFactory() or getFactories() methods) to have any effect.
     * Defaults to "640x480" (640x480 resolution, no preferred color bit depth,
     * no preferred refresh rate).
     * 
     *@since GameFrame for Java 0.9
     */
    private String m_requiredResolution = "640x480";
    
    /**
     * The required resolutions width.
     * Defaults to 640 pixels.
     * 
     *@since GameFrame for Java 0.9
     */
    private int m_width = 640;

    /**
     * The required resolutions height.
     * Defaults to 480 pixels.
     * 
     *@since GameFrame for Java 0.9
     */
    private int m_height = 480;

    /**
     * The required fullscreen graphics engines bits per pixel value.
     * Defaults to the value of GraphicsEngine.DEFAULT_COLOURDEPTH.
     * 
     *@since GameFrame for Java 0.9
     */
    private int m_bpp = GraphicsEngine.DEFAULT_COLOURDEPTH;

    /**
     * The required fullscreen graphics engines refresh rate.
     * Deafults to the value of GraphicsEngine.NO_PREFERRED_REFRESHRATE.
     * 
     *@since GameFrame for Java 0.9
     */
    private int m_refresh = GraphicsEngine.NO_PREFERRED_REFRESHRATE;
    
    /**
     * The title of the window used to show the graphics engine
     * (in component modes this is ignored). Defaults to
     * GraphicsEngine.DEFAULT_TITLE.
     */
    private String m_strTitle = GraphicsEngine.DEFAULT_TITLE;


    /**
     * <p>Sets the resolution that is required from the graphics engine implementation.
     * Must be invoked before engine factory enumeration occurs (before first invocation
     * of getRecommendedEngineFactory() or getFactories() methods) to have any effect.
     * Resolution is represented as string that have one of the following syntaxes
     * (the values used are purely examplary):</p>
     * <ul><li>640x480 (640x480 resolution, default color bit depth, no preferred refresh rate)</li>
     * <li>640x480x16 (640x480 resolution, 16 bit color depth, no preferred refresh rate).</li>
     * <li>640x480@60 (640x480 resolution, default color bit depth, 60Hz refresh rate).</li>
     * <li>640x480x16@60 (640x480 resolution, 16 bit color depth, 60Hz refresh rate).</li></ul>
     * 
     * <p>Note: The bitdepth (if defined) must be one of the GameFrameSettings.COLOURDEPTH_xxBITS
     * values, otherwise it is ignored and the default bits per pixel value (defined in
     * GraphicsEngine.DEFAULT_COLOURDEPTH) is used.</p>
     * 
     * <p>Note: The bitdepth & refresh rate are usually ignored for non-fullscreen graphics
     * engine implementations and the desktop's refresh rate & bitdepth are used.</p>
     * 
     *@see #getRequiredResolution()
     *@param requiredResolution The resolution the graphics engine must support.
     *@since GameFrame for Java 0.9
     */
    public final void setRequiredResolution( String requiredResolution )
    {
        m_requiredResolution = requiredResolution;
        
        // Get the indexes of the separators
        int idxSeparator = requiredResolution.indexOf('x');
        int idxBpp       = requiredResolution.indexOf('x', idxSeparator+1);
        int idxRefresh   = requiredResolution.indexOf('@');
        
        // Set initial values
        m_height  = 0;
        m_refresh = GraphicsEngine.NO_PREFERRED_REFRESHRATE;
        m_bpp     = GraphicsEngine.DEFAULT_COLOURDEPTH;

        // Parse the values
        m_width   = Integer.parseInt(requiredResolution.substring(0, idxSeparator));
        if (idxRefresh != -1)
        {
            m_refresh = Integer.parseInt(requiredResolution.substring(idxRefresh+1));

            if (idxBpp != -1)
            {
                m_bpp = checkBppValue(  Integer.parseInt( requiredResolution.substring(idxBpp+1, idxRefresh) ) );
                m_height = Integer.parseInt(requiredResolution.substring(idxSeparator+1, idxBpp));
            }
            else
            {
                m_height = Integer.parseInt(requiredResolution.substring(idxSeparator+1, idxRefresh));
            }
        }
        else
        {
            if (idxBpp != -1)
            {
                m_bpp    = checkBppValue( Integer.parseInt( requiredResolution.substring(idxBpp+1) ) );
                m_height = Integer.parseInt(requiredResolution.substring(idxSeparator+1, idxBpp));
            }
            else
            {
                m_height = Integer.parseInt(requiredResolution.substring(idxSeparator+1));
            }
        }
    }

    /**
     * Returns allways a checked and valid bits per pixel value. If the given value
     * is not valid it will be replaced with the default bits per pixel value
     * (GraphicsEngine.DEFAULT_COLOURDEPTH).
     *@param bpp The value to be checked for validity.
     *@return A checked and valid bits per pixel value.
     */
    private int checkBppValue( int bpp )
    {
        if ( ( bpp != GraphicsEngine.COLOURDEPTH_16BITS ) &&
             ( bpp != GraphicsEngine.COLOURDEPTH_24BITS ) &&
             ( bpp != GraphicsEngine.COLOURDEPTH_32BITS ) )
        {
            bpp = GraphicsEngine.DEFAULT_COLOURDEPTH;
        }
        
        return bpp;
    }

    /**
     * Returns the resolution that is required from the graphics engine implementation.
     * 
     *@see #setRequiredResolution(java.lang.String)
     *@return The resolution the wanted graphics engine must support.
     *@since GameFrame for Java 0.9
     */
    public final String getRequiredResolution()
    {
        return m_requiredResolution;
    }

    /**
     * Returns the bits per pixel value that is required from the graphics
     * engine implementation. Normally only fullscreen engine implementations
     * can use this value, windowed graphics engine implementations usually
     * simply ignore this value and use the current desktop BPP value. If no
     * specific value is required returns the value of GraphicsEngine.NO_PREFERRED_BPP.
     * 
     *@see #setRequiredResolution(java.lang.String)
     *@return Required fullscreen graphics engines BPP value.
     *@since GameFrame for Java 0.9
     */
    public final int getRequiredBitdepth()
    {
        return m_bpp;
    }

    /**
     * Returns the refresh rate that is required from the graphics engine
     * implementation. Normally only fullscreen engine implementations can use
     * this value, windowed graphics engine implementations usually
     * simply ignore this value and use the current desktop BPP value. If no
     * specific value is required returns the value of
     * GraphicsEngine.NO_PREFERRED_REFRESHRATE.
     * 
     *@see #setRequiredResolution(java.lang.String)
     *@return Required fullscreen graphics engines refresh rate.
     *@since GameFrame for Java 0.9
     */
    public final int getRequiredRefreshRate()
    {
        return m_refresh;
    }

    /**
     * Returns the required width of the resolution.
     * 
     *@see #setRequiredResolution(java.lang.String)
     *@return Required width.
     *@since GameFrame for Java 0.9
     */
    public final int getRequiredResolutionWidth()
    {
        return m_width;
    }

    /**
     * Returns the required height of the resolution.
     * 
     *@see #setRequiredResolution(java.lang.String)
     *@return Required height.
     *@since GameFrame for Java 0.9
     */
    public final int getRequiredResolutionHeight()
    {
        return m_height;
    }

    /**
     * Sets the required screen mode for the graphics engine.
     * Default value is SCREENMODE_FULLSCREEN.
     *@see #getScreenMode
     *@param eMode One of the SCREENMODE_xxx values.
     *@since GameFrame for Java 0.9
     */
    public final void setScreenMode( int eMode )
    {
        if ( ( eMode < SCREENMODE_FULLSCREEN ) || ( eMode > SCREENMODE_COMPONENT ) )
        {
            throw new IllegalArgumentException(
                "You must use one of the SCREENMODE_xxx values when setting the screen mode." );
        }
        
        m_eScreenMode = eMode;
    }

    /**
     * Returns the flag that indicates if fullscreen modes are allowed.
     *@see #setScreenMode(int)
     *@return True if fullscreen modes are allowed.
     *@since GameFrame for Java 0.9
     */
    public final int getScreenMode()
    {
        return m_eScreenMode;
    }
    
    /**
     * Sets the title that is shown when possible for the graphics engine.
     *@param strTitle The title that is to be shown.
     *@since GameFrame for Java 0.9.4
     */
    public void setTitle( String strTitle )
    {
        m_strTitle = strTitle;
    }
    
    /**
     * Reutnrs the title that is shown when possible for the graphics engine.
     *@return The title that is to be shown.
     *@since GameFrame for Java 0.9.4
     */
    public String getTitle()
    {
        return m_strTitle;
    }

    /**
     * Sets the flag that indicates if Microsoft Java VM specific implementations are allowed.
     *@see #isMsJavaAllowed
     *@param fAllow True if Microsoft Java VM specific libraries are allowed, false if not.
     *@since GameFrame for Java 0.9
     */
    public final void setAllowMsJava( boolean fAllow )
    {
        m_fAllowMSJava = fAllow;
    }

    /**
     * Returns the flag that indicates if Microsoft Java VM specific implementations are allowed.
     *@see #setAllowMsJava(boolean)
     *@return True if Microsoft Java VM specific libraries are allowed, false if not.
     *@since GameFrame for Java 0.9
     */
    public final boolean isMsJavaAllowed()
    {
        return m_fAllowMSJava;
    }

    /**
     * Sets the flag that indicates if Java Media Framework 1.1 specific implementations are allowed.
     *@see #isJmf11Allowed
     *@param fAllow True if Java Media Framework 1.1 specific implementations are allowed, false if not.
     *@since GameFrame for Java 0.9
     */
    public final void setAllowJmf11( boolean fAllow )
    {
        m_fAllowJmf11 = fAllow;
    }

    /**
     * Returns the flag that indicates if Java Media Framework 1.1 specific
     * implementations are allowed.
     *@see #setAllowJmf11(boolean)
     *@return True if Java Media Framework 1.1 specific implementations are
     *        allowed, false if not.
     *@since GameFrame for Java 0.9
     */
    public final boolean isJmf11Allowed()
    {
        return m_fAllowJmf11;
    }

    /**
     * Sets the flag that indicates if native code implementations are allowed.
     *@see #isNativeAllowed
     *@param fAllow True if native libraries are allowed, false if not.
     *@since GameFrame for Java 0.9
     */
    public final void setAllowNative( boolean fAllow )
    {
        m_fAllowNative = fAllow;
    }

    /**
     * Returns the flag that indicates if native code implementations are allowed.
     *@see #setAllowNative(boolean)
     *@return True if native libraries are allowed, false if not.
     *@since GameFrame for Java 0.9
     */
    public final boolean isNativeAllowed()
    {
        return m_fAllowNative;
    }

    /**
     * <p>Queries the given applet for parameters and sets the contained settings 
     * accordingly. This method should be called AFTER you have set your own settings in
     * order to allow users to override your settings. Recognizes the following Applet
     * parameters:</p>
     * <ul>
     * <li>"forcewin" (forces usage of windowed graphics mode)</li>
     * <li>"forcefull" (forces usage of fullscreen graphics mode, if mode is available)</li>
     * <li>"nomsjava" (disallows usage of Microsoft Java specific engines)</li>
     * <li>"nojmf" (disallows usage of Java Media Framework specific engines)</li>
     * <li>"nonative" (disallows usage of Java Native Interface engines)</li>
     * </ul>
     *
     *@param applet The applet queried for parameters.
     *@since GameFrame for Java 0.9.4
     */
    public final void setFromApplet( java.applet.Applet applet )
    {
        if ( applet.getParameter( "nofs" ) != null )
        {
            if ( m_eScreenMode == SCREENMODE_FULLSCREEN )
            {
                m_eScreenMode = SCREENMODE_WINDOWED;
            }
        }

        if ( applet.getParameter( "nomsjava" ) != null )
        {
            setAllowMsJava(false);
        }
            
        if ( applet.getParameter( "forcewin" ) != null )
        {
            m_eScreenMode = SCREENMODE_WINDOWED;
        }

        if ( applet.getParameter( "forcefull" ) != null )
        {
            m_eScreenMode = SCREENMODE_FULLSCREEN;
        }

        if ( applet.getParameter( "nojmf" ) != null )
        {
            setAllowJmf11(false);
        }
            
        if ( applet.getParameter( "nonative" ) != null )
        {
            setAllowNative(false);
        }
    }
    
    /**
     * <p>Parses the given command line arguments and sets the contained settings
     * accordingly. This method should be called AFTER you have set your own settings in
     * order to allow users to override your settings. Recognizes the following strings:</p>
     * <ul>
     * <li>"-forcewin" (forces usage of windowed graphics mode)</li>
     * <li>"-forcefull" (forces usage of fullscreen graphics mode, if mode is available)</li>
     * <li>"-nomsjava" (disallows usage of Microsoft Java specific engines)</li>
     * <li>"-nojmf" (disallows usage of Java Media Framework specific engines)</li>
     * <li>"-nonative" (disallows usage of Java Native Interface engines)</li>
     * </ul>
     * <p>Note: The prefix can be also other than "-" as the prefix is ignored.</p>
     *
     *@param aStrArgs Array of command line arguments.
     *@since GameFrame for Java 0.9
     */
    public void parseCommandLineArgs( String[] aStrArgs )
    {
        if (aStrArgs.length < 1)
        {
            return;
        }
        
        for (int i = 0; i < aStrArgs.length; i++)
        {
            if (aStrArgs[i].endsWith("nofs"))
            {
                if ( m_eScreenMode == SCREENMODE_FULLSCREEN )
                {
                    m_eScreenMode = SCREENMODE_WINDOWED;
                }
            }

            if (aStrArgs[i].endsWith("nomsjava"))
            {
                setAllowMsJava(false);
            }
            
            if (aStrArgs[i].endsWith("forcewin"))
            {
                m_eScreenMode = SCREENMODE_WINDOWED;
            }

            if (aStrArgs[i].endsWith("forcefull"))
            {
                m_eScreenMode = SCREENMODE_FULLSCREEN;
            }

            if (aStrArgs[i].endsWith("nojmf"))
            {
                setAllowJmf11(false);
            }
            
            if (aStrArgs[i].endsWith("nonative"))
            {
                setAllowNative(false);
            }
        }
    }

    /**
     * Returns a copy of the settings.
     * 
     *@return A copy of the settings.
     *@since GameFrame for Java 0.9
     */
    public GameFrameSettings getCopy()
    {
        try
        {
            return (GameFrameSettings) clone();
        }
        catch(CloneNotSupportedException noClone)
        {
        }
    
        return null;
    }
}
