/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.appletengines.jdk11x;

import gameframe.graphics.*;
import gameframe.engines.*;
import gameframe.*;

import java.awt.*;
import java.awt.image.*;

/**
 * Java 1.1 AWT implementation of an emulated fullscreen graphics engine for Java Applets.
 * Uses the component graphics engine inside the frameless window.
 *
 *@see CComponentGraphicsEngine
 *@author Pasi Kernen 04. April 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class CEmulatedFullscreenGraphicsEngine extends Window implements Finalizable
{
    /** The singleton instance of this graphics engine. */
    private static CEmulatedFullscreenGraphicsEngine mStatic_graphicsEngine;
    /** Frame that acts as a parent to the windows. */
    private static Frame mStatic_parentFrame;
    /** Window that masks the background if the wanted resolution is smaller than current resolution. */
    private static Window mStatic_maskWindow;
    /** The contained component graphics engine. */
    private static java.awt.Component mStatic_componentGraphicsEngine;
    /** Insets from the window frames. */
    private Insets m_insets = new Insets( 0,0,0,0 );
    
    /**
     * The default constructor that calls the superclass constructor that
     * sets the title of the frame to "GameFrame" and sets the created
     * frame to be non-resizable.
     *@param parentFrame The frame to be used as the parent for the game window.
     *@param componentGrfxEngine The component graphics engine used.
     *@param settings The settings object that contains settings for this graphics engine.
     */
    private CEmulatedFullscreenGraphicsEngine(
        Frame parentFrame,
        java.awt.Component componentGrfxEngine,
        GameFrameSettings settings )
    {
        super( parentFrame );
        setLayout( new BorderLayout() );
        show();
        
        m_insets = getInsets();
        mStatic_componentGraphicsEngine = componentGrfxEngine;
    
        // Center the frameless window in the middle of the screen
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        int width  = settings.getRequiredResolutionWidth();
        int height = settings.getRequiredResolutionHeight();
        int x = ( dim.width  - width  )  / 2;
        int y = ( dim.height - height ) / 2;
        setBounds(
            x,
            y,
            width  + m_insets.left + m_insets.right,
            height + m_insets.top  + m_insets.bottom );

        add( mStatic_componentGraphicsEngine, BorderLayout.CENTER );
        doLayout();

    
        // Create a mask window if necessary
        if ( ( dim.width > width ) || ( dim.height > height ) )
        {
            mStatic_maskWindow = new Window( mStatic_parentFrame );
            mStatic_maskWindow.setBackground( Color.black );
            mStatic_maskWindow.setBounds( 0, 0, dim.width, dim.height );
            mStatic_maskWindow.show();
            mStatic_maskWindow.addFocusListener( new java.awt.event.FocusAdapter()
            {
                public void focusGained(java.awt.event.FocusEvent e)
                {
                    if ( mStatic_graphicsEngine != null )
                    {
                        toFront();
                        requestFocus();
                        mStatic_componentGraphicsEngine.requestFocus();
                    }
                }
            } );

            // Make sure the graphics engine window is left on top
            toFront();
            requestFocus();
            mStatic_componentGraphicsEngine.requestFocus();
        }
    }

    /**
     * Returns an instance of this class. The instance is a window that
     * has the given title and size.
     *@param settings The settings object that contains settings for this graphics engine.
     */
    public static CEmulatedFullscreenGraphicsEngine getInstance(
        java.awt.Component componentGrfxEngine, GameFrameSettings settings )
    {
        if ( mStatic_graphicsEngine == null )
        {
            // Create and hide the parent frame
            mStatic_parentFrame = new Frame( settings.getTitle() );
            Image defaultIcon = GameFrame.getInstance().getDefaultIcon();
            if ( defaultIcon != null )
            {
                mStatic_parentFrame.setIconImage( defaultIcon );
            }
            mStatic_parentFrame.setBounds( -100, -100, 50, 50 );
            mStatic_parentFrame.setResizable( false );

            // Create the singleton instance of this graphics engine
            mStatic_graphicsEngine = new CEmulatedFullscreenGraphicsEngine(
                mStatic_parentFrame, componentGrfxEngine, settings );

            // When parent frame gets focus bring the graphics engine window to front
            mStatic_parentFrame.addFocusListener( new java.awt.event.FocusAdapter()
            {
                public void focusGained(java.awt.event.FocusEvent e)
                {
                    if ( mStatic_graphicsEngine != null )
                    {
                        mStatic_graphicsEngine.toFront();
                        mStatic_graphicsEngine.requestFocus();
                        mStatic_componentGraphicsEngine.requestFocus();
                    }
                }
            } );
        }
        
        return mStatic_graphicsEngine;
    }

	/**
	 * Invoked when the program exiting. Should do any
	 * necessary clean up, switch back to desktop etc.
	 */
	public void finalize()
    {
        if ( mStatic_graphicsEngine != null )
        {
            // Dispose the singleton instance reference
            mStatic_graphicsEngine = null;

            // Dispose the graphics engine
            remove( mStatic_componentGraphicsEngine );
            mStatic_componentGraphicsEngine = null;

            // Hide this window and the parent frame
            setVisible( false );
            mStatic_parentFrame.setVisible( false );

            // Dispose the mask window
            if ( mStatic_maskWindow  != null )
            {
                mStatic_maskWindow.setVisible( false );
                mStatic_maskWindow.dispose();
                mStatic_maskWindow = null;
            }
            
            // Dispose this window and the parent frame
            dispose();
            mStatic_parentFrame.dispose();
            mStatic_parentFrame = null;
        }
    }
}
