/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.appletengines.jdk11x;

import gameframe.engines.*;
import gameframe.input.*;
import gameframe.*;

import java.awt.*;
import java.awt.event.*;

/**
 * Implementation of the input engine for Java Applets running in 1.1.x JVM's.
 *
 *@author Pasi Kernen 04. April 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class CInputEngine implements InputEngine, Finalizable
{
    /** The keyboard engine or null if not created yet. */
    private static CKeyboard mStatic_keyboard;
    /** The mouse engine or null if not created yet. */
    private static CPointer mStatic_pointer;
    /** The singleton instance of the input engine. */
    private static CInputEngine mStatic_inputEngine;
    
    /** The focus component that is listened for events. */
    private Component m_focusComponent;
    
    /**
     * Constructs an input engine.
     *@param focusComponent Component that is listened for events.
     */
    private CInputEngine( Component focusComponent )
        throws GameFrameException
    {
        if ( focusComponent == null )
        {
            throw new GameFrameException( "Focuswindow is null." );
        }

        m_focusComponent = focusComponent;
    }
    
    /**
     * Returns the singleton input engine instance.
     *@param focusComponent Component that is listened for events.
     *@return The singleton input engine instance
     */
    public static CInputEngine getInstance( Component focusComponent )
        throws GameFrameException
    {
        if ( mStatic_inputEngine == null )
        {
            mStatic_inputEngine = new CInputEngine( focusComponent );
            System.err.println("Input engine created and initialized" );
        }
        
        return mStatic_inputEngine;
    }
    
    /**
     * Returns the default keyboard type of device in the system.
     *@return The default keyboard
     *@see gameframe.input.KeyboardDevice
     */
    public KeyboardDevice getDefaultKeyboardDevice()
        throws GameFrameException
    {
        if (mStatic_keyboard == null)
        {
            mStatic_keyboard = new CKeyboard(m_focusComponent);
        }

        return mStatic_keyboard;
    }
    
    /**
     * Returns all keyboard type of devices found on the system.
     *@return All keyboard type of devices found on the system.
     *@see gameframe.input.KeyboardDevice
     */
    public KeyboardDevice[] getAllKeyboardDevices()
        throws GameFrameException
    {
        if (mStatic_keyboard == null)
        {
            mStatic_keyboard = new CKeyboard(m_focusComponent);
        }
        
        CKeyboard[] aKeyboards = new CKeyboard[1];
        aKeyboards[0] = mStatic_keyboard;
        return aKeyboards;
    }

    /**
     * Returns the default pointer type of device in the system.
     *@return The default pointer device.
     *@see gameframe.input.PointerDevice
     */
    public PointerDevice getDefaultPointerDevice()
        throws GameFrameException
    {
        if (mStatic_pointer == null)
        {
            mStatic_pointer = new CPointer(m_focusComponent);
        }

        return mStatic_pointer;
    }
    
    /**
     * Returns all pointer type of devices found on the system.
     *@return All pointer type of devices.
     *@see gameframe.input.PointerDevice
     */
    public PointerDevice[] getAllPointerDevices()
        throws GameFrameException
    {
        if (mStatic_pointer == null)
        {
            mStatic_pointer = new CPointer(m_focusComponent);
        }
        
        CPointer[] aPointers = new CPointer[1];
        aPointers[0] = mStatic_pointer;
        return aPointers;
    }
    
    /**
     * Returns the short name (max 8 characters) of the
     * implementation "Java11".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "Java11";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Input engine that uses plain Java 1.1 AWT listener interfaces.";
	}

    /**
     * Releases all resources held by the all input device implementations.
     */
    public void finalize()
    {
        // Finalize the keyboard input class
        if (mStatic_keyboard != null)
        {
            mStatic_keyboard.finalize();
            mStatic_keyboard = null;
        }
        
        // Finalize the mouse input class
        if (mStatic_pointer != null)
        {
            mStatic_pointer.finalize();
            mStatic_pointer = null;
        }
        
        // Reset the singleton reference
        mStatic_inputEngine = null;
        System.err.println("Input engine finalized" );
    }
}
