/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.appletengines.jdk11x;

import gameframe.*;
import gameframe.input.*;

import java.awt.*;
import java.awt.event.*;

/**
 * Implementation of the keyboard device for Java Applets running in 1.1.x JVMs.
 *
 *@author Pasi Kernen 04. April 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
class CKeyboard implements KeyboardDevice, KeyListener
{
    /** Reference to the focus component that is the source of keyboard events. */
    private Component m_focusComponent;
    /** States of the keys. Can have one of the enumerated KEYSTATE_ values. */
    private int[] m_aKeyStates = new int[0xFF];
    /** Number of times the key was stroken. */
    private int[] m_aNumKeystrokes = new int[0xFF];
    /** Enumerated value for state key is up. */
    private static final int KEY_UP                = 0;
    /** Enumerated value for state key is down. */
    private static final int KEY_DOWN              = 1;
    /** Enumerated value for state where key was down but was not read during that time. */
    private static final int KEY_WAS_DOWN_NOT_READ = 2;
    /** Enumerated value for state where key was down and was read during that time. */
    private static final int KEY_WAS_DOWN_READ     = 3;
    /** The last alphanumeric key value typed. */
    private int m_lastAlphanumericKey = -1;

    /**
     * Constructs a keyboard engine that has the given component set as the focus component.
     *@param component The component that where listeners are registered.
     */
    CKeyboard(Component component)
    {
        m_focusComponent = component;
        m_focusComponent.addKeyListener( this );
        System.err.println("Keyboard created and initialized" );
    }

    /**
     * Returns true if the key is currently down 
     *@param keyCode The keycode of the key we want to query.
     *@return True if the key is or was down.
     */
    public boolean isKeyDown(int keyCode)
    {
        if (m_aKeyStates[keyCode] == KEY_DOWN)
        {
            m_aKeyStates[keyCode] = KEY_WAS_DOWN_READ;
            return true;
        }

        return (m_aKeyStates[keyCode] == KEY_WAS_DOWN_READ);
    }

    /**
     * Returns true if the key was briefly kept down
     * after last reading of the keys state (after last call made to isKeyDown or
     * getKeyDownCount methods with the same key code).
     */
    public boolean wasKeyDown(int keyCode)
    {
        if (m_aKeyStates[keyCode] == KEY_WAS_DOWN_NOT_READ)
        {
            m_aNumKeystrokes[keyCode] = 0;
            m_aKeyStates[keyCode] = KEY_UP;
            return true;
        }
        
        return false;
    }

    /**
     * Returns the number of times the key has been stroken since last reading
     * (after last call made to isKeyDown or getKeyDownCount methods with the
     * same key code).
     *@param keyCode The keycode of the key we want to query.
     *@return The number of times the key was stroken (pressed and released)
     */
    public int getKeyDownCount(int keyCode)
    {
        int numStrokes = m_aNumKeystrokes[keyCode];
        m_aNumKeystrokes[keyCode] = 0;
        if (m_aKeyStates[keyCode] == KEY_WAS_DOWN_NOT_READ)
        {
            m_aKeyStates[keyCode] = KEY_UP;
        }

        return numStrokes;
    }

    public void keyPressed(KeyEvent event)
    {
        int keycode = event.getKeyCode();
        m_aNumKeystrokes[keycode]++;
        m_aKeyStates[keycode] = KEY_DOWN;
    }

    public void keyReleased(KeyEvent event)
    {
        int keyCode = event.getKeyCode();
        
        // If the button was down and the event was read, set
        // button state back to up
        if (m_aKeyStates[keyCode] == KEY_WAS_DOWN_READ)
        {
            m_aKeyStates[keyCode] = KEY_UP;
        }
        
        // If the button was down and the event was not read, set
        // button state accordingly
        m_aKeyStates[keyCode] = KEY_WAS_DOWN_NOT_READ;
    }

    public void keyTyped(KeyEvent event)
    {
        char character = event.getKeyChar();
        if (Character.isLetterOrDigit(character))
        {
            m_lastAlphanumericKey = character;
        }
        else
        {
            if (character == ' ')
            {
                m_lastAlphanumericKey = ' ';
            }
            else if (character == '.')
            {
                m_lastAlphanumericKey = '.';
            }
            else
            {
                m_lastAlphanumericKey = -1;
            }
        }
    }

    /**
     * Returns any typed alphanumeric keys (including space and dot) as int values that
     * correspond to that keys char value or -1 if no such keys were pressed
     * after last invokation of this method.
     *@return Any typed alphanumeric keys
     */
    public int getTypedAlphanumericKey()
    {
        int temp = m_lastAlphanumericKey;
        m_lastAlphanumericKey = -1;
        return temp;
    }

    
    /**
     * Releases all resources held by the keyboard device implementation.
     */
    public void finalize()
    {
        if ( m_focusComponent != null )
        {
            m_focusComponent.removeKeyListener(this);
            m_focusComponent = null;
            m_aKeyStates     = null;
            m_aNumKeystrokes = null;
            System.err.println("Keyboard finalized" );
        }
    }
}
