/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.appletengines.jdk11x;

import gameframe.*;
import gameframe.input.*;

import java.awt.*;
import java.awt.event.*;

/**
 * Implementation of the pointer device for the Java 1.1 platform.
 *
 *@author Pasi Kernen 04. April 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
class CPointer implements PointerDevice, MouseListener, MouseMotionListener
{
    /** Reference to the focus component that is the source of keyboard events. */
    private Component m_focusComponent;
    /** The current X-coordinate of the mouse. */
    private int m_x;
    /** The current Y-coordinate of the mouse. */
    private int m_y;
    /** States of the buttons. Can have one of the enumerated BUTTONSTATE_ values. */
    private int[] m_aButtonStates = new int[10];
    /** Number of times the button was stroken. */
    private int[] m_aNumButtonPresses = new int[10];
    /** Enumerated value for state button is up. */
    private static final int BUTTONSTATE_UP                = 0;
    /** Enumerated value for state button is down. */
    private static final int BUTTONSTATE_DOWN              = 1;
    /** Enumerated value for state where button was down but was not read during that time. */
    private static final int BUTTONSTATE_WAS_DOWN_NOT_READ = 2;
    /** Enumerated value for state where button was down and was read during that time. */
    private static final int BUTTONSTATE_WAS_DOWN_READ     = 3;

    /**
     * Constructs a pointer device whose focus component is the given window.
     *@param focusComponent The component that receives the mouse events.
     */
    CPointer(Component focusComponent)
    {
        m_focusComponent = focusComponent;
        m_focusComponent.addMouseListener(this);
        m_focusComponent.addMouseMotionListener(this);
        System.err.println("Pointer created and initialized" );
    }

    /**
     * Returns the X-coordinate of the pointer devices current location. If the mouse
     * is outside of the screen returns -1.
     *@return Pointers current X-location or -1 if not on screen.
     */
    public int getX()
    {
        return m_x;
    }

    /**
     * Returns the Y-coordinate of the pointer devices current location. If the mouse
     * is outside of the screen returns -1.
     *@return Pointers current Y-location or -1 if not on screen.
     */
    public int getY()
    {
        return m_y;
    }

    /**
     * Returns true if the button is down. Id numbers should be
     * 0 = primary button, 1 = secondary or "popup menu" button, 2 etc. for rest of
     * the buttons. Max. 3 buttons are supported (button id's from 0 to 2).
     *@param buttonId The number of the button we want to query.
     *@return True if the button is or was down.
     */
    public boolean isButtonDown(int buttonId)
    {
        if (m_aButtonStates[buttonId] == BUTTONSTATE_DOWN)
        {
            m_aButtonStates[buttonId] = BUTTONSTATE_WAS_DOWN_READ;
            return true;
        }
        
        return (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_READ);
    }

    /**
     * Returns true if the button was briefly kept down after last reading
     * of the buttons state and that event hasn't been read by any other method
     * (after last call made to isButtonDown, wasButtonDown or getButtonDownCount
     * methods with the same button id number). Id numbers should be
     * 0 = primary button, 1 = secondary or "popup menu" button, 2 etc. for rest of
     * the buttons. Max. 3 buttons are supported (button id's from 0 to 2).
     *@param buttonId The number of the button we want to query.
     *@return True if the button is or was down.
     */
    public boolean wasButtonDown(int buttonId)
    {
        if (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_NOT_READ)
        {
            m_aNumButtonPresses[buttonId] = 0;
            m_aButtonStates[buttonId] = BUTTONSTATE_UP;
            return true;
        }
        
        return false;
    }

    /**
     * Returns the number of times the button has been stroken since last reading
     * (after last call made to wasButtonDown or getButtonDownCount methods with the
     * same button id number). Id numbers should be
     * 0 = primary button, 1 = secondary or "popup menu" button, 2 = third button.
     * Max. 3 buttons are supported (button id's from 0 to 2).
     *@param buttonId The number of the button we want to query.
     *@return The number of times the button was stroken (pressed and released)
     */
    public int getButtonDownCount(int buttonId)
    {
        int numPresses = m_aNumButtonPresses[buttonId];
        m_aNumButtonPresses[buttonId] = 0;
        if (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_NOT_READ)
        {
            m_aButtonStates[buttonId] = BUTTONSTATE_UP;
        }
        return numPresses;
    }

    /**
     * Called when mouse enters the focus window.
     *@param event The event object that describes the event in more detail.
     */
    public void mouseEntered(MouseEvent event)
    {
        m_x = event.getX();
        m_y = event.getY();
    }

    /**
     * Called when mouse leaves the focus window.
     *@param event The event object that describes the event in more detail.
     */
    public void mouseExited(MouseEvent event)
    {
        m_x = -1;
        m_y = -1;
    }

    /**
     * Called when mouse button is clicked in the focus window.
     *@param event The event object that describes the event in more detail.
     */
    public void mouseClicked(MouseEvent event)
    {
        m_aNumButtonPresses[getMouseButtonId(event)] += event.getClickCount();
    }

    /**
     * Called when mouse is moved inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    public void mouseMoved(MouseEvent event)
    {
        m_x = event.getX();
        m_y = event.getY();
    }

    /**
     * Called when mouse is pressed inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    public void mousePressed(MouseEvent event)
    {
        m_aButtonStates[getMouseButtonId(event)] = BUTTONSTATE_DOWN;
        m_focusComponent.requestFocus();
    }

    /**
     * Called when mouse is released inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    public void mouseReleased(MouseEvent event)
    {
        int buttonId = getMouseButtonId(event);
        
        // If the button was down and the event was read, set
        // button state back to up
        if (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_READ)
        {
            m_aButtonStates[buttonId] = BUTTONSTATE_UP;
        }
        
        // As the button was down and the event was not read,
        // set button state accordingly
        m_aButtonStates[buttonId] = BUTTONSTATE_WAS_DOWN_NOT_READ;
    }


    /**
     * Called when mouse is dragged inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    public void mouseDragged(MouseEvent event)
    {
        m_x = event.getX();
        m_y = event.getY();
    }

    /**
     * Returns the mouse button ID (number) from the given event.
     *@param event The event object that describes the event in more detail.
     *@return The number of the mouse button that was pressed (0-2).
     */
    private int getMouseButtonId(MouseEvent event)
    {
        if ( (event.getModifiers() & InputEvent.BUTTON1_MASK) != 0)
        {
            return 0;
        }
        else if ( (event.getModifiers() & InputEvent.BUTTON2_MASK) != 0)
        {
            return 1;
        }
        else
        {
            return 2;
        }
    }

    /**
     * Releases all resources held by the pointer device implementation.
     */
    public void finalize()
    {
        if (m_focusComponent != null)
        {
            m_focusComponent.removeMouseListener(this);
            m_focusComponent.removeMouseMotionListener(this);
            m_focusComponent       = null;
            m_aButtonStates     = null;
            m_aNumButtonPresses = null;
            System.err.println("Pointer finalized" );
        }
    }
}
