/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.appletengines.jdk11x;

import gameframe.engines.*;
import gameframe.sound.*;
import gameframe.util.*;
import gameframe.*;
import gameframe.engines.jdk11x.CLoopingSoundStream;
import gameframe.engines.jdk11x.CSoundStream;

import java.io.*;
import java.util.*;
import sun.audio.*;

/**
 * Implementation of the sound engine for Java Applets running on JDK 1.1.x JVMs.
 * Uses the well known "hack" to play samples on plain vanilla Java 1.1 platforms
 * without Java Media Framework. Loads mono/stereo Microsoft PCM RIFF WAVE files.
 *
 *@author Pasi Kernen 04. April 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
class CSoundEngine implements SoundEngine, Finalizable
{
    /** All the sounds created through this sound engine. */
    private Vector m_createdSounds = new Vector(10,10);
    /** Name of the directory that contains loaded sounds. */
    private String m_strDataDirectory = "";

    /**
     * Constructs a JDK 1.1.x sound engine.
     */
    public CSoundEngine( )
    {
        // Just to check that we have a player object
		sun.audio.AudioPlayer player = sun.audio.AudioPlayer.player;
        player.getName();
        System.err.println("Sound engine created and initialized" );
    }
    
	/**
	 * Loads an mono 8-bit 11025 bytes/sec AU formatted sample from the given file name
	 * and return an abstract reference for the loaded sound. This implementation is limited
	 * in sample lenght to the maximum positive value of integer...
	 * @param strFileName The name of the file to be loaded (without the file extension)
	 * @return An unique Sample object that specifies the loaded sample
	 */
	public Sample loadSample( String strFileName )
        throws FileNotFoundException, GameFrameException
	{
        strFileName = m_strDataDirectory+strFileName+".wav";

        // Load the wave file
        RiffWavFile waveFile  = new RiffWavFile();
        waveFile.loadWAVURL( strFileName );
        
        // Resample the PCM data to 8 KHz
        waveFile.resampleTo(8000);

        // u-law encode the PCM data
        ULawEncoder codec = new ULawEncoder();
        byte[] array = codec.encode(waveFile.getPcmData());
        waveFile = null;
        System.gc();

        // Create a soundstream from the encoded audio data
		InputStream soundStream = new CSoundStream(array);
        CJava11Sound sound = new CJava11Sound((CSoundStream) soundStream);
        m_createdSounds.addElement(sound);
		return sound;
	}

    /**
     * First tries to load a music file as streamed Microsoft RIFF WAV file,
     * if no such file is found from the current data directory, tries to load
     * a music file in MIDI format. When suitable file is found returns a
     * control object for the music file if no suitable file is found returns
     * NullSound object. Music files are loaded by default from the
     * current directory, the directory that is used to load the samples
     * can be set with the setDataDirectory() method. 
     * 
     *@see #setDataDirectory(java.lang.String)
     *@param fileName The name of the file to be loaded (without the file extension)
     *@return An unique Sample object that represents the loaded sample
     *@since GameFrame for Java 0.9.2
     */
    public Music loadMusic( String fileName )
        throws FileNotFoundException, GameFrameException
    {
        return new NullMusic();
    }

    /**
     * Sets the data directory wherefrom the sounds are loaded.
     * Defaults to the current directory. Do not add the last directory separator.
     *@param strDirectoryName The name of the directory that contains the sounds.
     */
    public void setDataDirectory(String strDirectoryName)
        throws java.io.FileNotFoundException
    {
        // If there is a path separator, remove it
        if ( strDirectoryName.charAt( strDirectoryName.length() - 1 ) == File.separatorChar )
        {
            strDirectoryName = strDirectoryName.substring( 0, strDirectoryName.length() - 1 );
        }

        m_strDataDirectory = strDirectoryName + File.separator;
    }
    

    /**
     * Returns the data directory wherefrom the sounds are loaded.
     * The source directory defaults to the current directory.
     * 
     * Note: The returned path will have the path separator as the last character.
     * 
     *@see #loadSample(java.lang.String)
     *@see #loadMusic(java.lang.String)
     *@return The name of the directory that wherefrom the sounds are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory()
    {
        return m_strDataDirectory;
    }
    
    /**
	 * Finalises all sounds.
	 */
	public void finalize()
	{
        // Finalize sounds
        if ( m_createdSounds != null )
        {
            for (Enumeration enum = m_createdSounds.elements(); enum.hasMoreElements();)
            {
                Sample sound = (Sample) enum.nextElement();
                sound.finalize();
            }
            
            m_createdSounds = null;
            System.err.println("Sound engine finalized" );
        }
    }
	
    /**
     * Returns the short name (max 8 characters) of the
     * implementation "Java11".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "Java11";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Sound engine that uses the unofficial Java 1.1 sound hack.";
	}

	/**
	 * Implementation of Sample that implements the sound playing in an "hacked"
	 * Java 1.1 way.
	 */
	class CJava11Sound implements Sample
	{
        private CSoundStream m_soundStream;
		//public sun.audio.AudioStream m_audioStream;
		CLoopingSoundStream m_loopedStream;
		boolean m_fPlaying;
        float m_pan;
        float m_volume;
		
		CJava11Sound(CSoundStream soundStream)
		{
			m_fPlaying = false;
            m_soundStream = soundStream;
			m_soundStream.mark(2000000);
		}
		
		/**
 		 * Plays this sample once and then stops
		 */
		public void playOnce()
		{
            stop();
			m_fPlaying = true;
			sun.audio.AudioPlayer.player.start(m_soundStream);
		}
		
		/**
		 * Plays the given sample over and over again (looping).
		 */
		public void playLooped()
		{
            stop();
			m_loopedStream = new CLoopingSoundStream(m_soundStream);
			sun.audio.AudioPlayer.player.start(m_loopedStream);
			m_fPlaying = true;
		}
		
		/**
		 * Stops playing the given sample
		 */
		public void stop()
		{
			if (m_fPlaying)
			{
				m_fPlaying = false;
				if (m_loopedStream != null)
				{
					sun.audio.AudioPlayer.player.stop(m_loopedStream);
					m_loopedStream = null;
					m_soundStream.reset();
					m_soundStream.mark(2000000);
				}
				else
				{
					sun.audio.AudioPlayer.player.stop(m_soundStream);
					m_soundStream.reset();
					m_soundStream.mark(2000000);
				}
			}
		}
		
        /**
         * Returns a copy of this sound object that is independent from the
         * original sound object. This new sound object can be started/stopped
         * independently from the original sound object. Using this method to
         * get multiple instances of one sound will save memory on some
         * implementations and is thus preferred way to duplicate sound objects.
         *@return A independent copy of this sound or null if copy fails.
         */
        public Sample getCopy()
        {
            CJava11Sound sound = new CJava11Sound(new CSoundStream(m_soundStream.getInternalData()));
            m_createdSounds.addElement(sound);
		    return sound;
        }

        /**
		 * Upon finalization stop everything (just in case).
		 */
		public void finalize()
		{
            if (m_soundStream != null)
            {
                if (m_fPlaying)
                {
			        stop();
                }
                m_createdSounds.removeElement(this);
                m_soundStream = null;
                m_loopedStream = null;
            }
		}
		
		/**
		 * Returns true if sound should be playing at the moment
		 */
		public boolean isPlaying()
		{
			return m_fPlaying;
		}
        
        /**
         * Sets the volume of the sound. Value of 0.0 means that the sound should be
         * inaudible and value of 1.0 means the sound is played at its normal volume.
         * Values above 1.0 are treated as 1.0.
         *@param volume The sounds volume.
         */
        public void setVolume(float volume)
        {
            if (volume > 1.0F)
            {
                volume = 1.0F;
            }
            
            if (volume < 0.0F)
            {
                volume = 0.0F;
            }
            m_volume = volume;
            m_soundStream.setVolume(volume);
        }

        /**
         * Sets the pan of the sound. Value of -1.0 means that the sound will be
         * heard only from the left speaker and value of 1.0 means that the sound
         * will be heard only from the right speaker. Value of 0.0 means that
         * the sound is heard with equal volume from both speakers.
         *@param pan The pan of the sound.
         */
        public void setPan(float pan)
        {
            if (pan > 1.0F)
            {
                pan = 1.0F;
            }
            
            if (pan < -1.0F)
            {
                pan = -1.0F;
            }
            m_pan = pan;
        }

        /**
         * Returns the volume of the sound. Value of 0.0 means that the sound should be
         * inaudible and value of 1.0 means the sound is played at its normal volume.
         *@return The sounds volume.
         */
        public float getVolume()
        {
            return m_volume;
        }
    
        /**
         * Returns the pan of the sound. Value of -1.0 means that the sound will be
         * heard only from the left speaker and value of 1.0 means that the sound
         * will be heard only from the right speaker. Value of 0.0 means that
         * the sound is heard with equal volume from both speakers.
         *@return The pan of the sound.
         */
        public float getPan()
        {
            return m_pan;
        }
	}
}
