/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines;

import gameframe.graphics.*;
import gameframe.input.*;
import gameframe.sound.*;
import gameframe.util.*;
import gameframe.*;


/**
 * <p>Defines an interface to an engine factory. An engine factory is used to
 * create all the needed engines (graphics, sound, input etc.) that then
 * take care of one part of the game frame functionality. When an engine
 * factory is finalized, it will finalize all the engines that were created by
 * it. This interface is used internally by the library to wrap access to engine
 * factories in MainFactory class.</p>
 * 
 * <p>To engine implementors:<br>
 * To make your own engine factory work with GameFrame for Java you must do these things:<ul>
 * <li>Make your engine factory implement this interface.</li>
 * <li>Implement in your engine factory a static method
 *     <code>public static EngineFactory getInstance(GameFrameSettings)</code>
 *     that returns an (preferrably singleton) instance of your engine factory.</li>
 * <li>Implement in your engine factory a static method
 *     <code>public static boolean isRunnableOnPlatform()</code>
 *     that returns true if the engine factory is runnable on current platform and false
 *     if it isn't runnable on current platform.</li>
 * <li>Place your engine factory under package directly under gameframe package.</li></ul>
 * </p>
 * 
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public interface EngineFactory
{
    /**
     * Enumerated value for the suitability of the engine factory for the platform.
     * This value means that the engine factory can't run on this platform (e.g.
     * Microsoft Java VM specific DirectX implementation on JavaSoft Java VM).
     * 
     *@since GameFrame for Java 0.9
     */
    public static final int SUITABILITY_NOT_SUITED = 0;

    /**
     * Enumerated value for the suitability of the engine factory for the platform.
     * This value means that the engine factory can run on this platform, but likely is not
     * the best option available (e.g. Java 1.1 based implementation on Microsoft Java VM).
     * 
     *@since GameFrame for Java 0.9
     */
    public static final int SUITABILITY_POOR = 1;

    /**
     * Enumerated value for the suitability of the engine factory for the platform.
     * This value means that the engine factory can run on this platform and is most likely
     * a good (even the best) option available for this platform (e.g. JNI DirectX
     * implementation on JavaSoft Java VM).
     * 
     *@since GameFrame for Java 0.9
     */
    public static final int SUITABILITY_GOOD = 2;
    
    /**
     * Enumerated value for the suitability of the engine factory for the platform.
     * This value means that the engine factory is the ultimate version available for
     * this platform (e.g. Direct/J DirectX implementation on Microsoft Java VM).
     * This value should only be used in EXTREME cases. Even the examplary case might
     * not be a good place to use this value, for example a JNI DirectX implementation
     * might be faster on the Microsoft Java VM.
     * 
     *@since GameFrame for Java 0.9
     */
    public static final int SUITABILITY_ULTIMATE = 3;

    
    /**
     * Returns the short name (max 8 characters) of the engine factory implementation
     * (e.g. "Java 1.1" or "MSDX").
     * 
     *@return The short name of the engine factory.
     *@since GameFrame for Java 0.9
     */
    public String getName();
    
    /**
     * Returns the (possibly multiline) description of the engine factory implementation.
     * 
     *@return A description of the engine factory.
     *@since GameFrame for Java 0.9
     */
    public String getDescription();
    
    /**
     * Returns the suitability of this engine implementation for the current platform as
     * one of the enumerated SUITABILITY_xxx values.
     * 
     *@return Suitability of the engine factory for this platform.
     *@since GameFrame for Java 0.9
     */
    public int getSuitabilityForPlatform();

    /**
     * <p>Returns all the fullscreen graphics modes supported by graphics engines that
     * are created by this factory. If the factory created graphics engines support
     * only windowed modes, this method returns null.</p>
     * 
     * <p>Note to engine developers: This method should always do a full cleanup after
     * it has been run. This means releasing all of the memory and resources that were
     * used.</p>
     * 
     *@return Supported fullscreen modes or null.
     *@since GameFrame for Java 0.9
     */
    public String[] getSupportedResolutions();
    
    /**
     * Returns the sound engine implementation for the current platform. When this method
     * returns the sound engine is in fully usable state (fully initialized).
     * If not successfull returns the do-nothing implementation of a sound engine (CNullSoundEngine
     * in package gameframe) or throws GameFrameException exception.
     * 
     *@return The single sound engine for this platform.
     *@exception GameFrameException Thrown if sound engine creation fails.
     *@since GameFrame for Java 0.9
     */
    public SoundEngine getSoundEngine()
        throws GameFrameException;

    /**
     * Returns the graphics engine implementation for the current platform. When this method
     * returns the graphics engine is in fully usable state (fully initialized).
     * If not successfull throws GameFrameException exception. The requirements for the created
     * graphics engine are taken from the user specified MainFactory settings in the MainFactory
     * class.
     * 
     *@return The single graphics engine for this platform.
     *@exception GameFrameException Thrown if graphics engine creation fails.
     *@since GameFrame for Java 0.9
     */
    public GraphicsEngine getGraphicsEngine()
        throws GameFrameException;

    /**
     * Returns the input engine implementation for the current platform. When this method
     * returns the input engine is in fully usable state (fully initialized). If not successfull
     * throws GameFrameException exception.
     * 
     *@return The single input engine for this platform.
     *@exception GameFrameException Thrown if input engine creation fails.
     *@since GameFrame for Java 0.9
     */
    public InputEngine getInputEngine()
        throws GameFrameException;

    /**
     * Returns a new timer instance. Several timers can be created with this method. If a native
     * timer can't be for some reason created, this method returns the basic implementation
     * CTimer from the GameFrame.Java11 package. If even this doesn't succeed throws GameFrameException
     * exception.
     * 
     *@return A new timer instance.
     *@exception GameFrameException Thrown if timer creation fails.
     *@since GameFrame for Java 0.9
     */
    public Timer createTimer()
        throws GameFrameException;
    
    /**
     * Notifies that the given timer has been finalized.
     */
    public void finalizationNotify ( Timer timer );

    /**
     * Invoked before application exits. Does all the cleanup that is
     * necessary to exit cleanly (closes windows, stops playing sounds, releases
     * any objects referenced etc.). Also finalizes all the engines and timers
     * that were created with this engine factory.
     * 
     *@since GameFrame for Java 0.9
     */
    public void finalize();
}
