/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines;

import gameframe.*;
import gameframe.sound.*;

import java.io.*;

/**
 * Do nothing implementation of the sound engine, used e.g.
 * when no soundcard is found on the current platform.
 *
 *@see gameframe.SoundEngine
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class NullSoundEngine implements SoundEngine, Finalizable
{
    /** The currently set datadirectory.  */
    private String m_strDataDirectory = "";
    
    /**
     * The singleton instance of Null Sample Engine class.
     *@since GameFrame for Java 0.9.1
     */
    public static NullSoundEngine mStatic_singletonInstance = null;
    
    /**
     * Returns the single instance of Null Sample Engine class.
     *@return The one and only instance of Null Sample Engine class.
     *@since GameFrame for Java 0.9.1
     */
    public static NullSoundEngine getInstance( )
    {
        if ( mStatic_singletonInstance == null )
        {
            mStatic_singletonInstance = new NullSoundEngine( );
        }
        return mStatic_singletonInstance;
    }

    /**
     * Private constructor of this class.
     *@since GameFrame for Java 0.9
     */
    private NullSoundEngine()
    {
    }
    
    /**
     * Returns an do nothing implementation of sample object.
     * 
     *@param fileName Ignored.
     *@return Do nothing implementation of sample object.
     *@since GameFrame for Java 0.9
     */
    public Sample loadSample( String fileName )
    {
        return ( new NullSample() );
    }

    /**
     * Returns an do nothing implementation of music object.
     * 
     *@param fileName Ignored.
     *@return Do nothing implementation of music object.
     *@since GameFrame for Java 0.9.2
     */
    public Music loadMusic( String fileName )
    {
        return ( new NullMusic() );
    }
    
    /**
     * Checks if the directory exists (like the other library implementations do).
     * 
     *@param strDirectoryName Ignored.
     *@since GameFrame for Java 0.9
     */
    public void setDataDirectory( String strDirectoryName )
        throws FileNotFoundException
    {
        // If there is a path separator, remove it
        if ( strDirectoryName.charAt( strDirectoryName.length() - 1 ) == File.separatorChar )
        {
            strDirectoryName = strDirectoryName.substring( 0, strDirectoryName.length() - 1 );
        }
        
        File file = new File( strDirectoryName );
        if ( ( !file.isDirectory() ) || ( !file.exists() ) )
        {
            throw new java.io.FileNotFoundException(
                "The given datadirectory "+
                strDirectoryName+
                " doesn't exist.");
        }

        m_strDataDirectory = strDirectoryName + File.separator;
    }

    /**
     * Returns the data directory wherefrom the sounds are loaded.
     * The source directory defaults to the current directory.
     * 
     * Note: The returned path will have the path separator as the last character.
     * 
     *@see #loadSample(java.lang.String)
     *@see #loadMusic(java.lang.String)
     *@return The name of the directory that wherefrom the sounds are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory()
    {
        return m_strDataDirectory;
    }

    /**
     * Returns the short name (max 8 characters) of the
     * implementation "NullSnd".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "NullSnd";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Do nothing sound engine implementation.";
	}

	/**
     * Clears the singleton instance.
     * 
     *@since GameFrame for Java 0.9.0
     */
    public void finalize()
    {
        mStatic_singletonInstance = null;
    }
}
