/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk11x;

import gameframe.engines.*;
import gameframe.graphics.*;
import gameframe.input.*;
import gameframe.sound.*;
import gameframe.util.*;
import gameframe.*;

import java.util.Vector;
import java.util.Enumeration;

/**
 * Implementation of an engine factory for Java 1.1.x JVM's.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.3
 *@since GameFrame for Java 0.9
 */
public class CEngineFactory implements gameframe.engines.EngineFactory
{
    /** The singleton reference to an instance of this engine factory. */
    private static CEngineFactory mStatic_engineFactory     = null;

    /** Reference to the singleton sound engine implementation. */
    private SoundEngine m_soundEngine           = null;
    /** Reference to the singleton graphics engine implementation. */
    private GraphicsEngine m_graphicsEngine     = null;
    /** Reference to the singleton input engine implementation. */
    private InputEngine m_inputEngine           = null;
    /** Reference to the singleton focus component. */
    private java.awt.Component m_focusComponent       = null;
    /** All objects that must be finalized upon engine factory finalization. */
    private Vector m_finalizableObjects         = new Vector(3,3);
    /** The main factory settings used for this engine factory. */
    private GameFrameSettings m_settings        = null;
    
    /**
     * Constructs the engine factory without constructing any engines.
     *@param settings The settings to be used in this engine factory.
     */
    private CEngineFactory(GameFrameSettings settings)
    {
        m_settings = settings;
    }
    
    /**
     * Returns the singleton instance of this engine factory (the instace
     * will be created upon the first call to this method).
     *@param settings The settings to be used in this engine factory.
     *@return The singleton instance of this engine factory.
     */
    public static CEngineFactory getInstance(GameFrameSettings settings)
    {
        if (mStatic_engineFactory == null)
        {
            mStatic_engineFactory = new CEngineFactory( settings );
        }
        
        return mStatic_engineFactory;
    }

    /**
     * Returns the short name (max 8 characters) of this engine factory
     * implementation which is "Java1.1"
     *@return The short name of the engine factory.
     */
    public String getName()
    {
        return "Java 1.1";
    }

    /**
     * Returns the description of the engine factory implementation.
     *@return A description of the engine factory.
     */
    public String getDescription()
    {
        return "Generic engine factory suited for Java 1.1.x implementations of Java runtime.";
    }

    /**
     * Returns the suitability of this engine implementation for
     * the current platform as one of the enumerated values.
     * Currently return POOR so that any better implementation will
     * override this implementation.
     *@return Suitability of the engine factory for this platform.
     */
    public int getSuitabilityForPlatform()
    {
        String strVersion = (String) System.getProperties().get("java.version");
        
        // This might not work on all runtimes... anyone know a better way?
        if (strVersion.startsWith("1.1") || strVersion.startsWith("JDK1.1"))
        {
            return CEngineFactory.SUITABILITY_GOOD;
        }

        if ((strVersion.startsWith("1.2"))|| strVersion.startsWith("JDK1.2"))
        {
            return CEngineFactory.SUITABILITY_POOR;
        }
        
        return CEngineFactory.SUITABILITY_NOT_SUITED;
    }
    
    /**
     * Returns true if this engine factory is runnable on the platform.
     */
    public static boolean isRunnableOnPlatform()
    {
        String strVersion = (String) System.getProperties().get("java.version");
        
        if (strVersion.startsWith("1.1") || strVersion.startsWith("JDK1.1"))
        {
            return true;
        }

        if ((strVersion.startsWith("1.2"))|| strVersion.startsWith("JDK1.2"))
        {
            return true;
        }
        
        return false;
    }

    /**
     * As the graphics engines created by this factory don't support
     * fullscreen modes, this method returns null.
     *@return Allways null.
     */
    public String[] getSupportedResolutions()
    {
        return null;
    }
    
    /**
     * Returns the sound engine implementation for the current platform.
     * If not successfull returns the do-nothing implementation of a sound engine.
     *@return The sound engine for this platform.
     */
    public SoundEngine getSoundEngine()
        throws GameFrameException
    {
        if ( m_soundEngine == null )
        {
            // Try to create the audio engine and if it fails create the null sound engine
            try
		    {
                m_soundEngine = new CSoundEngine();
		    }
            catch( Exception anyException )
            {
                m_soundEngine = NullSoundEngine.getInstance();
            }
            catch( Error anyError )
            {
                m_soundEngine = NullSoundEngine.getInstance();
            }
            
            m_finalizableObjects.addElement( m_soundEngine );
        }

        return m_soundEngine;
    }
    
    /**
     * Returns the graphics engine implementation for the current platform.
     * If not successfull returns null.
     *@return The graphics engine for this platform.
     */
    public GraphicsEngine getGraphicsEngine()
        throws GameFrameException
    {
        if ( m_graphicsEngine == null )
        {
            CComponentGraphicsEngine componentEngine = null;

            // Create a component graphics engine
            try
            {
                componentEngine = CComponentGraphicsEngine.getInstance( m_settings );

                // Add the graphics engine finalizable objects
                m_finalizableObjects.addElement( componentEngine );
            }
            catch(Exception anyException)
            {
                throw new GameFrameException("Couldn't create graphics engine. "+anyException);
            }
            catch(Error anyError)
            {
                throw new GameFrameException("Couldn't create graphics engine. "+anyError);
            }

            switch( m_settings.getScreenMode() )
            {
                case GameFrameSettings.SCREENMODE_FULLSCREEN:
                    // Create an emulated fullscreen graphics engine
                    try
                    {
                        CEmulatedFullscreenGraphicsEngine fullscreenEngine =
                            CEmulatedFullscreenGraphicsEngine.getInstance(
                                componentEngine, m_settings);

                        // Set the references to focus component and graphics engine
                        m_focusComponent    = (java.awt.Component) componentEngine;

                        // Set the reference to the grapchis engine
                        m_graphicsEngine = componentEngine;

                        // Add the graphics engine finalizable objects
                        m_finalizableObjects.addElement( fullscreenEngine );

                        // We'll wait until the peer for
                        // the component engine has been created
                        while( !componentEngine.isAddNotified() )
                        {
                            try
                            {
                                Thread.currentThread().sleep(10);
                            }
                            catch(InterruptedException ie)
                            {
                            }
                        }

                    }
                    catch(Exception anyException)
                    {
                        throw new GameFrameException("Couldn't create graphics engine. "+anyException);
                    }
                    catch(Error anyError)
                    {
                        throw new GameFrameException("Couldn't create graphics engine. "+anyError);
                    }

                    break;
                case GameFrameSettings.SCREENMODE_WINDOWED:
                    // Create a windowed graphics engine
                    try
                    {
                        CWindowedGraphicsEngine windowedEngine = CWindowedGraphicsEngine.getInstance(
                            componentEngine, m_settings);

                        // Set the references to focus component and graphics engine
                        m_focusComponent    = (java.awt.Component) componentEngine;

                        // Set the reference to the grapchis engine
                        m_graphicsEngine = componentEngine;

                        // Add the graphics engine finalizable objects
                        m_finalizableObjects.addElement( windowedEngine );

                        // We'll wait until the peer for
                        // the component engine has been created
                        while( !componentEngine.isAddNotified() )
                        {
                            try
                            {
                                Thread.currentThread().sleep(10);
                            }
                            catch(InterruptedException ie)
                            {
                            }
                        }
                    }
                    catch(Exception anyException)
                    {
                        throw new GameFrameException("Couldn't create graphics engine. "+anyException);
                    }
                    catch(Error anyError)
                    {
                        throw new GameFrameException("Couldn't create graphics engine. "+anyError);
                    }

                    break;
                case GameFrameSettings.SCREENMODE_COMPONENT:
                    
                    // Set the references to focus component and graphics engine
                    m_focusComponent    = (java.awt.Component) componentEngine;

                    // Set the reference to the grapchis engine
                    m_graphicsEngine = componentEngine;
                    break;
            }
        }

        return m_graphicsEngine;
    }

    /**
     * Returns the input engine implementation for the current platform.
     * If not successfull returns null.
     *@return The input engine for this platform.
     */
    public InputEngine getInputEngine()
        throws GameFrameException
    {
        if ( m_inputEngine == null )
        {
            if ( getGraphicsEngine() == null )
            {
                return null;
            }

            if ( m_focusComponent == null )
            {
                return null;
            }
            
            m_inputEngine = CInputEngine.getInstance( m_focusComponent );
            m_finalizableObjects.addElement( m_inputEngine );
        }
        
        return m_inputEngine;
    }

    /**
     * Returns a new timer instance. Several timers can be created with this method. If a native
     * timer can't be for some reason created, this method returns the basic implementation
     * CTimer from the GameFrame.Java11 package.
     *@return Timer instance.
     */
    public Timer createTimer()
        throws GameFrameException
    {
        CTimer timer = new CTimer( this );
        m_finalizableObjects.addElement( timer );
        return timer;
    }
    
    /**
     * Notifies that the given timer has been finalized.
     */
    public void finalizationNotify ( Timer timer )
    {
        m_finalizableObjects.removeElement( timer );
    }

    /**
     * Invoked before application exits. Calls finalize on all created
     * engines.
     */
    public void finalize()
    {
        // Finalize all objects that must be finalized
        for ( Enumeration enum = m_finalizableObjects.elements(); enum.hasMoreElements(); )
        {
            Finalizable finalizableObject = (Finalizable) enum.nextElement();
            finalizableObject.finalize();
        }
        m_finalizableObjects.removeAllElements();
        m_focusComponent = null;
        
        // Reset static variables, except flag that indicates
        // if Java Media Framework is available as it is set
        // only once in static initialiser
        mStatic_engineFactory = null;
    }
}
