/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk11x;
import java.io.*;

/**
 * Looping sound stream that uses the given CSoundStream as the
 * source for audio data. This means that the sample data is not
 * duplicated (which of course saves memory).
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class CLoopingSoundStream extends InputStream
{
    /** The sound stream object that is used as the base for looping sound stream. */
    private CSoundStream m_soundStream;

    /**
     * Constructs a looping sound stream from the given sound stream.
     *@param soundStream The sound stream used as the base.
     */
    public CLoopingSoundStream(CSoundStream soundStream)
    {
        m_soundStream = soundStream;
    }
    
    /**
     * Overrides the read method of input stream to
     * read from the underlying sound stream with
     * looping.
     *@return The read value.
     */
    public int read() throws IOException
    {
        int value = m_soundStream.read();

        if(value == -1)
        {
            reset();
            value = m_soundStream.read();
        }
        return value;
    }

    /**
     * Overrides the read method of input stream to
     * read from the underlying sound stream with
     * looping.
     *@param aBuffer The buffer to read into.
     *@return The number of read values.
     */
    public int read(byte[] aBuffer) throws IOException
    {
        return read(aBuffer, 0, aBuffer.length);
    }
    
    /**
     * Overrides the read method of input stream to
     * read from the underlying sound stream with
     * looping.
     *@param aBuffer The buffer to read into.
     *@param offset The offset to the buffer.
     *@param length The length of read.
     *@return The number of total read values.
     */
    public int read(byte aBuffer[], int offset, int length) throws IOException
    {
        int i;
        for(i = 0; i < length;)
        {
            int numRead = m_soundStream.read(aBuffer, offset + i, length - i);
            if(numRead >= 0)
            {
                i += numRead;
            }
            else
            {
                reset();
            }
        }

        return i;
    }

    /**
     * Redirected to underlying CSoundStream object.
     */
    public long skip(long numSkippedBytes) throws IOException
    {
        return m_soundStream.skip(numSkippedBytes);
    }

    /**
     * Redirected to underlying CSoundStream object.
     */
    public int available() throws IOException
    {
        return m_soundStream.available();
    }
    
    /**
     * Redirected to underlying CSoundStream object.
     */
    public void close() throws IOException
    {
        m_soundStream.close();
    }
    
    /**
     * Redirected to underlying CSoundStream object.
     */
    public void mark(int readLimit)
    {
        m_soundStream.mark(readLimit);
    }
    
    /**
     * Redirected to underlying CSoundStream object.
     */
    public void reset() throws IOException
    {
        m_soundStream.reset();
    }
    
    /**
     * Redirected to underlying CSoundStream object.
     */
    public boolean markSupported()
    {
        return m_soundStream.markSupported();
    }
}
