/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk11x;
import java.io.*;
import sun.audio.*;

/**
 * Implementation of a sound stream that uses an array as the source
 * for the streamed sound (sample) data.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class CSoundStream extends ByteArrayInputStream
{
    /** The volume. */
    private float m_volume;
    /** The calculated amount to sift to right (to achieve the volume). */
    private int m_siftAmount;
    /** The sound stream buffer. */
    private byte[] m_aBuffer;

    /**
     * Creates a new byte array input stream that reads data from the 
     * specified byte array. The byte array is not copied. 
     * @param   buffer  The buffer.
     */
    public CSoundStream(byte aBuffer[])
    {
        super(aBuffer);
        m_aBuffer = aBuffer;
    }


    /**
     * Sets the volume adjustment to the given value.
     *@param volume The current volume.
     */
    public void setVolume(float volume)
    {
        m_volume = volume;
        m_siftAmount = (int) ( (1.0F - volume) * 12.0F);
    }
    
    /**
     * Reads up to <code>len</code> bytes of data into an array of bytes 
     * from this input stream. This <code>read</code> method cannot block. 
     *
     * @param   buffer the buffer into which the data is read.
     * @param   offset the start offset of the data.
     * @param   length the maximum number of bytes read.
     * @return  the total number of bytes read into the buffer, or
     *          <code>-1</code> if there is no more data because the end of
     *          the stream has been reached.
     */
    public int read(byte buffer[], int offset, int length)
    {
        byte readValue;
        int segment;
        int value;
        int sign;
        int siftAmount;

        // If sifting is zero we need no processing...
        if (m_siftAmount == 0)
        {
            return super.read(buffer, offset, length);
        }

        int numRead = super.read(buffer, offset, length);

        if (numRead > 0)
        {
            for(int i = 0; i < numRead; i++)
            {
                siftAmount = m_siftAmount;
                readValue = (byte) (0xFF ^ buffer[i]);
                sign      = readValue & 0x80;
                segment   = (readValue >>> 4) & 0x07;
                
                value     = readValue & 0x0F;
                
                if (siftAmount > segment)
                {
                    siftAmount -= segment;
                    segment = 0;
                    value >>>= siftAmount;
                }
                else
                {
                    segment -= siftAmount;
                }

                buffer[i] = (byte) ( ( (segment << 4) | value | sign ) ^ 0xFF );
            }
        }        
        return numRead;
    }
    
    /**
     * Returns reference to the internal data array that contains the sample data.
     * Usefull for e.g. making copies of sounds.
     *@return Reference to the internal data array.
     */
    public byte[] getInternalData()
    {
        return m_aBuffer;
    }
}
