/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk11x;

import gameframe.*;
import gameframe.engines.*;
import gameframe.util.Timer;

/**
 * Interface for a generic timer class that times the (average or exact, depending
 * on implementation) time between the calls made to the calculateTime() method.
 * Useful for e.g. finding out how long a main game loop takes and refining some
 * timing accordingly.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class CTimer implements Timer, Finalizable
{
    /** The average time that has gone in one invokation of calculateTime(). */
    private float m_averageTime;
    /** The number of invokations of calculateTime(). */
    private int m_count;
    /** The last time the average value was calculated. */
    private long m_lastCalculationTime;
    /** The engine factory that created this timer. */
    private EngineFactory m_engineFactory;

    /**
     * Constructs a timer.
     */
    public CTimer( EngineFactory engineFactory )
    {
        m_lastCalculationTime = System.currentTimeMillis();
        m_engineFactory = engineFactory;
    }
    
    /**
     * When invoked calculates the time that has expired since the last invokation in
     * milliseconds.
     *@return The time that has gone since last invokation.
     */
    public float calculateTime()
    {
        m_count++;
        long tms = System.currentTimeMillis();
        
        if (tms == m_lastCalculationTime)
        {
            return m_averageTime;
        }
        
        m_averageTime = ( (float) ( tms - m_lastCalculationTime ) ) / ( (float) m_count );
        m_lastCalculationTime = tms;
        m_count = 0;
        
        return m_averageTime;
    }
    
    /**
     * Frees all the resources used by this class.
     */
    public void finalize()
    {
        m_engineFactory.finalizationNotify( this );
    }
}
