/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk11x;

import gameframe.graphics.*;
import gameframe.engines.*;
import gameframe.*;

import java.awt.*;

/**
 * Java 1.1 AWT implementation of a windowed graphics engine.
 * Uses the component graphics engine inside the frame window.
 *
 *@see CComponentGraphicsEngine
 *@author Pasi Kernen 04. April 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class CWindowedGraphicsEngine extends Frame implements Finalizable
{
    /** The singleton instance of this graphics engine. */
    private static CWindowedGraphicsEngine mStatic_graphicsEngine;
    /** The contained component graphics engine. */
    private static java.awt.Component mStatic_componentGraphicsEngine;
    /** Insets from the window frames. */
    private Insets m_insets = new Insets( 0,0,0,0 );
    /** Indicates whether the window has its peer created or not. */
    private boolean m_fAddNotified = false;

    /**
     * The default constructor that calls the superclass constructor that
     * sets the title of the frame to the title in the settings and sets the created
     * frame to be non-resizable.
     *@param componentGrfxEngine The component graphics engine used.
     *@param settings The settings object that contains settings for this graphics engine.
     */
    private CWindowedGraphicsEngine(
        java.awt.Component componentGrfxEngine, GameFrameSettings settings )
    {
        super( settings.getTitle() );

        // Show the frame to make the insets valid
        show();

        // Wait until the peer of the window has been created
        while( !m_fAddNotified )
        {
            try
            {
                Thread.currentThread().sleep( 20 );
            }
            catch( InterruptedException interrupted )
            {
            }
        }

        // Get the insets
        m_insets = getInsets();

        // Center the window in to the middle of the screen
        // and make it large enough (include insets)
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        int width     = settings.getRequiredResolutionWidth();
        int height    = settings.getRequiredResolutionHeight();
        int x         = ( dim.width  - width  ) / 2;
        int y         = ( dim.height - height ) / 2;
        setBounds(
            x,
            y,
            width  + m_insets.left + m_insets.right,
            height + m_insets.top  + m_insets.bottom );

        // Add the component graphics engine
        setLayout( new BorderLayout() );
        add( componentGrfxEngine, BorderLayout.CENTER );
        setResizable( false );
    }

    /**
     * Capture the creation of the peer and allow the constructor
     * to continue only when the peer has been created.
     */
    public void addNotify()
    {
        super.addNotify();
        m_fAddNotified = true;
    }

    /**
     * Returns an instance of this class. The instance is a window that
     * has the given title and size.
     *@param componentGrfxEngine The component graphics engine used.
     *@param settings The settings object that contains settings for this graphics engine.
     */
    public static CWindowedGraphicsEngine getInstance(
        java.awt.Component componentGrfxEngine, GameFrameSettings settings )
    {
        if ( mStatic_graphicsEngine == null )
        {
            // Preserve the component graphics engine reference
            mStatic_componentGraphicsEngine = componentGrfxEngine;

            // Create the singleton instance of this graphics engine
            mStatic_graphicsEngine = new CWindowedGraphicsEngine(
                componentGrfxEngine, settings );


            // When frame gets focus give it to the graphics engine component
            mStatic_graphicsEngine.addFocusListener( new java.awt.event.FocusAdapter()
            {
                public void focusGained( java.awt.event.FocusEvent event )
                {
                    if ( mStatic_graphicsEngine != null )
                    {
                        mStatic_componentGraphicsEngine.requestFocus();
                    }
                }
            } );
        
            mStatic_componentGraphicsEngine.requestFocus();
        }
        
        return mStatic_graphicsEngine;
    }

	/**
	 * Invoked when the program exiting. Should do any
	 * necessary clean up, switch back to desktop etc.
	 */
	public void finalize()
    {
        if ( mStatic_graphicsEngine != null )
        {
            // The window no longer has a peer
            m_fAddNotified = false;

            // Dispose the singleton instance reference
            mStatic_graphicsEngine = null;

            // Dispose the graphics engine
            remove( mStatic_componentGraphicsEngine );
            mStatic_componentGraphicsEngine = null;

            // Hide this frame
            setVisible( false );

            // Dispose this frame
            dispose();
        }
    }
}
