/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk12x;

import gameframe.graphics.*;
import gameframe.engines.*;
import gameframe.*;

import java.awt.*;
import javax.swing.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Java 2 AWT implementation of the graphics engine. For some odd reason this
 * works only if the component is Canvas object.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.3
 *@since GameFrame for Java 0.9
 */
public class CComponentGraphicsEngine extends Canvas
    implements GraphicsEngine, ImageObserver, Finalizable
{
    /** The singleton instance of this graphics engine. */
    private static CComponentGraphicsEngine mStatic_graphicsEngine;
    /** The size of this component. */    
    private Dimension mStatic_size = new Dimension();

    /** All the bitmaps created through this graphics engine. */
    private Vector m_createdBitmaps;
    /** All the drawable bitmaps created through this graphics engine. */
    private Vector m_createdDrawableBitmaps;
    /** Name of the directory that contains loaded bitmaps. */
    private String m_strDataDirectory = "";

    /** The color that is used when painting the backbuffer in the clear operation. */
    private Color m_backgroundColor = Color.black;
	/** The offscreen image used for double buffering. */
	private Image m_backBuffer;
	/** The graphics context of the current offscreen buffer. */
	private Graphics2D m_backBufferGraphics;
	/** The image that is shown on the screen upon repaint. */
	private Image m_frontBuffer;
	/** The graphics context of the image shown on the screen. */
	private Graphics2D m_frontBufferGraphics;
	/** The graphics context of the component. */
    private Graphics2D m_componentGraphics;
	/** Indicates if this frame has been addNotified. */
	private boolean m_fIsNotified;
    /** Width of this frame. */
    private int m_width;
    /** Height of this frame. */
    private int m_height;
    /** The bitmap that contains all the letters in the current font. */
    private Bitmap m_fontBitmap;
    /** Width of all the letters in the font bitmap. */
    private int m_letterWidth = 0;
    /** Height of all the letters in the font bitmap. */
    private int m_letterHeight = 0;
    /** Bitmaps of the fonts. */
    private Bitmap[] m_aFonts = new Bitmap[256];
    /** The graphics settings that are set to every graphics object. */
    private static Hashtable mStatic_graphicsSettings;

    static
    {
        mStatic_graphicsSettings = new Hashtable();
        mStatic_graphicsSettings.put(
            RenderingHints.KEY_ALPHA_INTERPOLATION,
            RenderingHints.VALUE_ALPHA_INTERPOLATION_SPEED );
        mStatic_graphicsSettings.put(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_OFF );
        mStatic_graphicsSettings.put(
            RenderingHints.KEY_COLOR_RENDERING,
            RenderingHints.VALUE_COLOR_RENDER_SPEED  );
        mStatic_graphicsSettings.put(
            RenderingHints.KEY_DITHERING,
            RenderingHints.VALUE_DITHER_DISABLE );
        mStatic_graphicsSettings.put(
            RenderingHints.KEY_FRACTIONALMETRICS,
            RenderingHints.VALUE_FRACTIONALMETRICS_OFF );
        mStatic_graphicsSettings.put(
            RenderingHints.KEY_INTERPOLATION  ,
            RenderingHints.VALUE_INTERPOLATION_BILINEAR );
        mStatic_graphicsSettings.put(
            RenderingHints.KEY_RENDERING,
            RenderingHints.VALUE_RENDER_SPEED );
    }

    /**
     * The default constructor that calls the superclass constructor that
     * sets the title of the frame to "GameFrame" and sets the created
     * frame to be non-resizable.
     *@param parentFrame The frame to be used as the parent for the game window.
     *@param settings The settings object that contains settings for this graphics engine.
     */
    private CComponentGraphicsEngine( GameFrameSettings settings )
    {
        m_width  = settings.getRequiredResolutionWidth();
        m_height = settings.getRequiredResolutionHeight();
        mStatic_size.width  = m_width;
        mStatic_size.height = m_height;
        
        m_createdBitmaps = new Vector( 20, 20 );
        m_createdDrawableBitmaps = new Vector( 20, 20 );
        super.requestFocus();
    }

    /**
     * Returns an instance of this class. The instance is a window that
     * has the given title and size.
     *@param settings The settings object that contains settings for this
     *       graphics engine.
     */
    static CComponentGraphicsEngine getInstance( GameFrameSettings settings )
    {
        if ( mStatic_graphicsEngine == null )
        {
            // Create graphics engine
            mStatic_graphicsEngine =
                new CComponentGraphicsEngine( settings );
        }

        return mStatic_graphicsEngine;
    }

    /**
     * Returns the short name (max 8 characters) of the
     * implementation "Java2".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "Java2";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Graphics engine that uses Java 2 AWT.";
	}

    /**
     * This method is used by the graphics engine to wait
     * until this frame has been add notified.
     *@return True if the frames peer is created.
     */
    boolean isAddNotified()
    {
        return m_fIsNotified;
    }

	/**
	 * Load a bitmap (used bitmap format is Windows BMP, no alpha values are loaded)
	 * from the given file and return an abstract reference for the loaded bitmap.
	 * Bitmap is loaded from the bitmap resource directory specified by MainFactory.
	 *@param strFileName The name of the file to be loaded (without the file extension).
	 *@return An unique Bitmap object that represents the loaded bitmap image.
	 */
    public Bitmap loadBitmap(String strFileName)
        throws java.io.FileNotFoundException, GameFrameException
    {
        strFileName += ".bmp";
        BitmapData bitmapData = BitmapFile.loadBMP( m_strDataDirectory + strFileName, false );

        // Create the image from the bitmap data
	    Image image = createImage ( new MemoryImageSource (
            bitmapData.getWidth(),
            bitmapData.getHeight(),
            bitmapData.getPixels(),
            0,
            bitmapData.getWidth() ) );

        // Wait until the image has been completely prepared for rendering
        while ( !prepareImage( image, this ) );

        Bitmap bitmap = new CBitmap( image );
        m_createdBitmaps.addElement( bitmap );
        return bitmap;
    }

	/**
	 * Load a bitmap (used bitmap format is Windows BMP, alpha keycolor is set to the
	 * pixel color at bitmap location 0,0 which is the pixel at upper left corner of
	 * the bitmap)from the given file and return an abstract reference for the loaded bitmap.
	 * Bitmap is loaded from the bitmap resource directory specified by MainFactory.
	 *@param strFileName The name of the file to be loaded (without the file extension).
	 *@return An unique Bitmap object that represents the loaded bitmap image.
	 */
    public Bitmap loadAlphaBitmap(String strFileName)
        throws java.io.FileNotFoundException, GameFrameException
    {
        strFileName += ".bmp";
        BitmapData bitmapData = BitmapFile.loadBMP( m_strDataDirectory + strFileName, true );

        // Create the image from the bitmap data
	    Image image = createImage ( new MemoryImageSource (
            bitmapData.getWidth(),
            bitmapData.getHeight(),
            bitmapData.getPixels(),
            0,
            bitmapData.getWidth() ) );

        // Wait until the image has been completely prepared for rendering
        while ( !prepareImage( image, this ) );

        Bitmap bitmap = new CBitmap( image );
        m_createdBitmaps.addElement( bitmap );
        return bitmap;
    }

    /**
     * Loads and returns a transparent bitmap (used bitmap format is Windows
     * BMP 8-bit, 16-bit or 24-bit) from the given filename. Bitmap is loaded
     * by default from the current data directory, the directory that is used
     * to load the bitmaps can be set with the setDataDirectory() method.
     * Alpha keycolor (the color that is interpreted as the transparent color)
     * is taken from the given red, green and blue values. If the bitmap is
     * larger than current resolution it will be clipped to the dimensions of
     * the current resolution.
     *
     *@param strFileName The name of the file to be loaded (without the file extension).
     *@param red The red intensity value of the key color (in range 0x00-0xFF).
     *@param green The green intensity value of the key color (in range 0x00-0xFF).
     *@param blue The blue intensity value of the key color (in range 0x00-0xFF).
     *@return Bitmap object that represents the loaded bitmap image or null if the loading failed.
     *@exception FileNotFoundException Thrown if the given file doesn't exist.
     *@exception GameFrameException Thrown if other error occur while loading the file.
     *@since GameFrame for Java 0.9
     */
    public Bitmap loadAlphaBitmap( String strFileName, byte red, byte green, byte blue )
        throws FileNotFoundException, GameFrameException
    {
        strFileName += ".bmp";
        BitmapData bitmapData = BitmapFile.loadBMP(
            m_strDataDirectory + strFileName, red, green, blue );

        // Create the image from the bitmap data
	    Image image = createImage ( new MemoryImageSource (
            bitmapData.getWidth(),
            bitmapData.getHeight(),
            bitmapData.getPixels(),
            0,
            bitmapData.getWidth() ) );

        // Wait until the image has been completely prepared for rendering
        while ( !prepareImage( image, this ) );

        Bitmap bitmap = new CBitmap( image );
        m_createdBitmaps.addElement( bitmap );
        return bitmap;
    }

    /**
     * Creates a non-transparent bitmap (the color is set to the current background color) that
     * can be drawn into with other bitmaps.
     * If the creation of the bitmap fails this method returns null.
     *@param width The width of the bitmap.
     *@param height The height of the bitmap
     *@return The created bitmap or null if failed to create.
     */
    public DrawableBitmap getDrawableBitmap(int width, int height)
        throws GameFrameException
    {
        Image image = createImage( width, height );

        if ( image != null )
        {
            Graphics2D graphics = (Graphics2D) image.getGraphics();
            if (graphics == null)
            {
                throw new GameFrameException(
                    "Couldn't get the graphics of the drawable bitmap.");
            }

            //graphics.setClip(0,0,width, height);
		    graphics.setColor(m_backgroundColor);
		    graphics.fillRect(0, 0, width, height);
            DrawableBitmap bitmap = new CDrawableBitmap(image, graphics);
            m_createdDrawableBitmaps.addElement(bitmap);
            return bitmap;
        }
        else
        {
            throw new GameFrameException("Couldn't create image.");
        }
    }

    /**
     * Creates an alpha bitmap of the given size.
     *@param width The width of the alpha bitmap.
     *@param height The height of the alpha bitmap.
     */
    private CBitmap createAlphaBitmap(int width, int height)
        throws GameFrameException
    {
        // Create an array of bytes whose every cell is initialized to value 0
        int[] aPixels = new int[width*height];

        // Create an image based on the array
        Image image = createImage( new MemoryImageSource ( width, height, aPixels, 0, width ) );

        // Check that we are successfull
        if (image != null)
        {
            CBitmap bitmap = new CBitmap( image );
            m_createdBitmaps.addElement( bitmap );
            return bitmap;
        }

        throw new GameFrameException( "Couldn't create image." );
    }

    /**
     * Moves by copying a rectangular area of the backbuffer by a distance
     * specified by dx and dy. From the point specified by x and y, this
     * method moves downwards and to the right. To move an area of the
     * component to the left or upwards, specify a negative value for dx or
     * dy. If a portion of the source rectangle lies outside the bounds of
     * the component moveArea will be unable to move the associated pixels.
     *@param x The x-coordinate of the source rectangle.
     *@param y The y-coordinate of the source rectangle.
     *@param width The width of the source rectangle.
     *@param height The height of the source rectangle.
     *@param dx The horizontal distance to copy the pixels.
     *@param dy The vertical distance to copy the pixels.
     */
    public void copyArea( int x, int y, int width, int height, int dx, int dy )
    {
        m_backBufferGraphics.setClip( 0, 0, m_width, m_height );
        m_backBufferGraphics.copyArea( x, y, width, height, dx, dy );
    }

    /**
     * Switches the backbuffer and the currently displayed buffer.
     */
    public void flip()
    {
        // Switch the front and backbuffers
        Image tempImage         = m_frontBuffer;
        Graphics2D tempGraphics = m_frontBufferGraphics;
        m_frontBuffer           = m_backBuffer;
        m_frontBufferGraphics   = m_backBufferGraphics;
        m_backBuffer            = tempImage;
        m_backBufferGraphics    = tempGraphics;

        try
        {
            // Draw the new frontbuffer to the screen
 		    m_componentGraphics.drawImage( m_frontBuffer, 0, 0, null );
        }
        catch(Exception anyException)
        {
            // Try to fix the error by getting newest graphics context
            m_componentGraphics = (Graphics2D) getGraphics();
 		    m_componentGraphics.drawImage(m_frontBuffer, 0, 0, null);
        }
        catch(Error anyError)
        {
            // Try to fix the error by getting newest graphics context
            m_componentGraphics = (Graphics2D) getGraphics();
 		    m_componentGraphics.drawImage(m_frontBuffer, 0, 0, null);
        }
   }

    /**
     * Clears the backbuffer with the current backround color.
     * All exceptions are held inside this method.
     */
    public void clearBackbuffer()
    {
        try
        {
		    m_backBufferGraphics.setColor( m_backgroundColor );
		    m_backBufferGraphics.fillRect( 0, 0, m_width, m_height );
        }
        catch(Exception anyException)
        {
            // Try to fix the error by getting newest graphics context
            m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
            m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
		    m_backBufferGraphics.setColor( m_backgroundColor );
		    m_backBufferGraphics.fillRect( 0, 0, m_width, m_height );
        }
        catch(Error anyError)
        {
            // Try to fix the error by getting newest graphics context
            m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
            m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
		    m_backBufferGraphics.setColor( m_backgroundColor );
		    m_backBufferGraphics.fillRect( 0, 0, m_width, m_height );
        }
    }

    /**
     * Sets the data directory wherefrom the bitmaps are loaded.
     * Defaults to the current directory. Do not add the last directory separator.
     *@param strDirectoryName The name of the directory that contains the bitmaps.
     *@throws FileNotFoundException Thrown if the data directory doesn't exist.
     */
    public void setDataDirectory( String strDirectoryName )
        throws java.io.FileNotFoundException
    {
        // If there is a path separator, remove it
        if ( strDirectoryName.charAt( strDirectoryName.length() - 1 ) == File.separatorChar )
        {
            strDirectoryName = strDirectoryName.substring( 0, strDirectoryName.length() - 1 );
        }

        // Make sure the given directory exists
        File file = new File( strDirectoryName );
        if ( ( !file.isDirectory() ) || ( !file.exists() ) )
        {
            throw new java.io.FileNotFoundException(
                "Given data directory \""+
                strDirectoryName+
                "\" doesn't exist.");
        }

        m_strDataDirectory = strDirectoryName + File.separator;
    }

    /**
     * Returns the data directory wherefrom the bitmaps are loaded.
     * The source directory defaults to the current directory.
     *
     * Note: The returned path will have the path separator as the last character.
     *
     *@see #loadBitmap(java.lang.String)
     *@see #loadAlphaBitmap(java.lang.String)
     *@return The name of the directory that wherefrom the bitmaps are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory()
    {
        return m_strDataDirectory;
    }

    /**
     * Returns false as this engine is allways in windowed mode.
     *@return False as the engine is allways in windowed mode.
     */
    public boolean isInFullscreenMode()
    {
        return false;
    }

    /**
     * Overrides the java.awt.Component's method to just draw the
     * current frontbuffer to the screen.
     *@param grfx The graphics context used in drawing.
     */
    public void update( Graphics grfx )
    {
        if ( m_frontBuffer != null )
        {
            grfx.drawImage( m_frontBuffer, 0, 0, this );
        }
    }

    /**
     * Overrides the java.awt.Component's method to just draw the
     * current frontbuffer to the screen.
     *@param grfx The graphics context used in drawing.
     */
    public void paint( Graphics grfx )
    {
        update( grfx );    
    }



    /**
     * Sets the currently used background color (defaults to Color.black).
     */
    public void setBackgroundColor( java.awt.Color color )
    {
        m_backgroundColor = color;

        Container parent = getParent();
        if ( parent != null )
        {
            parent.setBackground( color );
        }
    }

    /**
     * Returns the size of the display area that this engine uses. For fullscreen
     * modes it is the size of the fullscreen mode. For windowed modes the size is
     * size of the window.
     *@return The size of the display area.
     */
    public java.awt.Dimension getSize()
    {
        return super.getSize();
    }

    /**
     * Returns the maximum size of this component graphics engine.
     * The values are gotten from the GameFrameSettings object.
     *@return The user set width and height of this component.
     */
    public Dimension getMaximumSize()
    {
        return mStatic_size;
    }

    /**
     * Returns the minimum size of this component graphics engine.
     * The values are gotten from the GameFrameSettings object.
     *@return The user set width and height of this component.
     */
    public Dimension getMinimumSize()
    {
        return mStatic_size;        
    }

    /**
     * Returns the preferred size of this component graphics engine.
     * The values are gotten from the GameFrameSettings object.
     *@return The user set width and height of this component.
     */
    public Dimension getPreferredSize()
    {
        return mStatic_size;        
    }

    /**
     * Returns true, which means this component is focus traversable with
     * TAB and SHIFT-TAB keys.
     *@return True.
     */
    public boolean isFocusTraversable()
    {
        return true;
    }

    /**
     * Overridden to allways return true.
     *@return True.
     */
    public boolean isOpaque()
    {
        return true;
    }

	/**
	 * Invoked when the program exiting. Should do any
	 * necessary clean up, switch back to desktop etc.
	 */
	public void finalize()
    {
        if ( mStatic_graphicsEngine != null )
        {
            // Dispose the singleton instance reference
            mStatic_graphicsEngine = null;

            // Dispose all created bitmaps
            for ( Enumeration enum = m_createdBitmaps.elements(); enum.hasMoreElements(); )
            {
                Bitmap bitmap = (Bitmap) enum.nextElement();
                bitmap.finalize();
            }
            m_createdBitmaps = null;

            // Dispose all created drawable bitmaps
            for ( Enumeration enum = m_createdDrawableBitmaps.elements(); enum.hasMoreElements(); )
            {
                DrawableBitmap bitmap = (DrawableBitmap) enum.nextElement();
                bitmap.finalize();
            }
            m_createdDrawableBitmaps = null;
        }
    }

    /**
     * Sets the title of the window (if in windowed mode).
     *@param title The title of the window.
     */
    public void setTitle( String title )
    {
        setTitle( title );
    }
    
    /**
     * Loads the default font bitmap that is used when printing text on the screen.
     * Either this method or the setFontBitmap method must be called before using
     * any of the getAsString() or drawTextAt() methods.
     */
    public void loadDefaultFontBitmap()
        throws java.io.FileNotFoundException, GameFrameException
    {
        // Store the original datapath and set nothing to be the datapath
        String originalPath = m_strDataDirectory;
        m_strDataDirectory = "";

        // Check that the font bitmap name actually exists.
        String strDefaultFontBitmapFilename = GameFrame.getInstance().getDefaultFontBitmapFilename();
        if ( ( strDefaultFontBitmapFilename != null ) &&
             ( !strDefaultFontBitmapFilename.startsWith( "null" ) ) )
        {
            try
            {
                loadFontBitmap( strDefaultFontBitmapFilename );
            }
            finally
            {
                m_strDataDirectory = originalPath;
            }
        }
        else
        {
            throw new java.io.FileNotFoundException(
                "Default fontbitmap directory \""+
                strDefaultFontBitmapFilename+
                "\" not found." );
        }

        // We can't load the default font bitmap
        m_fontBitmap = null;
        m_strDataDirectory = originalPath;
    }

    /**
     * Sets the used font set to be loaded from the given strFileName using the specified
     * fixed size for every letter. The strFileName must specify a window BMP formatted
     * bitmap that is then loaded as transparent (alpha) bitmap and is then divided in
     * pieces that represent one letter each piece. The order in the bitmap must use
     * the standard ASCII letter order (can be obtained by running the PrintFontOrder
     * class in the GameFrame.Util package) and it must contain 16 columns of letters
     * in 16 rows (that is a 16x16 matrix of letters = 256 letters in whole). If a
     * nonexisting bitmap or fontbitmap of invalid size is used, the method will do
     * nothing. Either this method or the setDefaultFontBitmap method must be called
     * before using any of the getAsString() or drawTextAt() methods.
     *@param strFileName Name of the bitmap file that contains the letters.
     */
    public void loadFontBitmap( String strFileName )
        throws java.io.FileNotFoundException, GameFrameException
    {
        // Load the font bitmap
        m_fontBitmap = loadAlphaBitmap( strFileName );
        
        // Check that the font bitmap has reasonably big size
        if ( ( m_fontBitmap.getWidth()  / 16 == 0 ) ||
             ( m_fontBitmap.getHeight() / 16 == 0 ) )
        {
            throw new GameFrameException(
                "Given fontbitmap \""+
                strFileName+
                "\" is too small." );
        }
        
        // Calculate the size of one letter
        int width  = m_fontBitmap.getWidth()  / 16;
        int height = m_fontBitmap.getHeight() / 16;
        m_letterWidth  = width;
        m_letterHeight = height;

        // Divide the loaded bitmap to 256 letters
        for ( int i = 0; i < 256; i++ )
        {
            Bitmap letter = m_fontBitmap.getSubBitmap( ( i% 16 ) * width, ( i / 16 ) * height, width, height );
            m_aFonts[i]   = letter;
        }
    }

    /**
     * Returns the given one line of text's width when we are using the current font bitmap.
     * The method doesn't take into account possible clipping to current screen size.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param strText The text that is written using the current font.
     *@return The width of the given text.
     *@since GameFrame for Java 0.9.3
     */
    public int getTextWidth( String strText )
    {
        return m_letterWidth * strText.length();
    }

    /**
     * Returns the given one line of text's width when we are using the current font bitmap.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param aByteText The byte array containing the text to be measured.
     *@return The width of the given text.
     *@since GameFrame for Java 0.9.3
     */
    public int getTextWidth( byte[] aByteText )
    {
        return m_letterWidth * aByteText.length;
    }

    /**
     * Returns the current font width.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@since GameFrame for Java 0.9.3
     */
    public int getFontWidth()
    {
        return m_letterWidth;
    }
    
    /**
     * Returns the current font height.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@since GameFrame for Java 0.9.3
     */
    public int getFontHeight()
    {
        return m_letterHeight;
    }
    
    /**
     * Returns the given one line of text as an bitmap that is painted using
     * the current font. The returned bitmap will have the background color set
     * to the current background color of the graphics engine. This means also
     * that the returned bitmap is never transparent (due to limitations on
     * some platforms).
     *@param strText The text that is written using the current font.
     *@return The text as bitmap or null if not successfull.
     */
    public DrawableBitmap getAsBitmap( String strText )
        throws GameFrameException
    {
        return getAsBitmap( strText.getBytes() );
    }
    
    /**
     * Returns the given one line of text as an bitmap that is painted using
     * the current font.
     *@param aByteText The byte array containing the text to be printed.
     *@return The text as bitmap or null if not successfull.
     */
    public DrawableBitmap getAsBitmap( byte[] aByteText )
        throws GameFrameException
    {
        if ( m_aFonts == null )
        {
            throw new GameFrameException ( "Fontbitmap not loaded." );
        }
        
        int width  = m_letterWidth;
        int length = aByteText.length;
        
        // Create the bitmap where the letters are put
        CDrawableBitmap bitmap = (CDrawableBitmap) getDrawableBitmap( length * m_letterWidth, m_letterHeight );
        for ( int i = 0; i < length; i++ )
        {
            m_aFonts[ aByteText[i] ].blitTo( bitmap, i * width, 0 );
        }

        return bitmap;
    }
    
    /**
     * Prints the given one line of text to the screen so that the uppermost and
     * leftmost coordinate of the text is at the given location.
     *@param strText The text that is written using the current font.
     *@param x The leftmost x-coordinate for the text.
     *@param y The topmost y-coordinate for the text.
     */
    public void drawTextAt( String strText, int x, int y )
        throws GameFrameException
    {
        drawTextAt( strText.getBytes(), x, y );
    }
    
    /**
     * Prints the given string to the screen so that the uppermost and leftmost coordinate
     * of the text is at the given location.
     *@param aByteText The byte array containing the text to be printed.
     *@param x The leftmost x-coordinate for the text.
     *@param y The topmost y-coordinate for the text.
     */
    public void drawTextAt(byte[] aByteText, int x, int y)
        throws GameFrameException
    {
        if ( m_aFonts == null )
        {
            throw new GameFrameException ( "Fontbitmap not loaded." );
        }
        
        int width  = m_letterWidth;
        int length = aByteText.length;
        
        for ( int i = 0; i < length; i++ )
        {
            m_aFonts[ aByteText[i] ].blitTo( x, y );
            x += width;
        }
    }

    /**
     * Invoked when the peer has been created for this frame.
     */
	public void addNotify()
	{
		super.addNotify();
		if ( !m_fIsNotified )
		{
            m_componentGraphics = (Graphics2D) getGraphics();
            m_componentGraphics.setRenderingHints( mStatic_graphicsSettings );
            m_componentGraphics.setClip( 0, 0, m_width, m_height );

            // Set the flag that indicates we have been notified
			m_fIsNotified = true;

            // Create the front and back buffers
			m_backBuffer  = createImage( m_width, m_height );
            m_frontBuffer = createImage( m_width, m_height );

            // Get the graphics contexts of both buffers
			m_backBufferGraphics  = (Graphics2D) m_backBuffer.getGraphics();
            m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
            m_frontBufferGraphics = (Graphics2D) m_frontBuffer.getGraphics();
            m_frontBufferGraphics.setRenderingHints( mStatic_graphicsSettings );

            // Clear the buffers to the current background color
            clearBackbuffer();
            flip();
            clearBackbuffer();
		}
	}


    /**
     * Implementation of the bitmap image for plain Java 2 graphics engine.
     */
    class CBitmap implements Bitmap,ImageObserver
    {
        /** Reference to the AWT image that represents the bitmap in this graphics engine. */
        Image m_image;
        /** Width of the bitmap. */
        int m_bitmapWidth;
        /** Height of the bitmap. */
        int m_bitmapHeight;
        /** Clipping region used when drawing the bitmap with clipping. */
        Rectangle m_clipper = new Rectangle();

        /**
         * Constructs a bitmap control object from the given AWT image.
         *@param image The image that this bitmap represents.
         */
        CBitmap(java.awt.Image image)
        {
            m_image          = image;
            m_bitmapWidth    = image.getWidth(this);
            m_bitmapHeight   = image.getHeight(this);

            if (m_bitmapWidth > m_width)
            {
                m_bitmapWidth = m_width;
            }

            if (m_bitmapHeight > m_height)
            {
                m_bitmapHeight = m_height;
            }
            m_clipper.x      = 0;
            m_clipper.y      = 0;
            m_clipper.width  = m_bitmapWidth;
            m_clipper.height = m_bitmapHeight;
        }

        /**
         * Draws the bitmap to the graphics engine that loaded it at the given location.
         * The location can be outside of the visible screen area in which case the bitmap
         * is clipped accordingly.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         */
        public void blitTo(int x, int y)
        {
            try
            {
                m_backBufferGraphics.drawImage( m_image, x, y, null );
            }
            catch(Exception anyException)
            {
                System.out.println(
                    "CComponentGraphicsEngine.CBitmap.blitTo(x,y):Exception\n"+
                    "m_backBufferGraphics"+m_backBufferGraphics+" m_image="+m_image+" x="+x+", y="+y+
                    ""+anyException );
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage( m_image, x, y, null );
            }
            catch( Error anyError )
            {
                System.out.println(
                    "CComponentGraphicsEngine.CBitmap.blitTo(x,y):Error\n"+
                    "m_backBufferGraphics"+m_backBufferGraphics+" m_image="+m_image+" x="+x+", y="+y+
                    ""+anyError );
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage( m_image, x, y, null );
            }
        }

        /**
         * Draws the bitmap to the graphics engine that loaded it at the given location and
         * at the given size. If this bitmap is larger or smaller than the destination size
         * the bitmap will automatically be scaled. Note: This method is slow, so it's use is not
         * preferred during actual game play.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         *@param width  The with of the destination where the image is drawn to.
         *@param height The height of the destination where the image is drawn to.
         */
        public void strecthTo( int x, int y, int width, int height )
        {
            try
            {
                m_backBufferGraphics.drawImage(m_image,
                                               x, y,
                                               x+width, y+height,
                                               0, 0,
                                               m_bitmapWidth,
                                               m_bitmapHeight,
                                               null );
            }
            catch(Exception anyException)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage(m_image,
                                               x, y,
                                               x+width, y+height,
                                               0, 0,
                                               m_bitmapWidth,
                                               m_bitmapHeight,
                                               null );
            }
            catch(Error anyError)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage(m_image,
                                               x, y,
                                               x+width, y+height,
                                               0, 0,
                                               m_bitmapWidth,
                                               m_bitmapHeight,
                                               null );
            }
        }

        /**
         * Draws the given bitmap to this drawable bitmap at the given location.
         * The location can be outside of the visible screen area in which case the bitmap
         * is clipped accordingly.
         *@param bitmap The bitmap that will be drawn to this bitmap.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         */
        public void blitTo(DrawableBitmap bitmap, int x, int y)
        {
            ( ( CDrawableBitmap ) bitmap ).m_graphics.drawImage(
                m_image, x, y, null );
        }

        /**
         * This method traps false usage where the user tries to blit a bitmap that was not
         * loaded by this graphics engine to a bitmap that was created by this graphics engine.
         * This method does nothing.
         *@param bitmap The bitmap that will be drawn to this bitmap.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         *@param width  The with of the destination where the image is drawn to.
         *@param height The height of the destination where the image is drawn to.
         */
        public void strecthTo(DrawableBitmap bitmap, int x, int y, int width, int height)
        {
            ((CDrawableBitmap) bitmap).m_graphics.drawImage(
                                               m_image,
                                               x, y,
                                               x+width, y+height,
                                               0, 0,
                                               m_bitmapWidth,
                                               m_bitmapHeight,
                                               null );
        }

        /**
         * Returns the width of the bitmap.
         *@return Width of the bitmap if available.
         */
        public int getWidth()
        {
            return m_bitmapWidth;
        }

        /**
         * Returns the height of the bitmap.
         *@return Height of the bitmap if available.
         */
        public int getHeight()
        {
            return m_bitmapHeight;
        }

        /**
         * Returns an effect processed clone of this bitmap.
         *@param effect The effect to be applied to the clone bitmap,
         *       or null if none are to be applied.
         *@return An effect processed clone of this bitmap.
         *@exception GameFrameException thrown if any errors occur.
         */
        public Bitmap getClone( BitmapEffect effect )
            throws GameFrameException
        {
            // Check if we really need to apply effects
            if ( effect == null )
            {
                return getSubBitmapClone( 0, 0, m_bitmapWidth, m_bitmapHeight );
            }

            try
            {
        	    int[] aPixels   = new int[m_bitmapWidth * m_bitmapHeight];
	            PixelGrabber pg = new PixelGrabber(
                    m_image,
                    0, 0,
                    m_bitmapWidth, m_bitmapHeight,
                    aPixels, 0, m_bitmapWidth );

                try
                {
                    pg.grabPixels();
                }
                catch (InterruptedException e)
                {
                     throw new GameFrameException(
                        "Pixel grabbing failed.");
                }

                if ( ( pg.getStatus() & ImageObserver.ABORT ) != 0 )
                {
                     throw new GameFrameException(
                        "Pixel grabbing failed.");
                }

                BitmapData data = new BitmapData(
                    aPixels, m_bitmapWidth, m_bitmapHeight, m_bitmapWidth, null );
                data = effect.processData( data );

    		    Image image = createImage( new MemoryImageSource(
                    data.getWidth(), data.getHeight(),
                    pg.getColorModel(),
                    data.getPixels(),
                    0, data.getWidth() ) );

                if ( image == null )
                {
                     throw new GameFrameException("Couldn't create image.");
                }

                // Wait until image is prepared
                while ( !prepareImage( image, this ) );

                Bitmap bitmap = new CBitmap( image );
                m_createdBitmaps.addElement( bitmap );
                return bitmap;
            }
            catch( OutOfMemoryError outOfMemory )
            {
                throw new GameFrameException( "Not enough memory available for the operation." );
            }
        }

        /**
         * Returns a bitmap that represents a part of this bitmap. Useful for e.g.
         * extracting animation frames from one bitmap. If for any reason the
         * creation of the new bitmap fails (out of memory etc.) this method returns null.
         * The given dimensions are checked for validity.
         *@param srcX The area of the bitmap to be returned as a new bitmap.
         *@param srcY The area of the bitmap to be returned as a new bitmap.
         *@param srcWidth The area of the bitmap to be returned as a new bitmap.
         *@param srcHeight The area of the bitmap to be returned as a new bitmap.
         *@return A new bitmap representing the requested part of the original bitmap.
         */
        public Bitmap getSubBitmapClone(
            int srcX, int srcY, int srcWidth, int srcHeight)
            throws GameFrameException
        {
            // Check that the dimensions are valid
            if ( (srcWidth < 1) || (srcHeight < 1) )
            {
                throw new GameFrameException("Given source area size is zero.");
            }

            // Check that the bounds stay inside the current clipping rectangle
            if ( (srcX+srcWidth  > m_bitmapWidth) ||
                 (srcY+srcHeight > m_bitmapHeight) ||
                 (srcX < 0) ||
                 (srcY < 0) )
            {
                throw new GameFrameException(
                    "Given source rectangle "+srcX+", "+srcY+"->"+srcWidth+", "+srcHeight+" doesn't fit inside image size.");
            }

            try
            {
        	    int[] aPixels   = new int[srcWidth * srcHeight];
	            PixelGrabber pg =
                    new PixelGrabber(
                        m_image, srcX, srcY,
                        srcWidth, srcHeight,
                        aPixels, 0, srcWidth);

                try
                {
                    pg.grabPixels();
                }
                catch (InterruptedException e)
                {
                     throw new GameFrameException(
                        "Pixel grabbing failed.");
                }

                if ((pg.getStatus() & ImageObserver.ABORT) != 0)
                {
                     throw new GameFrameException(
                        "Pixel grabbing failed.");
                }

    		    Image image = Toolkit.getDefaultToolkit().createImage(
                    new MemoryImageSource(
                        srcWidth, srcHeight,
                        pg.getColorModel(), aPixels, 0, srcWidth ) );
                if (image == null)
                {
                     throw new GameFrameException(
                        "Couldn't create image." );
                }

                while ( !prepareImage( image, this ) );

                Bitmap bitmap = new CBitmap( image );
                m_createdBitmaps.addElement( bitmap );
                return bitmap;
            }
            catch( OutOfMemoryError outOfMemory )
            {
                throw new GameFrameException(
                    "Not enough memory available for the operation." );
            }
        }

        /**
         * Returns a bitmap that represents a part of this bitmap by using a new
         * control object, but using the old bitmap data (saves memory, but is not
         * that fast on many platforms. On the platforms where this method is
         * too slow for interactive use the implementation might actually copy the
         * data as with the getSubBitmapClone() method to speed things up). Anyhow
         * this method is preferred over getSubBitmapClone() method as in some platforms
         * it can save a lot of memory (e.g. under pure Java 1.1 creation of new bitmaps
         * by cloning the data is VERY memory consuming).
         *
         * Useful for e.g. extracting animation frames from one bitmap. This method
         * isn't usually fast and should be preferrably called during game initialization.
         * If for any reason the creation of the new bitmap fails (out of memory etc.)
         * this method returns null. The given dimensions are checked for validity.
         *@param srcX The area of the bitmap to be returned as a new bitmap.
         *@param srcY The area of the bitmap to be returned as a new bitmap.
         *@param srcWidth The area of the bitmap to be returned as a new bitmap.
         *@param srcHeight The area of the bitmap to be returned as a new bitmap.
         *@return A new bitmap representing the requested part of the original bitmap.
         */
        public Bitmap getSubBitmap(
            int srcX, int srcY, int srcWidth, int srcHeight )
                throws GameFrameException
        {
            // Check that the dimensions are valid
            if ( ( srcWidth < 1 ) || ( srcHeight < 1 ) )
            {
                throw new GameFrameException(
                    "Given source area size is zero." );
            }

            // Check that the bounds stay inside the current clipping rectangle
            if ( ( srcX+srcWidth > m_bitmapWidth) ||
                 ( srcY+srcHeight > m_bitmapHeight) ||
                 ( srcX < 0 ) ||
                 ( srcY < 0 ) )
            {
                throw new GameFrameException(
                    "Given source rectangle "+srcX+", "+srcY+"->"+srcWidth+", "+srcHeight+" doesn't fit inside image size." );
            }

            return new CClippedBitmap(
                this, srcX, srcY, srcWidth, srcHeight );
        }

        /**
         * Frees all the resources that are used by the bitmap implementation.
         */
        public void finalize()
        {
            if (m_image != null)
            {
                if ( m_createdBitmaps != null )
                {
                    m_createdBitmaps.removeElement(this);
                }
                m_image = null;
            }
        }

        /**
         * The image observer method that gets updated when an image is loading.
         */
	    public boolean imageUpdate(
            Image  img, int  infoflags, int  x, int  y, int  width, int  height)
	    {
            return true;
	    }
    }

    /**
     * Implementation of the bitmap image for plain Java 2 graphics engine.
     * This bitmap represents a bitmap that uses partial data of some other
     * bitmap and thus has to use clip regions when it is blitted to screen.
     */
    class CClippedBitmap implements Bitmap,ImageObserver
    {

        /** Reference to the AWT image that represents the bitmap in this graphics engine. */
        Image m_image;
        /** Width of the bitmap. */
        int m_bitmapWidth;
        /** Height of the bitmap. */
        int m_bitmapHeight;
        /** Clipping region used when drawing the bitmap. */
        Rectangle m_clipper = new Rectangle();
        /** The X-offset that is added when drawing this bitmap. */
        int m_xOffset = 0;
        /** The Y-offset that is added when drawing this bitmap. */
        int m_yOffset = 0;

        /**
         * Constructs a bitmap control object from the given bitmap.
         * This constructor is used when we want to create only a new
         * control object without duplicating the data.
         *
         *@param original The original bitmap that is used as the data source.
         *@param srcX The source images x-offset.
         *@param srcY The source images y-offset.
         *@param imageWidth The width of this image.
         *@param imageHeight The height of this image.
         */
        CClippedBitmap(
            CBitmap original,
            int srcX, int srcY,
            int imageWidth, int imageHeight)
        {
            m_image          = original.m_image;
            m_bitmapWidth    = imageWidth;
            m_bitmapHeight   = imageHeight;
            if (m_bitmapWidth > m_width)
            {
                m_bitmapWidth = m_width;
            }

            if (m_bitmapHeight > m_height)
            {
                m_bitmapHeight = m_height;
            }
            m_xOffset        = -srcX;
            m_yOffset        = -srcY;
            m_clipper.x      = 0;
            m_clipper.y      = 0;
            m_clipper.width  = m_bitmapWidth;
            m_clipper.height = m_bitmapHeight;
        }

        /**
         * Constructs a bitmap control object from the given bitmap.
         * This constructor is used when we want to create only a new
         * control object without duplicating the data.
         *
         *@param original The original bitmap that is used as the data source.
         *@param srcX The source images x-offset.
         *@param srcY The source images y-offset.
         *@param imageWidth The width of this image.
         *@param imageHeight The height of this image.
         */
        CClippedBitmap(
            CClippedBitmap original,
            int srcX, int srcY,
            int imageWidth, int imageHeight)
        {
            m_image          = original.m_image;
            m_bitmapWidth    = imageWidth;
            m_bitmapHeight   = imageHeight;
            if (m_bitmapWidth > m_width)
            {
                m_bitmapWidth = m_width;
            }

            if (m_bitmapHeight > m_height)
            {
                m_bitmapHeight = m_height;
            }
            m_xOffset        = -srcX + original.m_xOffset;
            m_yOffset        = -srcY + original.m_yOffset;
            m_clipper.x      = 0;
            m_clipper.y      = 0;
            m_clipper.width  = m_bitmapWidth;
            m_clipper.height = m_bitmapHeight;
        }

        /**
         * Draws the bitmap to the given drawable bitmap at the given location.
         * The drawing is clipped to the rectangle of given width and height. The rectangles
         * upperleft corner is at the given drawing location. No scaling will be done to make
         * the bitmap fit or fill that size.
         *@param bitmap The bitmap that will be drawn to this bitmap.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         *@param width The width of the clipping rectangle that starts from the drawing location.
         *@param height The height of the clipping rectangle that starts from the drawing location.
         */
        public void clipTo(DrawableBitmap bitmap, int x, int y, int width ,int height)
        {
            Graphics grfx = ((CDrawableBitmap) bitmap).m_graphics;
            Shape originalClip = grfx.getClip();
            m_clipper.x = x;
            m_clipper.y = y;
            m_clipper.width = width;
            m_clipper.height = height;
            grfx.setClip( m_clipper );
            grfx.drawImage(
                m_image, x+m_xOffset, y+m_yOffset, null );
            m_clipper.width = m_bitmapWidth;
            m_clipper.height = m_bitmapHeight;
            grfx.setClip( originalClip );
        }

        /**
         * Draws the bitmap to the graphics engine that loaded it at the given location.
         * The location can be outside of the visible screen area in which case the bitmap
         * is clipped accordingly.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         */
        public void blitTo(int x, int y)
        {
            Shape originalClip = m_backBufferGraphics.getClip();
            m_clipper.x = x;
            m_clipper.y = y;

            try
            {
                m_backBufferGraphics.setClip(m_clipper);
                m_backBufferGraphics.drawImage(
                    m_image, x+m_xOffset, y+m_yOffset, null );
            }
            catch(Exception anyException)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.setClip(m_clipper);
                m_backBufferGraphics.drawImage(
                    m_image, x+m_xOffset, y+m_yOffset, null );
                System.out.println(
                    "CComponentGraphicsEngine.CClippedBitmap.blitTo(x,y):Exception\n"+
                    "m_backBufferGraphics"+m_backBufferGraphics+" m_image="+m_image+" x="+x+", y="+y+
                    ""+anyException );
            }
            catch(Error anyError)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.setClip(m_clipper);
                m_backBufferGraphics.drawImage(
                    m_image, x+m_xOffset, y+m_yOffset, null );
                System.out.println(
                    "CComponentGraphicsEngine.CClippedBitmap.blitTo(x,y):Error\n"+
                    "m_backBufferGraphics"+m_backBufferGraphics+" m_image="+m_image+" x="+x+", y="+y+
                    ""+anyError );
            }

            m_backBufferGraphics.setClip( originalClip );
        }

        /**
         * Draws the bitmap to the graphics engine that loaded it at the given location and
         * at the given size. If this bitmap is larger or smaller than the destination size
         * the bitmap will automatically be scaled. Note: This method is slow, so it's use is not
         * preferred during actual game play.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         *@param width  The with of the destination where the image is drawn to.
         *@param height The height of the destination where the image is drawn to.
         */
        public void strecthTo(int x, int y, int width, int height)
        {
            Shape originalClip = m_backBufferGraphics.getClip();
            m_clipper.x = x;
            m_clipper.y = y;
            m_clipper.width = width;
            m_clipper.height = height;
            try
            {
                m_backBufferGraphics.setClip(m_clipper);
                m_backBufferGraphics.drawImage(
                    m_image,
                    x, y, x+width, y+height,  // Destination rectangle
                    -m_xOffset, -m_yOffset,   // Source rectangle
                    -m_xOffset+m_bitmapWidth,
                    -m_yOffset+m_bitmapHeight,
                    null );
            }
            catch(Exception anyException)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.setClip(m_clipper);
                m_backBufferGraphics.drawImage(
                    m_image,
                    x, y, x+width, y+height,  // Destination rectangle
                    -m_xOffset, -m_yOffset,   // Source rectangle
                    -m_xOffset+m_bitmapWidth,
                    -m_yOffset+m_bitmapHeight,
                    null );
            }
            catch(Error anyError)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.setClip(m_clipper);
                m_backBufferGraphics.drawImage(
                    m_image,
                    x, y, x+width, y+height,  // Destination rectangle
                    -m_xOffset, -m_yOffset,   // Source rectangle
                    -m_xOffset+m_bitmapWidth,
                    -m_yOffset+m_bitmapHeight,
                    null );
            }

            m_backBufferGraphics.setClip( originalClip );
            m_clipper.width = m_bitmapWidth;
            m_clipper.height = m_bitmapHeight;
        }

        /**
         * Draws the given bitmap to this drawable bitmap at the given location.
         * The location can be outside of the visible screen area in which case the bitmap
         * is clipped accordingly.
         *@param bitmap The bitmap that will be drawn to this bitmap.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         */
        public void blitTo(DrawableBitmap bitmap, int x, int y)
        {
            Graphics grfx = ((CDrawableBitmap) bitmap).m_graphics;
            Shape originalClip = grfx.getClip();
            m_clipper.x = x;
            m_clipper.y = y;
            grfx.setClip( m_clipper );
            grfx.drawImage(
                m_image, x + m_xOffset, y + m_yOffset, null );
            grfx.setClip( originalClip );
        }

        /**
         * This method traps false usage where the user tries to blit a bitmap that was not
         * loaded by this graphics engine to a bitmap that was created by this graphics engine.
         * This method does nothing.
         *@param bitmap The bitmap that will be drawn to this bitmap.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         *@param width  The with of the destination where the image is drawn to.
         *@param height The height of the destination where the image is drawn to.
         */
        public void strecthTo(DrawableBitmap bitmap, int x, int y, int width, int height)
        {
            Graphics grfx = ((CDrawableBitmap) bitmap).m_graphics;
            Shape originalClip = grfx.getClip();
            m_clipper.x = x;
            m_clipper.y = y;
            m_clipper.width = width;
            m_clipper.height = height;
            grfx.setClip(m_clipper);
            grfx.drawImage(m_image,
                x, y, x+width, y+height,  // Destination rectangle
                -m_xOffset, -m_yOffset,   // Source rectangle
                -m_xOffset+m_bitmapWidth,
                -m_yOffset+m_bitmapHeight,
                null );
            m_clipper.width = m_bitmapWidth;
            m_clipper.height = m_bitmapHeight;
            grfx.setClip( originalClip );
        }

        /**
         * Returns the width of the bitmap.
         *@return Width of the bitmap if available.
         */
        public int getWidth()
        {
            return m_bitmapWidth;
        }

        /**
         * Returns the height of the bitmap.
         *@return Height of the bitmap if available.
         */
        public int getHeight()
        {
            return m_bitmapHeight;
        }

        /**
         * Returns an effect processed clone of this bitmap.
         *@param effect The effect to be applied to the clone bitmap,
         *       or null if none are to be applied.
         *@return An effect processed clone of this bitmap.
         *@exception GameFrameException thrown if any errors occur.
         */
        public Bitmap getClone( BitmapEffect effect )
            throws GameFrameException
        {
            // Check if we really need to apply effects
            if ( effect == null )
            {
                return getSubBitmapClone( 0, 0, m_bitmapWidth, m_bitmapHeight );
            }

            try
            {
        	    int[] aPixels   = new int[m_bitmapWidth * m_bitmapHeight];
	            PixelGrabber pg = new PixelGrabber(
                    m_image,
                    0, 0,
                    m_bitmapWidth, m_bitmapHeight,
                    aPixels, 0, m_bitmapWidth );

                try
                {
                    pg.grabPixels();
                }
                catch (InterruptedException e)
                {
                     throw new GameFrameException("Pixel grabbing failed.");
                }

                if ( ( pg.getStatus() & ImageObserver.ABORT ) != 0 )
                {
                     throw new GameFrameException("Pixel grabbing failed.");
                }

                BitmapData data = new BitmapData( aPixels, m_bitmapWidth, m_bitmapHeight, m_bitmapWidth, null );
                effect.processData( data );

    		    Image image = Toolkit.getDefaultToolkit().createImage( new MemoryImageSource(
                    data.getWidth(), data.getHeight(),
                    pg.getColorModel(),
                    data.getPixels(),
                    0, data.getWidth() ) );

                if ( image == null )
                {
                     throw new GameFrameException("Couldn't create image.");
                }

                // Wait until image is prepared
                while ( !prepareImage( image, this ) );

                Bitmap bitmap = new CBitmap( image );
                m_createdBitmaps.addElement( bitmap );
                return bitmap;
            }
            catch( OutOfMemoryError outOfMemory )
            {
                throw new GameFrameException( "Not enough memory available for the operation." );
            }
        }

        /**
         * Returns a bitmap that represents a part of this bitmap. Useful for e.g.
         * extracting animation frames from one bitmap. If for any reason the
         * creation of the new bitmap fails (out of memory etc.) this method returns null.
         * The given dimensions are checked for validity.
         *@param srcX The area of the bitmap to be returned as a new bitmap.
         *@param srcY The area of the bitmap to be returned as a new bitmap.
         *@param srcWidth The area of the bitmap to be returned as a new bitmap.
         *@param srcHeight The area of the bitmap to be returned as a new bitmap.
         *@return A new bitmap representing the requested part of the original bitmap.
         */
        public Bitmap getSubBitmapClone(int srcX, int srcY, int srcWidth, int srcHeight)
            throws GameFrameException
        {
            // Check that the dimensions are valid
            if ( (srcWidth < 1) || (srcHeight < 1) )
            {
                throw new GameFrameException("Given source area size is zero.");
            }

            // Check that the bounds stay inside the current clipping rectangle
            if ( ( srcWidth > m_clipper.width   - m_clipper.x ) ||
                 ( srcHeight > m_clipper.height - m_clipper.y ) ||
                 ( srcX < 0 ) ||
                 ( srcY < 0 ) )
            {
                throw new GameFrameException("Given source rectangle "+srcX+", "+srcY+"->"+srcWidth+", "+srcHeight+" doesn't fit inside image size.");
            }

            try
            {
        	    int[] aPixels   = new int[srcWidth * srcHeight];
	            PixelGrabber pg = new PixelGrabber(m_image, srcX, srcY, srcWidth, srcHeight, aPixels, 0, srcWidth);

                try
                {
                    pg.grabPixels();
                }
                catch (InterruptedException e)
                {
                     throw new GameFrameException("Pixel grabbing failed.");
                }

                if ((pg.getStatus() & ImageObserver.ABORT) != 0)
                {
                     throw new GameFrameException("Pixel grabbing failed.");
                }

    		    Image image = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(srcWidth, srcHeight, pg.getColorModel(), aPixels, 0, srcWidth));
                if (image == null)
                {
                     throw new GameFrameException("Couldn't create image.");
                }

                while (!prepareImage(image, this));

                Bitmap bitmap = new CBitmap(image);
                m_createdBitmaps.addElement(bitmap);
                return bitmap;
            }
            catch(OutOfMemoryError outOfMemory)
            {
                throw new GameFrameException("Not enough memory available for the operation.");
            }
        }

        /**
         * Returns a bitmap that represents a part of this bitmap by using a new
         * control object, but using the old bitmap data (saves memory, but is not
         * that fast on many platforms. On the platforms where this method is
         * too slow for interactive use the implementation might actually copy the
         * data as with the getSubBitmapClone() method to speed things up). Anyhow
         * this method is preferred over getSubBitmapClone() method as in some platforms
         * it can save a lot of memory (e.g. under pure Java 1.1 creation of new bitmaps
         * by cloning the data is VERY memory consuming).
         *
         * Useful for e.g. extracting animation frames from one bitmap. This method
         * isn't usually fast and should be preferrably called during game initialization.
         * If for any reason the creation of the new bitmap fails (out of memory etc.)
         * this method returns null. The given dimensions are checked for validity.
         *@param srcX The area of the bitmap to be returned as a new bitmap.
         *@param srcY The area of the bitmap to be returned as a new bitmap.
         *@param srcWidth The area of the bitmap to be returned as a new bitmap.
         *@param srcHeight The area of the bitmap to be returned as a new bitmap.
         *@return A new bitmap representing the requested part of the original bitmap.
         */
        public Bitmap getSubBitmap(int srcX, int srcY, int srcWidth, int srcHeight)
            throws GameFrameException
        {
            // Check that the dimensions are valid
            if ( (srcWidth < 1) || (srcHeight < 1) )
            {
                throw new GameFrameException("Given source area size is zero.");
            }

            // Check that the bounds stay inside the current clipping rectangle
            if ( ( srcWidth > m_clipper.width   - m_clipper.x ) ||
                 ( srcHeight > m_clipper.height - m_clipper.y ) ||
                 ( srcX < 0 ) ||
                 ( srcY < 0 ) )
            {
                throw new GameFrameException(
                    "Given source rectangle "+srcX+", "+srcY+"->"+
                    srcWidth+", "+srcHeight+" doesn't fit inside image size." );
            }

            return new CClippedBitmap( this, srcX, srcY, srcWidth, srcHeight );
        }

        /**
         * Frees all the resources that are used by the bitmap implementation.
         */
        public void finalize()
        {
            if (m_image != null)
            {
                if ( m_createdBitmaps != null )
                {
                    m_createdBitmaps.removeElement(this);
                }
                m_image = null;
            }
        }

        /**
         * The image observer method that gets updated when an image is loading.
         */
	    public boolean imageUpdate(
            Image  img, int  infoflags, int  x, int  y, int  width, int  height)
	    {
            return true;
	    }
    }

    /**
     * Implementation of the drawable bitmap image for plain Java 2 graphics engine.
     */
    class CDrawableBitmap implements DrawableBitmap,ImageObserver
    {
        /** Reference to the AWT image that represents the bitmap in this graphics engine. */
        private Image m_image;
        /** The graphics context of this image. */
        public Graphics2D m_graphics;
        /** Width of the bitmap. */
        private int m_bitmapWidth;
        /** Height of the bitmap. */
        private int m_bitmapHeight;
        /** Clipping region used when drawing the bitmap. */
        private Rectangle m_clipper = new Rectangle();

        /**
         * Constructs an drawable bitmap from the given AWT image. The image that is given
         * as parameter must be an image whose graphics context can be gotten!
         *@param image The image that this bitmap represents.
         */
        CDrawableBitmap(java.awt.Image image, java.awt.Graphics2D graphics)
        {
            m_image          = image;
            m_graphics       = graphics;
            m_bitmapWidth    = image.getWidth(this);
            m_bitmapHeight   = image.getHeight(this);
            if (m_bitmapWidth > m_width)
            {
                m_bitmapWidth = m_width;
            }

            if (m_bitmapHeight > m_height)
            {
                m_bitmapHeight = m_height;
            }
        }



        /**
         * Draws the bitmap to the graphics engine that loaded it at the given location.
         * The location can be outside of the visible screen area in which case the bitmap
         * is clipped accordingly.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         */
        public void blitTo(int x, int y)
        {
            try
            {
                m_backBufferGraphics.drawImage(m_image, x, y, null);
            }
            catch(Exception anyException)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage(m_image, x, y, null);
            }
            catch(Error anyError)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage(m_image, x, y, null);
            }
        }

        /**
         * Draws the bitmap to the graphics engine that loaded it at the given location and
         * at the given size. If this bitmap is larger or smaller than the destination size
         * the bitmap will automatically be scaled. Note: This method is slow, so it's use is not
         * preferred during actual game play.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         *@param width  The with of the destination where the image is drawn to.
         *@param height The height of the destination where the image is drawn to.
         */
        public void strecthTo(int x, int y, int width, int height)
        {
            try
            {
                m_backBufferGraphics.drawImage(m_image, x, y, width, height, null);
            }
            catch(Exception anyException)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage(m_image, x, y, width, height, null);
            }
            catch(Error anyError)
            {
                m_backBufferGraphics = (Graphics2D) m_backBuffer.getGraphics();
                m_backBufferGraphics.setRenderingHints( mStatic_graphicsSettings );
                m_backBufferGraphics.drawImage(m_image, x, y, width, height, null);
            }
        }

        /**
         * Draws the given bitmap to this drawable bitmap at the given location.
         * The location can be outside of the visible screen area in which case the bitmap
         * is clipped accordingly.
         *@param bitmap The bitmap that will be drawn to this bitmap.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         */
        public void blitTo(DrawableBitmap bitmap, int x, int y)
        {
            ((CDrawableBitmap) bitmap).m_graphics.drawImage(m_image, x, y, null);
        }

        /**
         * This method traps false usage where the user tries to blit a bitmap that was not
         * loaded by this graphics engine to a bitmap that was created by this graphics engine.
         * This method does nothing.
         *@param bitmap The bitmap that will be drawn to this bitmap.
         *@param x The x-coordinate of the location where the image is drawn to.
         *@param y The y-coordinate of the location where the image is drawn to.
         *@param width  The with of the destination where the image is drawn to.
         *@param height The height of the destination where the image is drawn to.
         */
        public void strecthTo(DrawableBitmap bitmap, int x, int y, int width, int height)
        {
            ((CDrawableBitmap) bitmap).m_graphics.drawImage(
                m_image, x, y, width, height, null );
        }

        /**
         * Returns the width of the bitmap.
         *@return Width of the bitmap if available.
         */
        public int getWidth()
        {
            return m_bitmapWidth;
        }

        /**
         * Returns the height of the bitmap.
         *@return Height of the bitmap if available.
         */
        public int getHeight()
        {
            return m_bitmapHeight;
        }

        /**
         * Frees all the resources that are used by the bitmap implementation.
         */
        public void finalize()
        {
            if (m_image != null)
            {
                if ( m_createdBitmaps != null )
                {
                    m_createdBitmaps.removeElement(this);
                }
                m_image    = null;
                m_graphics = null;
            }
        }

        /**
         * The image observer method that gets updated when an image is loading.
         */
	    public boolean imageUpdate(Image  img, int  infoflags, int  x, int  y, int  width, int  height)
	    {
            return true;
	    }
    }
}
