/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk12x;

import gameframe.*;
import gameframe.input.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.EventQueue;

/**
 * Implementation of the keyboard and pointer device for Java 2 platforms.
 * The implementation is actually a replacement for system event queue that
 * monitors the event queue and picks up all mouse and keyboard related events
 * directly (slightly faster than the normal listener architecture). Other types
 * of events are handled normally by the java.awt.EventQueue superclass.
 *
 *@author Pasi Kernen 16. March 2000
 *@version GameFrame for Java 0.9.3
 *@since GameFrame for Java 0.9.3
 */
class CEventQueue extends EventQueue implements KeyboardDevice, PointerDevice
{
    /** The focus component that is listened for events. */
    private Component m_focusComponent;
    /** The singleton instance of this event queue. */
    private static CEventQueue mStatic_queueInstance;
    /** States of the keys. Can have one of the enumerated KEY_ values. */
    private int[] m_aKeyStates     = new int[0xFF];
    /** Number of times the key was stroken. */
    private int[] m_aNumKeystrokes = new int[0xFF];
    /** Enumerated value for state key is up. */
    private static final int KEY_UP                = 0;
    /** Enumerated value for state key is down. */
    private static final int KEY_DOWN              = 1;
    /** Enumerated value for state where key was down but was not read during that time. */
    private static final int KEY_WAS_DOWN_NOT_READ = 2;
    /** Enumerated value for state where key was down and was read during that time. */
    private static final int KEY_WAS_DOWN_READ     = 3;
    /** The last alphanumeric key value typed. */
    private int m_lastAlphanumericKey = -1;

    /** The current X-coordinate of the mouse. */
    private int m_x;
    /** The current Y-coordinate of the mouse. */
    private int m_y;
    /** States of the buttons. Can have one of the enumerated BUTTONSTATE_ values. */
    private int[] m_aButtonStates = new int[10];
    /** Number of times the button was stroken. */
    private int[] m_aNumButtonPresses = new int[10];
    /** Enumerated value for state button is up. */
    private static final int BUTTONSTATE_UP                = 0;
    /** Enumerated value for state button is down. */
    private static final int BUTTONSTATE_DOWN              = 1;
    /** Enumerated value for state where button was down but was not read during that time. */
    private static final int BUTTONSTATE_WAS_DOWN_NOT_READ = 2;
    /** Enumerated value for state where button was down and was read during that time. */
    private static final int BUTTONSTATE_WAS_DOWN_READ     = 3;

    /**
     * Constructs a keyboard engine that has the given window set as the focus window.
     *@param focusComponent The component that where listeners are registered.
     */
    private CEventQueue( Component focusComponent )
    {
        super();
        m_focusComponent = focusComponent;

        Toolkit.getDefaultToolkit().getSystemEventQueue().push( this );
    }

    /**
     * Returns the singleton instance.
     *@return The single instance of the keyboard.
     */
    public static CEventQueue getInstance( Component focusComponent )
    {
        if ( mStatic_queueInstance == null )
        {
            mStatic_queueInstance = new CEventQueue( focusComponent );
        }

        return mStatic_queueInstance;
    }

    /**
     * Returns true if the key is currently down
     *@param keyCode The keycode of the key we want to query.
     *@return True if the key is or was down.
     */
    public boolean isKeyDown( int keyCode )
    {
        if (m_aKeyStates[keyCode] == KEY_DOWN)
        {
            m_aKeyStates[keyCode] = KEY_WAS_DOWN_READ;
            return true;
        }

        return (m_aKeyStates[keyCode] == KEY_WAS_DOWN_READ);
    }

    /**
     * Returns true if the key was briefly kept down
     * after last reading of the keys state (after last call made to isKeyDown or
     * getKeyDownCount methods with the same key code).
     */
    public boolean wasKeyDown( int keyCode )
    {
        if ( m_aKeyStates[keyCode] == KEY_WAS_DOWN_NOT_READ )
        {
            m_aNumKeystrokes[keyCode] = 0;
            m_aKeyStates[keyCode] = KEY_UP;
            return true;
        }

        return false;
    }

    /**
     * Returns the number of times the key has been stroken since last reading
     * (after last call made to isKeyDown or getKeyDownCount methods with the
     * same key code).
     *@param keyCode The keycode of the key we want to query.
     *@return The number of times the key was stroken (pressed and released)
     */
    public int getKeyDownCount( int keyCode )
    {
        int numStrokes = m_aNumKeystrokes[keyCode];
        m_aNumKeystrokes[keyCode] = 0;
        if ( m_aKeyStates[keyCode] == KEY_WAS_DOWN_NOT_READ )
        {
            m_aKeyStates[keyCode] = KEY_UP;
        }

        return numStrokes;
    }

    /**
     * Overrides the default event queues AWT event dispatch method to
     * capture mouse and keyboard events from the queue.
     *@param event The dispatched AWT event.
     */
    protected void dispatchEvent( AWTEvent event )
    {
        if ( m_focusComponent != event.getSource() )
        {
            super.dispatchEvent( event );
            return;
        }

        switch( event.getID() )
        {
            case KeyEvent.KEY_PRESSED:
                keyPressed( (KeyEvent) event );
                break;
            case KeyEvent.KEY_RELEASED:
                keyReleased( (KeyEvent) event );
                break;
            case KeyEvent.KEY_TYPED:
                keyTyped( (KeyEvent) event );
                break;
            case MouseEvent.MOUSE_ENTERED:
                mouseEntered( (MouseEvent) event );
                break;
            case MouseEvent.MOUSE_EXITED:
                mouseExited( (MouseEvent) event );
                break;
            case MouseEvent.MOUSE_CLICKED:
                mouseClicked( (MouseEvent) event );
                break;
            case MouseEvent.MOUSE_PRESSED:
                mousePressed( (MouseEvent) event );
                break;
            case MouseEvent.MOUSE_RELEASED:
                mouseReleased( (MouseEvent) event );
                break;
            case MouseEvent.MOUSE_DRAGGED:
                mouseDragged( (MouseEvent) event );
                break;
            case MouseEvent.MOUSE_MOVED:
                mouseMoved( (MouseEvent) event );
                break;
            default:
                super.dispatchEvent( event );
                break;
        }
    }


    /**
     * Called when a key has been pressed.
     *@param event The event object that describes the event in more detail.
     */
    private final void keyPressed( KeyEvent event )
    {
        int keycode = event.getKeyCode();
        m_aKeyStates[keycode] = KEY_DOWN;
        m_aNumKeystrokes[keycode]++;
        event.consume();
    }

    /**
     * Called when a key has been released.
     *@param event The event object that describes the event in more detail.
     */
    private final void keyReleased(KeyEvent event)
    {
        int keyCode = event.getKeyCode();

        // If the button was down and the event was read, set
        // button state back to up
        if (m_aKeyStates[keyCode] == KEY_WAS_DOWN_READ)
        {
            m_aKeyStates[keyCode] = KEY_UP;
        }

        // If the button was down and the event was not read, set
        // button state accordingly
        m_aKeyStates[keyCode] = KEY_WAS_DOWN_NOT_READ;
        event.consume();
    }

    /**
     * Called when a key has been pressed and then released.
     *@param event The event object that describes the event in more detail.
     */
    private final void keyTyped( KeyEvent event )
    {
        char character = event.getKeyChar();
        if (Character.isLetterOrDigit(character))
        {
            m_lastAlphanumericKey = character;
        }
        else
        {
            if (character == ' ')
            {
                m_lastAlphanumericKey = ' ';
            }
            else if (character == '.')
            {
                m_lastAlphanumericKey = '.';
            }
            else
            {
                m_lastAlphanumericKey = -1;
            }
        }

        event.consume();
    }

    /**
     * Returns any typed alphanumeric keys (including space and dot) as int values that
     * correspond to that keys char value or -1 if no such keys were pressed
     * after last invokation of this method.
     *@return Any typed alphanumeric keys
     */
    public int getTypedAlphanumericKey()
    {
        int temp = m_lastAlphanumericKey;
        m_lastAlphanumericKey = -1;
        return temp;
    }


    /**
     * Releases all resources held by the keyboard device implementation.
     */
    public void finalize()
    {
        mStatic_queueInstance = null;
        pop();
    }


// Mouse related methods
    /**
     * Returns the X-coordinate of the pointer devices current location. If the mouse
     * is outside of the screen returns -1.
     *@return Pointers current X-location or -1 if not on screen.
     */
    public int getX()
    {
        return m_x;
    }

    /**
     * Returns the Y-coordinate of the pointer devices current location. If the mouse
     * is outside of the screen returns -1.
     *@return Pointers current Y-location or -1 if not on screen.
     */
    public int getY()
    {
        return m_y;
    }

    /**
     * Returns true if the button is down. Id numbers should be
     * 0 = primary button, 1 = secondary or "popup menu" button, 2 etc. for rest of
     * the buttons. Max. 3 buttons are supported (button id's from 0 to 2).
     *@param buttonId The number of the button we want to query.
     *@return True if the button is or was down.
     */
    public boolean isButtonDown(int buttonId)
    {
        if (m_aButtonStates[buttonId] == BUTTONSTATE_DOWN)
        {
            m_aButtonStates[buttonId] = BUTTONSTATE_WAS_DOWN_READ;
            return true;
        }

        return (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_READ);
    }

    /**
     * Returns true if the button was briefly kept down after last reading
     * of the buttons state and that event hasn't been read by any other method
     * (after last call made to isButtonDown, wasButtonDown or getButtonDownCount
     * methods with the same button id number). Id numbers should be
     * 0 = primary button, 1 = secondary or "popup menu" button, 2 etc. for rest of
     * the buttons. Max. 3 buttons are supported (button id's from 0 to 2).
     *@param buttonId The number of the button we want to query.
     *@return True if the button is or was down.
     */
    public boolean wasButtonDown(int buttonId)
    {
        if (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_NOT_READ)
        {
            m_aNumButtonPresses[buttonId] = 0;
            m_aButtonStates[buttonId] = BUTTONSTATE_UP;
            return true;
        }

        return false;
    }

    /**
     * Returns the number of times the button has been stroken since last reading
     * (after last call made to wasButtonDown or getButtonDownCount methods with the
     * same button id number). Id numbers should be
     * 0 = primary button, 1 = secondary or "popup menu" button, 2 = third button.
     * Max. 3 buttons are supported (button id's from 0 to 2).
     *@param buttonId The number of the button we want to query.
     *@return The number of times the button was stroken (pressed and released)
     */
    public int getButtonDownCount(int buttonId)
    {
        int numPresses = m_aNumButtonPresses[buttonId];
        m_aNumButtonPresses[buttonId] = 0;
        if (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_NOT_READ)
        {
            m_aButtonStates[buttonId] = BUTTONSTATE_UP;
        }
        return numPresses;
    }

    /**
     * Called when mouse enters the focus window.
     *@param event The event object that describes the event in more detail.
     */
    private final void mouseEntered(MouseEvent event)
    {
        m_x = event.getX();
        m_y = event.getY();
        event.consume();
    }

    /**
     * Called when mouse leaves the focus window.
     *@param event The event object that describes the event in more detail.
     */
    private final void mouseExited(MouseEvent event)
    {
        m_x = -1;
        m_y = -1;
        event.consume();
    }

    /**
     * Called when mouse button is clicked in the focus window.
     *@param event The event object that describes the event in more detail.
     */
    private final void mouseClicked(MouseEvent event)
    {
        m_aNumButtonPresses[getMouseButtonId(event)] += event.getClickCount();
        event.consume();
    }

    /**
     * Called when mouse is moved inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    private final void mouseMoved(MouseEvent event)
    {
        m_x = event.getX();
        m_y = event.getY();
        event.consume();
    }

    /**
     * Called when mouse is pressed inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    private final void mousePressed(MouseEvent event)
    {
        m_focusComponent.requestFocus();
        
        m_aButtonStates[getMouseButtonId(event)] = BUTTONSTATE_DOWN;
        event.consume();
    }

    /**
     * Called when mouse is released inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    private final void mouseReleased(MouseEvent event)
    {
        int buttonId = getMouseButtonId(event);

        // If the button was down and the event was read, set
        // button state back to up
        if (m_aButtonStates[buttonId] == BUTTONSTATE_WAS_DOWN_READ)
        {
            m_aButtonStates[buttonId] = BUTTONSTATE_UP;
        }

        // As the button was down and the event was not read,
        // set button state accordingly
        m_aButtonStates[buttonId] = BUTTONSTATE_WAS_DOWN_NOT_READ;
        event.consume();
    }


    /**
     * Called when mouse is dragged inside the focus window.
     *@param event The event object that describes the event in more detail.
     */
    private final void mouseDragged(MouseEvent event)
    {
        m_x = event.getX();
        m_y = event.getY();
        event.consume();
    }

    /**
     * Returns the mouse button ID (number) from the given event.
     *@param event The event object that describes the event in more detail.
     *@return The number of the mouse button that was pressed (0-2).
     */
    private final int getMouseButtonId( MouseEvent event )
    {
        if ( (event.getModifiers() & InputEvent.BUTTON1_MASK) != 0)
        {
            return 0;
        }
        else if ( (event.getModifiers() & InputEvent.BUTTON2_MASK) != 0)
        {
            return 1;
        }
        else
        {
            return 2;
        }
    }
}
