/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk12x;

import gameframe.*;
import gameframe.input.*;
import gameframe.engines.*;

import java.awt.*;
import java.awt.event.*;

/**
 * Implementation of the input engine for Java 2 JVM's.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.3
 *@since GameFrame for Java 0.9
 */
public class CInputEngine extends EventQueue
    implements InputEngine, Finalizable
{
    /** The singleton instance of the input engine. */
    private static CInputEngine mStatic_inputEngine;
    /** The focus component that is listened for events. */
    private Component m_focusComponent;

    /**
     * Constructs an input engine.
     *@param focusComponent Component that is listened for events.
     *@param settings The settings object that contains settings for this
     *       input engine.
     */
    public CInputEngine( Component focusComponent, GameFrameSettings settings )
        throws GameFrameException
    {
        if ( focusComponent == null )
        {
            throw new GameFrameException(
                "Focus component was null." );
        }

        m_focusComponent = focusComponent;
    }

    /**
     * Returns the default keyboard type of device in the system.
     *@return The default keyboard
     *@see gameframe.input.KeyboardDevice
     */
    public KeyboardDevice getDefaultKeyboardDevice()
        throws GameFrameException
    {
        return CEventQueue.getInstance( m_focusComponent );
    }

    /**
     * Returns all keyboard type of devices found on the system.
     *@return All keyboard type of devices found on the system.
     *@see gameframe.input.KeyboardDevice
     */
    public KeyboardDevice[] getAllKeyboardDevices()
        throws GameFrameException
    {
        KeyboardDevice[] aKeyboards = new KeyboardDevice[1];
        aKeyboards[0] = CEventQueue.getInstance( m_focusComponent );
        return aKeyboards;
    }

    /**
     * Returns the default pointer type of device in the system.
     *@return The default pointer device.
     *@see gameframe.input.PointerDevice
     */
    public PointerDevice getDefaultPointerDevice()
        throws GameFrameException
    {
        return CEventQueue.getInstance( m_focusComponent );
    }

    /**
     * Returns all pointer type of devices found on the system.
     *@return All pointer type of devices.
     *@see gameframe.input.PointerDevice
     */
    public PointerDevice[] getAllPointerDevices()
        throws GameFrameException
    {
        PointerDevice[] aPointers = new PointerDevice[1];
        aPointers[0] = CEventQueue.getInstance( m_focusComponent );
        return aPointers;
    }

    /**
     * Returns the short name (max 8 characters) of the
     * implementation "Java2".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "Java2";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Input engine that uses Java 2 event queue replacement.";
	}

    /**
     * Releases all resources held by the all input device implementations.
     */
    public void finalize()
    {
        // Finalize the keyboard input class
        CEventQueue.getInstance( m_focusComponent ).finalize();

        // Reset the singleton reference
        mStatic_inputEngine = null;
        m_focusComponent    = null;
    }
}

