/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.jdk12x;

import gameframe.*;
import gameframe.engines.*;
import gameframe.sound.*;

import java.io.*;
import java.util.*;
import sun.audio.*;
import java.net.*;
import java.applet.*;

/**
 * Java 2 implementation of the sound engine. Uses the well known "hack"
 * to play samples on plain vanilla Java 2 platforms without Java Media
 * Framework. Loads mono/stereo Microsoft PCM RIFF WAVE files.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.3
 *@since GameFrame for Java 0.9
 */
class CSoundEngine implements SoundEngine, Finalizable
{
    /** The singleton instance of the sound engine. */
    private static SoundEngine mStatic_soundEngine;
    /** The currently playing music. */
    private static CJava2Music mStatic_currentMusic;
    /** The focus component that is listened for events. */
    private java.awt.Component m_focusComponent;

    /** All the samples created through this sound engine. */
    private Vector m_createdSamples = new Vector(10,10);
    /** All the musics created through this sound engine. */
    private Vector m_createdMusics = new Vector(10,10);
    /** Name of the directory that contains loaded sounds. */
    private String m_strDataDirectory = "";

    /**
     * Constructs a sound engine.
     *@param focusComponent The component that needs to be focused in order
     *       for the sound to be played.
     *@param settings The settings object that contains settings for this
     *       sound engine.
     */
    public CSoundEngine(
        java.awt.Component focusComponent,
        GameFrameSettings settings )
    {
        m_focusComponent = focusComponent;
    }

	/**
	 * Loads an mono 8-bit 11025 bytes/sec AU formatted sample from the given file name
	 * and return an abstract reference for the loaded sound. This implementation is limited
	 * in sample lenght to the maximum positive value of integer...
	 * @param strFileName The name of the file to be loaded (without the file extension)
	 * @return An unique Sound object that specifies the loaded sample
	 */
	public Sample loadSample( String strFileName )
        throws FileNotFoundException, GameFrameException
	{
        strFileName = m_strDataDirectory+strFileName+".wav";

		File file = new File(strFileName);
        if (!file.exists())
        {
            throw new FileNotFoundException(
                "Sample file \""+
                strFileName+
                "\" doesn't exist.");
        }

		int length = (int) file.length();
		if (length <= 0)
		{
            throw new GameFrameException(
                "Size of the sample file "+
                strFileName+
                " is zero.");
		}

        strFileName = "file:///"+file.getAbsolutePath();
        strFileName.replace(file.separatorChar, '/');

        URL fileLocation = null;
        try
        {
            fileLocation = new URL(strFileName);
        }
        catch(MalformedURLException invalidUrl)
        {
            throw new FileNotFoundException(
                "Couldn't load sample file "+
                strFileName+
                ".");
        }


        AudioClip audioClip = java.applet.Applet.newAudioClip( fileLocation );
        if (audioClip == null)
        {
            throw new FileNotFoundException(
                "Couldn't load sample file "+
                strFileName+
                ".");
        }

        CJava2Sample sound = new CJava2Sample( audioClip, fileLocation );
        m_createdSamples.addElement(sound);
		return sound;
	}


    /**
     * First tries to load a music file as streamed Microsoft RIFF WAV file,
     * if no such file is found from the current data directory, tries to load
     * a music file in MIDI format. When suitable file is found returns a
     * control object for the music file if no suitable file is found returns
     * NullSound object. Music files are loaded by default from the
     * current directory, the directory that is used to load the samples
     * can be set with the setDataDirectory() method.
     *
     *@see #setDataDirectory(java.lang.String)
     *@param fileName The name of the file to be loaded (without the file extension)
     *@return An unique Sample object that represents the loaded sample
     *@since GameFrame for Java 0.9.2
     */
    public Music loadMusic( String strFileName )
        throws FileNotFoundException, GameFrameException
    {
        strFileName = m_strDataDirectory+strFileName+".mid";

		File file = new File( strFileName );
        if ( !file.exists() )
        {
            throw new FileNotFoundException(
                "MIDI file \""+
                strFileName+
                "\" doesn't exist." );
        }

		int length = (int) file.length();
		if ( length <= 0 )
		{
            throw new GameFrameException(
                "Size of the MIDI file "+
                strFileName+
                " is zero." );
		}

        strFileName = "file:///"+file.getAbsolutePath();
        strFileName.replace( file.separatorChar, '/' );

        URL fileLocation = null;
        try
        {
            fileLocation = new URL( strFileName );
        }
        catch( MalformedURLException invalidUrl )
        {
            throw new FileNotFoundException(
                "Couldn't load MIDI file "+
                strFileName+
                "." );
        }

        AudioClip audioClip = java.applet.Applet.newAudioClip( fileLocation );
        if (audioClip == null)
        {
            throw new FileNotFoundException(
                "Couldn't load MIDI file "+
                strFileName+
                "." );
        }

        CJava2Music music = new CJava2Music( audioClip, fileLocation );
        m_createdMusics.addElement( music );
		return music;
    }

    /**
     * Sets the data directory wherefrom the sounds are loaded.
     * Defaults to the current directory. Do not add the last directory separator.
     *@param strDirectoryName The name of the directory that contains the sounds.
     */
    public void setDataDirectory(String strDirectoryName)
        throws java.io.FileNotFoundException
    {
        // If there is a path separator, remove it
        if ( strDirectoryName.charAt( strDirectoryName.length() - 1 ) == File.separatorChar )
        {
            strDirectoryName = strDirectoryName.substring( 0, strDirectoryName.length() - 1 );
        }

        File file = new File(strDirectoryName);
        if ( ( !file.isDirectory() ) || ( !file.exists() ) )
        {
            throw new java.io.FileNotFoundException(
                "Data directory \""+
                strDirectoryName+
                "\" doesn't exist." );
        }

        m_strDataDirectory = strDirectoryName + File.separator;
    }


    /**
     * Returns the data directory wherefrom the sounds are loaded.
     * The source directory defaults to the current directory.
     *
     * Note: The returned path will have the path separator as the last character.
     * 
     *@see #loadSample(java.lang.String)
     *@see #loadMusic(java.lang.String)
     *@return The name of the directory that wherefrom the sounds are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory()
    {
        return m_strDataDirectory;
    }

    /**
     * Returns the short name (max 8 characters) of the
     * implementation "Java2".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "Java2";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Sound engine that uses Java 2 official SoundClip interface.";
	}

	/**
	 * Finalises all sounds.
	 */
	public void finalize()
	{
        // Finalize samples
        if ( m_createdSamples != null )
        {
            for (Enumeration enum = m_createdSamples.elements(); enum.hasMoreElements();)
            {
                Sample sample = (Sample) enum.nextElement();
                sample.finalize();
            }

            m_createdSamples = null;
        }

        // Finalize musics
        if ( m_createdMusics != null )
        {
            for (Enumeration enum = m_createdMusics.elements(); enum.hasMoreElements();)
            {
                Music music = (Music) enum.nextElement();
                music.finalize();
            }

            m_createdMusics = null;
        }
	}

	/**
	 * Implementation of Sound that implements a Java 2 sound clip.
	 */
	class CJava2Sample implements Sample
	{
        private AudioClip m_audioClip;
        private URL m_fileLocation;
		private boolean m_fPlaying;
        private float m_pan;
        private float m_volume;

        /**
         * Constructs a new Java 2 sound clip from the given parameters.
         *@param audioClip The audio clip object used to play the sounds.
         *@param fileLocation The audio clip file location URL.
         */
		CJava2Sample(AudioClip audioClip, URL fileLocation)
		{
			m_fPlaying = false;
            m_audioClip = audioClip;
            m_fileLocation = fileLocation;
		}

		/**
 		 * Plays this sample once and then stops
		 */
		public void playOnce()
		{
            if( m_fPlaying )
            {
                m_audioClip.stop();
            }

    	    m_fPlaying = true;
        	m_audioClip.play();
		}

		/**
		 * Plays the given sample over and over again (looping).
		 */
		public void playLooped()
		{
            if( m_fPlaying )
            {
                m_audioClip.stop();
            }

    	    m_fPlaying = true;
   			m_audioClip.loop();
		}

		/**
		 * Stops playing the given sample
		 */
		public void stop()
		{
			if (m_fPlaying)
			{
                m_audioClip.stop();
				m_fPlaying = false;
			}
		}

        /**
         * Returns a copy of this sound object that is independent from the
         * original sound object. This new sound object can be started/stopped
         * independently from the original sound object. Using this method to
         * get multiple instances of one sound will save memory on some
         * implementations and is thus preferred way to duplicate sound objects.
         *@return A independent copy of this sound or null if copy fails.
         */
        public Sample getCopy()
        {
            AudioClip newAudioClip = Applet.newAudioClip( m_fileLocation );
            CJava2Sample sound = new CJava2Sample(newAudioClip, m_fileLocation);
            m_createdSamples.addElement(sound);
		    return sound;
        }

        /**
		 * Upon finalization stop everything (just in case) and remove this
         * sound clip from the list of created sound clips.
		 */
		public void finalize()
		{
            if (m_audioClip != null)
            {
                if (m_fPlaying)
                {
			        stop();
                }
                m_createdSamples.removeElement(this);
                m_audioClip = null;
            }
		}

		/**
		 * Returns true if sound should be playing at the moment
		 */
		public boolean isPlaying()
		{
			return m_fPlaying;
		}
        
        /**
         * Sets the volume of the sound. Value of 0.0 means that the sound should be
         * inaudible and value of 1.0 means the sound is played at its normal volume.
         * Values above 1.0 are treated as 1.0.
         *@param volume The sounds volume.
         */
        public void setVolume(float volume)
        {
            if (volume > 1.0F)
            {
                volume = 1.0F;
            }

            if (volume < 0.0F)
            {
                volume = 0.0F;
            }
            m_volume = volume;
        }

        /**
         * Sets the pan of the sound. Value of -1.0 means that the sound will be
         * heard only from the left speaker and value of 1.0 means that the sound
         * will be heard only from the right speaker. Value of 0.0 means that
         * the sound is heard with equal volume from both speakers.
         *@param pan The pan of the sound.
         */
        public void setPan(float pan)
        {
            if (pan > 1.0F)
            {
                pan = 1.0F;
            }

            if (pan < -1.0F)
            {
                pan = -1.0F;
            }
            m_pan = pan;
        }

        /**
         * Returns the volume of the sound. Value of 0.0 means that the sound should be
         * inaudible and value of 1.0 means the sound is played at its normal volume.
         *@return The sounds volume.
         */
        public float getVolume()
        {
            return m_volume;
        }

        /**
         * Returns the pan of the sound. Value of -1.0 means that the sound will be
         * heard only from the left speaker and value of 1.0 means that the sound
         * will be heard only from the right speaker. Value of 0.0 means that
         * the sound is heard with equal volume from both speakers.
         *@return The pan of the sound.
         */
        public float getPan()
        {
            return m_pan;
        }
	}

	/**
	 * Implementation of music that implements a Java 2 sound clip.
	 */
	class CJava2Music implements Music
	{
        private AudioClip m_audioClip;
        private URL m_fileLocation;
		private boolean m_fPlaying;
        private float m_pan;
        private float m_volume;

        /**
         * Constructs a new Java 2 sound clip from the given parameters.
         *@param audioClip The audio clip object used to play the sounds.
         *@param fileLocation The audio clip file location URL.
         */
		CJava2Music(AudioClip audioClip, URL fileLocation)
		{
			m_fPlaying = false;
            m_audioClip = audioClip;
            m_fileLocation = fileLocation;
		}

		/**
 		 * Plays this sample once and then stops
		 */
		public void playOnce()
		{
            if ( mStatic_currentMusic != null )
            {
                mStatic_currentMusic.stop();
            }
            mStatic_currentMusic = this;

			m_fPlaying = true;
			m_audioClip.play();
		}

		/**
		 * Plays the given sample over and over again (looping).
		 */
		public void playLooped()
		{
            if ( mStatic_currentMusic != null )
            {
                mStatic_currentMusic.stop();
            }
            mStatic_currentMusic = this;

            m_audioClip.loop();
			m_fPlaying = true;
		}

		/**
		 * Stops playing the given sample
		 */
		public void stop()
		{
			if (m_fPlaying)
			{
                m_audioClip.stop();
				m_fPlaying = false;
			}
            mStatic_currentMusic = null;
		}

        /**
		 * Upon finalization stop everything (just in case) and remove this
         * music from the list of created music clips.
		 */
		public void finalize()
		{
            if ( m_audioClip != null )
            {
                if ( m_fPlaying )
                {
			        stop();
                }
                m_createdMusics.removeElement(this);
                m_audioClip = null;
            }
		}

		/**
		 * Returns true if sound should be playing at the moment
		 */
		public boolean isPlaying()
		{
			return m_fPlaying;
		}
	}
}
