/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.msjava4x;

import gameframe.*;
import gameframe.util.*;
import gameframe.graphics.*;

import com.ms.directX.*;
import com.ms.dll.DllLib;
import com.ms.com.*;

import java.util.*;
import java.io.*;

/**
 * Baseclass for all Microsoft Java DirectX bitmap implementations.
 * 
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
abstract class CBitmapBase
{
    /** All created temporary files. */
    protected static Vector mStatic_tempFiles = new Vector( 20, 20 );
    /** The DirectDraw surface this bitmap class represents. */
    protected DirectDrawSurface m_ddSurface;
    /** The DirectDraw surface to where this bitmap is blitted. */
    protected DirectDrawSurface m_ddBackBuffer;
    /** Width of the DirectDraw surface this bitmap class represents. */
    protected int m_width;
    /** Height of the DirectDraw surface this bitmap class represents. */
    protected int m_height;
    /** Width of the backbuffer DirectDraw surface. */
    protected int m_backBufferWidth;
    /** Height of the backbuffer DirectDraw surface. */
    protected int m_backBufferHeight;
    /** Flag that indicates if the bitmap is in video memory or not. */
    protected boolean m_fInVideoMem = false;

    /** The rectangle that covers the DirectDraw surfaces part this bitmap class represents.*/
    protected Rect m_srcRectangle;
    /** The rectangle that covers the destination are of the blit. */
    protected static Rect mStatic_dstRectangle = new Rect();
    /** Reference to the DirectDraw object that can create new bitmaps. */
    protected DirectDraw m_dd;
    /** The bitmap storage that stores all the bitmaps created by this bitmap. */
    protected IBitmapStorage m_bitmapStorage;
    /**
     * The superbitmap whose subbitmap this bitmap is or null if this is not a subbitmap.
     * The subbitmap clones are not counted as subbitmaps in this case.
     */
    protected CBitmapBase m_superBitmap;
    /** Name of the originating bitmap file. */    
    protected String m_strOriginBitmapFilename;
    /** The rectangle that this bitmap represents from the originating bitmap. */    
    protected Rect m_srcRectFromOriginBitmap;
    
    /**
     * Constructs a bitmap from the given paramters.
     *@param dd The DirectDraw object that can create new DD surfaces.
     *@param ddSurface The DirectDraw surface this bitmap class represents.
     *@param width Width of the DirectDraw surface this bitmap class represents.
     *@param height Height of the DirectDraw surface this bitmap class represents.
     *@param bitmapStorage The storage that stores all bitmaps created by this bitmap.
     */
    CBitmapBase( DirectDraw dd, DirectDrawSurface ddBackBuffer, DirectDrawSurface ddSurface,
                 int width, int height, IBitmapStorage bitmapStorage, String strOriginFile)
    {
        this( dd, ddBackBuffer, ddSurface, width, height, bitmapStorage, strOriginFile, null );
    }

    /**
     * Constructs a bitmap from the given paramters.
     *@param dd The DirectDraw object that can create new DD surfaces.
     *@param ddSurface The DirectDraw surface this bitmap class represents.
     *@param width Width of the DirectDraw surface this bitmap class represents.
     *@param height Height of the DirectDraw surface this bitmap class represents.
     *@param bitmapStorage The storage that stores all bitmaps created by this bitmap.
     *@param strOriginFile The originating bitmap file.
     *@param originRect The originating bitmaps rectangle that is this bitmap.
     */
    CBitmapBase(DirectDraw dd, DirectDrawSurface ddBackBuffer, DirectDrawSurface ddSurface,
                int width, int height, IBitmapStorage bitmapStorage, String strOriginFile,
                Rect originRect )
    {
        m_bitmapStorage = bitmapStorage;
        m_dd            = dd;
        m_ddSurface     = ddSurface;
        m_ddBackBuffer  = ddBackBuffer;
        
        DDSurfaceDesc backSurfaceDesc = new DDSurfaceDesc();
        m_ddBackBuffer.getSurfaceDesc( backSurfaceDesc );
        m_backBufferWidth  = backSurfaceDesc.width;
        m_backBufferHeight = backSurfaceDesc.height;
        
        DDSurfaceDesc surfaceDesc = new DDSurfaceDesc();
        m_ddSurface.getSurfaceDesc( surfaceDesc );
        if ( (surfaceDesc.flags & DirectXConstants.DDSD_CAPS ) != 0 )
        {
            m_fInVideoMem =
                ( ( surfaceDesc.ddsCaps & DirectXConstants.DDSCAPS_VIDEOMEMORY ) != 0 );
        }
        
        m_srcRectangle = new Rect();
        if ( originRect == null )
        {
            m_width  = surfaceDesc.width;
            m_height = surfaceDesc.height;
            m_srcRectangle.left   = 0;
            m_srcRectangle.right  = m_width;
            m_srcRectangle.top    = 0;
            m_srcRectangle.bottom = m_height;
        }
        else
        {
            m_width  = originRect.right - originRect.left;
            m_height = originRect.bottom - originRect.top;

            m_srcRectangle.left   = originRect.left;
            m_srcRectangle.right  = originRect.left + m_width;
            m_srcRectangle.top    = originRect.top;
            m_srcRectangle.bottom = originRect.top + m_height;
        }

        m_strOriginBitmapFilename = strOriginFile;
        m_srcRectFromOriginBitmap = originRect;
    }

    /**
     * Deletes all temporary files.
     */
    public void finalize()
    {
        if ( mStatic_tempFiles.size() > 0 )
        {
            for ( Enumeration enum = mStatic_tempFiles.elements();
                  enum.hasMoreElements(); )
            {
                File file = (File) enum.nextElement();
                file.delete();
            }
            
            mStatic_tempFiles.removeAllElements();
        }
    }
    
    /**
     * Restores the DirectDraw surface if it is lost. Cannot restore subbitmap clones
     * or created bitmaps correctly!
     */
    void restore()
    {
        try
        {
            m_ddSurface.restore();
        }
        catch( ComException ce )
        {
            // Surface restoration failed...
            return;
        }
        
        if ( m_strOriginBitmapFilename != null)
        {
		    java.io.File file = new java.io.File( m_strOriginBitmapFilename );
            if ( !file.exists() )
            {
                return;
            }

		    int length = (int) file.length();
		    if ( length <= 0 )
		    {
                return;
		    }

    	    DirectDrawBitmap ddBitmap = new DirectDrawBitmap();
	        ddBitmap.filename ( m_strOriginBitmapFilename );
    	    ddBitmap.initWidth(0);
	        ddBitmap.initHeight(0);

		    if( ddBitmap.loaded() == 0 )
		    {
                return;
            }

            if ( m_srcRectFromOriginBitmap == null )
            {
		        m_ddSurface.copyBitmap( ddBitmap, 0, 0, ddBitmap.width(), ddBitmap.height() );
            }
            else
            {
		        m_ddSurface.copyBitmap( ddBitmap,
                                    m_srcRectFromOriginBitmap.left,
                                    m_srcRectFromOriginBitmap.top,
                                    m_srcRectFromOriginBitmap.right,
                                    m_srcRectFromOriginBitmap.bottom );
            }
        }
        
        DDSurfaceDesc surfaceDesc = new DDSurfaceDesc();
        m_ddSurface.getSurfaceDesc( surfaceDesc );
        if ( ( surfaceDesc.flags & DirectXConstants.DDSD_CAPS ) != 0 )
        {
            m_fInVideoMem =
                ( ( surfaceDesc.ddsCaps & DirectXConstants.DDSCAPS_VIDEOMEMORY ) != 0 );
        }
        
        if ( m_superBitmap != null )
        {
            m_superBitmap.restore();
        }
    }

    /**
     * Returns an effect processed DirectDraw surface.
     *@param effect The effect that is applied.
     *@return The processed DirectDraw surface.
     */
    protected DirectDrawSurface getEffectSurface( BitmapEffect effect )
        throws GameFrameException
    {
        Rect srcRect   = new Rect();
        if ( m_srcRectFromOriginBitmap == null )
        {
            srcRect.left   = 0;
            srcRect.top    = 0;
            srcRect.right  = m_width;
            srcRect.bottom = m_height;
        }
        else
        {
            srcRect.left   = m_srcRectFromOriginBitmap.left;
            srcRect.top    = m_srcRectFromOriginBitmap.top;
            srcRect.right  = m_srcRectFromOriginBitmap.right;
            srcRect.bottom = m_srcRectFromOriginBitmap.bottom;
        }
        
        //System.out.print( "get pixel data..." );
        BitmapData bitmapData = getData( m_ddSurface, srcRect );
        
        ////System.out.print( "process data..." );
        bitmapData = effect.processData( bitmapData );
        //System.out.print( "get surface from data..." );
        return getSurfaceFromData( bitmapData );
    }

    /**
     * Returns a DirectDraw surface that has the given bitmap data.
     *@param bitmapData The bitmap data that is to be turned into DirectDraw surface.
     *@return DirectDraw surface that has the given bitmap data.
     */
    protected DirectDrawSurface getSurfaceFromData( BitmapData bitmapData )
        throws GameFrameException
    {
        // create a DirectDrawSurface for this bitmap
        DirectDrawSurface newDDSurface;
        
        try
        {
            //System.out.print("create surface...");
            if ( m_bitmapStorage instanceof CFullscreenGraphicsEngine )
            {
                CFullscreenGraphicsEngine engine = (CFullscreenGraphicsEngine) m_bitmapStorage;
                newDDSurface = engine.createSurface( bitmapData.getWidth(), bitmapData.getHeight() );
            }
            else
            {
                DDSurfaceDesc ddsd  = new DDSurfaceDesc();
                ddsd.flags          = DirectXConstants.DDSD_CAPS   |
                                      DirectXConstants.DDSD_HEIGHT |
                                      DirectXConstants.DDSD_WIDTH;
                ddsd.ddsCaps        = DirectXConstants.DDSCAPS_OFFSCREENPLAIN;
                ddsd.width          = bitmapData.getWidth();
                ddsd.height         = bitmapData.getHeight();
                newDDSurface        = m_dd.createSurface( ddsd );
            }
            
            if ( newDDSurface == null )
            {
                throw new GameFrameException( IConstants.COULDNT_CREATE_DDSURFACE );
            }
                
            Rect dstRect   = new Rect();
            dstRect.left   = 0;
            dstRect.top    = 0;
            dstRect.right  = bitmapData.getWidth();
            dstRect.bottom = bitmapData.getHeight();

            //System.out.print("set data to surface...");
            setDataToSurface( newDDSurface, dstRect, bitmapData );
        }
        catch( ComFailException cfe )
        {
            throw new GameFrameException( IConstants.IMAGE_CREATION_FAILED+cfe );
        }
        
        return newDDSurface;
    }
    
    /**
     * Returns the bitmap data from the given rectangle in the
     * given surface.
     *@param ddSurface The DirectDraw surface where the pixels are gotten from.
     *@param srcRect The source rectangle.
     *@return The bitmap data.
     */
    BitmapData getData( DirectDrawSurface ddSurface, Rect srcRect )
        throws GameFrameException
    {
        //System.out.print( "pixel format..." );
        DDPixelFormat pixelFormat = new DDPixelFormat();
        ddSurface.getPixelFormat( pixelFormat );

        DDSurfaceDesc ddsdSrc = new DDSurfaceDesc();
        ddSurface.getSurfaceDesc( ddsdSrc );

        int width  = srcRect.right  - srcRect.left;
        int height = srcRect.bottom - srcRect.top;
        int pitch  = ddsdSrc.pitch;
        int iBufferSize = pitch * height;
        int memToPixelRatio = 1;
        switch( pixelFormat.rgbBitCount )
        {
            case 16:
                memToPixelRatio = 2;
                break;
            case 24:
                memToPixelRatio = 3;
                break;
            case 32:
                memToPixelRatio = 4;
                break;
            default:
                throw new GameFrameException( IConstants.UNSUPPORTED_RGB_BIT_COUNT );    
        }

        byte[] abRawPixelData = new byte[iBufferSize * memToPixelRatio];
            
        //System.out.print( "buffer["+abRawPixelData.length+"]..." );


        //System.out.print( "locking..." );
        try
        {
            ddSurface.lock(
                srcRect,
                ddsdSrc,
                DirectXConstants.DDLOCK_READONLY |
                DirectXConstants.DDLOCK_WAIT,
                0,
                abRawPixelData );
        }
        catch( ComFailException lockFail )
        {
            throw new GameFrameException( IConstants.SURFACE_LOCK_FAILED );
        }

        //System.out.print( "buffer["+abRawPixelData.length+"]..." );

        //System.out.print( "unlocking..." );
        
        try
        {
            ddSurface.unlock( srcRect, ddsdSrc, abRawPixelData );
        }
        catch( ComFailException unlockFail )
        {
            throw new GameFrameException( IConstants.SURFACE_UNLOCK_FAILED );
        }

        //System.out.print(" 1. byte:"+Integer.toHexString( (int) abRawPixelData[0] & 0xFF ) );
        //System.out.print(" 2. byte:"+Integer.toHexString( (int) abRawPixelData[1] & 0xFF ) );
        //System.out.print(" 3. byte:"+Integer.toHexString( (int) abRawPixelData[2] & 0xFF ) );
        //System.out.print(" 4. byte:"+Integer.toHexString( (int) abRawPixelData[3] & 0xFF ) +" " );

        
        int[] aiBuffer = new int[ width * height ];
        int dstWidth   = width * memToPixelRatio;
        int srcIdx     = 0;
        int dstIdx     = 0;
        int byteCount  = 0;
        switch( pixelFormat.rgbBitCount )
        {
            case 16:
                for ( int y = 0; y < height; y++ )
                {                    
                    for ( int x = 0; x < dstWidth; x++ )
                    {
                        srcIdx = x + y*pitch;      
                        if ( ( x % 2 ) == 0 )
                        {
                            aiBuffer[dstIdx] = ( (int) abRawPixelData[srcIdx] ) & 0xFF;
                        }
                        else
                        {
                            aiBuffer[dstIdx] |= ( ( abRawPixelData[srcIdx] & 0xFF ) << 8 );
                            dstIdx++;
                        }
                    }
                }
                break;
            case 24:
                for ( int y = 0; y < height; y++ )
                {                    
                    for ( int x = 0; x < dstWidth; x++ )
                    {
                        srcIdx = x + y*pitch;      
                        switch( byteCount )
                        {
                            case 0:
                                aiBuffer[dstIdx] = ( (int) abRawPixelData[srcIdx] ) & 0xFF;
                                byteCount++;
                                break;
                            case 1:
                                aiBuffer[dstIdx] |= ( ( ( (int) abRawPixelData[srcIdx] ) & 0xFF ) << 8 );
                                byteCount++;
                                break;
                            case 2:
                                aiBuffer[dstIdx] |= ( ( ( (int) abRawPixelData[srcIdx] ) & 0xFF ) << 16 );
                                dstIdx++;
                                byteCount = 0;
                                break;
                        }
                    }
                }
                break;
            case 32:
                for ( int y = 0; y < height; y++ )
                {                    
                    for ( int x = 0; x < dstWidth; x++ )
                    {
                        srcIdx = x + y*pitch;      
                        switch( byteCount )
                        {
                            case 0:
                                aiBuffer[dstIdx] = ( (int) abRawPixelData[srcIdx] ) & 0xFF;
                                byteCount++;
                                break;
                            case 1:
                                aiBuffer[dstIdx] |= ( ( ( (int) abRawPixelData[srcIdx] ) & 0xFF ) << 8 );
                                byteCount++;
                                break;
                            case 2:
                                aiBuffer[dstIdx] |= ( ( ( (int) abRawPixelData[srcIdx] ) & 0xFF ) << 16 );
                                byteCount++;
                                break;
                            case 3:
                                aiBuffer[dstIdx] |= ( ( ( (int) abRawPixelData[srcIdx] ) & 0xFF ) << 24 );
                                byteCount = 0;
                                dstIdx++;
                                break;
                        }
                    }
                }
                break;
            default:
                throw new GameFrameException( IConstants.UNSUPPORTED_RGB_BIT_COUNT );    
        }
        
        //System.out.print(" 1. value:"+Integer.toHexString( aiBuffer[0] ) );
        //System.out.print(" 2. value:"+Integer.toHexString( aiBuffer[1] ) );

        //System.out.print( "converting pixels..." );
        // Convert the pixels to Java native 0xAARRGGBB format
        CPixelConverter converter = new CPixelConverter( pixelFormat );
        converter.convertToJavaNative( aiBuffer );

        //System.out.print( "returning..." );
        //System.out.print( "bitmap data..." );
        BitmapData data = new BitmapData(
            aiBuffer,
            width,
            height,
            width );
        
        data.setColourBits(
            converter.getNumRedBits(), 
            converter.getNumGreenBits(),
            converter.getNumBlueBits() );
        return data;           
    }

    /**
     * Sets the bitmap data to the given DirectDraw surface.
     *@param ddSurface The DirectDraw surface whose data is to be set.
     *@param srcRect   The rectangle area of the surface or null if whole
     *                 surface is to be set.
     *@return Bitmap data from the given DirectDraw surface
     */
    protected void setDataToSurface(
        DirectDrawSurface targetSurface, Rect dstRect, BitmapData bitmapData )
        throws GameFrameException
    {
        // Get the surface pixel format and calculate needed byte array size
        //System.out.print("getting pixelformat...");
        
        DDSurfaceDesc dstDDSurfaceDesc  = new DDSurfaceDesc();
        targetSurface.getSurfaceDesc( dstDDSurfaceDesc );
        
        DDPixelFormat pixelFormat = new DDPixelFormat();
        try
        {
            targetSurface.getPixelFormat( pixelFormat );
        }
        catch( ComFailException cfe )
        {
            throw new GameFrameException( IConstants.COULDNT_GET_PIXELFORMAT + cfe );
        }
        
        int width           = dstRect.right  - dstRect.left;
        int height          = dstRect.bottom - dstRect.top;
        int pitch           = dstDDSurfaceDesc.pitch;
        int iBufferSize     = pitch * height;
        int memToPixelRatio = 1;
        switch( pixelFormat.rgbBitCount )
        {
            case 16:
                memToPixelRatio = 2;
                break;
            case 24:
                memToPixelRatio = 3;
                break;
            case 32:
                memToPixelRatio = 4;
                break;
            default:
                throw new GameFrameException( IConstants.UNSUPPORTED_RGB_BIT_COUNT );    
        }
        byte[] abRawPixelData = new byte[iBufferSize * memToPixelRatio];
        //System.out.println("buffer["+abRawPixelData.length+"]");


        // Lock the surface for writing
        try
        {
            //System.out.print( "locking..." );
            targetSurface.lock(
                dstRect,
                dstDDSurfaceDesc,
                DirectXConstants.DDLOCK_WRITEONLY |
                DirectXConstants.DDLOCK_WAIT,
                0,
                abRawPixelData );
        }
        catch( ComFailException lockFail )
        {
            throw new GameFrameException( IConstants.SURFACE_LOCK_FAILED );
        }

        // Set the raw pixeldata to whatever...
        //System.out.print( "converting..." );
        int[] aiBuffer = bitmapData.getPixels();
        CPixelConverter converter = new CPixelConverter( pixelFormat );
        converter.convertFromJavaNative( aiBuffer );

        // Convert the pixeldata to raw memory byte array
        //System.out.print( "memory dumping..." );
        int dstWidth   = width * memToPixelRatio;
        int srcIdx     = 0;
        int dstIdx     = 0;
        int byteCount  = 0;
        switch( pixelFormat.rgbBitCount )
        {
            case 16:
                for ( int y = 0; y < height; y++ )
                {                    
                    for ( int x = 0; x < dstWidth; x++ )
                    {
                        dstIdx = x + y*pitch;      
                        if ( ( x % 2 ) == 0 )
                        {
                            abRawPixelData[dstIdx] = (byte) ( aiBuffer[srcIdx] & 0xFF );
                        }
                        else
                        {
                            abRawPixelData[dstIdx] = (byte) ( ( aiBuffer[srcIdx] >>> 8 ) & 0xFF );
                            srcIdx++;
                        }
                    }
                }
                break;
            case 24:
                for ( int y = 0; y < height; y++ )
                {                    
                    for ( int x = 0; x < dstWidth; x++ )
                    {
                        dstIdx = x + y*pitch;      
                        switch( byteCount )
                        {
                            case 0:
                                abRawPixelData[dstIdx] = (byte) ( aiBuffer[srcIdx] & 0xFF );
                                byteCount++;
                                break;
                            case 1:
                                abRawPixelData[dstIdx] = (byte) ( ( aiBuffer[srcIdx] >>> 8 ) & 0xFF );
                                byteCount++;
                                break;
                            case 2:
                                abRawPixelData[dstIdx] = (byte) ( ( aiBuffer[srcIdx] >>> 16 ) & 0xFF );
                                srcIdx++;
                                byteCount = 0;
                                break;
                        }
                    }
                }
                break;
            case 32:
                for ( int y = 0; y < height; y++ )
                {                    
                    for ( int x = 0; x < dstWidth; x++ )
                    {
                        dstIdx = x + y*pitch;      
                        switch( byteCount )
                        {
                            case 0:
                                abRawPixelData[dstIdx] =
                                    (byte) ( aiBuffer[srcIdx] & 0xFF );
                                byteCount++;
                                break;
                            case 1:
                                abRawPixelData[dstIdx] =
                                    (byte) ( ( aiBuffer[srcIdx] >>> 8 ) & 0xFF );
                                byteCount++;
                                break;
                            case 2:
                                abRawPixelData[dstIdx] =
                                    (byte) ( ( aiBuffer[srcIdx] >>> 16 ) & 0xFF );
                                byteCount++;
                                break;
                            case 3:
                                abRawPixelData[dstIdx] =
                                    (byte) ( ( aiBuffer[srcIdx] >>> 24 ) & 0xFF );
                                byteCount = 0;
                                srcIdx++;
                                break;
                        }
                    }
                }
                break;
            default:
                throw new GameFrameException( IConstants.UNSUPPORTED_RGB_BIT_COUNT );    
        }

        
        //System.out.print("unlocking...");
        //System.out.print("bytes:"+Integer.toHexString( (int) abRawPixelData[0] & 0xFF ) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[1] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[2] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[3] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[4] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[5] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[6] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[7] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[8] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[9] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[10] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[11] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[12] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[13] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[14] & 0xFF) );
        //System.out.print(", "+Integer.toHexString( (int) abRawPixelData[15] & 0xFF) +" " );

        
        try
        {
            // The instruction at "0x7c00503a" referenced memory at "0xFFFFFFFC".
            // The memory could not be "read".
            // Apply the changes and unlock the surface
            targetSurface.restore();
            targetSurface.unlock( dstRect, dstDDSurfaceDesc, abRawPixelData );
        }
        catch( ComFailException unlockFail )
        {
            throw new GameFrameException( IConstants.SURFACE_UNLOCK_FAILED + "\n"+ unlockFail);
        }
    }
}
