/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.msjava4x;

import gameframe.*;
import gameframe.graphics.*;

import java.io.*;

import com.ms.directX.*;
import com.ms.com.*;

/**
 * Implementation of the non transparent bitmap image for the
 * Microsoft Java DirectX platform.
 * 
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
class CDDBitmap extends CBitmapBase implements Bitmap, DrawableBitmap
{
    /**
     * Constructs a bitmap from the given paramters.
     *@param dd The DirectDraw object that can create new DD surfaces.
     *@param ddSurface The DirectDraw surface this bitmap class represents.
     *@param width Width of the DirectDraw surface this bitmap class represents.
     *@param height Height of the DirectDraw surface this bitmap class represents.
     *@param bitmapStorage The storage that stores all bitmaps created by this bitmap.
     *@param strOriginFile The originating bitmap file.
     */
    CDDBitmap(DirectDraw dd, DirectDrawSurface ddBackBuffer, DirectDrawSurface ddSurface,
              int width, int height, IBitmapStorage bitmapStorage, String strOriginFile)
    {
        super ( dd, ddBackBuffer, ddSurface, width, height, bitmapStorage, strOriginFile );
    }

    /**
     * Constructs a bitmap from the given paramters.
     *@param dd The DirectDraw object that can create new DD surfaces.
     *@param ddSurface The DirectDraw surface this bitmap class represents.
     *@param width Width of the DirectDraw surface this bitmap class represents.
     *@param height Height of the DirectDraw surface this bitmap class represents.
     *@param bitmapStorage The storage that stores all bitmaps created by this bitmap.
     *@param strOriginFile The originating bitmap file.
     *@param originRect The originating bitmaps rectangle that is this bitmap.
     */
    CDDBitmap(DirectDraw dd, DirectDrawSurface ddBackBuffer, DirectDrawSurface ddSurface,
              int width, int height, IBitmapStorage bitmapStorage, String strOriginFile,
              Rect originRect )
    {
        super ( dd, ddBackBuffer, ddSurface, width, height, bitmapStorage,
                strOriginFile, originRect );
    }

    /**
     * Releases all resources held by this bitmap representation.
     */
    public void finalize()
    {
        super.finalize();

        // Release the DirectDraw surface COM object
        if (m_ddSurface != null)
        {
            m_bitmapStorage.removeBitmap(this);
            ComLib.release(m_ddSurface);
            m_ddSurface    = null;
            m_ddBackBuffer = null;
            m_dd           = null;
            m_srcRectangle = null;
        }
    }
        
    /**
     * Draws the bitmap to the graphics engine that loaded it at the given location.
     * The location can be outside of the visible screen area in which case the bitmap
     * is clipped accordingly.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     */
    public void blitTo( int x, int y )
    {
        mStatic_dstRectangle.left   = x;
        mStatic_dstRectangle.right  = x + m_width;
        mStatic_dstRectangle.top    = y;
        mStatic_dstRectangle.bottom = y + m_height;

        // Check if we can do a fast blit
        if ( ( mStatic_dstRectangle.left   >= 0 ) &&
             ( mStatic_dstRectangle.top    >= 0 ) &&
             ( mStatic_dstRectangle.right  <= m_backBufferWidth ) &&
             ( mStatic_dstRectangle.bottom <= m_backBufferHeight ) &&
             m_fInVideoMem )
        {
            if ( m_ddBackBuffer.bltFast(
                    x,
                    y,
                    m_ddSurface,
                    m_srcRectangle,
                    DirectXConstants.DDBLTFAST_WAIT |
                    DirectXConstants.DDBLTFAST_NOCOLORKEY ) == DirectXConstants.DD_OK )
            {
                // Fast blit was successfull, we can return
                return;
            }
        }
        
        // We must do a slow blit
        if ( m_ddBackBuffer.blt(
            mStatic_dstRectangle,
            m_ddSurface,
            m_srcRectangle,
            DirectXConstants.DDBLT_WAIT ) != DirectXConstants.DD_OK )
        {
            restore();
            m_ddBackBuffer.blt(
                mStatic_dstRectangle,
                m_ddSurface,
                m_srcRectangle,
                DirectXConstants.DDBLT_WAIT );
        }
    }
    
    /**
     * Draws the bitmap to the graphics engine that loaded it at the given location and
     * at the given size. If this bitmap is larger or smaller than the destination size
     * the bitmap will automatically be scaled. Note: This method is slow, so it's use is not
     * preferred during actual game play.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with of the destination where the image is drawn to.
     *@param height The height of the destination where the image is drawn to.
     */
    public void strecthTo(int x, int y, int width, int height)
    {
        mStatic_dstRectangle.left   = x;
        mStatic_dstRectangle.right  = x + width;
        mStatic_dstRectangle.top    = y;
        mStatic_dstRectangle.bottom = y + height;
        if ( m_ddBackBuffer.blt(
            mStatic_dstRectangle,
            m_ddSurface,
            m_srcRectangle,
            DirectXConstants.DDBLT_WAIT ) != DirectXConstants.DD_OK )
        {
            restore();
            m_ddBackBuffer.blt(
                mStatic_dstRectangle,
                m_ddSurface,
                m_srcRectangle,
                DirectXConstants.DDBLT_WAIT );
        }
    }

    /**
     * Returns the width of the bitmap.
     *@return Width of the bitmap if available.
     */
    public int getWidth()
    {
        return m_width;
    }

    /**
     * Returns the height of the bitmap.
     *@return Height of the bitmap if available.
     */
    public int getHeight()
    {
        return m_height;
    }

    /**
     * Returns an effect processed clone of this bitmap.
     *@param effect The effects to be applied to the clone bitmap,
     *       or null if none are to be applied.
     *@return An effect processed clone of this bitmap.
     *@exception GameFrameException thrown if any errors occur.
     */
    public Bitmap getClone( BitmapEffect effect )
        throws GameFrameException
    {
        // If no effect processing is required, return a copy of this bitmap
        if ( effect == null )
        {
            return getSubBitmapClone( 0, 0, m_width, m_height );
        }

        // Create the effect processed surface
        DirectDrawSurface dds = getEffectSurface( effect );

        // Create the new bitmap, add it to bitmap storage and return it
        CDDBitmap bitmap = new CDDBitmap(
            m_dd,
            m_ddBackBuffer,
            dds,
            m_width,
            m_height,
            m_bitmapStorage,
            null,
            null );
        m_bitmapStorage.addBitmap( bitmap );
        return bitmap;
    }
    
    /**
     * Returns a bitmap that represents a part of this bitmap. Useful for e.g.
     * extracting animation frames from one bitmap. If for any reason the
     * creation of the new bitmap fails (out of memory etc.) this method returns null.
     * If the given rectangle is outside of the bitmap area of the original bitmap
     * or even larger than the original bitmap area this method functions in an
     * unpredictable manner (e.g. returns null or return junk).
     *
     *@param srcX The area of the bitmap to be returned as a new bitmap.
     *@param srcY The area of the bitmap to be returned as a new bitmap.
     *@param srcWidth The area of the bitmap to be returned as a new bitmap.
     *@param srcHeight The area of the bitmap to be returned as a new bitmap.
     *@return A new bitmap representing the requested part of the original bitmap.
     */
    public Bitmap getSubBitmapClone( int srcX, int srcY, int srcWidth, int srcHeight )
        throws GameFrameException
    {
        // Check that the dimensions are valid
        if ( ( srcWidth < 1 ) || ( srcHeight < 1 ) )
        {
            throw new GameFrameException( IConstants.AREA_SIZE_IS_ZERO );
        }
            
        // Check that the bounds stay inside the current clipping rectangle
        if ( ( srcWidth > m_srcRectangle.right - m_srcRectangle.left ) ||
             ( srcX < 0 ) ||
             ( srcHeight > m_srcRectangle.bottom - m_srcRectangle.top ) ||
             ( srcY < 0 ) )
        {
                throw new GameFrameException(
                    "The given bounds "+srcX+", "+srcY+"->"+srcWidth+", "+srcHeight+
                    " can't fit inside the bitmap size of "+
                    (m_srcRectangle.right - m_srcRectangle.left)+", "+
                    (m_srcRectangle.bottom - m_srcRectangle.top) );
        }
            
        // create a DirectDrawSurface for this bitmap
        DirectDrawSurface dds;
        try
        {
            if ( m_bitmapStorage instanceof CFullscreenGraphicsEngine )
            {
                CFullscreenGraphicsEngine engine = (CFullscreenGraphicsEngine) m_bitmapStorage;
                dds = engine.createSurface( srcWidth, srcHeight );
            }
            else
            {
                DDSurfaceDesc ddsd  = new DDSurfaceDesc();
                ddsd.flags          = DirectXConstants.DDSD_CAPS |
                                        DirectXConstants.DDSD_HEIGHT |
                                        DirectXConstants.DDSD_WIDTH;
                ddsd.ddsCaps        = DirectXConstants.DDSCAPS_OFFSCREENPLAIN;
                ddsd.width          = srcWidth;
                ddsd.height         = srcHeight;
                dds                 = m_dd.createSurface( ddsd );
            }
            
            Rect dstRect   = new Rect();
            dstRect.left   = 0;
            dstRect.top    = 0;
            dstRect.right  = srcWidth;
            dstRect.bottom = srcHeight;
        
            Rect srcRect   = new Rect();
            srcRect.left   = srcX;
            srcRect.top    = srcY;
            srcRect.right  = srcX + srcWidth;
            srcRect.bottom = srcY + srcHeight;
        
            Rect originatorRect   = new Rect();
            originatorRect.left   = srcRect.left;
            originatorRect.top    = srcRect.top;
            originatorRect.right  = srcRect.right;
            originatorRect.bottom = srcRect.bottom;
            
            if ( m_srcRectFromOriginBitmap != null )
            {
                originatorRect.left   += m_srcRectFromOriginBitmap.left;
                originatorRect.right  += m_srcRectFromOriginBitmap.left;
                originatorRect.top    += m_srcRectFromOriginBitmap.top;
                originatorRect.bottom += m_srcRectFromOriginBitmap.top;
            }
         
            dds.blt( dstRect, m_ddSurface, srcRect, DirectXConstants.DDBLT_WAIT );

            CDDBitmap bitmap = new CDDBitmap(
                m_dd,
                m_ddBackBuffer,
                dds,
                srcWidth,
                srcHeight,
                m_bitmapStorage,
                m_strOriginBitmapFilename,
                null ); //originatorRect );
            m_bitmapStorage.addBitmap( bitmap );
            return bitmap;
        }
        catch( ComFailException cfe )
        {
            throw new GameFrameException(IConstants.IMAGE_CREATION_FAILED+cfe);
        }
    }

    /**
     * Returns a bitmap that represents a part of this bitmap by using a new
     * control object, but using the old bitmap data (saves memory, but is not
     * that fast on many platforms, on the platforms where this method is really
     * too slow for interactive use the implementation might actually copy the
     * data as with the getSubBitmapClose() method to speed things up).
     *
     * Useful for e.g. extracting animation frames from one bitmap. This method
     * is not very fast and should be preferrably called during game initialization.
     * If for any reason the creation of the new bitmap fails (out of memory etc.)
     * this method returns null. If the given rectangle is outside of the bitmap
     * area of the original bitmap or even larger than the original bitmap area
     * this method functions in an unpredictable manner (e.g. returns null or junk).
     *
     *@param srcX The area of the bitmap to be returned as a new bitmap.
     *@param srcY The area of the bitmap to be returned as a new bitmap.
     *@param srcWidth The area of the bitmap to be returned as a new bitmap.
     *@param srcHeight The area of the bitmap to be returned as a new bitmap.
     *@return A new bitmap representing the requested part of the original bitmap.
     */
    public Bitmap getSubBitmap(int srcX, int srcY, int srcWidth, int srcHeight)
        throws GameFrameException
    {
        // Check that the dimensions are valid
        if ( (srcWidth < 1) || (srcHeight < 1) )
        {
            throw new GameFrameException(IConstants.AREA_SIZE_IS_ZERO);
        }
            
        // Check that the bounds stay inside the current clipping rectangle
        if ( ( srcWidth > m_srcRectangle.right - m_srcRectangle.left ) ||
             ( srcX < 0 ) ||
             ( srcHeight > m_srcRectangle.bottom - m_srcRectangle.top ) ||
             ( srcY < 0 ) )
        {
                throw new GameFrameException(
                    "The given bounds "+srcX+", "+srcY+"->"+srcWidth+", "+srcHeight+
                    " can't fit inside the bitmap size of "+
                    (m_srcRectangle.right - m_srcRectangle.left)+", "+
                    (m_srcRectangle.bottom - m_srcRectangle.top) );
        }

        Rect originatorRect   = new Rect();
        originatorRect.left   = srcX;
        originatorRect.right  = srcX + srcWidth;
        originatorRect.top    = srcY;
        originatorRect.bottom = srcY + srcHeight;

        if ( m_srcRectFromOriginBitmap != null )
        {
            originatorRect.left   += m_srcRectFromOriginBitmap.left;
            originatorRect.right  += m_srcRectFromOriginBitmap.left;
            originatorRect.top    += m_srcRectFromOriginBitmap.top;
            originatorRect.bottom += m_srcRectFromOriginBitmap.top;
        }

        CDDBitmap bitmap = new CDDBitmap(
            m_dd,
            m_ddBackBuffer,
            m_ddSurface,
            srcWidth,
            srcHeight,
            m_bitmapStorage,
            m_strOriginBitmapFilename,
            originatorRect );
        
        bitmap.m_srcRectangle.left    = originatorRect.left;
        bitmap.m_srcRectangle.top     = originatorRect.top;
        bitmap.m_srcRectangle.right   = originatorRect.right;
        bitmap.m_srcRectangle.bottom  = originatorRect.bottom;
        bitmap.m_superBitmap = this;
        return bitmap;
    }

    /**
     * Draws the bitmap to the given drawable bitmap at the given location.
     * The location can be outside of the visible screen area in which case the bitmap
     * is clipped accordingly. This method is fast and is usable during the game execution.
     *@param bitmap The bitmap that will be drawn to this bitmap.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     */
    public void blitTo(DrawableBitmap bitmap, int x, int y)
    {
        CBitmapBase dstBitmap = (CBitmapBase) bitmap;
        mStatic_dstRectangle.left   = x;
        mStatic_dstRectangle.right  = x + m_width;
        mStatic_dstRectangle.top    = y;
        mStatic_dstRectangle.bottom = y + m_height;
        if ( dstBitmap.m_ddSurface.blt(
            mStatic_dstRectangle,
            m_ddSurface,
            m_srcRectangle,
            DirectXConstants.DDBLT_WAIT ) != DirectXConstants.DD_OK )
        {
            restore();
            dstBitmap.m_ddSurface.blt(
                mStatic_dstRectangle,
                m_ddSurface,
                m_srcRectangle,
                DirectXConstants.DDBLT_WAIT );
        }
    }

    /**
     * Draws the bitmap to the given drawable bitmap at the given location
     * at the given size. If this bitmap is larger or smaller than the destination size
     * the bitmap will automatically be scaled. Note: This method is slow, so it's use is not
     * preferred during actual game play.
     *@param bitmap The bitmap that will be drawn to this bitmap.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with of the destination where the image is drawn to.
     *@param height The height of the destination where the image is drawn to.
     */
    public void strecthTo(DrawableBitmap bitmap, int x, int y, int width, int height)
    {
        CBitmapBase dstBitmap = (CBitmapBase) bitmap;
        mStatic_dstRectangle.left   = x;
        mStatic_dstRectangle.right  = x + width;
        mStatic_dstRectangle.top    = y;
        mStatic_dstRectangle.bottom = y + height;
        if ( dstBitmap.m_ddSurface.blt(
            mStatic_dstRectangle,
            m_ddSurface,
            m_srcRectangle,
            DirectXConstants.DDBLT_WAIT ) != DirectXConstants.DD_OK )
        {
            restore();
            dstBitmap.m_ddSurface.blt(
                mStatic_dstRectangle,
                m_ddSurface,
                m_srcRectangle,
                DirectXConstants.DDBLT_WAIT );
        }
    }
}