/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.msjava4x;

import gameframe.engines.*;
import gameframe.engines.jdk11x.CEmulatedFullscreenGraphicsEngine;
import gameframe.engines.jdk11x.CComponentGraphicsEngine;
import gameframe.engines.jdk11x.CWindowedGraphicsEngine;
import gameframe.engines.jdk11x.CInputEngine;
import gameframe.graphics.*;
import gameframe.input.*;
import gameframe.sound.*;
import gameframe.util.*;
import gameframe.*;

import java.util.*;

/**
 * Implementation of an engine factory for Microsoft Java JVM's
 * that support DirectX interfacing from Java. Uses the emulated
 * fullscreen graphics engine (if needed), component graphics engine
 * (if needed), windowed graphics engine (if needed) and the input
 * engine (allways) from the gameframe.engines.jdk11x package.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class CEngineFactory implements gameframe.engines.EngineFactory
{
    /** Reference to the singleton instance of this engine factory. */
    private static CEngineFactory mStatic_engineFactory;
    
    /** All objects that must be finalized upon engine factory finalization. */
    private Vector m_finalizableObjects         = new Vector(3,3);
	/** Reference to the java.awt.Component for the DirectSound sound engine */
	private java.awt.Component m_focusComponent;
    /** Reference to the singleton sound engine implementation. */
    private SoundEngine m_soundEngine;
    /** Reference to the singleton graphics engine implementation. */
    private GraphicsEngine m_graphicsEngine;
    /** Reference to the singleton input engine implementation. */
    private InputEngine m_inputEngine;
    /** The valid fullscreen modes this engine factory can handle. */
    private String[] m_aFullscreenGraphicsModes;
    /** The main factory settings used for this engine factory. */
    private GameFrameSettings m_settings;
    
    /**
     * Constructs the engine factory.
     *@param settings The settings to be used in this engine factory.
     */
    private CEngineFactory( GameFrameSettings settings )
    {
        m_settings = settings;
    }
    
    /**
     * Returns the singleton instance of this engine factory (the instace
     * will be created upon the first call to this method).
     *@param settings The settings to be used in this engine factory.
     *@return The singleton instance of this engine factory.
     */
    public static CEngineFactory getInstance( GameFrameSettings settings )
    {
        if (mStatic_engineFactory == null)
        {
            mStatic_engineFactory = new CEngineFactory(settings);
        }
        
        return mStatic_engineFactory;
    }
    
    /**
     * Returns the short name (max 8 characters) of this
     * engine factory implementation which is "MSJ4DX".
     *@return The short name of the engine factory.
     */
    public String getName()
    {
        return "MSJ4DX";
    }

    /**
     * Returns the description of the engine factory implementation.
     *@return A description of the engine factory.
     */
    public String getDescription()
    {
        return "Engine factory suited for Microsoft 4.x JVM's that support interfacing to DirectX";
    }

    /**
     * Returns the suitability of this engine implementation for
     * the current platform as one of the enumerated values.
     * Currently return POOR so that any better implementation will
     * override this implementation.
     *@return Suitability of the engine factory for this platform.
     */
    public int getSuitabilityForPlatform()
    {
		try
		{
            // Check if DirectX Java extensions can be found
            com.ms.directX.DDCaps ddcaps = new com.ms.directX.DDCaps();
            ddcaps = null;
            
            if ( m_settings.isMsJavaAllowed() )
            {
    			return CEngineFactory.SUITABILITY_ULTIMATE;
            }
		}
		catch(NoClassDefFoundError ncdfe)
		{
		}
        catch(Exception anyException)
        {
        }
        catch(Error anyError)
        {
        }

        return CEngineFactory.SUITABILITY_NOT_SUITED;
    }
    
    /**
     * Returns true if this engine factory is runnable on the platform.
     */
    public static boolean isRunnableOnPlatform()
    {
		try
		{
            // Check if Microsoft Java extensions can be found
            com.ms.wfc.ui.Region region = new com.ms.wfc.ui.Region();
            region.dispose();
            com.ms.directX.DDCaps ddcaps = new com.ms.directX.DDCaps();
            ddcaps = null;
            
			return true;
		}
		catch(NoClassDefFoundError ncdfe)
		{
		}
        catch(Exception anyException)
        {
        }
        catch(Error anyError)
        {
        }

        return false;
    }

    /**
     * Returns the fullscreen graphics modes supported by graphics engines that
     * are created by this factory. If the factory created graphics engines support
     * only windowed modes, this method returns null.
     * Note: This method should always do a full cleanup after it has been run!
     *@return Supported fullscreen modes or null.
     */
    public String[] getSupportedResolutions()
    {
        if ( m_aFullscreenGraphicsModes == null )
        {
            Vector modes;

            if ( m_graphicsEngine instanceof CFullscreenGraphicsEngine )
            {
                CFullscreenGraphicsEngine fullscreenEngine = (CFullscreenGraphicsEngine) m_graphicsEngine;
                modes = fullscreenEngine.getSupportedModes();
            }
            else
            {
                try
                {
                    CFullscreenGraphicsEngine fullscreenEngine =
                        CFullscreenGraphicsEngine.getInstance( new GameFrameSettings() );
                    modes = fullscreenEngine.getSupportedModes();
                    fullscreenEngine.finalize();
                }
                catch( GameFrameException gfe )
                {
                    return null;
                }
            }

            m_aFullscreenGraphicsModes = new String[modes.size()];
            modes.copyInto(m_aFullscreenGraphicsModes);
        }
        
        return m_aFullscreenGraphicsModes;
    }
    
    /**
     * Returns the sound engine implementation for the current platform.
     * If not successfull returns the do-nothing implementation of a sound engine.
     *@return The sound engine for this platform.
     */
    public SoundEngine getSoundEngine()
        throws GameFrameException
    {
        if ( m_soundEngine == null )
        {
            // DirectSound needs a java.awt.Window, so we must create a window before
            // creating DirectSound based sound engine. This is done by creating the
            // graphics engine
            if ( getGraphicsEngine() == null )
            {
                m_soundEngine = NullSoundEngine.getInstance();
                m_finalizableObjects.addElement( m_soundEngine );
                return m_soundEngine;
            }

            if ( m_focusComponent == null )
            {
                m_soundEngine = NullSoundEngine.getInstance();
                m_finalizableObjects.addElement( m_soundEngine );
                return m_soundEngine;
            }
            
            // Try to create the DirectSound sound engine
            try
            {
                m_soundEngine = new CSoundEngine( m_focusComponent );
            }
            catch( Exception anyException )
            {
                // DirectSound sound engine failed to initialize, we use the default null sound engine.
                m_soundEngine = NullSoundEngine.getInstance();
            }

            m_finalizableObjects.addElement( m_soundEngine );
        }

        return m_soundEngine;
    }
    
    /**
     * Returns the most suitable graphics engine implementation for the current platform.
     * Only the fullscreen graphics engine differs from the JDK 1.1.x versions so other
     * engines are used from the <code>gameframe.engines.jdk11x</code> package.
     *@return The graphics engine for this platform.
     *@see gameframe.engines.msjava4x.CFullscreenGraphicsEngine
     *@see gameframe.engines.jdk11x.CEmulatedFullscreenGraphicsEngine
     *@see gameframe.engines.jdk11x.CWindowedGraphicsEngine
     *@see gameframe.engines.jdk11x.CComponentGraphicsEngine
     */
    public GraphicsEngine getGraphicsEngine()
        throws GameFrameException
    {
        if ( m_graphicsEngine == null )
        {
            switch( m_settings.getScreenMode() )
            {
                case GameFrameSettings.SCREENMODE_FULLSCREEN:
                {
                    CFullscreenGraphicsEngine graphicsEngine = null;
                        
                    try
                    {
                        // Try to create the real fullscreen graphics engine
                        graphicsEngine
                            = CFullscreenGraphicsEngine.getInstance( m_settings );
                            
                        m_focusComponent = (java.awt.Window) graphicsEngine;
                        m_graphicsEngine = graphicsEngine;
                        m_finalizableObjects.addElement( graphicsEngine );
                    }
                    catch( Exception anyException )
                    {
                        // Create an emulated fullscreen graphics engine
                        // Create a component graphics engine
                        CComponentGraphicsEngine componentEngine = getComponentGraphicsEngine();
                        
                        try
                        {
                            CEmulatedFullscreenGraphicsEngine emulatedEngine =
                                CEmulatedFullscreenGraphicsEngine.getInstance(
                                    componentEngine, m_settings);

                            // Set the references to focus component and graphics engine
                            m_focusComponent = (java.awt.Component) emulatedEngine;

                            // Set the reference to the grapchis engine
                            m_graphicsEngine = componentEngine;
                                    
                            // Add the graphics engine finalizable objects
                            m_finalizableObjects.addElement( emulatedEngine );
                        }
                        catch( Exception anyException2 )
                        {
                            throw new GameFrameException(
                                "Couldn't create graphics engine. "+anyException2 );
                        }
                        catch( Error anyError2 )
                        {
                            throw new GameFrameException(
                                "Couldn't create graphics engine. "+anyError2 );
                        }
                    }
                    break;
                }
                case GameFrameSettings.SCREENMODE_WINDOWED:
                {
                    // Create a component graphics engine
                    CComponentGraphicsEngine componentEngine = getComponentGraphicsEngine();

                    CWindowedGraphicsEngine windowedEngine =
                        CWindowedGraphicsEngine.getInstance( componentEngine, m_settings );
                
                    m_focusComponent = componentEngine;
                    m_graphicsEngine = componentEngine;
                    m_finalizableObjects.addElement( windowedEngine );
                    break;                    
                }
                case GameFrameSettings.SCREENMODE_COMPONENT:
                {
                    // Create a component graphics engine
                    CComponentGraphicsEngine componentEngine = getComponentGraphicsEngine();

                    // Set the references to focus component and graphics engine
                    m_focusComponent    = (java.awt.Component) componentEngine;

                    // Set the reference to the grapchis engine
                    m_graphicsEngine = componentEngine;
                            
                    break;
                }
            }
        }
        
        return m_graphicsEngine;
    }

    /**
     * Creates and returns the component graphics engine.
     *@return The used component graphics engine.
     *@throw GameFrameException thrown if operation fails.
     */
    private CComponentGraphicsEngine getComponentGraphicsEngine()
        throws GameFrameException
    {
        CComponentGraphicsEngine componentEngine = null;
        // Create a component graphics engine
        try
        {
            componentEngine = CComponentGraphicsEngine.getInstance(
                m_settings);
            // Add the graphics engine finalizable objects
            m_finalizableObjects.addElement( componentEngine );
        }
        catch(Exception anyException)
        {
            throw new GameFrameException("Couldn't create graphics engine. "+anyException);
        }
        catch(Error anyError)
        {
            throw new GameFrameException("Couldn't create graphics engine. "+anyError);
        }
        
        return componentEngine;
    }

    /**
     * Returns the input engine implementation for the current platform.
     * If not successfull returns null.
     *@return The input engine for this platform.
     *@see gameframe.engines.jdk11x.CInputEngine
     */
    public InputEngine getInputEngine()
        throws GameFrameException
    {
        if ( m_inputEngine == null )
        {
            if ( getGraphicsEngine() == null )
            {
                return null;
            }

            if ( m_focusComponent == null )
            {
                return null;
            }
            
            m_inputEngine = CInputEngine.getInstance( m_focusComponent );
            m_finalizableObjects.addElement( m_inputEngine );
        }
        
        return m_inputEngine;
    }

    /**
     * Returns a new timer instance. Several timers can be created with this method. If a native
     * timer can't be for some reason created, this method returns the basic implementation
     * CTimer from the GameFrame.Java11 package.
     *@return Timer instance.
     */
    public Timer createTimer()
        throws GameFrameException
    {
        CTimer timer = new CTimer( this );
        m_finalizableObjects.addElement(timer);
        return timer;
    }

    /**
     * Notifies that the given timer has been finalized.
     */
    public void finalizationNotify ( Timer timer )
    {
        m_finalizableObjects.removeElement( timer );
    }

    /**
     * Invoked before application exits. Should do all the cleanup that is
     * necessary to exit cleanly (close window, stop playing sounds, release
     * any objects referenced etc.).
     */
    public void finalize()
    {
        // Finalize all objects that must be finalized
        for ( Enumeration enum = m_finalizableObjects.elements(); enum.hasMoreElements(); )
        {
            Finalizable finalizableObject = (Finalizable) enum.nextElement();
            finalizableObject.finalize();
        }
        m_finalizableObjects.removeAllElements();
        m_focusComponent = null;
        
        // Reset static variables, except flag that indicates
        // if Java Media Framework is available as it is set
        // only once in static initialiser
        mStatic_engineFactory = null;
    }
}
