/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.msjava4x;

import gameframe.*;
import gameframe.graphics.*;
import gameframe.util.*;
import gameframe.engines.*;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.image.*;

import com.ms.com.*;
import com.ms.dll.*;
import com.ms.awt.*;
import com.ms.directX.*;
import com.ms.awt.peer.ComponentPeerX;

/**
 * Microsoft DirectDraw (from DirectX 3) implementation of the graphics engine.
 * Uses Microsoft Java 3.x specific DirectX classes and Direct/J API.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class CFullscreenGraphicsEngine extends Window
    implements GraphicsEngine, IBitmapStorage, Finalizable
{
    /** Frame that acts as a parent to the window. */
    private static Frame mStatic_parentFrame;
    /** The rectangle that covers the DirectDraw surface this bitmap class represents.*/
    static Rect mStatic_srcRectangle;
    /** The rectangle that covers the destination are of the blit. */
    static Rect mStatic_dstRectangle;

    /** The bits per pixel value of the display. If value is NO_PREFERRED_BPP it is set to be the same as system BPP value. */
    private int m_modeBpp       = DEFAULT_COLOURDEPTH;
    /** The refresh rate of the display. Initially set to NO_PREFERRED_REFRESHRATE which means that all refresh modes are enumerated. */
    private int m_modeRefresh   = NO_PREFERRED_REFRESHRATE;
    /** DirectDraw object. */
    private DirectDraw          m_directDraw;
    /** DirectDraw primary surface. */
    private DirectDrawSurface   m_ddsPrimary;
    /** DirectDraw backbuffer surface. */
    private DirectDrawSurface   m_ddsBack;
    /** DirectDraw palette. */
    private DirectDrawPalette   m_ddPalette;
    /** The DirectDrawClipper for front and back surfaces. */
    private DirectDrawClipper   m_ddclipper;
    /** Flag that indicates if we are at the moment in fullscreen mode or not. */
    private boolean m_fInfullscreenMode = false;
    /** Flag that indicates if we have at the moment focus or not. */
    private boolean m_fFocused = true;
    
    /** The color that is used when painting the backbuffer in the clear operation. */
    private Color m_backgroundColor = Color.black;
    /** The actual pixel value that is used to fill the backbuffer with background color. */
    private int m_backgroundFillValue = 0;
    /** Width of this frame. */
    private int m_width;
    /** Height of this frame. */
    private int m_height;
    /** Every bitmap that has been loaded since last finalize method call. */
    private Vector m_loadedBitmaps = new Vector(10,10);
    /** The rectangle object that holds the size of this screen. */
    private Rect m_screenRectangle = new Rect();
    /** Name of the directory that contains loaded bitmaps. */
    private String m_strDataDirectory = "";
    /** DirectDraw surface that contains the letter for the current font. */
    private DirectDrawSurface m_ddsFontBitmap;
    /** Width of all the letters in the font bitmap. */
    private int m_letterWidth = 0;
    /** Height of all the letters in the font bitmap. */
    private int m_letterHeight = 0;

    /**
     * Returns an instance of this class. The instance is a window that
     * has the given title and size. Note: This class is not a singleton!
     *@param settings The settings used to create this graphics engine.
     */
    static CFullscreenGraphicsEngine getInstance(
        GameFrameSettings settings )
        throws GameFrameException
    {
        mStatic_parentFrame = new Frame( settings.getTitle() );
        Image defaultIcon = GameFrame.getInstance().getDefaultIcon();
        if ( defaultIcon != null )
        {
            mStatic_parentFrame.setIconImage( defaultIcon );
        }

        // Hide the parent frame
        mStatic_parentFrame.setBounds( -100, -100, 50, 50 );
        mStatic_parentFrame.setResizable( false );

        CFullscreenGraphicsEngine graphicsEngine =
            new CFullscreenGraphicsEngine( mStatic_parentFrame, settings );
        graphicsEngine.init();
        
        return graphicsEngine;
    }

    /**
     * Private constructor for this singleton instance.
     *@param parentFrame The parent frame of this window
     *@param settings The settings used to create this graphics engine.
     */
    private CFullscreenGraphicsEngine(
        Frame parentFrame, GameFrameSettings settings )
    {
        this( parentFrame ,
              settings.getRequiredResolutionWidth(),
              settings.getRequiredResolutionHeight(),
              settings.getRequiredBitdepth(),
              settings.getRequiredRefreshRate() );
    }

    /**
     * Private constructor for this singleton instance.
     *@param parentFrame The parent frame of this window
     *@param settings The settings used to create this graphics engine.
     */
    private CFullscreenGraphicsEngine(
        Frame parentFrame, int width, int height, int bpp, int refresh )
    {
        super( parentFrame );
        
        super.addFocusListener( new java.awt.event.FocusListener()
        {
            public void focusGained( java.awt.event.FocusEvent event )
            {
                m_fFocused = true;
            }
            
            public void focusLost( java.awt.event.FocusEvent event )
            {
                m_fFocused = false;
            }
        } );

        mStatic_srcRectangle     = new Rect();
        mStatic_dstRectangle     = new Rect();
        m_width                  = width;
        m_height                 = height;
        m_screenRectangle.left   = 0;
        m_screenRectangle.top    = 0;
        m_screenRectangle.right  = m_width;
        m_screenRectangle.bottom = m_height;
        
        m_modeBpp                = bpp;
        m_modeRefresh            = refresh;
        
        setVisible(true);
        requestFocus();
    }

    /**
     * Initializes this graphics engine to fullscreen mode with doublebuffering.
     */
    void init()
        throws GameFrameException
    {
        DDSurfaceDesc       ddsd;

        setBounds(0, 0, m_width, m_height);
        show();

        // Create a surface description that is used when enumerating the display modes
        ddsd             = new DDSurfaceDesc();
        ddsd.width       = m_width;
        ddsd.height      = m_height;
        ddsd.flags       = DirectXConstants.DDSD_HEIGHT | DirectXConstants.DDSD_WIDTH;
                
        // Create the main DirectDraw object
        try
        {
            m_directDraw = new DirectDraw();
        }
        catch(ComException ce)
        {
            finalize();
            throw new GameFrameException(
                IConstants.COULDNT_CREATE_ENGINE+" "+
                "ComFailException: Couldn't create DirectDraw object (ComException)");
        }

        // If we have no preferred BPP, get the systems BPP that is allways a "legal" value
        if ( m_modeBpp == DEFAULT_COLOURDEPTH )
        {
            m_modeBpp = m_directDraw.systemBpp();
        }
        
        // Enumerate all available video modes
        CModeEnumerator enumerator = new CModeEnumerator(m_modeBpp, NO_PREFERRED_REFRESHRATE);
        m_directDraw.enumDisplayModes(DirectXConstants.DDEDM_REFRESHRATES, ddsd, null, enumerator);

        Vector aAvailableModes = enumerator.getAvailableModes();
        // If a correct video mode is not found exit...
        if (aAvailableModes.size() == 0)
        {
            finalize();
            throw new GameFrameException(
                IConstants.COULDNT_CREATE_ENGINE+" "+
                "No fullscreen videomodes available");
        }
        
        DDSurfaceDesc mode = null;
        for (Enumeration enum = aAvailableModes.elements(); enum.hasMoreElements(); )
        {
            DDSurfaceDesc testedMode = (DDSurfaceDesc) enum.nextElement();
            
            // Pick the first mode that has the required resolution...
            if ((testedMode.width == m_width) && (testedMode.height == m_height))
            {
                m_modeRefresh = testedMode.refreshRate;
                mode          = testedMode;
                break;
            }
        }
        
        if (mode == null)
        {
            finalize();
            throw new GameFrameException(IConstants.COULDNT_CREATE_ENGINE+" Requested fullscreen videomode not found");
        }
        

        try
        {
            m_directDraw.setDisplayMode(m_width,
                                m_height,
                                m_modeBpp,
                                m_modeRefresh,
                                0);
            m_fInfullscreenMode = true;
        }
        catch(ComFailException cfe)
        {
            m_fInfullscreenMode = false;
            finalize();
            throw new GameFrameException(
                IConstants.COULDNT_CREATE_ENGINE+" "+
                "ComFailException: Couldn't set display mode to requested mode" );
        }

        try
        {
            m_directDraw.setCooperativeLevel(
                this,
                DirectXConstants.DDSCL_EXCLUSIVE | DirectXConstants.DDSCL_FULLSCREEN);
            m_fInfullscreenMode = true;
        }
        catch(ComFailException cfe)
        {
            m_fInfullscreenMode = false;
            finalize();
            throw new GameFrameException(
                IConstants.COULDNT_CREATE_ENGINE+" "+
                "ComFailException: Couldn't set mode to fullscreen" );
        }

        // Give the setCooperativeLevel() method some time to complete
        try
        {
            Thread.currentThread().sleep( 100 );
        }
        catch(InterruptedException ie)
        {
        }
        
        DDCaps driverCaps = new DDCaps();
        DDCapsFx driverCapsFx = new DDCapsFx();
        DDCaps helCaps = new DDCaps();
        DDCapsFx helCapsFx = new DDCapsFx();
        
        m_directDraw.getCaps(driverCaps, driverCapsFx, helCaps, helCapsFx);
        
        // Create the flipping chain that has primary surface with one backbuffer
        // First try to create them to video memory...
        ddsd                 = new DDSurfaceDesc();
        ddsd.flags           = DirectXConstants.DDSD_CAPS | DirectXConstants.DDSD_BACKBUFFERCOUNT;
        ddsd.ddsCaps         = DirectXConstants.DDSCAPS_PRIMARYSURFACE |
                                DirectXConstants.DDSCAPS_FLIP |
                                DirectXConstants.DDSCAPS_COMPLEX | DirectXConstants.DDSCAPS_VIDEOMEMORY;
        ddsd.backBufferCount = 1;

        try
        {
            m_ddsPrimary         = m_directDraw.createSurface(ddsd);
            if (m_ddsPrimary == null)
            {
                // Couldn't create in video memory, try to create in system memory
                ddsd                 = new DDSurfaceDesc();
                ddsd.flags           = DirectXConstants.DDSD_CAPS | DirectXConstants.DDSD_BACKBUFFERCOUNT;
                ddsd.ddsCaps         = DirectXConstants.DDSCAPS_PRIMARYSURFACE |
                                        DirectXConstants.DDSCAPS_FLIP |
                                        DirectXConstants.DDSCAPS_COMPLEX | DirectXConstants.DDSCAPS_SYSTEMMEMORY;
                ddsd.backBufferCount = 1;
                
                try
                {
                    m_ddsPrimary = m_directDraw.createSurface(ddsd);
                    if (m_ddsPrimary == null)
                    {
                        finalize();
                        throw new GameFrameException(
                            IConstants.COULDNT_CREATE_ENGINE+" "+
                            "Couldn't create primary DirectDrawSurface!" );
                    }
                }
                catch(ComFailException cfe1)
                {
                    finalize();
                    throw new GameFrameException(
                        IConstants.COULDNT_CREATE_ENGINE+" "+
                        "ComFailException: Couldn't create primary DirectDrawSurface" );
                }
                catch(Exception error1)
                {
                    throw new GameFrameException(
                        IConstants.COULDNT_CREATE_ENGINE+" "+
                        error1 );
                }
            }
        }
        catch(ComFailException cfe2)
        {
            // Couldn't create in video memory, try to create in system memory
            ddsd                 = new DDSurfaceDesc();
            ddsd.flags           = DirectXConstants.DDSD_CAPS | DirectXConstants.DDSD_BACKBUFFERCOUNT;
            ddsd.ddsCaps         = DirectXConstants.DDSCAPS_PRIMARYSURFACE |
                                    DirectXConstants.DDSCAPS_FLIP |
                                    DirectXConstants.DDSCAPS_COMPLEX | DirectXConstants.DDSCAPS_SYSTEMMEMORY;
            ddsd.backBufferCount = 1;

            try
            {
                m_ddsPrimary         = m_directDraw.createSurface(ddsd);
                if (m_ddsPrimary == null)
                {
                    finalize();
                    throw new GameFrameException(
                        IConstants.COULDNT_CREATE_ENGINE+" "+
                        "Couldn't create primary DirectDrawSurface." );
                }
            }
            catch(ComFailException cfe3)
            {
                finalize();
                throw new GameFrameException(
                    IConstants.COULDNT_CREATE_ENGINE+" "+
                    "ComFailException: Couldn't create primary DirectDrawSurface." );
            }
            catch(Exception error2)
            {
                finalize();
                throw new GameFrameException(
                    IConstants.COULDNT_CREATE_ENGINE+" "+
                    error2 );
            }
        }

		// Create a clipper for the primary surface
		m_ddclipper = m_directDraw.createClipper(0);
		m_ddclipper.setComponent(this);			
		m_ddsPrimary.setClipper( m_ddclipper );

        try
        {
            m_ddsBack = m_ddsPrimary.getAttachedSurface(DirectXConstants.DDSCAPS_BACKBUFFER);
            if (m_ddsBack == null)
            {
                finalize();
                throw new GameFrameException(
                    IConstants.COULDNT_CREATE_ENGINE+" "+
                    "Couldn't get backbuffer DirectDrawSurface." );
            }
        }
        catch(ComFailException cfe)
        {
            finalize();
            throw new GameFrameException(
                IConstants.COULDNT_CREATE_ENGINE+" "+
                "ComFailException: Couldn't get backbuffer DirectDrawSurface" );
        }
        catch(Exception error3)
        {
            throw new GameFrameException(
                IConstants.COULDNT_CREATE_ENGINE+" "+
                error3 );
        }

        // Set a clipper for the secondary surface
		m_ddsBack.setClipper( m_ddclipper );
        
        // Hide the cursor to offscreen area
        com.ms.wfc.ui.Cursor.setPosition(new com.ms.wfc.ui.Point(mode.width, mode.height));

        // Request input focus
        requestFocus();
        
        // Give the engine some time to settle down
        try
        {
            Thread.currentThread().sleep( 50 );
        }
        catch( InterruptedException ie )
        {
        }
    }

	/**
	 * Load a bitmap (used bitmap format is Windows BMP, no alpha values are loaded)
	 * from the given file and return an abstract reference for the loaded bitmap.
	 * Bitmap is loaded from the bitmap resource directory specified by MainFactory.
	 *@param strFileName The name of the file to be loaded (without the file extension).
	 *@return An unique Bitmap object that represents the loaded bitmap image.
	 */
    public Bitmap loadBitmap(String strFileName)
        throws java.io.FileNotFoundException, GameFrameException
    {
        strFileName = m_strDataDirectory+strFileName+".bmp";
        return loadBitmapInternal( strFileName );
    }
    
	/**
	 * Load a bitmap (used bitmap format is Windows BMP, no alpha values are loaded)
	 * from the given file and return an abstract reference for the loaded bitmap.
	 *@param strFileName The exact name of the file to be loaded.
	 *@return An unique Bitmap object that represents the loaded bitmap image.
	 */
    Bitmap loadBitmapInternal( String strFileName )
        throws java.io.FileNotFoundException, GameFrameException
    {
        DirectDrawSurface ddSurface = null;
        DirectDrawBitmap ddBitmap   = loadDDBitmap( strFileName );
        
        ddSurface = createSurface( ddBitmap.width(), ddBitmap.height() );
		ddSurface.copyBitmap(ddBitmap, 0, 0, 0, 0);

        CDDBitmap bitmap = new CDDBitmap(
            m_directDraw, m_ddsBack, ddSurface,
            ddBitmap.width(), ddBitmap.height(),
            this, strFileName );
        addBitmap( bitmap );
        return bitmap;
    }

	/**
	 * Load a bitmap (used bitmap format is Windows BMP, alpha keycolor is set to the
	 * pixel color at bitmap location 0,0 which is the pixel at upper left corner of
	 * the bitmap)from the given file and return an abstract reference for the loaded bitmap.
	 * Bitmap is loaded from the bitmap resource directory specified by MainFactory.
	 *@param strFileName The name of the file to be loaded (without the file extension).
	 *@return An unique Bitmap object that represents the loaded bitmap image.
	 */
    public Bitmap loadAlphaBitmap( String strFileName )
        throws java.io.FileNotFoundException, GameFrameException
    {
        strFileName = m_strDataDirectory+strFileName+".bmp";
        
        BitmapData data = BitmapFile.loadBMP( strFileName, true );
        return loadAlphaBitmapInternal(
            strFileName,
            (byte) data.getKeyColorRed(),
            (byte) data.getKeyColorGreen(),
            (byte) data.getKeyColorBlue() );
    }
    
    /**
     * Loads and returns a transparent bitmap (used bitmap format is Windows
     * BMP 8-bit, 16-bit or 24-bit) from the given filename. Bitmap is loaded
     * by default from the current data directory, the directory that is used
     * to load the bitmaps can be set with the setDataDirectory() method.
     * Alpha keycolor (the color that is interpreted as the transparent color)
     * is taken from the given red, green and blue values. If the bitmap is
     * larger than current resolution it will be clipped to the dimensions of
     * the current resolution.
     *@param strFileName The name of the file to be loaded (without the file extension).
     *@param red The red intensity value of the key color (in range 0x00-0xFF).
     *@param green The green intensity value of the key color (in range 0x00-0xFF).
     *@param blue The blue intensity value of the key color (in range 0x00-0xFF).
     *@return Bitmap object that represents the loaded bitmap image or null if the loading failed.
     *@exception FileNotFoundException Thrown if the given file doesn't exist.
     *@exception GameFrameException Thrown if other error occur while loading the file.
     */
    public Bitmap loadAlphaBitmap( String strFileName, byte red, byte green, byte blue )
        throws FileNotFoundException, GameFrameException
    {
        strFileName = m_strDataDirectory+strFileName+".bmp";
        
        return loadAlphaBitmapInternal( strFileName, red, green, blue );
    }
    
	/**
	 * Load a bitmap (used bitmap format is Windows BMP, alpha keycolor is set to the
	 * pixel color at bitmap location 0,0 which is the pixel at upper left corner of
	 * the bitmap)from the given file and return an abstract reference for the loaded bitmap.
	 * Bitmap is loaded from the bitmap resource directory specified by MainFactory.
	 *@param strFileName The name of the file to be loaded (without the file extension).
     *@param red The red intensity value of the key color (in range 0x00-0xFF).
     *@param green The green intensity value of the key color (in range 0x00-0xFF).
     *@param blue The blue intensity value of the key color (in range 0x00-0xFF).
	 *@return An unique Bitmap object that represents the loaded bitmap image.
	 */
    Bitmap loadAlphaBitmapInternal( String strFileName, byte red, byte green, byte blue )
        throws java.io.FileNotFoundException, GameFrameException
    {
	    DirectDrawSurface ddSurface = null;
        DirectDrawBitmap ddBitmap = loadDDBitmap( strFileName );

        ddSurface = createSurface( ddBitmap.width(), ddBitmap.height() );
        setBmpKeyColor( ddSurface, red, green, blue );
		ddSurface.copyBitmap( ddBitmap, 0, 0, 0, 0 );
        
        CDDAlphaBitmap bitmap = new CDDAlphaBitmap(
            m_directDraw, m_ddsBack, ddSurface,
            ddBitmap.width(), ddBitmap.height(),
            this, strFileName,
            red, green, blue );
        addBitmap(bitmap);
        return bitmap;
    }

    /**
     * Loads the given bitmap file as a surface.
     *@param strFileName The filename of the bitmap.
     *@return The DirectDrawSurface that contains the bitmaps.
     */
    DirectDrawBitmap loadDDBitmap( String strFileName )
        throws java.io.FileNotFoundException, GameFrameException
    {
		DirectDrawBitmap	ddBitmap;
        
		File file = new File(strFileName);
        if (!file.exists())
        {
            throw new FileNotFoundException(
                IConstants.BITMAP_FILE_NOT_FOUND1+
    		    strFileName+
                IConstants.BITMAP_FILE_NOT_FOUND2);
        }

		int length = (int) file.length();
		if (length <= 0)
		{
            throw new GameFrameException(
                IConstants.FILE_SIZE_IS_ZERO1+
                strFileName+
                IConstants.FILE_SIZE_IS_ZERO2);
		}

    	ddBitmap = new DirectDrawBitmap();
	    ddBitmap.filename(strFileName);	
    	ddBitmap.initWidth(0);
	    ddBitmap.initHeight(0);

		if( ddBitmap.loaded() == 0 )
		{
            throw new GameFrameException(
                IConstants.COULDNT_LOAD_BITMAP1+
    		    strFileName+
                IConstants.COULDNT_LOAD_BITMAP2 );
        }
        
        return ddBitmap;
    }
    
    /**
     * Creates a non-transparent bitmap (the color is set to the current background color) that
     * can be drawn into with other bitmaps.
     * If the creation of the bitmap fails this method returns null.
     *@param width The width of the bitmap.
     *@param height The height of the bitmap
     *@return The created bitmap or null if failed to create.
     */
    public DrawableBitmap getDrawableBitmap(int width, int height)
        throws GameFrameException
    {
        DirectDrawSurface ddSurface = createSurface(width, height);
        if (ddSurface == null)
        {
            throw new GameFrameException(
                CFullscreenGraphicsEngine.class.getName()+".getDrawableBitmap("+width+","+height+") "+IConstants.COULDNT_CREATE_DDSURFACE );
        }
        
        try
        {
            DirectDrawClipper clipper = m_directDraw.createClipper(0);
            Rect rect   = new Rect();
            rect.left   = 0;
            rect.top    = 0;
            rect.right  = width;
            rect.bottom = height;
            
            int[] aRectDimList = new int[4];
            aRectDimList[0]    = 0;
            aRectDimList[1]    = 0;
            aRectDimList[2]    = width;
            aRectDimList[3]    = height;
            
            try
            {
                clipper.setClipList( rect, aRectDimList, 4 );
            }
            catch(ComException ce1)
            {
                //Couldn't set the clipList
                throw new GameFrameException(
                    CFullscreenGraphicsEngine.class.getName()+".getDrawableBitmap("+width+","+height+") Couldn't set the cliplist" );
            }
            
            
            ddSurface.setClipper(clipper);
            ddSurface.bltColorFill(rect, m_backgroundFillValue);

        }
        catch(ComException ce2)
        {
            //Couldnt set the clipper
            throw new GameFrameException(
                CFullscreenGraphicsEngine.class.getName()+".getDrawableBitmap("+width+","+height+") Couldn't set the DirectDrawClipper." );
        }
        
        return new CDDBitmap( m_directDraw, m_ddsBack, ddSurface, width, height, this, null );
    }

    /**
     * Creates an alpha bitmap of the given size.
     *@param width The width of the alpha bitmap.
     *@param height The height of the alpha bitmap.
     *@return The created alpha bitmap.
     */
    private CDDAlphaBitmap createAlphaBitmap(int width, int height)
        throws GameFrameException
    {
        DirectDrawSurface ddSurface = createSurface(width, height);
        if (ddSurface == null)
        {
            throw new GameFrameException(
                CFullscreenGraphicsEngine.class.getName()+".createAlphaBitmap("+width+","+height+") "+IConstants.COULDNT_CREATE_DDSURFACE );
        }
        DDPixelFormat pixelFormat = new DDPixelFormat();
        ddSurface.getPixelFormat(pixelFormat);
        int keyColor;
        keyColor  = CPixelConverter.getPixelValue( m_backgroundColor.getRed(),   pixelFormat.redBitMask);
        keyColor |= CPixelConverter.getPixelValue( m_backgroundColor.getGreen(), pixelFormat.greenBitMask);
        keyColor |= CPixelConverter.getPixelValue( m_backgroundColor.getBlue(),  pixelFormat.blueBitMask);
        DDColorKey colorKey = new DDColorKey();
        colorKey.colorSpaceLowValue  = keyColor;
        colorKey.colorSpaceHighValue = colorKey.colorSpaceLowValue;
        ddSurface.setColorKey( DirectXConstants.DDCKEY_SRCBLT, colorKey );
        return new CDDAlphaBitmap(
            m_directDraw, m_ddsBack, ddSurface, width, height, this, null,
            (byte) m_backgroundColor.getRed(),
            (byte) m_backgroundColor.getGreen(),
            (byte) m_backgroundColor.getBlue() );
    }

    /**
     * First tries to create a plain DD surface that has the given size to the
     * video memory. If that is not successfull, creates the DD surface to other
     * memory (system or AGP).
     *@param width Width of the surface to be created.
     *@param height Height of the surface to be created.
     *@return DirectDraw surface that has the given size.
     */
    DirectDrawSurface createSurface(int width, int height)
        throws GameFrameException
    {
        // Limit the maximum surface size to the screen size
        if ( width > m_width )
        {
            width = m_width;
        }
        
        if ( height > m_height )
        {
            height = m_height;
        }

        // create a DirectDrawSurface for this bitmap
        // First try to create it in video memory
        DDSurfaceDesc ddSurfaceDescription;
        DirectDrawSurface ddSurface = null;
        
		ddSurfaceDescription         = new DDSurfaceDesc();
	    ddSurfaceDescription.flags   = DirectXConstants.DDSD_CAPS | DirectXConstants.DDSD_HEIGHT | DirectXConstants.DDSD_WIDTH;
	    ddSurfaceDescription.ddsCaps = DirectXConstants.DDSCAPS_OFFSCREENPLAIN | DirectXConstants.DDSCAPS_VIDEOMEMORY;
	    ddSurfaceDescription.width   = width;
	    ddSurfaceDescription.height  = height;
        
        boolean fInVideoMemory = true;
        
        try
        {
            ddSurface =  m_directDraw.createSurface( ddSurfaceDescription );
            
            if (ddSurface == null)
            {
                // Couldn't create in videomemory
                fInVideoMemory = false;
            }
        }
        catch(ComFailException notAbleToCreateInVideoMemory)
        {
            // Couldn't create in videomemory
            fInVideoMemory = false;
        }
        
        // Couldn't create in videomemory, try to create to system memory
        if (!fInVideoMemory)
        {
		    ddSurfaceDescription          = new DDSurfaceDesc();
	        ddSurfaceDescription.flags    = DirectXConstants.DDSD_CAPS | DirectXConstants.DDSD_HEIGHT | DirectXConstants.DDSD_WIDTH;
	        ddSurfaceDescription.ddsCaps  = DirectXConstants.DDSCAPS_OFFSCREENPLAIN | DirectXConstants.DDSCAPS_SYSTEMMEMORY;
	        ddSurfaceDescription.width    = width;
	        ddSurfaceDescription.height   = height;

            try
            {
                ddSurface =  m_directDraw.createSurface( ddSurfaceDescription );
            }
            catch(ComFailException cfe2)
            {
                throw new GameFrameException( IConstants.COULDNT_CREATE_DDSURFACE+" "+cfe2.toString() );;
            }
        }

        return ddSurface;
    }
    
    /**
     * Adds a bitmap to the group of created bitmaps (that is used when finalizing
     * all objects).
     */
    public void addBitmap(CBitmapBase bitmap)
    {
        m_loadedBitmaps.addElement(bitmap);
    }
    
    /**
     * Removes a bitmap from the group of created bitmaps (that is used when finalizing
     * all objects).
     */
    public void removeBitmap(CBitmapBase bitmap)
    {
        m_loadedBitmaps.removeElement(bitmap);
    }

    /**
     * Moves by copying a rectangular area of the backbuffer by a distance
     * specified by dx and dy. From the point specified by x and y, this
     * method moves downwards and to the right. To move an area of the
     * component to the left or upwards, specify a negative value for dx or
     * dy. If a portion of the source rectangle lies outside the bounds of
     * the component moveArea will be unable to move the associated pixels.
     *@param x The x-coordinate of the source rectangle.
     *@param y The y-coordinate of the source rectangle.
     *@param width The width of the source rectangle.
     *@param height The height of the source rectangle.
     *@param dx The horizontal distance to copy the pixels.
     *@param dy The vertical distance to copy the pixels.
     */
    public void copyArea( int x, int y, int width, int height, int dx, int dy )
    {
        int dstX = x + dx;
        int dstY = y + dy;
        // Check the boundaries and return if they are invalid
        if ( ( x < 0 ) || ( y < 0 ) || ( x + width > m_width ) || ( y + height > m_height ) ||
             ( dstX < 0 ) || ( dstY < 0 ) || ( dstX + width > m_width ) || ( dstY + height > m_height ) )
        {
            return;
        }
        
        mStatic_srcRectangle.left    = x;
        mStatic_srcRectangle.top     = y;
        mStatic_srcRectangle.right   = x + width;
        mStatic_srcRectangle.bottom  = y + height;
        
        mStatic_dstRectangle.left    = dstX;
        mStatic_dstRectangle.top     = dstY;
        mStatic_dstRectangle.right   = dstX + width;
        mStatic_dstRectangle.bottom  = dstY + height;

        int code = m_ddsBack.blt( mStatic_dstRectangle,
                           m_ddsBack,
                           mStatic_srcRectangle,
                           DirectXConstants.DDBLT_WAIT );
        if ( code != 0 )
        {
            System.out.println("Copy Area Failed");
        }
    }
    
    /**
     * Switches the backbuffer and the currently displayed buffer.
     */
    public void flip()
    {
        // If this component is not focused, don't even bother trying to flip
        if ( ! m_fFocused )
        {
            return;
        }

        // Flip the back and front surfaces
		int numFails = 0;
		while(true)
		{
			try
			{
				m_ddsPrimary.flip(null, DirectXConstants.DDFLIP_WAIT);
				break;
			}
			catch(ComFailException cfe1)
			{
				// Restore any lost surfaces
			    if (m_ddsPrimary.isLost() != DirectXConstants.DD_OK)
			    {
	                try
    		        {
			            m_ddsPrimary.restore();
			        }
			        catch(ComFailException cfe2)
                    {
            			numFails++;
                    }
			    }
			    if (m_ddsBack.isLost() != DirectXConstants.DD_OK)
			    {
	                try
    		        {
			            m_ddsBack.restore();
			        }
			        catch(ComFailException cfe3)
                    {
            			numFails++;
                    }
			    }
			    numFails++;
			}
			
			if (numFails > 2)
			{
			    System.out.println("Too many fails in "+getClass().getName()+" while flipping surfaces. Exiting...");
				gameframe.GameFrame.exit(1);
			}
		}
    }

    /**
     * Sets the data directory wherefrom the bitmaps are loaded.
     * Defaults to the current directory.
     *@param strDirectoryName The name of the directory that contains the bitmaps.
     */
    public void setDataDirectory(String strDirectoryName)
        throws FileNotFoundException
    {
        // If there is a path separator, remove it
        if ( strDirectoryName.charAt( strDirectoryName.length() - 1 ) == File.separatorChar )
        {
            strDirectoryName = strDirectoryName.substring( 0, strDirectoryName.length() - 1 );
        }
        
        File file = new File( strDirectoryName );
        if ( ( !file.isDirectory() ) || ( !file.exists() ) )
        {
            throw new java.io.FileNotFoundException(
                IConstants.DATA_DIRECTORY_DOESNT_EXIST1+
                strDirectoryName+
                IConstants.DATA_DIRECTORY_DOESNT_EXIST2);
        }

        m_strDataDirectory = strDirectoryName + File.separator;
    }

    /**
     * Returns the data directory wherefrom the bitmaps are loaded.
     * The source directory defaults to the current directory.
     * 
     * Note: The returned path will have the path separator as the last character.
     * 
     *@see #loadBitmap(java.lang.String)
     *@see #loadAlphaBitmap(java.lang.String)
     *@return The name of the directory that wherefrom the bitmaps are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory()
    {
        return m_strDataDirectory;
    }
    
    /**
     * Paints the given surface with the current background color.
     *@param surface The surface to be painted with background color.
     */
    private void paintSurfaceWithBackgroundColor(DirectDrawSurface surface)
    {
        Rect rect = new Rect();
        DDSurfaceDesc desc = new DDSurfaceDesc();
        surface.getSurfaceDesc(desc);
        rect.left = 0;
        rect.top  = 0;
        rect.right = desc.width;
        rect.bottom = desc.height;
            
        surface.bltColorFill(rect, m_backgroundFillValue);
    }

    /**
     * Clears the backbuffer with the current backround color.
     */
    public void clearBackbuffer()
    {
        m_ddsBack.bltColorFill(m_screenRectangle, m_backgroundFillValue);
    }
    
    /**
     * Returns true as this engine is allways in fullscreen mode.
     *@return True as the engine is in fullscreen mode.
     */
    public boolean isInFullscreenMode()
    {
        return true;
    }

    /**
     * Loads the default font bitmap that is used when printing text on the screen.
     * Either this method or the setFontBitmap method must be called before using
     * any of the getAsString() or drawTextAt() methods.
     */
    public void loadDefaultFontBitmap()
        throws java.io.FileNotFoundException, GameFrameException
    {
        // Save the old data directory and set the datadirectory to nothing
        String oldDataDir = m_strDataDirectory;
        m_strDataDirectory   = "";
        
        String strDefaultFontBitmapFilename = GameFrame.getInstance().getDefaultFontBitmapFilename();
        if ( (strDefaultFontBitmapFilename != null) && (!strDefaultFontBitmapFilename.startsWith("null")) )
        {
            try
            {
                loadFontBitmap( strDefaultFontBitmapFilename );
            }
            finally
            {
                m_strDataDirectory = oldDataDir;
            }
        }
        else
        {
            throw new java.io.FileNotFoundException( IConstants.DEFAULT_FONT_BITMAP_DIRECTORY_NOT_FOUND );
        }

        m_strDataDirectory = oldDataDir;
    }

    /**
     * Loads and sets the used font set to be loaded from the given strFileName using the specified
     * fixed size for every letter. The strFileName must specify a window BMP formatted
     * bitmap that is then loaded as transparent (alpha) bitmap and is then divided in
     * pieces that represent one letter each piece. The order in the bitmap must use
     * the standard ASCII letter order (can be obtained by running the PrintFontOrder
     * class in the GameFrame.Util package) and it must contain 16 columns of letters
     * in 16 rows (that is a 16x16 matrix of letters = 256 letters in whole). If a
     * nonexisting bitmap or fontbitmap of invalid size is used, the method will do
     * nothing.
     *@param strFileName Name of the bitmap file that contains the letters.
     */
    public void loadFontBitmap(String strFileName)
        throws java.io.FileNotFoundException, GameFrameException
    {
        strFileName = m_strDataDirectory+strFileName+".bmp";
        // Check that the file exists
        File file = new File( strFileName );
        if ( !file.exists() )
        {
            throw new java.io.FileNotFoundException(
                IConstants.COULDNT_LOAD_FONTBITMAP_LETTERS1+
                strFileName+
                IConstants.COULDNT_LOAD_FONTBITMAP_LETTERS2);
        }

        // Check that the file is not zero length
		int length = (int) file.length();
		if ( length <= 0 )
		{
            throw new GameFrameException (
                IConstants.FILE_SIZE_IS_ZERO1+
                strFileName+
                IConstants.FILE_SIZE_IS_ZERO2);
		}

    	// Release the old font bitmap to recover the memory it was using
        if ( m_ddsFontBitmap != null )
        {
            ComLib.release( m_ddsFontBitmap );
            m_ddsFontBitmap = null;
            System.gc();
        }

        //Create and load the bitmap for the fonts
		DirectDrawBitmap bitmap = new DirectDrawBitmap();
	    bitmap.filename ( strFileName );	
    	bitmap.initWidth( 0 );
	    bitmap.initHeight( 0 );

		if( bitmap.loaded() == 0 )
		{
            throw new GameFrameException (
                IConstants.COULDNT_LOAD_BITMAP1+
                strFileName+
                IConstants.COULDNT_LOAD_BITMAP2);
        }
        
    	// Create a DirectDrawSurface for this bitmap
        DirectDrawSurface ddsFontBitmap;
		ddsFontBitmap = createSurface( bitmap.width(), bitmap.height() );
        
        
        BitmapData data = BitmapFile.loadBMP( strFileName, true );
        setBmpKeyColor( ddsFontBitmap,
            (byte) data.getKeyColorRed(),
            (byte) data.getKeyColorGreen(),
            (byte) data.getKeyColorBlue() );
		ddsFontBitmap.copyBitmap(bitmap, 0, 0, 0, 0);

        // Check that the size of the given bitmap is big enough
        if ((bitmap.width()/16 == 0) || (bitmap.height()/16 == 0))
        {
            throw new GameFrameException (
                IConstants.FONT_BITMAP_TOO_SMALL1+
                strFileName+
                IConstants.FONT_BITMAP_TOO_SMALL2);
        }

        m_ddsFontBitmap = ddsFontBitmap;
        m_letterWidth  = bitmap.width()/16;
        m_letterHeight = bitmap.height()/16;

        ComLib.release(bitmap);
        bitmap = null;
        System.gc();
    }
    
    /**
     * Returns the given one line of text's width when we are using the current font bitmap.
     * The method doesn't take into account possible clipping to current screen size.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param strText The text that is written using the current font.
     *@return The width of the given text.
     *@since GameFrame for Java 0.9.3
     */
    public int getTextWidth( String strText )
    {
        return m_letterWidth * strText.length();
    }

    /**
     * Returns the given one line of text's width when we are using the current font bitmap.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param aByteText The byte array containing the text to be measured.
     *@return The width of the given text.
     *@since GameFrame for Java 0.9.3
     */
    public int getTextWidth( byte[] aByteText )
    {
        return m_letterWidth * aByteText.length;
    }

    /**
     * Returns the current font width.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@since GameFrame for Java 0.9.3
     */
    public int getFontWidth()
    {
        return m_letterWidth;
    }
    
    /**
     * Returns the current font height.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@since GameFrame for Java 0.9.3
     */
    public int getFontHeight()
    {
        return m_letterHeight;
    }
    
    /**
     * Returns the given one line of text as an bitmap that is painted using
     * the current font.
     *@param strText The text that is written using the current font.
     *@return The text as bitmap or null if not successfull.
     */
    public DrawableBitmap getAsBitmap(String strText)
        throws GameFrameException
    {
        int width  = m_letterWidth;
        int height = m_letterHeight;
        int length = strText.length();
        CDDBitmap bitmap = (CDDBitmap) getDrawableBitmap(length*width, height);
        if  ( m_ddsFontBitmap == null )
        {
            throw new GameFrameException ( IConstants.FONT_BITMAP_NOT_SET );
        }

        byte[] aByteText = strText.getBytes();
        mStatic_dstRectangle.top     = 0;
        mStatic_dstRectangle.bottom  = height;
        for (int i = 0; i < length; i++)
        {
            mStatic_srcRectangle.left    = (aByteText[i]%16)*width;
            mStatic_srcRectangle.right   = mStatic_srcRectangle.left + width;
            mStatic_srcRectangle.top     = (aByteText[i]/16)*height;
            mStatic_srcRectangle.bottom  = mStatic_srcRectangle.top + height;

            mStatic_dstRectangle.left    = i*width;
            mStatic_dstRectangle.right   = mStatic_dstRectangle.left + width;
            bitmap.m_ddSurface.blt(mStatic_dstRectangle, m_ddsFontBitmap, mStatic_srcRectangle, DirectXConstants.DDBLT_WAIT | DirectXConstants.DDBLT_KEYSRC);
        }
        return bitmap;
    }
    
    /**
     * Returns the given one line of text as an bitmap that is painted using
     * the current font.
     *@param aByteText The byte array containing the text to be printed.
     *@return The text as bitmap or null if not successfull.
     */
    public DrawableBitmap getAsBitmap(byte[] aByteText)
        throws GameFrameException
    {
        int width  = m_letterWidth;
        int height = m_letterHeight;
        int length = aByteText.length;
        CDDBitmap bitmap = (CDDBitmap) getDrawableBitmap(length*width, height);
        if  ( m_ddsFontBitmap == null )
        {
            throw new GameFrameException ( IConstants.FONT_BITMAP_NOT_SET );
        }
        
        mStatic_dstRectangle.top     = 0;
        mStatic_dstRectangle.bottom  = height;
        for (int i = 0; i < length; i++)
        {
            mStatic_srcRectangle.left    = (aByteText[i]%16)*width;
            mStatic_srcRectangle.right   = mStatic_srcRectangle.left + width;
            mStatic_srcRectangle.top     = (aByteText[i]/16)*height;
            mStatic_srcRectangle.bottom  = mStatic_srcRectangle.top + height;

            mStatic_dstRectangle.left    = i*width;
            mStatic_dstRectangle.right   = mStatic_dstRectangle.left + width;
            bitmap.m_ddSurface.blt(mStatic_dstRectangle, m_ddsFontBitmap, mStatic_srcRectangle, DirectXConstants.DDBLT_WAIT | DirectXConstants.DDBLT_KEYSRC);
        }
        return bitmap;
    }

    /**
     * Prints the given one line of text to the screen so that the uppermost and
     * leftmost coordinate of the text is at the given location.
     *@param strText The text that is written using the current font.
     *@param x The leftmost x-coordinate for the text.
     *@param y The topmost y-coordinate for the text.
     */
    public void drawTextAt(String strText, int x, int y)
        throws GameFrameException
    {
        int width  = m_letterWidth;
        int height = m_letterHeight;
        int length = strText.length();
        if (m_ddsFontBitmap == null)
        {
            throw new GameFrameException ( IConstants.FONT_BITMAP_NOT_SET );
        }
        
        byte[] aByteText = strText.getBytes();
        mStatic_dstRectangle.top     = y;
        mStatic_dstRectangle.bottom  = y+height;
        for (int i = 0; i < length; i++)
        {
            mStatic_srcRectangle.left    = (aByteText[i]%16)*width;
            mStatic_srcRectangle.right   = mStatic_srcRectangle.left + width;
            mStatic_srcRectangle.top     = (aByteText[i]/16)*height;
            mStatic_srcRectangle.bottom  = mStatic_srcRectangle.top + height;

            mStatic_dstRectangle.left    = x+i*width;
            mStatic_dstRectangle.right   = mStatic_dstRectangle.left + width;
            m_ddsBack.blt(mStatic_dstRectangle, m_ddsFontBitmap, mStatic_srcRectangle, DirectXConstants.DDBLT_WAIT | DirectXConstants.DDBLT_KEYSRC);
        }
    }
    
    /**
     * Prints the given string to the screen so that the uppermost and leftmost coordinate
     * of the text is at the given location.
     *@param aByteText The byte array containing the text to be printed.
     *@param x The leftmost x-coordinate for the text.
     *@param y The topmost y-coordinate for the text.
     */
    public void drawTextAt(byte[] aByteText, int x, int y)
        throws GameFrameException
    {
        int width  = m_letterWidth;
        int height  = m_letterHeight;
        int length = aByteText.length;
        if (m_ddsFontBitmap == null)
        {
            throw new GameFrameException ( IConstants.FONT_BITMAP_NOT_SET );
        }
        
        mStatic_dstRectangle.top     = y;
        mStatic_dstRectangle.bottom  = y+height;
        for (int i = 0; i < length; i++)
        {
            byte byteLetter = aByteText[i];
            mStatic_srcRectangle.left    = (byteLetter%16) * width;
            mStatic_srcRectangle.right   = mStatic_srcRectangle.left + width;
            mStatic_srcRectangle.top     = (byteLetter/16)*height;
            mStatic_srcRectangle.bottom  = mStatic_srcRectangle.top + height;
            mStatic_dstRectangle.left    = x;
            x += width;
            mStatic_dstRectangle.right   = x;
            m_ddsBack.blt( mStatic_dstRectangle,
                           m_ddsFontBitmap,
                           mStatic_srcRectangle,
                           DirectXConstants.DDBLT_WAIT |
                           DirectXConstants.DDBLT_KEYSRC );
        }
    }

    /**
     * Sets the currently used background color (defaults to Color.black).
     */
    public void setBackgroundColor(java.awt.Color color)
    {
        m_backgroundColor     = color;
        
        try
        {
            m_backgroundFillValue = mapToPixelFormat(color, m_ddsBack);
        }
        catch(GameFrameException gfe)
        {
        }
    }

    /**
     * Returns the short name (max 8 characters) of the
     * implementation "DDraw".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "DDraw";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Graphics engine that uses Microsoft Java 3.x DirectDraw classes.";
	}

    /**
     * Returns a vector filled with the supported modes.
     *@return The supported modes.
     */
    Vector getSupportedModes()
    {
        CModeEnumerator enumerator = new CModeEnumerator(
            CFullscreenGraphicsEngine.DEFAULT_COLOURDEPTH,
            CFullscreenGraphicsEngine.NO_PREFERRED_REFRESHRATE );

        // If enumeration fails return null...
        if ( !enumerate( enumerator ) )
        {
            return null;
        }
        
        Vector aAvailableModes = enumerator.getAvailableModes();
        
        // If a correct video mode is not found exit...
        if ( aAvailableModes.size() == 0 )
        {
            return null;
        }
        
        Vector modes = new Vector(5,5);
        for ( Enumeration enum = aAvailableModes.elements(); enum.hasMoreElements(); )
        {
            com.ms.directX.DDSurfaceDesc testedMode = (com.ms.directX.DDSurfaceDesc) enum.nextElement();
            modes.addElement(""+testedMode.width+"x"+testedMode.height+
                             "x"+testedMode.rgbBitCount+"@"+testedMode.refreshRate);
        }
        
        return modes;
    }

    /**
     * Enumerates all available resolutions and all different refresh rates.
     */
    boolean enumerate(CModeEnumerator enumerator)
    {
        // Create a surface description that is used when enumerating the display modes
        DDSurfaceDesc ddsd = new DDSurfaceDesc();
                
        // Enumerate all available video modes
        try
        {
            m_directDraw.enumDisplayModes( DirectXConstants.DDEDM_REFRESHRATES, ddsd, null, enumerator );
        }
        catch(ComException ce)
        {
            System.out.println("Couldn't enumerate display modes (ComException)");
            return false;
        }
        
        return true;
    }

	/**
	 * Invoked when the program is exiting or user wants to get rid of this
	 * engine. Frees all COM stuff that needs to be freed.
	 */
    public void finalize()
    {
        if ( m_directDraw != null )
        {
            // Show the cursor
            com.ms.wfc.ui.Cursor.show();
            com.ms.wfc.ui.Cursor.setClip(null);

            // Finalize all loaded bitmaps
            if (m_loadedBitmaps.size() > 0)
            {
                for(Enumeration enum = m_loadedBitmaps.elements(); enum.hasMoreElements();)
                {
                    CBitmapBase bitmap = (CBitmapBase) enum.nextElement();
                    bitmap.finalize();
                }
                m_loadedBitmaps = null;
            }

            // Restore the display mode to what it was
            if ( m_fInfullscreenMode )
            {
                m_directDraw.restoreDisplayMode();
                m_fInfullscreenMode = false;

            }
            // Set the co-op level to normal as there is
            // no way to make the GC to destroy this window
            // before we might create a new one.
            try
            {
                m_directDraw.setCooperativeLevel(this, DirectXConstants.DDSCL_NORMAL);
            }
            catch(ComFailException cfe)
            {
            }

            // Release all COM objects
            if (m_ddPalette != null)
                ComLib.release(m_ddPalette);
            if (m_ddsPrimary != null)
                ComLib.release(m_ddsPrimary);
            if (m_directDraw != null)
                ComLib.release(m_directDraw);

            // "Destroy" this window
            if (isVisible())
            {
                setVisible(false);
                dispose();
            }

            // "Destroy" the host frame
            if (mStatic_parentFrame != null)
            {
                mStatic_parentFrame.setVisible(false);
                mStatic_parentFrame.dispose();
                mStatic_parentFrame = null;
            }

            // Set COM object references to null
            m_ddPalette         = null;
            m_ddsPrimary        = null;
            m_directDraw        = null;

            mStatic_srcRectangle = null;
            mStatic_dstRectangle = null;

        }
    }
    
    /**
     * Overrides the java.awt.Component's method to do nothing.
     */
    public void paint(Graphics g)
    {
    }
    
    /**
     * Overrides the java.awt.Component's method to do nothing.
     */
    public void update(Graphics g)
    {
    }

    /**
     * Returns the size of the display area that this engine uses. For fullscreen
     * modes it is the size of the fullscreen mode. For windowed modes the size is
     * size of the window.
     *@return The size of the display area.
     *@since GameFrame for Java 0.9.2
     */
    public java.awt.Dimension getSize()
    {
        return super.getSize();
    }

    /**
     * Overrides the java.awt.Window's setVisible() method to also
     * set the visibility of the frame that is the parent of this window.
     *@param fVisibility True if the window is to be visible, false if not.
     */
    public void setVisible(boolean fVisibility)
    {
        super.setVisible(fVisibility);
        mStatic_parentFrame.setVisible(fVisibility);
    }

    /**
     * Overrides the java.awt.Window's show() method to also
     * show the frame that is the parent of this window.
     */
    public void show()
    {
        super.show();
        mStatic_parentFrame.show();
    }

    /**
     * Sets the bitmap key color to the surface.
     *@param ddSurface The DirectDraw surface whose key color is to be set.
     *@param red The red key color component.
     *@param green The green key color component.
     *@param blue The blue key color component.
     */
    public void setBmpKeyColor ( DirectDrawSurface ddSurface, byte red, byte green, byte blue )
        throws GameFrameException
	{
        try
        {
            int keyColor = mapToPixelFormat( new Color( ( (int) red   ) & 0xFF,
                                                        ( (int) green ) & 0xFF,
                                                        ( (int) blue  ) & 0xFF ),
                                             ddSurface );
            DDColorKey colorKey          = new DDColorKey();
            colorKey.colorSpaceLowValue  = keyColor;
            colorKey.colorSpaceHighValue = colorKey.colorSpaceLowValue;
            ddSurface.setColorKey( DirectXConstants.DDCKEY_SRCBLT, colorKey );
        }
        catch( ComFailException error )
        {
            throw new GameFrameException( IConstants.COULDNT_SET_KEYCOLOR );
        }
	}

    /**
     * Maps the given color object to a pixel value that has the same
     * pixel format as the given DirectDraw surface. Returns 0xFFFFFFFF
     * in case of failure, but also when user wants to map a color
     * to such a value. User should note when errors occur...
     *@param color The color that is mapped.
     *@param surface The DirectDraw surface whose pixel format is used.
     *@return The given color mapped to a pixel value.
     */
    int mapToPixelFormat( Color color, DirectDrawSurface surface )
        throws GameFrameException
    {
        DDPixelFormat pixelFormat = new DDPixelFormat();
        surface.getPixelFormat( pixelFormat );
        
        int bpp = pixelFormat.rgbBitCount;
        switch( bpp )
        {
            case 4:
            case 8:
                // Indexed color model...
                {
                    DirectDrawPalette palette = surface.getPalette();
                    if (palette == null)
                    {
                        throw new GameFrameException( IConstants.NO_PALETTE_FOUND );
                    }
                    Color[] aColors = palette.getColorEntries(0, 0, bpp);
                    for (int i = 0; i < bpp; i++)
                    {
                        if (aColors[i].equals(color))
                        {
                            return i;
                        }
                    }
                        
                    // Here we could actually try to find the closest color
                    // that would apply, but its more easier to do it in C++
                    // implementation (the same way that DDX4 example determines
                    // the color key by using GDI), so it is left to do later on...
                    return 0xFFFFFFFF;
                }
            case 16:
            case 24:
            case 32:
                // Non-indexed color model...
                {
                    int pixelValue;
                    pixelValue  = CPixelConverter.getPixelValue(color.getRed(),   pixelFormat.redBitMask);
                    pixelValue |= CPixelConverter.getPixelValue(color.getGreen(), pixelFormat.greenBitMask);
                    pixelValue |= CPixelConverter.getPixelValue(color.getBlue(),  pixelFormat.blueBitMask);
                    return pixelValue;
                }
            default:
                throw new GameFrameException( IConstants.UNSUPPORTED_RGB_BIT_COUNT );    
        }
    }
}
