/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.msjava4x;

import gameframe.sound.*;

import com.ms.win32.*;
import java.io.*;
import com.ms.dll.*;

/**
 * A MIDI music clip. Uses Direct/J and Winmm.dll to play the MIDI file via MCI
 * commands.
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.2
 */
public class CMidiMusic implements Music, Runnable
{
    /** MCI Open parameters "structure". */
    private static MCI_OPEN_PARMS mStatic_mciOpenParms       = new MCI_OPEN_PARMS();
    /** MCI Play parameters "structure". */
    private static MCI_PLAY_PARMS mStatic_mciPlayParms       = new MCI_PLAY_PARMS();
    /** MCI Status parameters "structure". */
    private static MCI_STATUS_PARMS mStatic_mciStatusParms   = new MCI_STATUS_PARMS();
    /** MCI Stop parameters "structure". */
    private static MCI_GENERIC_PARMS mStatic_mciGenericParms = new MCI_GENERIC_PARMS();
    /** The device id when no device is open. */
    private static final int NO_DEVICE_ID = -1;
    /** The currently playing music. */
    private static CMidiMusic mStatic_currentlyPlayingMusic;
    
    /** The sound engine that created this object. */
    private CSoundEngine m_soundEngine;
    /** The MIDI file that this object represents. */
    private String m_strMidiFile;
    /** The device ID of the MCI MIDI device. If this is not NO_DEVICE_ID then a MCI device is open.*/
    private int mStatic_iDeviceId = NO_DEVICE_ID;
    /** True if the looper thread is running.*/
    private boolean m_fLoopActive = false;
    /** Flag that indicates if stop hasn't been called since playOnce or playLooped has been called. */
    private boolean m_fPlaying = false;
    /** Flag that indicates if looping is enabled or not. */
    private boolean m_fLooped = false;
    /** The thread that takes care of the looping. */
    private Thread m_loopThread;
    /** The component that uses the sound engine. */
    private java.awt.Component m_focusedComponent;

    /**
     * Constructs a MIDI music representation.
     *@param focusedComponent The component that uses this sound engine.
     *@param strMidiFile The MIDI file that this object represents.
     *@param soundEngine The sound engine that creates this object.
     */
    public CMidiMusic( java.awt.Component focusedComponent, String strMidiFile, CSoundEngine soundEngine)
    {
        m_strMidiFile       = strMidiFile;
        m_focusedComponent  = focusedComponent;
        m_soundEngine       = soundEngine;
    }

    /**
     * Plays the music once and then stops.
     */
    public void playOnce()
    {
        stopCurrentMusic();
        if ( m_strMidiFile != null )
        {
            mStatic_currentlyPlayingMusic = this;
        
            m_fLooped = false;
            if ( !openMCI() )
            {
                return;
            }
                
            if ( !playMCI() )
            {
                return;
            }
        }
    }

    /**
     * Plays the given MIDI file as looped.
     */
    public void playLooped()
    {
        stopCurrentMusic();
        if ( m_strMidiFile != null )
        {
            mStatic_currentlyPlayingMusic = this;

            m_loopThread = new Thread( this );
            m_loopThread.setDaemon( false );
            m_loopThread.start();
        }
    }

    /**
     * Stops playing of the MIDI file.
     */
    public void stop()
    {
        if ( m_fPlaying )
        {
            if ( m_fLoopActive )
            {
                stopLoopingAndWait();
            }
            else
            {
                stopMCI();
            }
        }
        
        closeMCI();
    }

    /**
     * Stops the currently playing music if such exists.
     */
    public static void stopCurrentMusic()
    {
        if ( mStatic_currentlyPlayingMusic != null )
        {
            mStatic_currentlyPlayingMusic.stop();
            mStatic_currentlyPlayingMusic = null;
        }
    }
    
    /**
     * Waits until the looping thread has died.
     */
    private void stopLoopingAndWait()
    {
        if ( m_fLooped )
        {
            m_fLooped = false;
            
            // Wait until playing is stopped...
            while ( m_fLoopActive )
            {
                try
                {
                    Thread.currentThread().sleep( 100 );
                }
                catch( InterruptedException interrupted )
                {
                }
            }
            
        }
    }
   
    /**
     * The run method for the thread that monitors the loop playing
     * and restarts the music when it has played once.
     */
    public void run()
    {
        int iReturn;
        int length;

        // Check that we still have a file to play
        if ( m_strMidiFile == null )
        {
            return;
        }
        
        // Open an MCI device
        if ( !openMCI() )
        {
            // Failed to open the device, return from this method...
            return;
        }

        // We are indeed looping
        m_fLooped  = true;
        m_fLoopActive = true;

        // While we are looping the music...
        while( m_fLooped )
        {
            // Try to play the music
            if ( !playMCI() )
            {
                // Music didn't play, close the device and return
                m_fLooped = false;
                stopMCI();
                closeMCI();
                m_fLoopActive = false;
                return;
            }
            
            // Update the length of the music
            mStatic_mciStatusParms.dwItem = win.MCI_STATUS_LENGTH;
            iReturn = Winmm.mciSendCommand(
                        mStatic_iDeviceId,
                        win.MCI_STATUS,
                        win.MCI_STATUS_ITEM | win.MCI_WAIT,
                        mStatic_mciStatusParms );
            if ( iReturn != 0 )
            {
                // Failed to get the length stop the music, close the MIDI device and return
                System.out.println( m_strMidiFile+":CMidiMusic.run():MIDI sequencer status query failed. Code:"+iReturn );
                System.out.println( "Desc:"+getErrorString( iReturn ) );
                m_fLooped = false;
                stopMCI();
                closeMCI();
                m_fLoopActive = false;
                return;
            }
            length = mStatic_mciStatusParms.dwReturn;

            // While the music hasn't reached its end...
            int position = 0;
            while ( ( position < length ) && m_fLooped )
            {
                // Take a short nap
                try
                {
                    Thread.currentThread().sleep(100);
                }
                catch( InterruptedException interrupted )
                {
                }

                // Update the music position
                mStatic_mciStatusParms.dwItem     = win.MCI_STATUS_POSITION;
                iReturn = Winmm.mciSendCommand(
                            mStatic_iDeviceId,
                            win.MCI_STATUS,
                            win.MCI_STATUS_ITEM,
                            mStatic_mciStatusParms );
                if ( iReturn != 0 )
                {
                    // Failed to get the position, stop the music, close the MIDI device and return
                    System.out.println(
                        m_strMidiFile+":CMidiMusic.run():MIDI sequencer status query failed. Code:"+iReturn );
                    System.out.println( "Desc:"+getErrorString( iReturn ) );
                    m_fLooped = false;
                    stopMCI();
                    closeMCI();
                    m_fLoopActive = false;
                    return;
                }

                position = mStatic_mciStatusParms.dwReturn;
            }            

            // Stop the playing and rewind to the beginning of the music
            // (we have to use the WAIT flag to be sure the device is
            //  stopped and rewound properly before next play command)
            iReturn = Winmm.mciSendCommand(
                        mStatic_iDeviceId,
                        win.MCI_SEEK,
                        win.MCI_SEEK_TO_START | win.MCI_WAIT,
                        null );
            if ( iReturn != 0 )
            {
                // Failed to stop and rewind, close the MIDI device and return
                System.out.println(
                    m_strMidiFile+":CMidiMusic.run():MIDI sequencer seek failed. Code:"+iReturn );
                System.out.println( "Desc:"+getErrorString( iReturn ) );
                m_fLooped = false;
                stopMCI();
                closeMCI();
                m_fLoopActive = false;
                return;
            }
        }

        // Someone stopped the loop, stop and close the MIDI device
        stopMCI();
        closeMCI();
        m_fLoopActive = false;
    }
    
    /**
     * Opens the MCI MIDI device.
     *@return True if successfull, false if failed to open.
     */
    synchronized private boolean openMCI()
    {
        // Check that no device is open allready
        if ( mStatic_iDeviceId !=  NO_DEVICE_ID )
        {
            // Something was open, close it
            closeMCI();
        }
        
        if ( m_strMidiFile == null )
        {
            return false;
        }

        // Open the device by specifying the device and filename.
        // MCI will attempt to choose the MIDI mapper as the output port.
        mStatic_mciOpenParms.lpstrDeviceType    = "sequencer";
        mStatic_mciOpenParms.lpstrElementName   = m_strMidiFile;
        int iReturn = Winmm.mciSendCommand(
            0,
            win.MCI_OPEN,
            win.MCI_OPEN_TYPE | win.MCI_OPEN_ELEMENT | win.MCI_WAIT,
            mStatic_mciOpenParms );
        if ( iReturn != 0 )
        {
            // Failed to open device. Don't close it, just return error.
            System.out.println(
                m_strMidiFile+":CMidiMusic.openMCI():MIDI sequencer open failed. Code:"+iReturn );
            System.out.println( "Desc:"+getErrorString( iReturn ) );
            return false;
        }
        
        // The device opened successfully, store the device ID.
        mStatic_iDeviceId = mStatic_mciOpenParms.wDeviceID;
        return true;
    }

    /**
     * Plays the MCI MIDI device.
     *@return True if successfull, false if failed to play.
     */
    synchronized private boolean playMCI()
    {
        if ( mStatic_iDeviceId != NO_DEVICE_ID )
        {
            if ( m_strMidiFile == null )
            {
                return false;
            }

            int iReturn = Winmm.mciSendCommand(
                mStatic_iDeviceId,
                win.MCI_PLAY,
                0,
                mStatic_mciPlayParms );
            if ( iReturn != 0 )
            {
                // Failed to play, close the MCI device
                System.out.println( m_strMidiFile+":MIDI sequencer play failed. Code:"+iReturn );
                System.out.println( "Desc:"+getErrorString( iReturn ) );
                closeMCI();
                return false;
            }
            
            m_fPlaying = true;
            return true;
        }

        return false;
    }
    
    
    /**
     * Stops the MCI MIDI device.
     */
    synchronized private void stopMCI()
    {
        if ( ( mStatic_iDeviceId != NO_DEVICE_ID ) && ( m_fPlaying ) )
        {
            int iReturn = Winmm.mciSendCommand(
                            mStatic_iDeviceId,
                            win.MCI_STOP,
                            win.MCI_WAIT,
                            mStatic_mciGenericParms );
            m_fPlaying = false;
            if ( iReturn != 0 )
            {
                System.out.println( m_strMidiFile+":CMidiMusic.stopMCI():MIDI sequencer stop failed. Code:"+iReturn );
                System.out.println( "Desc:"+getErrorString( iReturn ) );
            }
        }
    }
    
    /**
     * Deinitialises the MIDI MCI device.
     */
    synchronized private void closeMCI()
    {
        if ( mStatic_iDeviceId != NO_DEVICE_ID )
        {
            if ( m_fPlaying )
            {
                stopMCI();
            }

            int iReturn = Winmm.mciSendCommand(
                mStatic_iDeviceId,
                win.MCI_CLOSE,
                win.MCI_WAIT,
                0 );
            if ( iReturn != 0 )
            {
                System.out.println( m_strMidiFile+":CMidiMusic.closeMCI():MIDI sequencer close failed. Code:"+iReturn );
                System.out.println( "Desc:"+getErrorString( iReturn ) );
                return;
            }

            mStatic_iDeviceId = NO_DEVICE_ID;
        }
    }

    /**
     * Returns the error string that corresponds to the give error code.
     *@param iError The error code.
     *@return The error string corresponding to the code.
     */
    private static String getErrorString(int iError)
    {
        StringBuffer strBuffer = new StringBuffer(256);
        if ( Winmm.mciGetErrorString(iError, strBuffer, 256 ) )
        {
            return strBuffer.toString();
        }
        
        return "-";
    }
    
    /**
     * 
     * Time in TMSF format is expressed as a doubleword value with the least
     * significant byte containing tracks, the next least significant byte
     * containing minutes, the next least significant byte containing seconds,
     * and the most significant byte containing frames. 
     *@return The given values in packed TMSF format.
     */
    private static int MCI_MAKE_TMSF(byte track, byte minutes, byte seconds, byte frames)
    {
        return (int) ( track ) | ( minutes << 8) | ( ( seconds | (frames << 8) ) << 16 );
 
    }
    
    /**
     * Mimics the behaviour of Win32 macro LOWORD.
     *@return The low word of the given value.
     */
    private static short LOWORD( int iValue )
    {
        return ( (short) iValue );
    }
    
    /**
     * Returns true if play() or playLooped() has been invoked on the sound and
     * the stop() hasn't been called after that. This method doesn't reflect the fact
     * if the sound can be heard or not at the moment.
     * 
     *@since GameFrame for Java 0.9.2
     */
    public boolean isPlaying()
    {
        return m_fPlaying;
    }

    /**
     * Closes all MCI devices opened by this application.
     * Waits until devices are closed before returning.
     */
    public void finalize()
    {
        // Unregister ourselves from the music engine
        m_soundEngine.finalizationNotify( this );

        // Stop ourself if we are playing
        stop();
        
        // Release all possible object references
        m_strMidiFile      = null;
        m_focusedComponent = null;
        m_loopThread       = null;
        mStatic_iDeviceId  = NO_DEVICE_ID;

        m_soundEngine      = null;
    }
}
