/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.msjava4x;

/**
 * Implementation of an pixel converter that converts arrays containing bitmap
 * data from one pixel format to another.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
class CPixelConverter
{
    /** The alpha mask. */
    private int m_alphaMask         = 0;
    /** The red mask. */
    private int m_redMask           = 0;
    /** The green mask. */
    private int m_greenMask         = 0;
    /** The blue mask. */
    private int m_blueMask          = 0;
    
    /** The alpha mask beginning index. */
    private int m_alphaMaskBeginIdx = 0;
    /** The red mask beginning index. */
    private int m_redMaskBeginIdx   = 0;
    /** The green mask beginning index. */
    private int m_greenMaskBeginIdx = 0;
    /** The blue mask beginning index. */
    private int m_blueMaskBeginIdx  = 0;

    /** The alpha mask bit count. */
    private int m_numAlphaMaskBits  = 0;
    /** The red mask bit count. */
    private int m_numRedMaskBits    = 0;
    /** The green mask bit count. */
    private int m_numGreenMaskBits  = 0;
    /** The blue mask bit count. */
    private int m_numBlueMaskBits   = 0;
    
    /**
     * Constructs a pixel converter that converts between the given
     * format and the Java native 0xAARRGGBB format.
     *@param pixelFormat The DirectDraw pixel format.
     */
    public CPixelConverter( com.ms.directX.DDPixelFormat pixelFormat )
    {
        m_alphaMask         = pixelFormat.alphaBitMask;
        m_redMask           = pixelFormat.redBitMask;
        m_greenMask         = pixelFormat.greenBitMask;
        m_blueMask          = pixelFormat.blueBitMask;
        if ( m_alphaMask != 0 )
        {
            m_alphaMaskBeginIdx = getMaskBeginIdx( m_alphaMask );
            m_numAlphaMaskBits  = getMaskBits(     m_alphaMask );
        }
        if ( m_redMask != 0 )
        {
            m_redMaskBeginIdx   = getMaskBeginIdx( m_redMask );
            m_numRedMaskBits    = getMaskBits(     m_redMask );
        }
        if ( m_greenMask != 0 )
        {
            m_greenMaskBeginIdx = getMaskBeginIdx( m_greenMask );
            m_numGreenMaskBits  = getMaskBits(     m_greenMask );
        }
        if ( m_blueMask != 0 )
        {
            m_blueMaskBeginIdx  = getMaskBeginIdx( m_blueMask );
            m_numBlueMaskBits   = getMaskBits(     m_blueMask );
        }
                
        //System.out.println( "" );
        //System.out.println( "Alpha mask:"+Integer.toHexString( m_alphaMask ) + " begin:"+m_alphaMaskBeginIdx+" bits:"+m_numAlphaMaskBits );
        //System.out.println( "Red   mask:"+Integer.toHexString( m_redMask ) + " begin:"+m_redMaskBeginIdx+" bits:"+m_numRedMaskBits );
        //System.out.println( "Green mask:"+Integer.toHexString( m_greenMask ) + " begin:"+m_greenMaskBeginIdx+" bits:"+m_numGreenMaskBits );
        //System.out.println( "Blue  mask:"+Integer.toHexString( m_blueMask ) + " begin:"+m_blueMaskBeginIdx+" bits:"+m_numBlueMaskBits );
        //System.out.println( "" );
    }

    /**
     * Constructs a pixel converter that converts between the given
     * format and the Java native 0xAARRGGBB format.
     *@param alphaMask The alpha value mask.
     *@param redMask The red value mask.
     *@param greenMask The green value mask.
     *@param blueMask The blue value mask.
     */
    public CPixelConverter( int alphaMask, int redMask, int greenMask, int blueMask )
    {
        m_alphaMask = alphaMask;
        m_redMask   = redMask;
        m_greenMask = greenMask;
        m_blueMask  = blueMask;
        if ( alphaMask != 0 )
        {
            m_alphaMaskBeginIdx = getMaskBeginIdx( alphaMask );
            m_numAlphaMaskBits  = getMaskBits(     alphaMask );
        }
        if ( redMask != 0 )
        {
            m_redMaskBeginIdx   = getMaskBeginIdx( redMask );
            m_numRedMaskBits    = getMaskBits(     redMask );
        }
        if ( greenMask != 0 )
        {
            m_greenMaskBeginIdx = getMaskBeginIdx( greenMask );
            m_numGreenMaskBits  = getMaskBits(     greenMask );
        }
        if ( blueMask != 0 )
        {
            m_blueMaskBeginIdx  = getMaskBeginIdx( blueMask );
            m_numBlueMaskBits   = getMaskBits(     blueMask );
        }
    }
    
    /**
     * Returns the number of bits used to represent the red component.
     *@return Bits used for the red.
     */
    public int getNumRedBits()
    {
        return m_numRedMaskBits;
    }

    /**
     * Returns the number of bits used to represent the green component.
     *@return Bits used for the green.
     */
    public int getNumGreenBits()
    {
        return m_numGreenMaskBits;
    }
    
    /**
     * Returns the number of bits used to represent the blue component.
     *@return Bits used for the blue.
     */
    public int getNumBlueBits()
    {
        return m_numBlueMaskBits;
    }
    
    /**
     * Converts pixels in the pixel format given in the constructor
     * to Java native format.
     *@param aiPixels The pixels in original format, changed to Java native.
     */
    public void convertToJavaNative( int[] aiPixels )
    {
        int alpha;
        int red;
        int green;
        int blue;
        for ( int i = 0; i < aiPixels.length; i++ )
        {
            alpha = aiPixels[i] & m_alphaMask;
            red   = aiPixels[i] & m_redMask;
            green = aiPixels[i] & m_greenMask;
            blue  = aiPixels[i] & m_blueMask;

            // Rotate the value to be at the first 8 bits
            if ( m_alphaMaskBeginIdx > 0 )
            {
                alpha >>>= m_alphaMaskBeginIdx;
            }
            if ( m_redMaskBeginIdx > 0 )
            {
                red >>>= m_redMaskBeginIdx;
            }
            if ( m_greenMaskBeginIdx > 0 )
            {
                green >>>= m_greenMaskBeginIdx;
            }
            if ( m_blueMaskBeginIdx > 0 )
            {
                blue >>>= m_blueMaskBeginIdx;
            }
            
            // Scale the value to be in the range of 0x00-0xFF
            int numRotations = 8 - m_numAlphaMaskBits;
            if ( numRotations > 0 )
            {
                alpha <<= numRotations;
            }
            else if ( numRotations < 0 )
            {
                numRotations = -numRotations;
                alpha >>>= numRotations;
            }

            numRotations = 8 - m_numRedMaskBits;
            if ( numRotations > 0 )
            {
                red <<= numRotations;
            }
            else if ( numRotations < 0 )
            {
                numRotations = -numRotations;
                red >>>= numRotations;
            }

            numRotations = 8 - m_numGreenMaskBits;
            if ( numRotations > 0 )
            {
                green <<= numRotations;
            }
            else if ( numRotations < 0 )
            {
                numRotations = -numRotations;
                green >>>= numRotations;
            }

            numRotations = 8 - m_numBlueMaskBits;
            if ( numRotations > 0 )
            {
                blue <<= numRotations;
            }
            else if ( numRotations < 0 )
            {
                numRotations = -numRotations;
                blue >>>= numRotations;
            }
            
            aiPixels[i] = ( alpha << 24 ) |
                          ( red << 16 ) | 
                          ( green << 8 ) | 
                          ( blue );
        }
    }
    
    /**
     * Converts pixels in Java native format
     * to the pixel format given in the constructor.
     *@param aiPixels The pixels in Java native, changed to original format.
     */
    public void convertFromJavaNative( int[] aiPixels )
    {
        int alpha;
        int red;
        int green;
        int blue;
        for ( int i = 0; i < aiPixels.length; i++ )
        {
            alpha = ( aiPixels[i] & 0xFF000000 ) >>> 24;
            red   = ( aiPixels[i] & 0x00FF0000 ) >>> 16;
            green = ( aiPixels[i] & 0x0000FF00 ) >>> 8;
            blue  = ( aiPixels[i] & 0x000000FF );

            // Make the value use the same amount of bits as the mask uses
            int numRotations = 8 - m_numAlphaMaskBits;
            if (numRotations > 0)
            {
                alpha >>>= numRotations;
            }
            else if ( numRotations < 0 )
            {
                alpha <<= -numRotations;
            }
            
            numRotations = 8 - m_numRedMaskBits;
            if (numRotations > 0)
            {
                red >>>= numRotations;
            }
            else if ( numRotations < 0 )
            {
                red <<= -numRotations;
            }

            numRotations = 8 - m_numGreenMaskBits;
            if (numRotations > 0)
            {
                green >>>= numRotations;
            }
            else if ( numRotations < 0 )
            {
                green <<= -numRotations;
            }

            numRotations = 8 - m_numBlueMaskBits;
            if (numRotations > 0)
            {
                blue >>>= numRotations;
            }
            else if ( numRotations < 0 )
            {
                blue <<= -numRotations;
            }

            // Rotate the value to be at the same place that maskbits are
            alpha <<= m_alphaMaskBeginIdx;
            alpha &= m_alphaMask;
            red   <<= m_redMaskBeginIdx;
            red   &= m_redMask;
            green <<= m_greenMaskBeginIdx;
            green &= m_greenMask;
            blue  <<= m_blueMaskBeginIdx;
            blue  &= m_blueMask;
            
            aiPixels[i] = alpha | red | green | blue;
        }
    }

    /**
     * Calculates the beginning index of the given mask.
     *@param mask The bitmask whose beginning is calculated.
     *@return The beginning index of the given mask.
     */
    private final int getMaskBeginIdx( int mask )
    {
        int maskBeginIdx = 0;

        // Roll until we find the beginning of the mask
        while ( ( mask & 0x01) != 0x01 )
        {
            mask >>>= 1;
            maskBeginIdx++;
        }
        
        return maskBeginIdx;
    }
    
    /**
     * Calculates the number of bits in the given mask.
     *@param mask The bitmask whose bits are counted.
     *@return The number of bits in the mask.
     */
    private final int getMaskBits( int mask )
    {
        int maskBeginIdx = 0;
        int numMaskBits  = 0;

        // Roll until we find the beginning of the mask
        while ( ( mask & 0x01 ) != 0x01 )
        {
            mask >>>= 1;
            maskBeginIdx++;
        }
        
        // Roll and count the number of bits in the mask
        while ( ( mask & 0x01 ) == 0x01 )
        {
            mask >>>= 1;
            numMaskBits++;
        }
        
        return numMaskBits;
    }
    
    /**
     * Maps the given value (0x00-0xFF) to the given mask.
     *@param colorIntensity The value of a color intensity.
     *@param mask The mask for the specific color.
     *@return The byte value mapped to the mask value.
     */
    static final int getPixelValue( int colorIntensity, int mask )
    {
        int value        = colorIntensity & 0xFF;
        int maskBeginIdx = 0;
        int numMaskBits  = 0;
        int tempMask     = mask;

        // Analyze the mask
        while ( ( tempMask & 0x01) != 0x01 )
        {
            tempMask >>>= 1;
            maskBeginIdx++;
        }
        
        while ( (tempMask & 0x01) == 0x01 )
        {
            tempMask >>>= 1;
            numMaskBits++;
        }
        
        // Make the value use the same amount of bits as the mask uses
        int numRotations = 8 - numMaskBits;
        if (numRotations > 0)
        {
            value >>>= numRotations;
        }
        
        if (numRotations < 0)
        {
            numRotations = -numRotations;
            value <<= numRotations;
        }
        
        // Rotate the value to be at the same place that maskbits are
        value <<= maskBeginIdx;
        value &= mask;
        
        return value;
    }

    /**
     * Maps the given pixel value with the given mask to an
     * 8-bit color intensity value.
     *@param pixelValue The pixel.
     *@param mask The mask of the color component we want to extract
     *@return The color intensity.
     */
    static int getIntensity( int pixelValue, int mask )
    {
        int maskBeginIdx = 0;
        int numMaskBits  = 0;
        int tempMask     = mask;

        // Analyze the mask
        
        // Roll until we find the beginning of the mask
        while ( ( tempMask & 0x01) != 0x01 )
        {
            tempMask >>>= 1;
            maskBeginIdx++;
        }
        
        // Roll and count the number of bits in the mask
        while ( (tempMask & 0x01) == 0x01 )
        {
            tempMask >>>= 1;
            numMaskBits++;
        }
        
        
        // Take the value with the mask
        int intensity = pixelValue & mask;

        // Rotate the value to be at the first 8 bits
        if ( maskBeginIdx > 0 )
        {
            intensity >>>= maskBeginIdx;
        }

        // Scale the value to be in the range of 0x00-0xFF
        int numRotations = 8 - numMaskBits;
        if ( numRotations > 0 )
        {
            intensity <<= numRotations;
        }
        else if ( numRotations < 0 )
        {
            numRotations = -numRotations;
            intensity >>>= numRotations;
        }
        
        return intensity;
    }
}
