/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.engines.msjava4x;

import gameframe.*;
import gameframe.sound.*;
import gameframe.engines.*;

import java.io.*;
import java.awt.*;
import java.util.*;

import com.ms.com.*;
import com.ms.directX.*;

/**
 * Implementation of sound engine for Microsoft JVM's that support DirectX
 * interfacing.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
class CSoundEngine implements SoundEngine, Finalizable
{
    /** The DirectSound instance that is used to load the sounds. */
    private DirectSound m_dSound;
    /** All the sounds created through this sound engine. */
    private Vector m_createdSamples = new Vector(10,10);
    /** All the music files created through this sound engine. */
    private Vector m_createdMusics = new Vector(10,10);
    /** Name of the directory that contains loaded sounds. */
    private String m_strDataDirectory = "";
    /** The focused component that uses the sound engine. */
    private Component m_focusedComponent;

    /**
     * Constructs a DirectSound implementation of the sound engine.
     *@param component The component that is the focus monitored window for this sound engine.
     */
    public CSoundEngine( Component component ) throws Exception
    {
        m_dSound = new DirectSound();
        m_dSound.initialize( null );
        m_focusedComponent = component;

        // note: if there is no sound device available, this will throw an exception.
        try
        {
            m_dSound.setCooperativeLevel( component, com.ms.directX.DirectXConstants.DSSCL_NORMAL );
        }
        catch ( Exception error )
        {
            throw new GameFrameException( IConstants.NO_SOUNDCARD_FOUND + error );
        }
        
        // Give the DirectSound COM object some time to be initialized
        try
        {
            Thread.currentThread().sleep( 50 );
        }
        catch( InterruptedException ie )
        {
        }
    }
    
    /**
     * Releases the DirectSound COM objects and MIDI music objects.
     */
    public void finalize()
    {
        if ( m_createdMusics != null )
        {
            CMidiMusic.stopCurrentMusic();
            for ( Enumeration enum = m_createdMusics.elements(); enum.hasMoreElements(); )
            {
                CMidiMusic music = (CMidiMusic) enum.nextElement();
                music.finalize();
            }
            m_createdMusics.removeAllElements();
            m_createdMusics = null;
        }
        
        if ( m_dSound != null )
        {
            for (Enumeration enum = m_createdSamples.elements(); enum.hasMoreElements();)
            {
                Sample sound = (Sample) enum.nextElement();
                sound.finalize();
            }
            
            // Give the DirectSoundBuffer COM objects some time to be released
            try
            {
                Thread.currentThread().sleep( 50 );
            }
            catch( InterruptedException ie )
            {
            }

            m_createdSamples.removeAllElements();
            m_createdSamples = null;
            ComLib.release( m_dSound );

            // Give the DirectSound COM object some time to be released
            try
            {
                Thread.currentThread().sleep( 50 );
            }
            catch( InterruptedException ie )
            {
            }
            
            m_dSound = null;
        }
   }

    /**
     * Sets the new focused component if the application finalises the
     * old graphics engine and creates a new one.
     *@param component The component that owns this sound engine.
     */
    void setFocusedComponent( Component component )
    {
        m_focusedComponent = component;
        m_dSound.setCooperativeLevel( component, com.ms.directX.DirectXConstants.DSSCL_NORMAL );
    }
   

    /**
     * Removes the given MIDI music from the list of active MIDI musics.
     *@param music The MIDI music object to be removed.
     */   
    void finalizationNotify( CMidiMusic music )
    {
        m_createdMusics.removeElement( music );
    }

    /**
     * Returns the short name (max 8 characters) of the
     * implementation "DSound".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName()
	{
		return "DSound";
	}

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription()
	{
		return "Sound engine that uses Microsoft Java 3.x DirectSound classes.";
	}

   /**
     * Load a sample in WAV format from the given file and return an abstract
     * reference for the loaded sound.
     *@param fileName The name of the file to be loaded (without the file extension)
     *@return An unique Sample object that specifies the loaded sample
     */
    public Sample loadSample( String fileName )
        throws java.io.FileNotFoundException, GameFrameException
    {
        fileName = m_strDataDirectory+fileName+".wav";

        File file = new File(fileName);
        if ( !file.exists() )
        {
            throw new FileNotFoundException(
                IConstants.COULDNT_LOAD_WAV1+
                fileName+
                IConstants.COULDNT_LOAD_WAV2);
        }
            
        int length = (int) file.length();
        if (length <= 0)
        {
            throw new GameFrameException(
                IConstants.FILE_SIZE_IS_ZERO1+
                fileName+
                IConstants.FILE_SIZE_IS_ZERO2);
        }
        
        try 
        {
            DSResourceDesc dsrDesc    = new DSResourceDesc();
            DirectSoundResource dsr   = new DirectSoundResource();
            DSBufferDesc dsBufferDesc = new DSBufferDesc();

            // this guy returns us the size of the wave file in 
            // dwBufferBytes
            WaveFormatEx waveFormat   = dsr.loadWaveFile (fileName, dsrDesc);

            // need to specify DSBCAPS_CTRLDEFAULT so you can control the
            // volume, pan, frequency.
            dsBufferDesc.flags        = DirectXConstants.DSBCAPS_STATIC | DirectXConstants.DSBCAPS_CTRLDEFAULT;
            dsBufferDesc.bufferBytes  = dsrDesc.bufferBytes;

            DirectSoundBuffer   dsBuffer = m_dSound.createSoundBuffer (dsBufferDesc, waveFormat);
               
            // get the sound data from the internal memory from LoadWaveFile
            //  to the sound buffer.  The flags are the same as those used 
            //  for Lock()
            dsBuffer.transferToSoundBuffer (dsrDesc, 0);
                     
            CDXSound sound = new CDXSound(dsBuffer);
            m_createdSamples.addElement(sound);
            return sound;
        }
        catch (Exception anyException)
        {
            throw new GameFrameException(
                IConstants.COULDNT_LOAD_WAV1+
                fileName+
                IConstants.COULDNT_LOAD_WAV2+" "+ anyException);
        }
    }

    /**
     * First tries to load a music file as streamed Microsoft RIFF WAV file,
     * if no such file is found from the current data directory, tries to load
     * a music file in MIDI format. When suitable file is found returns a
     * control object for the music file if no suitable file is found returns
     * NullSound object. Music files are loaded by default from the
     * current directory, the directory that is used to load the samples
     * can be set with the setDataDirectory() method. 
     * 
     *@see #setDataDirectory(java.lang.String)
     *@param fileName The name of the file to be loaded (without the file extension)
     *@return An unique Sample object that represents the loaded sample
     *@since GameFrame for Java 0.9.2
     */
    public Music loadMusic( String fileName )
        throws FileNotFoundException, GameFrameException
    {
        fileName = m_strDataDirectory+fileName+".mid";
        File file = new File(fileName);
        if (!file.exists())
        {
            throw new FileNotFoundException(
                IConstants.COULDNT_LOAD_WAV1+
                fileName+
                IConstants.COULDNT_LOAD_WAV2);
        }
            
        int length = (int) file.length();
        if (length <= 0)
        {
            throw new GameFrameException(
                IConstants.FILE_SIZE_IS_ZERO1+
                fileName+
                IConstants.FILE_SIZE_IS_ZERO2);
        }
        
        CMidiMusic music = new CMidiMusic( m_focusedComponent, fileName, this );
        m_createdMusics.addElement(music);
        return music;
    }

    /**
     * Sets the data directory wherefrom the sounds are loaded.
     * Defaults to the current directory. Do not add the last directory separator.
     *@param strDirectoryName The name of the directory that contains the sounds.
     */
    public void setDataDirectory(String strDirectoryName)
        throws FileNotFoundException
    {
        // If there is a path separator, remove it
        if ( strDirectoryName.charAt( strDirectoryName.length() - 1 ) == File.separatorChar )
        {
            strDirectoryName = strDirectoryName.substring( 0, strDirectoryName.length() - 1 );
        }

        File file = new File(strDirectoryName);
        if ( ( !file.isDirectory() ) || ( !file.exists() ) )
        {
            throw new java.io.FileNotFoundException(
                IConstants.DATA_DIRECTORY_DOESNT_EXIST1+
                strDirectoryName+
                IConstants.DATA_DIRECTORY_DOESNT_EXIST2);
        }

        m_strDataDirectory = strDirectoryName + File.separator;
    }
    
    /**
     * Returns the data directory wherefrom the sounds are loaded.
     * The source directory defaults to the current directory.
     * 
     * Note: The returned path will have the path separator as the last character.
     * 
     *@see #loadSample(java.lang.String)
     *@see #loadMusic(java.lang.String)
     *@return The name of the directory that wherefrom the sounds are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory()
    {
        return m_strDataDirectory;
    }

    /**
     * DirectSound implementation of the sound object.
     */
    class CDXSound implements Sample
    {
        /** DirectSound buffer that contains the sample data. */
        private DirectSoundBuffer m_dsBuffer;
        /** Flag that indicates if the playing has been started. */
        private boolean m_fPlaying = false;
        /** The pan of the sound. */
        private float m_pan;
        /** The volume of the sound. */
        private float m_volume;

        /**
         * Constructs a DirectSound implementation of the sound object.
         *@param dsBuffer DirectSound buffer that contains the sample data.
         */
        CDXSound(DirectSoundBuffer dsBuffer)
        {
            m_dsBuffer = dsBuffer;
        }
        
        /**
         * Plays this sample once and then stops. If the sample is
         * already playing then it is stopped and restarted.
         */
        public void playOnce()
        {
            try 
            {
                m_dsBuffer.setCurrentPosition(0);
                m_dsBuffer.play (0);
                m_fPlaying = true;
            }
            catch (Exception e)
            {
            }
        }
        
        /**
         * Returns true if start has been invoked and stop hasn't yet been invoked.
         *@return True if the sound should be playing.
         */
        public boolean isPlaying()
        {
            return m_fPlaying;
        }
        
        /**
         * Plays the given sample over and over again (looping).
         * If the sample is already playing then it is stopped and
         * restarted.
         */
        public void playLooped()
        {
            try 
            {
                m_dsBuffer.setCurrentPosition(0);
                m_dsBuffer.play (DirectXConstants.DSBPLAY_LOOPING);
                m_fPlaying = true;
            }
            catch (Exception e)
            {
            }
        }
        
        /**
         * Stops playing the given sample
         */
        public void stop()
        {
            try
            {
                m_dsBuffer.stop();
                m_fPlaying = false;
            }
            catch (Exception e)
            {
            }
        }
        /**
         * Sets the volume of the sound. Value of 0.0 means that the sound should be
         * inaudible and value of 1.0 means the sound is played at its normal volume.
         * Values above 1.0 are treated as 1.0 and values below 0.0 are treated as 0.0.
         *@param volume The sounds volume.
         */
        public void setVolume( float volume )
        {
            m_volume = volume;
            if (volume > 1.0F)
            {
                volume = 1.0F;
            }
            
            if (volume < 0.0F)
            {
                volume = 0.0F;
            }
            
            int dsVolume = - (int) ( (1.0F - volume) * 10000.0F );
            m_dsBuffer.setVolume( dsVolume );
        }

        /**
         * Sets the pan of the sound. Value of -1.0 means that the sound will be
         * heard only from the left speaker and value of 1.0 means that the sound
         * will be heard only from the right speaker. Value of 0.0 means that
         * the sound is heard with equal volume from both speakers.
         * Values above 1.0 are treated as 1.0 and values below -1.0 are treated as -1.0.
         *@param pan The pan of the sound.
         */
        public void setPan( float pan )
        {
            m_pan = pan;
            
            if ( pan > 1.0F )
            {
                pan = 1.0F;
            }
            
            if ( pan < -1.0F )
            {
                pan = -1.0F;
            }
            
            int dsPan = (int) ( pan * 10000.0F );
            m_dsBuffer.setPan( dsPan );
        }

        /**
         * Returns the volume of the sound. Value of 0.0 means that the sound should be
         * inaudible and value of 1.0 means the sound is played at its normal volume.
         *@return The sounds volume.
         */
        public float getVolume()
        {
            return m_volume;
        }
    
        /**
         * Returns the pan of the sound. Value of -1.0 means that the sound will be
         * heard only from the left speaker and value of 1.0 means that the sound
         * will be heard only from the right speaker. Value of 0.0 means that
         * the sound is heard with equal volume from both speakers.
         *@return The pan of the sound.
         */
        public float getPan()
        {
            return m_pan;
        }
        
        /**
         * Returns a copy of this sound object that is independent from the
         * original sound object. This new sound object can be started/stopped
         * independently from the original sound object. Using this method to
         * get multiple instances of one sound will save memory on some
         * implementations and is thus preferred way to duplicate sound objects.
         *@return A independent copy of this sound or null if copy fails.
         */
        public Sample getCopy()
        {
            CDXSound sound = new CDXSound(m_dSound.duplicateSoundBuffer(m_dsBuffer));
            m_createdSamples.addElement(sound);
            return sound;
        }

        public void finalize()
        {
            if (m_dsBuffer != null)
            {
                m_dsBuffer.stop();
                m_createdSamples.removeElement(this);
                ComLib.release( m_dsBuffer );
                m_dsBuffer = null;
            }
        }
    }
}
