/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

import gameframe.*;

/**
 * Implementation of an animated bitmap. Has multiple animation modes
 * like ping-pong, one shot or looping. It implements the Bitmap interface
 * and adds methods for handling the animation.
 *
 *@author Pasi Kernen 04. Apr 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class AnimBitmap implements Bitmap
{
    /**
     * Enumerated animation mode. Plays the animation over and over again
     * starting from the first and looping back to first when it reaches
     * the last animation cell.
     */
    public static final int MODE_LOOP = 0;
    /**
     * Enumerated animation mode. Plays the animation once
     * and stops at the last animation cell.
     */
    public static final int MODE_ONCE = 1;
    /**
     * Enumerated animation mode. Plays the animation once
     * from first animation cell to the last and then reverses the
     * direction and plays the animation from last to first and reverses
     * the direction again.
     */
    public static final int MODE_PING_PONG = 2;

    /** Enumerated direction of the animation. Reverse playing sequence. */
    public static final int DIR_REVERSE = -1;
    /** Enumerated direction of the animation. Stopped sequence. */
    public static final int DIR_STOPPED = 0;
    /** Enumerated direction of the animation. Forward playing sequence. */
    public static final int DIR_FORWARD = 1;

    /** The current animation mode. Defaults to MODE_LOOP. */
    private int m_mode = MODE_LOOP;
    /** The current animation frames. */
    private Bitmap[] m_aBitmaps;
    /** The floating point index of the currently shown animation cell. */
    private float m_flAnimIdx = 0.0F;
    /** The integer index of the currently shown animation cell. */
    private int m_iAnimIdx = 0;
    /** The maximum index for the currently shown animation. */
    private int m_maxIdx;
    /** The the current animation mode. */
    private int m_eMode = 0;
    /** The direction of the animation. Defaults to DIR_FORWARD */
    private int m_eDirection = DIR_FORWARD;
    /** The previous direction of the animation. Defaults to DIR_STOPPED */
    private int m_eOldDirection = DIR_FORWARD;

    /**
     * Constructs an animated bitmap from the series of Bitmap objects
     * given.
     *@param bitmaps The array of bitmaps to use.
     *@throws GameFrameException when something goes wrong. Note: In this case
     *        the animated bitmap is not initialized and is not usable.
     */
    public AnimBitmap( Bitmap[] bitmaps )
        throws GameFrameException
    {
        m_aBitmaps = bitmaps;

        m_maxIdx = m_aBitmaps.length - 1;
        if ( m_maxIdx <= 0 )
        {
            throw new GameFrameException(
                "Too few animation cells ("+m_aBitmaps.length+
                ") at least two animation cells must be given" );
        }
    }

    /**
     * Constructs an animated bitmap from the given Bitmap that contains
     * the animation cells. Initially gets the subbitmaps from the given
     * bitmap object and uses those subbitmaps to create the animation.
     *@param tiledBitmap The bitmap containing the animation cells to use.
     *@param cellWidth The width of one animation cell in the tiled bitmap.
     *@param cellHeight The height of one animation cell in the tiled bitmap.
     *@throws GameFrameException when something goes wrong. Note: In this case
     *        the animated bitmap is not initialized and is not usable.
     */
    public AnimBitmap( Bitmap tiledBitmap, int cellWidth, int cellHeight )
        throws GameFrameException
    {
        int bitmapWidth  = tiledBitmap.getWidth();
        int bitmapHeight = tiledBitmap.getHeight();
        int numXCells    = bitmapWidth  / cellWidth;
        int numYCells    = bitmapHeight / cellHeight;

        // Check the sanity of the given parameters
        if ( ( numXCells == 0 ) || ( numYCells == 0 ) )
        {
            throw new GameFrameException(
                "Given animated bitmap is smaller than the animation cell." );
        }

        // Get the subbitmaps from the given tiled bitmap
        m_aBitmaps = new Bitmap[ numXCells * numYCells ];
        for ( int idxY = 0; idxY < numYCells; idxY++ )
        {
            for ( int idxX = 0; idxX < numXCells; idxX++ )
            {
                m_aBitmaps[idxX + idxY * numXCells] =
                    tiledBitmap.getSubBitmapClone(
                        idxX*cellWidth, idxY*cellHeight,
                        cellWidth, cellHeight );
            }
        }

        m_maxIdx = m_aBitmaps.length - 1;
        if ( m_maxIdx <= 0 )
        {
            throw new GameFrameException(
                "Too few animation cells ("+m_aBitmaps.length+
                ") at least two animation cells must be given" );
        }
    }

    /**
     * Sets the animation mode of this animated bitmap.
     *@param eMode An enumerated value of MODE_xxx type.
     */
    public void setMode( int eMode )
    {
        m_eMode = eMode;
    }

    /**
     * Returns the animation mode of this animated bitmap.
     *@return An enumerated value of MODE_xxx type.
     */
    public int getMode()
    {
        return m_eMode;
    }

    /**
     * Returns a copy of the used bitmaps in the animation.
     *@return Copy of used bitmaps.
     *@exception GameFrameException thrown if cloning isn't successfull.
     */
    public Bitmap[] getBitmapClones() throws GameFrameException
    {
        Bitmap[] aBitmaps = new Bitmap[m_aBitmaps.length];
        for ( int i = 0; i < aBitmaps.length; i++ )
        {
            aBitmaps[i] = m_aBitmaps[i].getClone( null );
        }
        return aBitmaps;
    }

    /**
     * Stops the animation. Even if the nextFrame() or
     * advanceFrame() methods are called the animation
     * doesn't advance.
     */
    public void stop()
    {
        m_eDirection = DIR_STOPPED;
    }
    
    /**
     * Tells the animation to play forward (from first
     * animation cell to the last). Note: In ping-pong
     * mode this method has a limited meaning as ping-
     * pong mode changes the direction of the animation
     * constantly.
     */
    public void playForward()
    {
        m_eDirection = DIR_FORWARD;
    }
    
    /**
     * Tells the animation to play backwards (from last
     * animation cell to the first). Note: In ping-pong
     * mode this method has a limited meaning as ping-
     * pong mode changes the direction of the animation
     * constantly.
     */
    public void playReverse()
    {
        m_eDirection = DIR_REVERSE;
    }
    
    /**
     * Returns the animation direction of this animated bitmap.
     *@return An enumerated value of DIR_xxx type.
     */
    public int getDirection()
    {
        return m_eDirection;
    }

    /**
     * Advances the animation one frame to the direction it is
     * configured to go.
     */
    public void nextFrame()
    {
        m_iAnimIdx += m_eDirection;
        m_flAnimIdx = (float) m_iAnimIdx;

        switch( m_eMode )
        {
            case MODE_LOOP:
                if ( m_iAnimIdx > m_maxIdx )
                {
                    m_iAnimIdx = 0;
                }

                if ( m_iAnimIdx < 0 )
                {
                    m_iAnimIdx = m_maxIdx;
                }

                break;
            case MODE_ONCE:
                if ( m_iAnimIdx > m_maxIdx )
                {
                    m_iAnimIdx    = m_maxIdx;
                    m_eOldDirection = m_eDirection;
                    m_eDirection = DIR_STOPPED;
                }

                if ( m_iAnimIdx < 0 )
                {
                    m_iAnimIdx    = 0;
                    m_eOldDirection = m_eDirection;
                    m_eDirection = DIR_STOPPED;
                }
                break;
            case MODE_PING_PONG:
                if ( m_iAnimIdx > m_maxIdx )
                {
                    m_iAnimIdx    = m_aBitmaps.length - 2;
                    m_eDirection = -m_eDirection;
                }

                if ( m_iAnimIdx < 0 )
                {
                    m_iAnimIdx    = 1;
                    m_eDirection = -m_eDirection;
                }
                break;
        }
    }

    /**
     * Advances the animation the number of frames given to the direction it is
     * configured to go. You should use the advanceFrames( float ) method instead
     * of this method as this method simply casts the given integer to a float in
     * invokes that version of the method.
     *@see #advanceFrames(float)
     *@param iNumFrames Positive number of frames to advance.
     *@deprecated Use advanceFrames( float ) instead of the advanceFrames( int ) version.
     */
    public void advanceFrames( int iNumFrames )
    {
        advanceFrames( (float) iNumFrames );
    }
    
    /**
     * Advances the animation the number of frames given to the direction it is
     * configured to go. The frame number is a floating point number so that
     * the animation can be advanced with e.g. 0.5 or 0.2 frames if needed
     *@param flNumFrames Positive number of frames to advance.
     */
    public void advanceFrames( float flNumFrames )
    {
        int iNumFrames = (int) flNumFrames;
        m_flAnimIdx   += ( ( (float) m_eDirection ) * flNumFrames );
        m_iAnimIdx     = (int) m_flAnimIdx;

        switch( m_eMode )
        {
            case MODE_LOOP:
                if ( m_iAnimIdx > m_maxIdx )
                {
                    m_iAnimIdx  = iNumFrames % m_maxIdx;
                    m_flAnimIdx = (float) m_iAnimIdx;
                }

                if ( m_iAnimIdx < 0 )
                {
                    m_iAnimIdx  = m_maxIdx - ( iNumFrames % m_maxIdx );
                    m_flAnimIdx  = (float) m_iAnimIdx;
                }

                break;
            case MODE_ONCE:
                if ( m_iAnimIdx > m_maxIdx )
                {
                    m_iAnimIdx   = m_maxIdx;
                    m_flAnimIdx  = (float) m_iAnimIdx;
                    m_eDirection = DIR_STOPPED;
                }

                if ( m_iAnimIdx < 0 )
                {
                    m_eDirection = DIR_STOPPED;
                    m_iAnimIdx   = 0;
                    m_flAnimIdx  = 0.0F;
                }
                break;
            case MODE_PING_PONG:
                if ( m_iAnimIdx > m_maxIdx )
                {
                    m_iAnimIdx   = m_maxIdx - ( iNumFrames % m_maxIdx );
                    m_flAnimIdx  = (float) m_iAnimIdx;
                    m_eDirection = -m_eDirection;
                }

                if ( m_iAnimIdx < 0 )
                {
                    m_iAnimIdx   = - ( iNumFrames % m_maxIdx );
                    m_flAnimIdx  = (float) m_iAnimIdx;
                    m_eDirection = -m_eDirection;
                }
                break;
        }
    }



// THE METHODS OF BITMAP INTERFACE


    /**
     * Draws the currently shown bitmap to the graphics engine that loaded it at the given
     * location. The location can be outside of the visible screen area in
     * which case the currently shown bitmap is clipped accordingly. This method is fast and
     * is usable during the game execution.
     *
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@since GameFrame for Java 0.9
     */
    public void blitTo( int x, int y )
    {
        m_aBitmaps[ m_iAnimIdx ].blitTo( x, y );
    }

    /**
     * Draws the currently shown bitmap to the graphics engine that loaded it at the given
     * location and at the given size. If currently shown bitmap is larger or smaller than
     * the destination size the currently shown bitmap will automatically be scaled. Note: This
     * method is usually a bit slow, so it's use is not preferred during
     * interactive periods in game execution.
     *
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with that is used when drawing the image.
     *@param height The height that is used when drawing the image.
     *@since GameFrame for Java 0.9
     */
    public void strecthTo( int x, int y, int width, int height )
    {
        m_aBitmaps[ m_iAnimIdx ].strecthTo( x, y, width, height );
    }

    /**
     * Draws the currently shown bitmap to the given drawable bitmap at the given location.
     * The location can be outside of the visible screen area in which case the
     * bitmap is clipped accordingly. This method is fast and is usable during
     * the game execution.
     *
     *@param bitmap The bitmap that will be drawn to.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@since GameFrame for Java 0.9
     */
    public void blitTo( DrawableBitmap bitmap, int x, int y )
    {
        m_aBitmaps[ m_iAnimIdx ].blitTo( bitmap, x, y );
    }

    /**
     * Draws the currently shown bitmap to the given drawable bitmap at the given location
     * at the given size. If currently shown bitmap is larger or smaller than the
     * destination size the currently shown bitmap will automatically be scaled. Note: This
     * method is usually quite slow, so it's use is not preferred during
     * interactive periods in game execution.
     *
     *@param bitmap The bitmap that will be drawn to.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with that is used when drawing the image.
     *@param height The height that is used when drawing the image.
     *@since GameFrame for Java 0.9
     */
    public void strecthTo(
        DrawableBitmap bitmap, int x, int y, int width, int height )
    {
        m_aBitmaps[ m_iAnimIdx ].strecthTo( bitmap, x, y, width, height );
    }

    /**
     * Returns the width of the currently shown bitmap.
     * This method is fast and is usable during the game execution.
     *
     *@return Width of the currently shown bitmap if available.
     *@since GameFrame for Java 0.9.3
     */
    public int getWidth()
    {
        return m_aBitmaps[ m_iAnimIdx ].getWidth();
    }


    /**
     * Returns the height of the currently shown bitmap.
     * This method is fast and is usable during the game execution.
     *
     *@return Height of the currently shown bitmap if available.
     *@since GameFrame for Java 0.9.3
     *@see Bitmap#getHeight()
     */
    public int getHeight()
    {
        return m_aBitmaps[ m_iAnimIdx ].getHeight();
    }

    /**
     * <P>Returns an effect processed copy of the currently shown bitmap.
     * This operation copies the actual bitmap data to a new memory location
     * in a processed form and thus consumes memory.</P>
     *
     * <P>Usually used to save graphics drawing time on game graphics as one
     * can generate an animation sequence on the fly using gradual effects.</P>
     *@param effect The bitmap effect to be applied to the clone bitmap,
     *       or null if none are to be applied.
     *@return An effect processed clone of the currently shown bitmap.
     *@exception GameFrameException thrown if any errors occur.
     *@see Bitmap#getHeight()
     */
    public Bitmap getClone( BitmapEffect effect )
        throws GameFrameException
    {
        return m_aBitmaps[ m_iAnimIdx ].getClone( effect );
    }
    /**
     * Returns the subbitmap clone of the current animation frame. Works as the
     * getSubBitmapClone() in Bitmap.
     *
     *@param srcX The leftmost coordinate of the currently shown bitmap to be returned as a new bitmap.
     *@param srcY The topmost coordinate of the currently shown bitmap to be returned as a new bitmap.
     *@param srcWidth The width of the currently shown bitmap to be returned as a new bitmap.
     *@param srcHeight The height of the currently shown bitmap to be returned as a new bitmap.
     *@return A new bitmap representing the requested part of the original bitmap.
     *@exception GameFrameException Thrown if clone sub-bitmap creation is not successfull.
     *@see Bitmap#getSubBitmapClone(int, int, int, int)
     */
    public Bitmap getSubBitmapClone( int srcX, int srcY, int srcWidth, int srcHeight )
        throws GameFrameException
    {
        return m_aBitmaps[ m_iAnimIdx ].getSubBitmapClone(
            srcX, srcY, srcWidth, srcHeight );
    }

    /**
     * Returns the subbitmap of the current animation frame. Works as the
     * getSubBitmap() in Bitmap.
     *
     *@param srcX The leftmost coordinate of the currently shown bitmap to be returned as a new bitmap.
     *@param srcY The topmost coordinate of the currently shown bitmap to be returned as a new bitmap.
     *@param srcWidth The width of the currently shown bitmap to be returned as a new bitmap.
     *@param srcHeight The height of the currently shown bitmap to be returned as a new bitmap.
     *@return A new bitmap representing the requested part of the original bitmap.
     *@exception GameFrameException Thrown if sub-bitmap creation is not successfull.
     *@see Bitmap#getSubBitmap(int, int, int, int)
     */
    public Bitmap getSubBitmap( int srcX, int srcY, int srcWidth, int srcHeight )
        throws GameFrameException
    {
        return m_aBitmaps[ m_iAnimIdx ].getSubBitmap(
            srcX, srcY, srcWidth, srcHeight );
    }

    /**
     * Frees all the resources that are used by the animated bitmap
     * implementation. This method is quite slow to execute and should
     * preferrably be called upon exiting the game or between level
     * changes etc.
     */
    public void finalize()
    {
        for ( int i = 0; i < m_aBitmaps.length; i++ )
        {
            m_aBitmaps[i].finalize();
        }

        m_aBitmaps = null;
    }
}
