/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

import gameframe.*;

/**
 * Defines an interface to a bitmap image that can blit (draw) or strecth
 * (scale) itself in the graphics engine's backbuffer or into drawable bitmap.
 * The graphics engine in question is allways the graphics engine that loaded
 * the bitmap. The drawable bitmap in question must be a drawable bitmap created
 * by the same graphics engine as this bitmap. If the drawable bitmap was
 * created by different graphics engine the results are unpredictable. Most of
 * the methods offered in this interface are implemented to be as fast as
 * possible. An exception is the method getSubBitmap() that tries to minimize
 * the memory usage to the extent that is possible (maybe even using the same
 * bitmap data as the original bitmap but with different clipping and blitting
 * regions).
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public interface Bitmap
{
    /**
     * Draws the bitmap to the graphics engine that loaded it at the given
     * location. The location can be outside of the visible screen area in
     * which case the bitmap is clipped accordingly. This method is fast and
     * is usable during the game execution.
     *
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@since GameFrame for Java 0.9
     */
    public void blitTo( int x, int y );

    /**
     * Draws the bitmap to the graphics engine that loaded it at the given
     * location and at the given size. If this bitmap is larger or smaller than
     * the destination size the bitmap will automatically be scaled. Note: This
     * method is usually a bit slow, so it's use is not preferred during
     * interactive periods in game execution.
     *
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with that is used when drawing the image.
     *@param height The height that is used when drawing the image.
     *@since GameFrame for Java 0.9
     */
    public void strecthTo( int x, int y, int width, int height );

    /**
     * Draws the bitmap to the given drawable bitmap at the given location.
     * The location can be outside of the visible screen area in which case the
     * bitmap is clipped accordingly. This method is fast and is usable during
     * the game execution.
     *
     *@param bitmap The bitmap that will be drawn to.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@since GameFrame for Java 0.9
     */
    public void blitTo( DrawableBitmap bitmap, int x, int y );

    /**
     * Draws the bitmap to the given drawable bitmap at the given location
     * at the given size. If this bitmap is larger or smaller than the
     * destination size the bitmap will automatically be scaled. Note: This
     * method is usually quite slow, so it's use is not preferred during
     * interactive periods in game execution.
     *
     *@param bitmap The bitmap that will be drawn to.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with that is used when drawing the image.
     *@param height The height that is used when drawing the image.
     *@since GameFrame for Java 0.9
     */
    public void strecthTo(
        DrawableBitmap bitmap, int x, int y, int width, int height );

    /**
     * Returns the width of the bitmap. This method is fast and is usable
     * during the game execution.
     *
     *@return Width of the bitmap if available.
     *@since GameFrame for Java 0.9
     */
    public int getWidth();

    /**
     * Returns the height of the bitmap. This method is fast and is usable
     * during the game execution.
     *
     *@return Height of the bitmap if available.
     *@since GameFrame for Java 0.9
     */
    public int getHeight();

    /**
     * <P>Returns an effect processed copy of this bitmap. This operation
     * copies the actual bitmap data to a new memory location in a processed
     * form and thus consumes memory.</P>
     * 
     * <P>Usually used to save graphics drawing time on game graphics as one
     * can generate an animation sequence on the fly using gradual effects.</P>
     *@param effect The bitmap effect to be applied to the clone bitmap,
     *       or null if none are to be applied.
     *@return An effect processed clone of this bitmap.
     *@exception GameFrameException thrown if any errors occur.
     */
    public Bitmap getClone( BitmapEffect effect )
        throws GameFrameException;
    
    /**
     * <P>Returns a bitmap that represents a part of this bitmap by copying the
     * actual part of the bitmap representation to different memory location.
     * This consumes memory, but the resulting bitmap is as fast to blit as the
     * original bitmap.</P>
     *
     * <P>Useful for e.g. extracting animation frames from one bitmap. This
     * method is not very fast and should be preferrably called during game
     * initialization. If for any reason the creation of the new bitmap fails
     * (out of memory etc.) this method throws an GameFrameException exception.
     * If the given rectangle is outside of the bitmap area of the original
     * bitmap or even larger than the original bitmap area this method returns
     * throws an GameFrameException exception (the input values are checked for
     * validity). Also if the area of the given dimensions is 0 then this method
     * also throws an GameFrameException exception.</P>
     *
     *@param srcX The leftmost coordinate of the bitmap to be returned as a new bitmap.
     *@param srcY The topmost coordinate of the bitmap to be returned as a new bitmap.
     *@param srcWidth The width of the bitmap to be returned as a new bitmap.
     *@param srcHeight The height of the bitmap to be returned as a new bitmap.
     *@return A new bitmap representing the requested part of the original bitmap.
     *@exception GameFrameException Thrown if clone sub-bitmap creation is not successfull.
     *@since GameFrame for Java 0.9
     */
    public Bitmap getSubBitmapClone( int srcX, int srcY, int srcWidth, int srcHeight )
        throws GameFrameException;

    /**
     * <P>Returns a bitmap that represents a part of this bitmap by using a new
     * control object, but using the old bitmap data. This saves memory, but the
     * resulting bitmap is not as fast to blit as the original one on many
     * platforms. On the platforms where this approach is really too slow for
     * interactive use the implementation might actually copy the data (as with
     * the getSubBitmapClone() method) to speed things up to an interactive level.</P>
     *
     * <P>Useful for e.g. extracting animation frames from one bitmap. This method
     * is not very fast and should be preferrably called during game initialization.
     * If for any reason the creation of the new bitmap fails (out of memory etc.)
     * this method throws an GameFrameException exception. If the given rectangle
     * is outside of the bitmap area of the original bitmap or even larger than
     * the original bitmap area this method throws an GameFrameException exception
     * (i.e. the input values are checked for validity). Also if the area of the given
     * dimensions is 0 then this method also throws an GameFrameException exception.<P>
     * 
     * <P>WARNING! If you create a sub-bitmap with this method and then finalize the
     * original bitmap, the sub-bitmap can also be considered as finalized.
     * This is because the original bitmap no longer contains any valid data
     * and the created sub-bitmap control object is dependant on that data.</P>
     *
     *@param srcX The leftmost coordinate of the bitmap to be returned as a new bitmap.
     *@param srcY The topmost coordinate of the bitmap to be returned as a new bitmap.
     *@param srcWidth The width of the bitmap to be returned as a new bitmap.
     *@param srcHeight The height of the bitmap to be returned as a new bitmap.
     *@return A new bitmap representing the requested part of the original bitmap.
     *@exception GameFrameException Thrown if sub-bitmap creation is not successfull.
     *@since GameFrame for Java 0.9
     */
    public Bitmap getSubBitmap( int srcX, int srcY, int srcWidth, int srcHeight )
        throws GameFrameException;
    
    /**
     * Frees all the resources that are used by the bitmap implementation.
     * This method is quite slow to execute and should preferrably be called upon
     * exiting the game or between level changes etc.
     * 
     *@since GameFrame for Java 0.9
     */
    public void finalize();
}
