/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

import java.util.Vector;

/**
 * Represents the bitmap data loaded from a bitmap file.
 *@author Pasi Kernen 16. Feb 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class BitmapData implements Cloneable
{
    /** The value of any key color component if it is not used. */
    public static final int NOT_USED = -1;
    /** The pixel data array. */
    protected int[] m_aPixels;
    /** The bitmap width. */
    protected int   m_width;
    /** The bitmap height. */
    protected int   m_height;
    /** The number of pixels you need to travel to step one pixel down. */
    protected int   m_pitch;
    /** The number of bits used to represent the red component in a pixel. */
    protected int   m_numRedBits = 8;
    /** The number of bits used to represent the green component in a pixel. */
    protected int   m_numGreenBits = 8;
    /** The number of bits used to represent the blue component in a pixel. */
    protected int   m_numBlueBits = 8;
    /**
     * The bitmap keycolor red value.
     * This field is valid only if the bitmap was loaded as alpha bitmap.
     */
    protected int   m_keyColorRed = NOT_USED;
    /**
     * The bitmap keycolor green value.
     * This field is valid only if the bitmap was loaded as alpha bitmap.
     */
    protected int   m_keyColorGreen = NOT_USED;
    /**
     * The bitmap keycolor blue value.
     * This field is valid only if the bitmap was loaded as alpha bitmap.
     */
    protected int   m_keyColorBlue = NOT_USED;
    /**
     * The palette of the bitmap (if paletted color bitmap) or
     * null if no palette available.
     */
    protected int[] m_aPalette;

    /**
     * Constructs a bitmap data object that has the given parameters.
     *@param aPixels The pixels in default colour format.
     *@param width The width of the bitmap.
     *@param height The height of the bitmap.
     *@param pitch The number of pixels to step one pixel downwards.
     */
    public BitmapData( int[] aPixels, int width, int height, int pitch )
    {
        this( aPixels, width, height, pitch, null );
    }

    /**
     * Constructs a bitmap data object that has the given parameters.
     *@param aPixels The pixels in default colour format.
     *@param width The width of the bitmap.
     *@param height The height of the bitmap.
     *@param pitch The number of pixels to step one pixel downwards.
     *@param keyColorRed The alpha key colours red component value.
     *@param keyColorGreen The alpha key colours green component value.
     *@param keyColorBlue The alpha key colours blue component value.
     *@param aPalette The palette array (in default RGB format).
     */
    public BitmapData( int[] aPixels, int width, int height, int pitch,
                       int keyColorRed, int keyColorGreen, int keyColorBlue )
    {
        this( aPixels, width , height, pitch,
              keyColorRed, keyColorGreen, keyColorBlue, null );
    }

    /**
     * Constructs a bitmap data object that has the given parameters.
     *@param aPixels The pixels in default colour format.
     *@param width The width of the bitmap.
     *@param height The height of the bitmap.
     *@param pitch The number of pixels to step one pixel downwards.
     *@param aPalette The palette array (in default RGB format).
     */
    public BitmapData( int[] aPixels, int width, int height, int pitch,
                       int[] aPalette )
    {
        this( aPixels, width, height, pitch,
              NOT_USED, NOT_USED, NOT_USED, aPalette);
    }

    /**
     * Constructs a bitmap data object that has the given parameters.
     *@param aPixels The pixels in default colour format.
     *@param width The width of the bitmap.
     *@param height The height of the bitmap.
     *@param pitch The number of pixels to step one pixel downwards.
     *@param keyColorRed The alpha key colours red component value.
     *@param keyColorGreen The alpha key colours green component value.
     *@param keyColorBlue The alpha key colours blue component value.
     *@param aPalette The palette array (in default RGB format).
     */
    public BitmapData( int[] aPixels, int width, int height, int pitch,
                       int keyColorRed, int keyColorGreen, int keyColorBlue,
                       int[] aPalette )
    {
        m_aPixels       = aPixels;
        m_width         = width;
        m_height        = height;
        m_pitch         = pitch;
        m_keyColorRed   = keyColorRed;
        m_keyColorGreen = keyColorGreen;
        m_keyColorBlue  = keyColorBlue;
        m_aPalette      = aPalette;
    }

    /**
     * Returns the pixel data array in the default color format.
     *@return Pixel data array
     */
    public int[] getPixels()
    {
        return this.m_aPixels;
    }

    /**
     * Returns the number of bits used for the red component in a pixel.
     *@return The number of bits for red colour.
     */
    public int getNumRedBits()
    {
        return m_numRedBits;
    }

    /**
     * Returns the number of bits used for the red component in a pixel.
     *@return The number of bits for red colour.
     */
    public int getNumGreenBits()
    {
        return m_numGreenBits;
    }

    /**
     * Returns the number of bits used for the red component in a pixel.
     *@return The number of bits for red colour.
     */
    public int getNumBlueBits()
    {
        return m_numBlueBits;
    }

    /**
     * Sets the colour bits (the number of bits used for each colour).
     *@param numRedBits The number of bits used for the red colour.
     *@param numGreenBits The number of bits used for the green colour.
     *@param numBlueBits The number of bits used for the blue colour.
     */
    public void setColourBits( int numRedBits, int numGreenBits, int numBlueBits )
    {
        m_numRedBits   = numRedBits;
        m_numGreenBits = numGreenBits;
        m_numBlueBits  = numBlueBits;
        //System.out.println("Bits:"+numRedBits+","+numGreenBits+","+numBlueBits);
    }

    /**
     * Returns whether this image data uses a palette or not.
     *@return True if the image is paletted, false if not.
     */
    public final boolean isPaletted()
    {
        return ( m_aPalette != null );
    }

    /**
     * Returns the palette array in the default color format.
     *@return Palette array.
     */
    public int[] getPalette()
    {
        return m_aPalette;
    }

    /**
     * Returns the width of the bitmap.
     *@return Width of the bitmap
     */
    public int getWidth()
    {
        return this.m_width;
    }

    /**
     * Returns the number of pixels that needs to be travelled in the pixel
     * array in order to step one pixel down in the bitmap.
     *@return The pitch of the bitmap.
     */
    public int getPitch()
    {
        return this.m_pitch;
    }

    /**
     * Returns the height of the bitmap.
     *@return Height of the bitmap
     */
    public int getHeight()
    {
        return this.m_height;
    }

    /**
     * Returns the red component of the keycolor.
     * Valid only if bitmap was loaded as alphabitmap.
     *@return Red component of the keycolor.
     */
    public int getKeyColorRed()
    {
        return m_keyColorRed;
    }

    /**
     * Returns the red component of the keycolor.
     * Valid only if bitmap was loaded as alphabitmap.
     *@return Red component of the keycolor.
     */
    public int getKeyColorGreen()
    {
        return m_keyColorGreen;
    }

    /**
     * Returns the red component of the keycolor.
     * Valid only if bitmap was loaded as alphabitmap.
     *@return Red component of the keycolor.
     */
    public int getKeyColorBlue()
    {
        return m_keyColorBlue;
    }

    /**
     * Returns whether this bitmap data uses the keycolor or not.
     *@return True if keycolour is defined or false if not.
     */
    public boolean isKeyColourInUse()
    {
        return ( ( m_keyColorRed   == NOT_USED ) &&
                 ( m_keyColorGreen == NOT_USED ) &&
                 ( m_keyColorBlue  == NOT_USED ) );
    }

    /**
     * Returns a copy of the bitmap data.
     *@return A copy of the bitmap data.
     */
    public BitmapData getCopy()
    {
        int[] aNewPixels = new int[m_aPixels.length];
        System.arraycopy( m_aPixels, 0, aNewPixels, 0, m_aPixels.length );
        return new BitmapData( aNewPixels, m_width, m_height, m_pitch, m_aPalette );
    }

    /**
     * Returns the given rectangle of the bitmap data.
     *@return The given rectangles bitmap data.
     */
    private BitmapData getRectData( int srcX, int srcY, int srcWidth, int srcHeight )
    {
        int[] aNewPixels = new int[ srcWidth * srcHeight ];

        for ( int row = srcY; row < srcY + srcHeight; row++ )
        {
            System.arraycopy( m_aPixels, srcX, aNewPixels, 0, srcWidth );
        }

        return new BitmapData( aNewPixels, srcWidth, srcHeight, m_pitch, m_aPalette );
    }
}
