/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

import gameframe.*;
import gameframe.util.*;

import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;
/**
 * Represents Windows Bitmap (BMP) file and has utility methods for loading
 * a Windows Bitmap file and returning its bitmap data.
 *@author Pasi Kernen 16. Feb 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class BitmapFile
{
    /** Bitmask array used to read the monochrome bitmaps */
    private static final int[] mStatic_aBitmask = { 1, 2, 4, 8, 16, 32, 64 ,128 };
    /** Bitmask that can be used to handle only the RGB part of the colours. */
    private static final int RGB_ONLY_MASK = 0x00FFFFFF;
    
    /**
     * The image observer method that gets updated when an image is loading.
     */
	public boolean imageUpdate(Image  img, int  infoflags, int  x, int  y, int  width, int  height)
	{
		return true;
	}
    
    /**
     * Saves the given bitmap to a Windows BMP formatted file.
     *@param strFileName The name of the file to save to.
     *@param bitmapData The bitmap data (width, height and raw pixel data).
     */
    public static void saveBMP( String strFileName, BitmapData bitmapData )
        throws GameFrameException
    {
        // Sanity check the filename
        if ( strFileName == null )
        {
            throw new NullPointerException(
                "Save filename was null for a bitmap." );
        }
        
        if ( bitmapData == null )
        {
            throw new NullPointerException(
                "Bitmapdata was null for save operation." );
        }

        File file = new File( strFileName );
        
        FileOutputStream outStream = null;
		try
		{
            outStream = new FileOutputStream( file );
            if ( bitmapData.getPalette() != null )
            {
                save8Bit( outStream, bitmapData );
            }
            else
            {
                save24Bit( outStream, bitmapData );
            }
		}
		catch (IOException e)
		{
			throw new GameFrameException(
                "Couldn't save to file "+
                strFileName+
                ".");
		}
        finally
        {
            // Close the input stream
            try
            {
			    outStream.close();
            }
            catch( IOException ioe )
            {
            }
        }
    }

    /**
     * Saves the bitmap and palette data to the outputstream in 8-bit BMP format.
     *@param outStream The stream to write to.
     *@param bitmapData The bitmap data to be written.
     */
    private static final void save8Bit( OutputStream outStream, BitmapData bitmapData )
        throws IOException
    {
        BigEndianDataOutputStream intelOut = new BigEndianDataOutputStream( outStream );
        int bmpWidth      = bitmapData.getWidth();
        int bmpHeight     = bitmapData.getWidth();
        int paddedWidth   = ( (bmpWidth * 8 + 31) / 32) * 4;
        char fileType     = '\u4D42';
        char offsetToData = '\u0436';
        int length        = paddedWidth * bitmapData.getHeight() + offsetToData;

        // Write BITMAPFILEHEADER
        intelOut.writeShort(fileType);      // Write filetype tag ('BM')
        intelOut.writeInt(length);          // Write size of the file
        intelOut.writeShort(0);             // Reserved, set to zero
        intelOut.writeShort(0);             // Reserved, set to zero
        intelOut.writeInt( offsetToData );  // Byte offset from the BITMAPFILEHEADER to the actual bitmap

        // Write BITMAPINFOHEADER
        intelOut.writeInt(40);          // Number of bytes required by the BITMAPINFOHEADER
        intelOut.writeInt( bmpWidth );  // The width of the bitmap
        intelOut.writeInt( bmpHeight ); // The height of the bitmap
        intelOut.writeShort(1);         // Number of planes (allways 1)
        intelOut.writeShort(8);         // Bits per pixel
        intelOut.writeInt(0);           // Compression (set to no compression)
        intelOut.writeInt(
            paddedWidth * bmpHeight );  // Image size in bytes
        intelOut.writeInt(0);           // horizontal resolution in pixels per meter
        intelOut.writeInt(0);           // vertical resolution in pixels per meter
        intelOut.writeInt(0);           // number of color indexes in the color table actually used by the bitmap
        intelOut.writeInt(0);           // number of color indexes that are considered important


        // Write the 8-bit palette
        int[] aPalette = bitmapData.getPalette();
        int value;
        byte blue;
        byte green;
        byte red;
        for( int idx = 0; idx < 256; idx++ )
        {
            if( idx < aPalette.length )
            {
                value = aPalette[idx];
                blue = (byte) ( value & 0xff ); // Blue
                value >>= 8;
                green = (byte) ( value & 0xff ); // Green
                value >>= 8;
                red   = (byte) ( value & 0xff ); // Red
                intelOut.writeByte( blue);
                intelOut.writeByte( green );
                intelOut.writeByte( red );
                intelOut.writeByte( 0 );
            }
            else
            {
                intelOut.writeInt(0);
            }
        }

        // Write the 8-bit pixel data
        int[] aPixelData = bitmapData.getPixels();
        int realWidth    = bmpWidth;
        int lastRowIdx   = bmpHeight - 1;
        byte aRow[]      = new byte[realWidth];
        int numPaddingBytes = paddedWidth - realWidth;
        for(int row = lastRowIdx; row > -1; row--)
        {
            for ( int col = 0; col < realWidth; col++ )
            {
                aRow[col] = mapFromPixelToPalette(
                    aPixelData[bmpWidth * row + col],
                    aPalette );
            }

            intelOut.write(aRow);
            for(int col = 0; col < numPaddingBytes; col++)
            {
                intelOut.write(0);
            }

        }
    }

    /**
     * Saves the bitmap to the outputstream in 8-bit BMP format.
     *@param outStream The stream to write to.
     *@param bitmapData The bitmap data to be written.
     */
    private static final void save24Bit( OutputStream outStream, BitmapData bitmapData )
        throws IOException
    {
        BigEndianDataOutputStream intelOut = new BigEndianDataOutputStream( outStream );
        int bmpWidth      = bitmapData.getWidth();
        int bmpHeight     = bitmapData.getWidth();
        char fileType     = '\u4D42';
        int paddedWidth   = ((bmpWidth * 24 + 31) / 32) * 4;
        byte offsetToData = 54;
        int length        = paddedWidth * bmpHeight + offsetToData;

        // Write BITMAPFILEHEADER
        intelOut.writeShort(fileType);      // Write filetype tag ('BM')
        intelOut.writeInt(length);          // Write size of the file
        intelOut.writeShort(0);             // Reserved, set to zero
        intelOut.writeShort(0);             // Reserved, set to zero
        intelOut.writeInt( offsetToData );  // Byte offset from the BITMAPFILEHEADER to the actual bitmap

        // Write BITMAPINFOHEADER
        intelOut.writeInt(40);          // Number of bytes required by the BITMAPINFOHEADER
        intelOut.writeInt( bmpWidth );  // The width of the bitmap
        intelOut.writeInt( bmpHeight ); // The height of the bitmap
        intelOut.writeShort(1);         // Number of planes (allways 1)
        intelOut.writeShort(24);         // Bits per pixel
        intelOut.writeInt(0);           // Compression (set to no compression)
        intelOut.writeInt(
            paddedWidth * bmpHeight );  // Image size in bytes
        intelOut.writeInt(0);           // horizontal resolution in pixels per meter
        intelOut.writeInt(0);           // vertical resolution in pixels per meter
        intelOut.writeInt(0);           // number of color indexes in the color table actually used by the bitmap
        intelOut.writeInt(0);           // number of color indexes that are considered important


        // Write the 24-bit pixel data
        int[] aPixelData = bitmapData.getPixels();
        int realWidth    = bmpWidth;
        int aRow[]       = new int[realWidth];
        int lastRowIdx   = bmpHeight - 1;

        int padAmount = paddedWidth - bmpWidth * 3;

        byte aPaddedRGBRow[] = new byte[3 * realWidth + padAmount];

        // Store the rows in the usual reverse order
        for( int row = lastRowIdx; row > -1; row-- )
        {
            for ( int col = 0; col < realWidth; col++ )
            {
                aRow[col] = aPixelData[bmpWidth * row + col];

            }

            int paddedRowColumn = 3 * realWidth;
            for( int l1 = realWidth; --l1 >= 0; )
            {
                aPaddedRGBRow[--paddedRowColumn] = (byte)((aRow[l1] & 0xff0000) >> 16);
                aPaddedRGBRow[--paddedRowColumn] = (byte)((aRow[l1] & 0xff00) >> 8);
                aPaddedRGBRow[--paddedRowColumn] = (byte)(aRow[l1] & 0xff);
            }

            paddedRowColumn = 3 * realWidth;
            for(int padCounter = padAmount; --padCounter >= 0;)
            {
                aPaddedRGBRow[paddedRowColumn++] = 0;
            }

            intelOut.write(aPaddedRGBRow);
        }
    }

    /**
     * Maps the pixel value to the value in the palette
     */
    private final static byte mapFromPixelToPalette( int pixel, int[] aPalette )
    {
        for ( int i = 0; i < aPalette.length; i++ )
        {
            if ( ( pixel & RGB_ONLY_MASK ) == ( aPalette[i] & RGB_ONLY_MASK ) )
            {
                return (byte) i;
            }
        }

        return 0;
    }

    /**
     * Loads the file at the given URL containing a Windows BMP formatted bitmap and
     * returns an object containing the raw pixel data and the dimensions
     * of the loaded bitmap.
     *@param strFileName The name of the file to load.
     *@param fEnableAlpha True if bitmap is to be loaded as alpha bitmap.
     *@return The bitmap data (width, height and raw pixel data).
     */
    public final static BitmapData loadBMPURL (
        String strFileName, boolean fEnableAlpha )
            throws FileNotFoundException, GameFrameException
    {
        if ( !( GameFrame.getInstance() instanceof AppletGameFrame ) )
        {
            throw new GameFrameException( "The loadBMPURL method can only be used when running as an applet." );
        }

        AppletGameFrame appletGF = (AppletGameFrame) GameFrame.getInstance();
        Applet applet = appletGF.getApplet();
        URL codeBase  = applet.getCodeBase();

        URL url = null;
        try
        {
            url = new URL(
                codeBase.getProtocol(),
                codeBase.getHost(),
                codeBase.getPort(),
                codeBase.getFile() + strFileName.replace('\\', '/' ) );
            System.err.println( "loadBMPURL:"+url);
        }
        catch( MalformedURLException urlError )
        {
            throw new GameFrameException( "URL for Bitmap was faulty:"+urlError );
        }

        InputStream inStream = null;
		try
		{
            inStream = url.openStream();
        }
		catch (IOException e)
		{
			throw new GameFrameException(
                "Error occured when trying to open stream to "+
                url.toString()+
                ".");
		}

        return loadBMP( inStream, url.toString(), fEnableAlpha );
    }

    /**
     * Loads the file at the given URL containing a Windows BMP formatted bitmap and
     * returns an object containing the raw pixel data and the dimensions
     * of the loaded bitmap.
     *@param strFileName The name of the file to load.
     *@param red The red key color intensity value (in range 0x00-0xFF).
     *@param green The green key color intensity value (in range 0x00-0xFF).
     *@param blue The blue key color intensity value (in range 0x00-0xFF).
     *@return The bitmap data (width, height and raw pixel data).
     */
    public final static BitmapData loadBMPURL (
        String strFileName, byte red, byte green, byte blue )
            throws FileNotFoundException, GameFrameException
    {
        if ( !( GameFrame.getInstance() instanceof AppletGameFrame ) )
        {
            throw new GameFrameException( "The loadBMPURL method can only be used when running as an applet." );
        }

        AppletGameFrame appletGF = (AppletGameFrame) GameFrame.getInstance();
        Applet applet = appletGF.getApplet();
        URL codeBase  = applet.getCodeBase();

        URL url = null;
        try
        {
            url = new URL(
                codeBase.getProtocol(),
                codeBase.getHost(),
                codeBase.getPort(),
                codeBase.getFile() + strFileName.replace('\\', '/' ) );
            System.err.println( "loadBMPURL:"+url);
        }
        catch( MalformedURLException urlError )
        {
            throw new GameFrameException( "URL for Bitmap was faulty:"+urlError );
        }

        InputStream inStream = null;
		try
		{
            inStream = url.openStream();
        }
		catch (IOException e)
		{
			throw new GameFrameException(
                "Error occured when trying to open stream to "+
                url.toString()+
                ".");
		}

        return loadBMP( inStream, url.toString(), red, green, blue );
    }

    /**
     * Loads the given file containing a Windows BMP formatted bitmap and
     * returns an object containing the raw pixel data and the dimensions
     * of the loaded bitmap.
     *@param strFileName The name of the file to load.
     *@param fEnableAlpha True if bitmap is to be loaded as alpha bitmap.
     *@return The bitmap data (width, height and raw pixel data).
     */
    public final static BitmapData loadBMP (
        String strFileName, boolean fEnableAlpha )
            throws FileNotFoundException, GameFrameException
	{
        // Sanity check the filename
        if ( strFileName == null )
        {
            throw new NullPointerException(
                "Given bitmap filename was null." );
        }
        
		File file = new File( strFileName );
        if ( !file.exists() )
        {
    		throw new FileNotFoundException(
                "Bitmap file \""+
                strFileName+
                "\" doesn't exist.");
        }
            
		int length = (int) file.length();
		if (length <= 0)
		{
            throw new GameFrameException(
                "Size of the bitmap file "+
                strFileName+
                " is zero.");
		}
        
        InputStream inStream = null;
		try
		{
            inStream = new FileInputStream( file );
        }
		catch (IOException e)
		{
			throw new GameFrameException(
                "Error occured while loading file "+
                strFileName+
                ".");
		}

        return loadBMP( inStream, strFileName, fEnableAlpha );
    }

    /**
     * Loads the given file containing a Windows BMP formatted bitmap and
     * returns an object containing the raw pixel data and the dimensions
     * of the loaded bitmap.
     *@param strFileName The name of the file to load.
     *@param red The red key color intensity value (in range 0x00-0xFF).
     *@param green The green key color intensity value (in range 0x00-0xFF).
     *@param blue The blue key color intensity value (in range 0x00-0xFF).
     *@return The bitmap data (width, height and raw pixel data).
     */
    public final static BitmapData loadBMP (
        String strFileName, byte red, byte green, byte blue )
            throws FileNotFoundException, GameFrameException
	{
        // Sanity check the filename
        if ( strFileName == null )
        {
            throw new NullPointerException(
                "Given bitmap filename was null." );
        }
        
		File file = new File( strFileName );
        if ( !file.exists() )
        {
    		throw new FileNotFoundException(
                "Bitmap file \""+
                strFileName+
                "\" doesn't exist.");
        }
            
		int length = (int) file.length();
		if (length <= 0)
		{
            throw new GameFrameException(
                "Size of the bitmap file "+
                strFileName+
                " is zero.");
		}
        
        InputStream inStream = null;
		try
		{
            inStream = new FileInputStream( file );
        }
		catch (IOException e)
		{
			throw new GameFrameException(
                "Error occured while loading file "+
                strFileName+
                ".");
		}

        return loadBMP( inStream, strFileName, red, green, blue );
    }

    /**
     * Handles the loading of the bitmap from the given input stream.
     *@param inStream The input stream to load from.
     *@param strFileName The filename used in error texts.
     *@param fEnableAlpha Flag indicating if key color should be used or not.
     */
    public final static BitmapData loadBMP(
        InputStream inStream, String strFileName,
        boolean fEnableAlpha )
            throws FileNotFoundException, GameFrameException
    {
        byte alphakeyRed   = 0;
        byte alphakeyGreen = 0;
        byte alphakeyBlue  = 0;
        BitmapData bitmapData;

        try
		{
            // Read the 14 byte BITMAPFILEHEADER
			int fileHeaderLength= 14;
			byte aFileHeader[]  = new byte[ fileHeaderLength ];
            fillBuffer( inStream, aFileHeader );

            // Read the 40-byte BITMAPINFOHEADER
			int infoHeaderLength    = 40;
			byte aInfoHeader[]      = new byte[infoHeaderLength];
            fillBuffer( inStream, aInfoHeader );

			// Interperet header data (use Intel int to Java int conversion)
			int bitmapWidth =
                  ( ( ( int ) aInfoHeader[7] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[6] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[5] & 0xff ) << 8  )
				|     ( int ) aInfoHeader[4] & 0xff;

			int bitmapHeight =
                  ( ( ( int ) aInfoHeader[11] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[10] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[9]  & 0xff ) << 8  )
				|     ( int ) aInfoHeader[8]  & 0xff;
			int bitmapBpp =
                  ( ( ( int ) aInfoHeader[15] & 0xff ) << 8 )
				|     ( int ) aInfoHeader[14] & 0xff;
			int bitmapSize =
                  ( ( ( int ) aInfoHeader[23] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[22] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[21] & 0xff ) << 8  )
				|     ( int ) aInfoHeader[20] & 0xff;
			int bitmapNumColors =
                  ( ( ( int ) aInfoHeader[35] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[34] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[33] & 0xff ) << 8  )
				|     ( int ) aInfoHeader[32] & 0xff;
			int compression =
                  ( ( ( int ) aInfoHeader[19] ) << 24 )
				| ( ( ( int ) aInfoHeader[18] ) << 16 )
				| ( ( ( int ) aInfoHeader[17] ) << 8  )
				|     ( int ) aInfoHeader[16];

            // Check that the bitmap isn't compressed
            if ( compression != 0 )
            {
                throw new GameFrameException(
                        "The file format of bitmap file "+
                        strFileName+
                        " is incorrect." );
            }

            // Decide what to do based on the bits per pixel value
            switch(bitmapBpp)
            {
                case 24:
                {
				    // No Palatte data for 24-bit format but scan lines are
				    // padded out to even 4-byte boundaries

                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - bitmapWidth * 3;
				    int aPixelData[]  = new int [ bitmapHeight * bitmapWidth ];
				    byte aRgbBuffer[] =
                        new byte [ ( ( bitmapWidth * 3 ) + pad ) * bitmapHeight ];

                    // Read the pixel data
                    fillBuffer( inStream, aRgbBuffer );

				    if ( fEnableAlpha )
				    {
                        // Alpha channel enabled set the alphakey values
                        // based on the first pixel
                        alphakeyBlue  = aRgbBuffer[0];
                        alphakeyGreen = aRgbBuffer[1];
                        alphakeyRed   = aRgbBuffer[2];
                    }
                    else
                    {
                        // Alpha keying disabled, set alphakey values
                        // to impossible values
                        alphakeyBlue  = -1;
                        alphakeyGreen = -1;
                        alphakeyRed   = -1;
                    }

                    // Interprete the pixel data
                    int idx = 0;
				    for ( int row = 0; row < bitmapHeight; row++ )
				    {
				    	for ( int col = 0; col < bitmapWidth; col++ )
				    	{

                            int offset = bitmapWidth * (bitmapHeight - row - 1) + col;
				    		if	( ( aRgbBuffer[idx+2] == alphakeyRed   ) &&
				    			  ( aRgbBuffer[idx+1] == alphakeyGreen ) &&
				    			  ( aRgbBuffer[idx]   == alphakeyBlue  ) )
				    		{
                                // Alphakey colored pixel found, set it to transparent
                                aPixelData [offset] = ( aPixelData [offset] & RGB_ONLY_MASK );
				    		}
				    		else
				    		{
                                // Normal pixel found, store it to the pixel data array
				    			aPixelData [ offset ] =
				    				      ( 0xFF )                          << 24
				    				| ( ( ( int ) aRgbBuffer[idx+2] & 0xff) << 16 )
				    				| ( ( ( int ) aRgbBuffer[idx+1] & 0xff) <<  8 )
				    				|     ( int ) aRgbBuffer[idx]   & 0xff;
				    		}

                            // Advance the index
		    	    		idx += 3;
				    	}

                        // Add the pad amount per line
				    	idx += pad;
				    }

                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        null );
                }
                break;


                case 8:
                {
				    // Have to determine the number of colors, the clrsused
				    // parameter is dominant if it is greater than zero.  If
				    // zero, calculate colors based on bitsperpixel.
				    int numColors = 0;
				    if (bitmapNumColors > 0)
				    {
				    	numColors = bitmapNumColors;
				    }
				    else
				    {
				    	numColors = ( 1 & 0xff ) << bitmapBpp;
				    }

				    // Some bitmaps do not have the image size field calculated
				    // in theses cases calculate the bitmap size
				    if ( bitmapSize == 0 )
				    {
				    	bitmapSize = ( ( ( ( bitmapWidth * bitmapBpp ) + 31 ) & ~31 ) >> 3 );
				    	bitmapSize *= bitmapHeight;
				    }

                    // Create palette and palette buffer
				    int  aPalette[] = new int [numColors];
				    byte aPaletteBuffer[] = new byte [numColors*4];

                    // Read the palette to the buffer
                    fillBuffer( inStream, aPaletteBuffer );

                    // Interprete the palette colors from the buffer to the palette
				    int idx = 0;
				    for ( int n = 0; n < numColors; n++ )
				    {
				    	aPalette[n] = 0xFF;                                  // Alpha
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+2] & 0xff ); // Red
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+1] & 0xff ); // Green
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx]   & 0xff ); // Blue
				    	idx += 4;
				    }
                
				    // Read the image data (actually indices into the palette)
				    // Scan lines are still padded out to even 4-byte
				    // boundaries.
                    
                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - bitmapWidth;
                    
                    // Create the pixel data array and pixel data buffer
				    int  aPixelData[] = new int [ bitmapWidth*bitmapHeight ];
				    byte aPixelBuffer[] = new byte [(bitmapWidth+pad)*bitmapHeight];

                    // Read the pixel data to the buffer
                    fillBuffer( inStream, aPixelBuffer );

                    // Set the alpha keying color in the palette
				    if ( fEnableAlpha )
				    {
                        // Alpha channel is enabled
                        int keyColorIndex = ( (int) aPixelBuffer[0] & 0xff );
                        alphakeyBlue  = (byte) ( ( aPalette[keyColorIndex] >>> 16 ) & 0xFF );
                        alphakeyGreen = (byte) ( ( aPalette[keyColorIndex] >>> 8 ) & 0xFF );
                        alphakeyRed   = (byte) ( aPalette[keyColorIndex] & 0xFF );

                        // set palette color that the first pixel uses to be the transparent color
                        aPalette[ keyColorIndex ] = ( aPalette[ keyColorIndex ] & RGB_ONLY_MASK );
				    }

                    // Interprete the pixel data from the buffer to the pixel array
                    idx = 0;
				    for ( int row = 0; row < bitmapHeight; row++ )
				    {
				    	for ( int col = 0; col < bitmapWidth; col++ )
				    	{
                            int pixel = aPalette [ ( ( int ) aPixelBuffer[idx] & 0xff ) ];
                            aPixelData [ bitmapWidth * ( bitmapHeight - row - 1 ) + col ] = pixel;
				    		idx++;
				    	}
                        
                        // Add the padding per line
				    	idx += pad;
				    }
                    
                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        aPalette );
                }
                break;
                
                case 4:
                {
				    // Have to determine the number of colors, the bitmapNumColors
				    // parameter is dominant if it is greater than zero.  If
				    // zero, calculate colors based on bits per pixel.
				    int numColors = 0;
				    if ( bitmapNumColors > 0 )
				    {
				    	numColors = bitmapNumColors;
				    }
				    else
				    {
				    	numColors = ( 1 & 0xff ) << bitmapBpp;
				    }
                    
				    // Some bitmaps do not have the image size field calculated
				    // in theses cases calculate the bitmap size
				    if ( bitmapSize == 0 )
				    {
				    	bitmapSize = ( ( ( ( bitmapWidth * bitmapBpp ) + 31 ) & ~31 ) >> 3 );
				    	bitmapSize *= bitmapHeight;
				    }
                    
                    // Create palette and palette buffer
				    int  aPalette[] = new int [ numColors ];
				    byte aPaletteBuffer[] = new byte [ numColors*4 ];
                    
                    // Read the palette to the buffer
                    fillBuffer( inStream, aPaletteBuffer );
                    
                    // Interprete the palette colors from the buffer to the palette
				    int idx = 0;
				    for ( int n = 0; n < numColors; n++ )
				    {
				    	aPalette[n] = 0xFF;                                  // Alpha
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+2] & 0xff ); // Red
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+1] & 0xff ); // Green
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx]   & 0xff ); // Blue
				    	idx += 4;
				    }
                
                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - bitmapWidth / 2;
                    if ( ( bitmapWidth % 2 ) != 0 )
                    {
                        pad--;
                    }
                    
                    // Create the pixel data array and pixel data buffer
                    int dataLength = ( bitmapWidth / 2 + pad ) * bitmapHeight;
                    if ( ( bitmapWidth % 2 ) != 0 )
                    {
                        dataLength = ( ( bitmapWidth / 2 ) + 1 + pad ) * bitmapHeight;
                    }
				    int  aPixelData[]   = new int  [ bitmapWidth * bitmapHeight ];
				    byte aPixelBuffer[] = new byte [ dataLength ];
                    
                    // Read the pixel data to the buffer
                    fillBuffer( inStream, aPixelBuffer );

                    // Set the alpha keying color in the palette
				    if ( fEnableAlpha )
				    {
                        // Alpha channel is enabled
                        int keyColorIndex = ( (int) aPixelBuffer[0] & 0xff ) >>> 4;
                        alphakeyBlue  = (byte) ( ( aPalette[keyColorIndex] >>> 16 ) & 0xFF );
                        alphakeyGreen = (byte) ( ( aPalette[keyColorIndex] >>> 8 ) & 0xFF );
                        alphakeyRed   = (byte) ( aPalette[keyColorIndex] & 0xFF );

                        // set palette color that the first pixel uses to be the transparent color
                        aPalette[ keyColorIndex ] = ( aPalette[ keyColorIndex ] & RGB_ONLY_MASK );
				    }

                    // Interprete the pixel data from the buffer to the pixel array
                    idx = 0;
				    for ( int row = 0; row < bitmapHeight; row++ )
				    {
				    	for ( int col = 0; col < bitmapWidth; col = col + 2 )
				    	{
                            // Get the first pixel bit
                            int offset = bitmapWidth * ( bitmapHeight - row - 1 ) + col;
                            int pixel = aPalette [ ( ( int ) aPixelBuffer[idx] & 0xF0 ) >>> 4 ];
                            aPixelData [ offset ] = pixel;

                            // Get the second pixel bit
                            pixel = aPalette [ ( ( int ) aPixelBuffer[idx] & 0x0F ) ];
                            aPixelData [ offset + 1 ] = pixel;
				    		idx++;
				    	}
                        
                        // Add the padding per line
				    	idx += pad;
				    }
                    
                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        aPalette );
                }
                break;
                
                case 1:
                {
				    // Have to determine the number of colors, the bitmapNumColors
				    // parameter is dominant if it is greater than zero.  If
				    // zero, calculate colors based on bits per pixel.
				    int numColors = 0;
				    if ( bitmapNumColors > 0 )
				    {
				    	numColors = bitmapNumColors;
				    }
				    else
				    {
				    	numColors = ( 1 & 0xff ) << bitmapBpp;
				    }
                    
				    // Some bitmaps do not have the image size field calculated
				    // in theses cases calculate the bitmap size
				    if ( bitmapSize == 0 )
				    {
				    	bitmapSize = ( ( ( ( bitmapWidth * bitmapBpp ) + 31 ) & ~31 ) >> 3 );
				    	bitmapSize *= bitmapHeight;
				    }
                    
                    // Create palette and palette buffer
				    int  aPalette[] = new int [ numColors ];
				    byte aPaletteBuffer[] = new byte [ numColors*4 ];
                    
                    // Read the palette to the buffer
				    fillBuffer ( inStream, aPaletteBuffer );
                    
                    // Interprete the palette colors from the buffer to the palette
				    int idx = 0;
				    for ( int n = 0; n < numColors; n++ )
				    {
				    	aPalette[n] = 0xFF;                                  // Alpha
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+2] & 0xff ); // Red
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+1] & 0xff ); // Green
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx]   & 0xff ); // Blue
				    	idx += 4;
				    }
                
                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - ( bitmapWidth / 8 );
                    if ( ( bitmapWidth % 8 ) != 0 )
                    {
                        pad--;
                    }
                    
                    // Create the pixel data array and pixel data buffer
                    int dataLength = ( bitmapWidth / 8 + pad ) * bitmapHeight;
                    if ( ( bitmapWidth % 8 ) != 0 )
                    {
                        dataLength = ( ( bitmapWidth / 8 ) + 1 + pad ) * bitmapHeight;
                    }
				    int  aPixelData[]   = new int  [ bitmapWidth * bitmapHeight ];
				    byte aPixelBuffer[] = new byte [ dataLength ];
                    
                    // Read the pixel data to the buffer
				    fillBuffer ( inStream, aPixelBuffer );

                    // Set the alpha keying color in the palette
				    if ( fEnableAlpha )
				    {
                        // Alpha channel is enabled
                        int keyColorIndex = ( (int) aPixelBuffer[0] & 0xff ) >>> 7;
                        alphakeyBlue  = (byte) ( ( aPalette[keyColorIndex] >>> 16 ) & 0xFF );
                        alphakeyGreen = (byte) ( ( aPalette[keyColorIndex] >>> 8 ) & 0xFF );
                        alphakeyRed   = (byte) ( aPalette[keyColorIndex] & 0xFF );

                        // set palette color that the first pixel uses to be the transparent color
                        aPalette[ keyColorIndex ] = ( aPalette[ keyColorIndex ] & RGB_ONLY_MASK );
				    }

                    // Interprete the pixel data from the buffer to the pixel array
                    idx = 0;
                    int pixel;
				    for ( int iY = 0; iY < bitmapHeight; iY++ )
				    {
				    	for ( int iX = 0; iX < bitmapWidth; )
				    	{
                            for ( int bit = 7; bit >= 0; bit-- )
                            {
                                if ( iX < bitmapWidth )
                                {
                                    pixel = aPalette [
                                        ( ( int ) aPixelBuffer[idx] & mStatic_aBitmask[ bit ] ) >>> bit ];
                                    aPixelData [ bitmapWidth * ( bitmapHeight - iY - 1 ) + iX ] = pixel;
                                }
                                else
                                {
                                    break;
                                }

                                iX++;
                            }
                            
                            idx++;
				    	}
                        
                        // Add the padding per line
				    	idx += pad;
				    }
                    
                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        aPalette );
                }
                break;

                default:
                    inStream.close();
                    // Unknown bits per pixel amount, throw an exception
                    throw new GameFrameException(
                            "The file format of file "+
                            strFileName+
                            " is incorrect." );
            }
		}
		catch (IOException e)
		{
			throw new GameFrameException(
                "Error occured while loading file "+
                strFileName+
                ".");
		}
        finally
        {
            // Close the input stream
            try
            {
			    inStream.close();
            }
            catch( IOException ioe )
            {
            }
        }

        return bitmapData;
    }

    /**
     * Handles the loading of the bitmap from the given input stream.
     *@param inStream The input stream to load from.
     *@param strFileName The filename used in error texts.
     *@param alphakeyRed The red alpha key value (in range 0x00-0xFF).
     *@param alphakeyGreen The green alpha key value (in range 0x00-0xFF).
     *@param alphakeyBlue The blue alpha key value (in range 0x00-0xFF).
     */
    public final static BitmapData loadBMP(
        InputStream inStream, String strFileName,
        byte alphakeyRed, byte alphakeyGreen, byte alphakeyBlue )
            throws FileNotFoundException, GameFrameException
    {
        BitmapData bitmapData;

        try
		{
            // Read the 14 byte BITMAPFILEHEADER
			int fileHeaderLength= 14;
			byte aFileHeader[]  = new byte[ fileHeaderLength ];
            fillBuffer( inStream, aFileHeader );

            // Read the 40-byte BITMAPINFOHEADER
			int infoHeaderLength    = 40;
			byte aInfoHeader[]      = new byte[infoHeaderLength];
            fillBuffer( inStream, aInfoHeader );

			// Interperet header data (use Intel int to Java int conversion)
			int bitmapWidth =
                  ( ( ( int ) aInfoHeader[7] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[6] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[5] & 0xff ) << 8  )
				|     ( int ) aInfoHeader[4] & 0xff;

			int bitmapHeight =
                  ( ( ( int ) aInfoHeader[11] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[10] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[9]  & 0xff ) << 8  )
				|     ( int ) aInfoHeader[8]  & 0xff;
			int bitmapBpp =
                  ( ( ( int ) aInfoHeader[15] & 0xff ) << 8 )
				|     ( int ) aInfoHeader[14] & 0xff;
			int bitmapSize =
                  ( ( ( int ) aInfoHeader[23] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[22] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[21] & 0xff ) << 8  )
				|     ( int ) aInfoHeader[20] & 0xff;
			int bitmapNumColors =
                  ( ( ( int ) aInfoHeader[35] & 0xff ) << 24 )
				| ( ( ( int ) aInfoHeader[34] & 0xff ) << 16 )
				| ( ( ( int ) aInfoHeader[33] & 0xff ) << 8  )
				|     ( int ) aInfoHeader[32] & 0xff;
			int compression =
                  ( ( ( int ) aInfoHeader[19] ) << 24 )
				| ( ( ( int ) aInfoHeader[18] ) << 16 )
				| ( ( ( int ) aInfoHeader[17] ) << 8  )
				|     ( int ) aInfoHeader[16];

            // Check that the bitmap isn't compressed
            if ( compression != 0 )
            {
                throw new GameFrameException(
                        "The file format of bitmap file "+
                        strFileName+
                        " is incorrect." );
            }

            // Decide what to do based on the bits per pixel value
            switch(bitmapBpp)
            {
                case 24:
                {
				    // No Palatte data for 24-bit format but scan lines are
				    // padded out to even 4-byte boundaries

                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - bitmapWidth * 3;
				    int aPixelData[]  = new int [ bitmapHeight * bitmapWidth ];
				    byte aRgbBuffer[] =
                        new byte [ ( ( bitmapWidth * 3 ) + pad ) * bitmapHeight ];

                    // Read the pixel data
                    fillBuffer( inStream, aRgbBuffer );

                    // Interprete the pixel data
                    int idx = 0;
				    for ( int row = 0; row < bitmapHeight; row++ )
				    {
				    	for ( int col = 0; col < bitmapWidth; col++ )
				    	{

                            int offset = bitmapWidth * (bitmapHeight - row - 1) + col;
				    		if	( ( aRgbBuffer[idx+2] == alphakeyRed   ) &&
				    			  ( aRgbBuffer[idx+1] == alphakeyGreen ) &&
				    			  ( aRgbBuffer[idx]   == alphakeyBlue  ) )
				    		{
                                // Alphakey colored pixel found, set it to transparent
                                aPixelData [offset] = ( aPixelData [offset] & RGB_ONLY_MASK );
				    		}
				    		else
				    		{
                                // Normal pixel found, store it to the pixel data array
				    			aPixelData [ offset ] =
				    				      ( 0xFF )                          << 24
				    				| ( ( ( int ) aRgbBuffer[idx+2] & 0xff) << 16 )
				    				| ( ( ( int ) aRgbBuffer[idx+1] & 0xff) <<  8 )
				    				|     ( int ) aRgbBuffer[idx]   & 0xff;
				    		}

                            // Advance the index
		    	    		idx += 3;
				    	}

                        // Add the pad amount per line
				    	idx += pad;
				    }

                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        null );
                    bitmapData.setColourBits( 8, 8, 8 );
                }
                break;


                case 8:
                {
				    // Have to determine the number of colors, the clrsused
				    // parameter is dominant if it is greater than zero.  If
				    // zero, calculate colors based on bitsperpixel.
				    int numColors = 0;
				    if (bitmapNumColors > 0)
				    {
				    	numColors = bitmapNumColors;
				    }
				    else
				    {
				    	numColors = ( 1 & 0xff ) << bitmapBpp;
				    }

				    // Some bitmaps do not have the image size field calculated
				    // in theses cases calculate the bitmap size
				    if ( bitmapSize == 0 )
				    {
				    	bitmapSize = ( ( ( ( bitmapWidth * bitmapBpp ) + 31 ) & ~31 ) >> 3 );
				    	bitmapSize *= bitmapHeight;
				    }

                    // Create palette and palette buffer
				    int  aPalette[] = new int [numColors];
				    byte aPaletteBuffer[] = new byte [numColors*4];

                    // Read the palette to the buffer
                    fillBuffer( inStream, aPaletteBuffer );

                    // Interprete the palette colors from the buffer to the palette
				    int idx = 0;
				    for ( int n = 0; n < numColors; n++ )
				    {
				    	aPalette[n] = 0xFF;                                  // Alpha
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+2] & 0xff ); // Red
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+1] & 0xff ); // Green
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx]   & 0xff ); // Blue
				    	idx += 4;
				    }

				    // Read the image data (actually indices into the palette)
				    // Scan lines are still padded out to even 4-byte
				    // boundaries.

                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - bitmapWidth;

                    // Create the pixel data array and pixel data buffer
				    int  aPixelData[] = new int [ bitmapWidth*bitmapHeight ];
				    byte aPixelBuffer[] = new byte [(bitmapWidth+pad)*bitmapHeight];

                    // Read the pixel data to the buffer
                    fillBuffer( inStream, aPixelBuffer );

                    // Interprete the pixel data from the buffer to the pixel array
                    idx = 0;
				    for ( int row = 0; row < bitmapHeight; row++ )
				    {
				    	for ( int col = 0; col < bitmapWidth; col++ )
				    	{
                            int pixel = aPalette [ ( ( int ) aPixelBuffer[idx] & 0xff ) ];
                            aPixelData [ bitmapWidth * ( bitmapHeight - row - 1 ) + col ] = pixel;
				    		idx++;
				    	}

                        // Add the padding per line
				    	idx += pad;
				    }

                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        aPalette );
                    bitmapData.setColourBits( 8, 8, 8 );
                }
                break;

                case 4:
                {
				    // Have to determine the number of colors, the bitmapNumColors
				    // parameter is dominant if it is greater than zero.  If
				    // zero, calculate colors based on bits per pixel.
				    int numColors = 0;
				    if ( bitmapNumColors > 0 )
				    {
				    	numColors = bitmapNumColors;
				    }
				    else
				    {
				    	numColors = ( 1 & 0xff ) << bitmapBpp;
				    }

				    // Some bitmaps do not have the image size field calculated
				    // in theses cases calculate the bitmap size
				    if ( bitmapSize == 0 )
				    {
				    	bitmapSize = ( ( ( ( bitmapWidth * bitmapBpp ) + 31 ) & ~31 ) >> 3 );
				    	bitmapSize *= bitmapHeight;
				    }

                    // Create palette and palette buffer
				    int  aPalette[] = new int [ numColors ];
				    byte aPaletteBuffer[] = new byte [ numColors*4 ];

                    // Read the palette to the buffer
                    fillBuffer( inStream, aPaletteBuffer );

                    // Interprete the palette colors from the buffer to the palette
				    int idx = 0;
				    for ( int n = 0; n < numColors; n++ )
				    {
				    	aPalette[n] = 0xFF;                                  // Alpha
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+2] & 0xff ); // Red
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+1] & 0xff ); // Green
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx]   & 0xff ); // Blue
				    	idx += 4;
				    }

                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - bitmapWidth / 2;
                    if ( ( bitmapWidth % 2 ) != 0 )
                    {
                        pad--;
                    }

                    // Create the pixel data array and pixel data buffer
                    int dataLength = ( bitmapWidth / 2 + pad ) * bitmapHeight;
                    if ( ( bitmapWidth % 2 ) != 0 )
                    {
                        dataLength = ( ( bitmapWidth / 2 ) + 1 + pad ) * bitmapHeight;
                    }
				    int  aPixelData[]   = new int  [ bitmapWidth * bitmapHeight ];
				    byte aPixelBuffer[] = new byte [ dataLength ];

                    // Read the pixel data to the buffer
                    fillBuffer( inStream, aPixelBuffer );

                    // Interprete the pixel data from the buffer to the pixel array
                    idx = 0;
				    for ( int row = 0; row < bitmapHeight; row++ )
				    {
				    	for ( int col = 0; col < bitmapWidth; col = col + 2 )
				    	{
                            // Get the first pixel bit
                            int offset = bitmapWidth * ( bitmapHeight - row - 1 ) + col;
                            int pixel = aPalette [ ( ( int ) aPixelBuffer[idx] & 0xF0 ) >>> 4 ];
                            aPixelData [ offset ] = pixel;

                            // Get the second pixel bit
                            pixel = aPalette [ ( ( int ) aPixelBuffer[idx] & 0x0F ) ];
                            aPixelData [ offset + 1 ] = pixel;
				    		idx++;
				    	}

                        // Add the padding per line
				    	idx += pad;
				    }

                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        aPalette );
                    bitmapData.setColourBits( 8, 8, 8 );
                }
                break;

                case 1:
                {
				    // Have to determine the number of colors, the bitmapNumColors
				    // parameter is dominant if it is greater than zero.  If
				    // zero, calculate colors based on bits per pixel.
				    int numColors = 0;
				    if ( bitmapNumColors > 0 )
				    {
				    	numColors = bitmapNumColors;
				    }
				    else
				    {
				    	numColors = ( 1 & 0xff ) << bitmapBpp;
				    }

				    // Some bitmaps do not have the image size field calculated
				    // in theses cases calculate the bitmap size
				    if ( bitmapSize == 0 )
				    {
				    	bitmapSize = ( ( ( ( bitmapWidth * bitmapBpp ) + 31 ) & ~31 ) >> 3 );
				    	bitmapSize *= bitmapHeight;
				    }

                    // Create palette and palette buffer
				    int  aPalette[] = new int [ numColors ];
				    byte aPaletteBuffer[] = new byte [ numColors*4 ];

                    // Read the palette to the buffer
				    fillBuffer ( inStream, aPaletteBuffer );

                    // Interprete the palette colors from the buffer to the palette
				    int idx = 0;
				    for ( int n = 0; n < numColors; n++ )
				    {
				    	aPalette[n] = 0xFF;                                  // Alpha
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+2] & 0xff ); // Red
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx+1] & 0xff ); // Green
                        aPalette[n] <<= 8;
                        aPalette[n] |= (int) ( aPaletteBuffer[idx]   & 0xff ); // Blue
				    	idx += 4;
				    }

                    // Calculate the pad amount
				    int pad = ( bitmapSize / bitmapHeight ) - ( bitmapWidth / 8 );
                    if ( ( bitmapWidth % 8 ) != 0 )
                    {
                        pad--;
                    }

                    // Create the pixel data array and pixel data buffer
                    int dataLength = ( bitmapWidth / 8 + pad ) * bitmapHeight;
                    if ( ( bitmapWidth % 8 ) != 0 )
                    {
                        dataLength = ( ( bitmapWidth / 8 ) + 1 + pad ) * bitmapHeight;
                    }
				    int  aPixelData[]   = new int  [ bitmapWidth * bitmapHeight ];
				    byte aPixelBuffer[] = new byte [ dataLength ];

                    // Read the pixel data to the buffer
				    fillBuffer ( inStream, aPixelBuffer );

                    // Interprete the pixel data from the buffer to the pixel array
                    idx = 0;
                    int pixel;
				    for ( int iY = 0; iY < bitmapHeight; iY++ )
				    {
				    	for ( int iX = 0; iX < bitmapWidth; )
				    	{
                            for ( int bit = 7; bit >= 0; bit-- )
                            {
                                if ( iX < bitmapWidth )
                                {
                                    pixel = aPalette [
                                        ( ( int ) aPixelBuffer[idx] & mStatic_aBitmask[ bit ] ) >>> bit ];
                                    aPixelData [ bitmapWidth * ( bitmapHeight - iY - 1 ) + iX ] = pixel;
                                }
                                else
                                {
                                    break;
                                }

                                iX++;
                            }

                            idx++;
				    	}

                        // Add the padding per line
				    	idx += pad;
				    }

                    bitmapData = new BitmapData (
                        aPixelData,
                        bitmapWidth,
                        bitmapHeight,
                        bitmapWidth,
                        alphakeyRed,
                        alphakeyGreen,
                        alphakeyBlue,
                        aPalette );
                    bitmapData.setColourBits( 8, 8, 8 );
                }
                break;

                default:
                    inStream.close();
                    // Unknown bits per pixel amount, throw an exception
                    throw new GameFrameException(
                            "The file format of file "+
                            strFileName+
                            " is incorrect." );
            }
		}
		catch (IOException e)
		{
			throw new GameFrameException(
                "Error occured while loading file "+
                strFileName+
                ".");
		}
        finally
        {
            // Close the input stream
            try
            {
			    inStream.close();
            }
            catch( IOException ioe )
            {
            }
        }

        return bitmapData;
	}

    /**
     * Fills (to full capacity) the given buffer from the given input stream.
     *@param inStream The input sream to read the data from.
     *@param aBuffer The buffer that is filled with the read data.
     */
    private final static void fillBuffer( InputStream inStream, byte[] aBuffer )
        throws IOException
    {
        int bufferLength = aBuffer.length;
	    int numReadBytes = inStream.read( aBuffer, 0, bufferLength );
        while ( numReadBytes < bufferLength )
        {
            numReadBytes += inStream.read(
                aBuffer, numReadBytes, bufferLength - numReadBytes );
        }
    }
}
