/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

import java.awt.Color;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Replaces the colors with other colours given as colour mappings.
 *
 *@author Pasi Kernen 18. Mar 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class ColorReplacementEffect extends BitmapEffect
{
	/** The color replacements to be applied. */
	protected Vector m_colorReplacements = new Vector();

    /**
     * Adds a colour to be replaced by another colour.
     *@param replacedColour The colour to be replaced.
     *@param replacementColour The colour that will replace the colour.
     */
    public void addColourMapping( int replacedColour, int replacementColour )
    {
        m_colorReplacements.addElement( new ColorMapping( replacedColour, replacementColour ) );
    }

    /**
     * Adds a colour mapping to the list of colour mappings to be applied.
     *@param colourMapping The colour mapping to be applied.
     */
    public void addColourMapping( ColorMapping colourMapping )
    {
        m_colorReplacements.addElement( colourMapping );
    }

    /**
     * Resets the effect settings to default values (no colour mappings are applied).
     */
    public void reset()
    {
        m_colorReplacements.removeAllElements();
    }

	/**
     * Maps the colours with the given colour maps.
	 *@param data The raw bitmap data that is to be processed.
     *@return The processed bitmap.
	 */
	public BitmapData processData( BitmapData data )
    {
		ColorMapping map;
		int[] aPixels = data.getPixels();
        int pixel;
        for ( Enumeration enum = m_colorReplacements.elements();
              enum.hasMoreElements(); )
        {
            map = (ColorMapping) enum.nextElement();
            if ( data.getNumRedBits() < 8 )
            {
                int rotation = 8 - data.getNumRedBits();
                //System.out.println("Rotation:"+rotation);
                int red = ( map.keyColor & 0x00FF0000 ) >>> 16;
                //System.out.println("Red:"+red);
                red >>>= rotation;
                red <<=  rotation;
                red <<= 16;
                map.keyColor = ( map.keyColor & 0xFF00FFFF ) | red;
            }
            if ( data.getNumGreenBits() < 8 )
            {
                int rotation = 8 - data.getNumGreenBits();
                int green = ( map.keyColor & 0x0000FF00 ) >>> 8;
                green >>>= rotation;
                green <<=  rotation;
                green <<= 8;
                map.keyColor = ( map.keyColor & 0xFFFF00FF ) | green;
            }
            if ( data.getNumBlueBits() < 8 )
            {
                int rotation = 8 - data.getNumBlueBits();
                int blue = ( map.keyColor & 0x000000FF );
                blue >>>= rotation;
                blue <<=  rotation;
                map.keyColor = ( map.keyColor & 0xFFFFFF00 ) | blue;
            }
        }
        for ( Enumeration enum = m_colorReplacements.elements();
			  enum.hasMoreElements(); )
		{
			map = (ColorMapping) enum.nextElement();
	        for ( int idx = 0; idx < aPixels.length; idx++ )
		    {
                pixel = aPixels[idx];
                // Compare ignoring Alpha value
                if ( ( pixel & 0x00FFFFFF ) == ( map.keyColor & 0x00FFFFFF ) )
                {
                    // Preserve alpha value and set the keycolor
                    pixel &= 0xFF000000;
                    pixel |= ( 0x00FFFFFF & map.valueColor );
                    aPixels[idx] = pixel;
                }
            }
        }                       

        return data;
    }
}
