/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

import gameframe.*;

/**
 * Defines an interface to a bitmap image that can blit (draw) or strecth (scale) itself
 * in the graphics engine's backbuffer or into drawable bitmap, but also can be blitted
 * into. The graphics engine in question is allways the graphics engine that loaded the
 * bitmap. The bitmap in question must be a bitmap created by the same graphics engine
 * as this drawable bitmap. If the bitmap was created by different graphics engine the
 * results are unpredictable. Most of the methods offered in this interface are implemented
 * to be as fast as possible. An exception is the method getSubBitmap() that tries to minimize
 * the memory usage to the extent that is possible (maybe even using the same bitmap data as the
 * original bitmap but with different clipping and blitting regions).
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public interface DrawableBitmap
{
    /**
     * Draws the bitmap to the graphics engine that loaded it at the given
     * location. The location can be outside of the visible screen area in which
     * case the bitmap is clipped accordingly. This method is fast and is usable
     * during the game execution.
     *
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@since GameFrame for Java 0.9
     */
    public void blitTo( int x, int y );

    /**
     * Draws the bitmap to the graphics engine that loaded it at the given
     * location and at the given size. If this bitmap is larger or smaller than
     * the destination size the bitmap will automatically be scaled. Note: This
     * method is usually a bit slow, so it's use is not preferred during
     * interactive periods in game execution.
     *
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with that is used when drawing the image.
     *@param height The height that is used when drawing the image.
     *@since GameFrame for Java 0.9
     */
    public void strecthTo( int x, int y, int width, int height );

    /**
     * Draws the bitmap to the given drawable bitmap at the given location.
     * The location can be outside of the visible screen area in which case
     * the bitmap is clipped accordingly. This method is fast and is usable
     * during the game execution.
     *
     *@param bitmap The bitmap that will be drawn to.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@since GameFrame for Java 0.9
     */
    public void blitTo( DrawableBitmap bitmap, int x, int y );

    /**
     * Draws the bitmap to the given drawable bitmap at the given location
     * at the given size. If this bitmap is larger or smaller than the
     * destination size the bitmap will automatically be scaled. Note: This
     * method is usually a bit slow, so it's use is not preferred during
     * interactive periods in game execution.
     *
     *@param bitmap The bitmap that will be drawn to.
     *@param x The x-coordinate of the location where the image is drawn to.
     *@param y The y-coordinate of the location where the image is drawn to.
     *@param width  The with that is used when drawing the image.
     *@param height The height that is used when drawing the image.
     *@since GameFrame for Java 0.9
     */
    public void strecthTo(
        DrawableBitmap bitmap, int x, int y, int width, int height );

    /**
     * Returns the width of the bitmap. This method is fast and is usable
     * during the game execution.
     *
     *@return Width of the bitmap if available.
     *@since GameFrame for Java 0.9
     */
    public int getWidth();

    /**
     * Returns the height of the bitmap. This method is fast and is usable
     * during the game execution.
     * 
     *@return Height of the bitmap if available.
     *@since GameFrame for Java 0.9
     */
    public int getHeight();
    
    /**
     * Frees all the resources that are used by the bitmap implementation.
     * This method is quite slow to execute and should preferrably be called
     * upon exiting the game or between level changes etc.
     * 
     *@since GameFrame for Java 0.9
     */
    public void finalize();
}
