/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

import gameframe.*;

import java.io.FileNotFoundException;

/**
 * <p>Defines an interface to a graphics engine that can be thought to be a "display device".
 * The graphics engine uses double buffering and offers a few methods for loading a bitmap in
 * Windows BMP (8-bit, 16-bit or 24-bit) format, handling the double buffering, converting
 * text strings to bitmap representations and drawing text strings directly.</p>
 * 
 * <p>Note: The actual implementation is recommended to use double buffering if such is available, if
 * no such feature is available backbuffering can be omitted. In the case double buffering is not
 * available all operations affect the one and only displayed buffer and the flip() method does
 * nothing.</p>
 * 
 * <p>Note2: The graphics engine will not create bitmaps larger than the currently used resolution.
 * If you try to create larger bitmaps they are allways automatically clipped to have the maximum width
 * and/or height. This is due to limitations in some platforms and those limitations must be forced
 * to every implementation to make the library work uniformly in all platforms. Writing code to handle
 * this internally by the GameFrame library (by dividing the bitmaps to smaller ones) is too big of a
 * task (at least for the time being).</p>
 *
 *@see gameframe.graphics.Bitmap
 *@see gameframe.graphics.DrawableBitmap
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public interface GraphicsEngine
{
    /** Constant that is set as the default title for all graphics engine windows. */
    public static final String DEFAULT_TITLE = "GameFrame for Java 0.9.4";
    /** Constant for refreshrate when there is no preferred refresh rate. */
    public static final int NO_PREFERRED_REFRESHRATE = 0;
    /** Constant for 16-bits colour depth. */
    public static int COLOURDEPTH_16BITS = 16;
    /** Constant for 24-bits colour depth. */
    public static int COLOURDEPTH_24BITS = 24;
    /** Constant for 32-bits colour depth. */
    public static int COLOURDEPTH_32BITS = 32;
    /** All supported colour depths. */
    public static int[] ALL_COLOURDEPTHS = { 16, 24, 32 };
    /** Constant for default colour depth when none is set by the user. */
    public static final int DEFAULT_COLOURDEPTH = COLOURDEPTH_16BITS;


    /**
     * Sets the data directory wherefrom the bitmaps are loaded.
     * The source directory defaults to the current directory.
     * 
     * Note: Remember to use the java.io.File.separator as the separator of directories.
     * 
     *@see #loadBitmap(java.lang.String)
     *@see #loadAlphaBitmap(java.lang.String)
     *@param strDirectoryName The name of the directory that contains the bitmaps.
     *@exception FileNotFoundException Thrown if the given directory doesn't exist.
     *@since GameFrame for Java 0.9
     */
    public void setDataDirectory( String strDirectoryName )
        throws FileNotFoundException;

    /**
     * Returns the data directory wherefrom the bitmaps are loaded.
     * The source directory defaults to the current directory.
     * 
     * Note: The returned path will have the path separator as the last character.
     * 
     *@see #loadBitmap(java.lang.String)
     *@see #loadAlphaBitmap(java.lang.String)
     *@return The name of the directory that wherefrom the bitmaps are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory();

    /**
     * Loads and returns a non-transparent bitmap (used bitmap format is Windows BMP 8-bit,
     * 16-bit or 24-bit) from the given filename. Bitmaps are loaded by default from the current
     * directory, the directory that is used to load the bitmaps can be set with the setDataDirectory()
     * method. If the bitmap is larger than current resolution it will be clipped to the dimensions of
     * the current resolution.
     *
     *
     *@see #setDataDirectory(java.lang.String)
     *@see #loadAlphaBitmap(java.lang.String)
     *@see Bitmap
     *@param fileName The name of the file to be loaded (without the file extension).
     *@return Bitmap object that represents the loaded bitmap image or null if the loading failed.
     *@exception FileNotFoundException Thrown if the given file doesn't exist.
     *@exception GameFrameException Thrown if other error occur while loading the file.
     *@since GameFrame for Java 0.9
     */
    public Bitmap loadBitmap( String fileName )
        throws FileNotFoundException, GameFrameException;

    /**
     * Loads and returns a transparent bitmap (used bitmap format is Windows BMP 8-bit,
     * 16-bit or 24-bit) from the given filename. Bitmap is loaded by default from the current
     * directory, the directory that is used to load the bitmaps can be set with the setDataDirectory()
     * method. Alpha keycolor (the color that is interpreted as the transparent color) is set to the
     * pixel color at bitmap location 0,0 which is the pixel at upper left corner of
     * the bitmap).If the bitmap is larger than current resolution it will be clipped to the dimensions of
     * the current resolution.
     *
     *@see #setDataDirectory(java.lang.String)
     *@see #loadBitmap(java.lang.String)
     *@see #loadAlphaBitmap(java.lang.String, byte, byte, byte)
     *@see Bitmap
     *@param fileName The name of the file to be loaded (without the file extension).
     *@return Bitmap object that represents the loaded bitmap image or null if the loading failed.
     *@exception FileNotFoundException Thrown if the given file doesn't exist.
     *@exception GameFrameException Thrown if other error occur while loading the file.
     *@since GameFrame for Java 0.9
     */
    public Bitmap loadAlphaBitmap( String fileName )
        throws FileNotFoundException, GameFrameException;

    /**
     * Loads and returns a transparent bitmap (used bitmap format is Windows
     * BMP 8-bit, 16-bit or 24-bit) from the given filename. Bitmap is loaded
     * by default from the current data directory, the directory that is used
     * to load the bitmaps can be set with the setDataDirectory() method.
     * Alpha keycolor (the color that is interpreted as the transparent color)
     * is taken from the given red, green and blue values. If the bitmap is
     * larger than current resolution it will be clipped to the dimensions of
     * the current resolution.
     *
     *@see #setDataDirectory(java.lang.String)
     *@see #loadBitmap(java.lang.String)
     *@see #loadAlphaBitmap(java.lang.String)
     *@see Bitmap
     *@param fileName The name of the file to be loaded (without the file extension).
     *@param red The red intensity value of the key color (in range 0x00-0xFF).
     *@param green The green intensity value of the key color (in range 0x00-0xFF).
     *@param blue The blue intensity value of the key color (in range 0x00-0xFF).
     *@return Bitmap object that represents the loaded bitmap image or null if the loading failed.
     *@exception FileNotFoundException Thrown if the given file doesn't exist.
     *@exception GameFrameException Thrown if other error occur while loading the file.
     *@since GameFrame for Java 0.9.4
     */
    public Bitmap loadAlphaBitmap( String fileName, byte red, byte green, byte blue )
        throws FileNotFoundException, GameFrameException;

    /**
     * Creates a non-transparent bitmap (the color is set to the current background color)
     * that can be drawn into with other bitmaps. If the bitmap is larger than current
     * resolution it will be clipped to the dimensions of the current resolution.
     *
     *@param width The width of the created bitmap.
     *@param height The height of the created bitmap
     *@return The created bitmap or null if creation failed.
     *@exception GameFrameException Thrown if errors occur while creating the bitmap.
     *@since GameFrame for Java 0.9
     */
    public DrawableBitmap getDrawableBitmap( int width, int height )
        throws GameFrameException;

    /**
     * Moves by copying a rectangular area of the backbuffer by a distance
     * specified by dx and dy. From the point specified by x and y, this
     * method moves downwards and to the right. To move an area of the
     * component to the left or upwards, specify a negative value for dx or
     * dy. If a portion of the source rectangle lies outside the bounds of
     * the component moveArea will be unable to move the associated pixels.
     *@param x The x-coordinate of the source rectangle.
     *@param y The y-coordinate of the source rectangle.
     *@param width The width of the source rectangle.
     *@param height The height of the source rectangle.
     *@param dx The horizontal distance to copy the pixels.
     *@param dy The vertical distance to copy the pixels.
     */
    //public void copyArea( int x, int y, int width, int height, int dx, int dy );

    /**
     * Switches the backbuffer and the currently displayed buffer. After this operation
     * returns the old backbuffer is visible on the screen and the old front buffer is
     * now the backbuffer that is affected by the drawing operations.
     * 
     *@since GameFrame for Java 0.9
     */
    public void flip();

    /**
     * Sets the currently used background color (which defaults to Color.black).
     * The background color is the color that is used to clear the backbuffer
     * and set to be the background color of created drawable bitmaps.
     *
     *@see #clearBackbuffer
     *@since GameFrame for Java 0.9
     */
    public void setBackgroundColor( java.awt.Color color );

    /**
     * Paints the current backbuffer with the current backround color.
     *
     *@see #setBackgroundColor(java.awt.Color)
     *@since GameFrame for Java 0.9
     */
    public void clearBackbuffer();

    /**
     * Loads the default font bitmap that is used when printing text on the screen.
     * Either this method or the setFontBitmap() method must be invoked before using
     * any of the getAsString() or drawTextAt() methods. If the font bitmap is larger
     * than current resolution it will be clipped to the dimensions of the current resolution.
     * 
     *@see #loadFontBitmap(java.lang.String)
     *@see #getAsBitmap(java.lang.String)
     *@see #getAsBitmap(byte[])
     *@see #drawTextAt(java.lang.String, int, int)
     *@see #drawTextAt(byte[], int, int)
     *@exception FileNotFoundException Thrown if the default font bitmap can't be found.
     *@exception GameFrameException Thrown if other errors occur while loading the default font bitmap.
     *@since GameFrame for Java 0.9.1
     */
    public void loadDefaultFontBitmap()
        throws FileNotFoundException, GameFrameException;

    /**
     * Loads and sets the used font set to be loaded from the given filename using a calculated
     * fixed size for every letter. The filename must specify a Windows BMP formatted
     * bitmap (8-bit, 16-bit or 24-bit colour) that is then loaded as transparent
     * (alpha) bitmap. The order in the bitmap must use the standard ASCII letter order
     * (order can be obtained by running the FontOrder class included with the GameFrame examples)
     * and it must contain 16 columns of letters in 16 rows (that is a 16x16 matrix of
     * letters = 256 letters in whole). If a nonexisting bitmap or fontbitmap of invalid
     * size is used, the method will do nothing. Either this method or the setDefaultFontBitmap()
     * method must be called before using any of the getAsString() or drawTextAt() methods.If the
     * font bitmap is larger than current resolution it will be clipped to the dimensions of the
     * current resolution.
     * 
     *@see #loadDefaultFontBitmap
     *@see #getAsBitmap(java.lang.String)
     *@see #getAsBitmap(byte[])
     *@see #drawTextAt(java.lang.String, int, int)
     *@see #drawTextAt(byte[], int, int)
     *@param filename Name of the bitmap file that contains the letters.
     *@exception FileNotFoundException Thrown if the given font bitmap can't be found.
     *@exception GameFrameException Thrown if other error occur while loading the font bitmap.
     *@since GameFrame for Java 0.9.1
     */
    public void loadFontBitmap( String filename )
        throws FileNotFoundException, GameFrameException;

    /**
     * Returns the given one line of text's width when we are using the current font bitmap.
     * The method doesn't take into account possible clipping to current screen size.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param strText The text that is written using the current font.
     *@return The width of the given text.
     *@since GameFrame for Java 0.9.3
     */
    public int getTextWidth( String strText );

    /**
     * Returns the given one line of text's width when we are using the current font bitmap.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param aByteText The byte array containing the text to be measured.
     *@return The width of the given text.
     *@since GameFrame for Java 0.9.3
     */
    public int getTextWidth( byte[] aByteText );

    /**
     * Returns the current font width.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@since GameFrame for Java 0.9.3
     */
    public int getFontWidth();
    
    /**
     * Returns the current font height.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@since GameFrame for Java 0.9.3
     */
    public int getFontHeight();
    
    /**
     * Returns the given one line of text as an bitmap that is painted using
     * the current font. The returned bitmap will have the background color set
     * to the current background color of the graphics engine. This means also
     * that the returned bitmap is never transparent (due to limitations on
     * some platforms). If the resulting bitmap is larger than current resolution
     * it will be clipped to the dimensions of the current resolution.
     *
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param strText The text that is written using the current font.
     *@return The text as bitmap or null if not successfull. Textual description of the error can be gotten with getLastError().
     *@exception GameFrameException Thrown if errors occur.
     *@since GameFrame for Java 0.9
     */
    public DrawableBitmap getAsBitmap( String strText )
        throws GameFrameException;

    /**
     * Returns the given one line of text as an bitmap that is painted using
     * the current font. The returned bitmap will have the background color set
     * to the current background color of the graphics engine. This means also
     * that the returned bitmap is never transparent (due to limitations on
     * some platforms). If the resulting bitmap is larger than current resolution
     * it will be clipped to the dimensions of the current resolution.
     * 
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param aByteText The byte array containing the text to be printed.
     *@return The text as bitmap or null if not successfull. Textual description of the error can be gotten with getLastError().
     *@exception GameFrameException Thrown if errors occur.
     *@since GameFrame for Java 0.9
     */
    public DrawableBitmap getAsBitmap( byte[] aByteText )
        throws GameFrameException;

    /**
     * Draws the given string to the screen so that the uppermost and leftmost coordinate
     * of the text is at the given location.
     * 
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param strText The text that is written using the current font.
     *@param x The leftmost x-coordinate for the text.
     *@param y The topmost y-coordinate for the text.
     *@exception GameFrameException Thrown if the font bitmap is not set correctly or other errors occur.
     *@since GameFrame for Java 0.9
     */
    public void drawTextAt( String strText, int x, int y )
        throws GameFrameException;

    /**
     * Prints the given string to the screen so that the uppermost and leftmost coordinate
     * of the text is at the given location.
     * 
     *@see #loadDefaultFontBitmap
     *@see #loadFontBitmap(java.lang.String)
     *@param aByteText The byte array containing the text to be printed.
     *@param x The leftmost x-coordinate for the text.
     *@param y The topmost y-coordinate for the text.
     *@exception GameFrameException Thrown if the font bitmap is not set correctly or other errors occur.
     *@since GameFrame for Java 0.9
     */
    public void drawTextAt( byte[] aByteText, int x, int y )
        throws GameFrameException;

    /**
     * Returns true if the engine is in fullscreen mode, false if in windowed mode.
     * Note: Window that is exactly the same size as the desktop resolution is still
     * considered to be in windowed mode.
     * 
     *@return True if the engine is in fullscreen mode, false if in windowed mode.
     *@since GameFrame for Java 0.9
     */
    public boolean isInFullscreenMode();
    
    /**
     * Returns the size of the display area that this engine uses. For fullscreen
     * modes it is the size of the fullscreen mode. For windowed modes the size is
     * size of the window.
     *@return The size of the display area.
     *@since GameFrame for Java 0.9.2
     */
    public java.awt.Dimension getSize();

    /**
     * Returns the short name (max 8 characters) of the
     * implementation e.g. "Java11" or "DDraw".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName();

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription();
}
