/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

/**
 * Rotates the image around its center the given amount.
 *
 *@author Pasi Kernen 18. Mar 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class RotationEffect extends BitmapEffect
{
    /** The rotation amount to be applied in radians. */
    protected float m_flRotation = 0.0F;
    
    /**
     * Sets the amount of rotation applied to the image in degrees.
     *@param degrees The amount of rotation applied to the image.
     *@since GameFrame for Java 0.9.3
     */
    public void setRotationDegrees( float degrees )
    {
        m_flRotation = (float) ( ( degrees / 180.0F ) * Math.PI );
    }
    
    /**
     * Sets the amount of rotation applied to the image in radians.
     *@param radians The amount of rotation applied to the image.
     *@since GameFrame for Java 0.9.3
     */
    public void setRotationRadians( float radians )
    {
        m_flRotation = radians;
    }
    
    /**
     * Returns the amount of rotation applied to the image in degrees.
     *@return The amount of rotation applied to the image.
     *@since GameFrame for Java 0.9.3
     */
    public float getRotationDegrees()
    {
        return (float) ( m_flRotation / Math.PI ) * 180.0F;
    }

    /**
     * Returns the amount of rotation applied to the image in radians.
     *@return The amount of rotation applied to the image.
     *@since GameFrame for Java 0.9.3
     */
    public float getRotationRadians()
    {
        return m_flRotation;
    }


    /**
     * Resets the effect settings to default values (no rotation is applied).
     *@since GameFrame for Java 0.9.3
     */
    public void reset()
    {
        m_flRotation = 0.0F;
    }
	
	/**
     * Rotates the image around its center the given amount.
	 *@param data The raw bitmap data that is to be processed.
     *@return The processed data.
	 */
	public BitmapData processData( BitmapData data )
    {
		int[] aPixels = data.getPixels();
		int iWidth = data.getWidth();
		int iHeight = data.getHeight();
		int iPitch  = data.getPitch();
		double sinValue = Math.sin(m_flRotation);
        double cosValue = Math.cos(m_flRotation);
        double width   = (double) data.getWidth();
        double height  = (double) data.getHeight();
        double centerX = width / 2.0D;
        double centerY = height / 2.0D;
        int dstRowOffset;
        int[] aOrigPixels = new int[ aPixels.length ];
        System.arraycopy( aPixels, 0, aOrigPixels, 0, aPixels.length );

        for(int y = 0; y < iHeight; y++)
        {
            dstRowOffset = y * iPitch;
            for(int x = 0; x < iWidth; x++)
            {
                double srcX =   ( ( (double) x - centerX ) * cosValue  )
                              + ( ( (double) y - centerY ) * sinValue )
                              + centerX;
                double srcY = ( ( ( (double) y - centerY ) * cosValue  )
                                - ( (double) x - centerX ) * sinValue )
                               + centerY;
                if( srcX < 0.0D || srcX > height || srcY < 0.0D || srcY > width )
                {
                    aPixels[ dstRowOffset + x ] = aOrigPixels[0];
                }
                else
                {
                    aPixels[ dstRowOffset + x ] = aOrigPixels[ (int) srcY * iPitch + (int) srcX ];
                }
            }
        }

        aOrigPixels = null;
        System.gc();

        return data;
    }
}
