/*
GameFrame for Java. Enable platform dependent features without giving up platform independence!
Copyright (C) 2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.graphics;

/**
 * Scales the image the given amount. The new image will have its size
 * set accordingly.
 *
 *@author Pasi Kernen 25. Apr 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class ScaleEffect extends BitmapEffect
{
    /** The scale factor to be applied. */
    protected float m_flScale = 1.0F;

    /**
     * Sets the amount of scaling applied to the image. Values of ]0.0, 1.0[
     * will make the resulting image smaller and values above 1.0 will make
     * the resulting image larger. Value of 1.0 will not change the image
     * at all. Values of ]-infinity, 0] or above 100.0 are not accepted.
     *@param scale The scale to be applied.
     */
    public void setScale( float scale )
    {
        if ( ( scale <= 0.0F ) || ( scale > 100.0F ) )
        {
            return;
        }
        
        m_flScale = scale;
    }

    /**
     * Returns the amount of scaling applied to the image.
     *@return The scale to be applied on the image.
     */
    public float getScale()
    {
        return m_flScale;
    }

    /**
     * Resets the effect settings to default values (no scaling is applied).
     */
    public void reset()
    {
        m_flScale = 1.0F;
    }

	/**
     * Scales the image the given amount.
	 *@param data The raw bitmap data that is to be processed.
     *@return The processed data.
	 */
	public BitmapData processData( BitmapData data )
    {
		int oldWidth  = data.getWidth();
		int oldHeight = data.getHeight();
        int oldPitch  = data.getPitch();

        int newWidth  = (int) ( ( (float) oldWidth  ) * m_flScale );
        int newPitch  = newWidth;
        int newHeight = (int) ( ( (float) oldHeight ) * m_flScale );

        int dstRowOffset;
        int[] aOrigPixels = data.getPixels();
        int[] aNewPixels  = new int[ newPitch * newHeight ];

        for( int y = 0; y < newHeight; y++ )
        {
            dstRowOffset = y * newPitch;
            for( int x = 0; x < newWidth; x++ )
            {
                double srcX = x / m_flScale;
                double srcY = y / m_flScale;
                if( srcX < 0.0D || srcX > oldHeight ||
                    srcY < 0.0D || srcY > oldWidth )
                {
                    aNewPixels[ dstRowOffset + x ] = aOrigPixels[0];
                }
                else
                {
                    aNewPixels[ dstRowOffset + x ] =
                        aOrigPixels[ (int) srcY * oldPitch + (int) srcX ];
                }
            }
        }

        return new BitmapData( aNewPixels, newWidth, newHeight, newPitch,
            data.getKeyColorRed(), data.getKeyColorGreen(),
            data.getKeyColorBlue(), data.getPalette() );
    }
}
