/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.input;

import gameframe.*;

/**
 * Represents the interface to an input engine that is used to
 * get to the input devices in the system. The engine allows
 * usage of both polled implementations of one device.
 * Implementations should only create one instance of
 * a polled device (as it can be shared).
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public interface InputEngine
{
    /**
     * Returns all keyboard type of devices found on the system as polled
     * devices (whenever the program that wants to know about
     * the state of the device it must ask it from the device).
     *
     *@return All keyboard type of devices.
     *@see gameframe.input.KeyboardDevice
     *@exception GameFrameException Thrown if errors occur while enumerating the keyboard devices.
     *@since GameFrame for Java 0.9
     */
    public KeyboardDevice[] getAllKeyboardDevices()
        throws GameFrameException;

    /**
     * Returns the default keyboard type of device in the system as polled
     * device (whenever the program that wants to know about the state of
     * the device it must ask it from the device).
     *
     *@return The default keyboard device.
     *@see gameframe.input.KeyboardDevice
     *@exception GameFrameException Thrown if errors occur while getting the default keyboard device.
     *@since GameFrame for Java 0.9
     */
    public KeyboardDevice getDefaultKeyboardDevice()
        throws GameFrameException;

    /**
     * Returns all pointer type of devices found on the system as polled
     * devices (whenever the program that wants to know about
     * the state of the device it must ask it from the device).
     *
     *@return All pointer type of devices.
     *@see gameframe.input.PointerDevice
     *@exception GameFrameException Thrown if errors occur while enumerating the pointer devices.
     *@since GameFrame for Java 0.9
     */
    public PointerDevice[] getAllPointerDevices()
        throws GameFrameException;
        
    /**
     * Returns the default pointer type of device in the system as
     * an polled device (whenever the program that wants to know about
     * the state of the device it must ask it from the device).
     *
     *@return The default pointer device.
     *@see gameframe.input.PointerDevice
     *@exception GameFrameException Thrown if errors occur while getting the default pointer device.
     *@since GameFrame for Java 0.9
     */
    public PointerDevice getDefaultPointerDevice()
        throws GameFrameException;

    /**
     * Returns the short name (max 8 characters) of the
     * implementation e.g. "Java11" or "DDraw".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName();

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription();
}
