/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.input;

import gameframe.*;

import java.awt.event.KeyListener;

/**
 * This interface is meant for input devices that are keyboard-like in their operation.
 * This means that the device has several (more than 10) buttons that are on/off type.
 * 
 * The keycodes used in this interface are those defined in java.awt.event.KeyEvent class.
 * If the devices buttons can't be directly mapped to those keycodes the mapping should
 * be made in a such way that order of the keys are preserved, when counting the keys
 * from topleft to the right, row by row. Also if the input device has directional control
 * buttons, those must be mapped to cursor keycodes (VK_UP, VK_DOWN, VK_LEFT, VK_RIGHT).
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public interface KeyboardDevice
{
    /**
     * Returns true if the key is down or if the key was briefly kept down
     * after last reading of the keys state (after last call made to isKeyDown or
     * getKeyDownCount methods with the same key code). The keycode is one of the
     * VK_xxx codes from java.awt.event.KeyEvent class.
     * 
     *@param keyCode The keycode of the key we want to query.
     *@return True if the key is down.
     *@since GameFrame for Java 0.9
     */
    public boolean isKeyDown( int keyCode );
    
    /**
     * Returns true if the key was briefly kept down after last reading of the keys
     * state (after last call made to isKeyDown or getKeyDownCount methods with the
     * same key code). The keycode is one of the VK_xxx codes from
     * java.awt.event.KeyEvent class.
     * 
     *@param keyCode The keycode of the key we want to query.
     *@return True if the key was down.
     *@since GameFrame for Java 0.9
     */
    public boolean wasKeyDown( int keyCode );
    
    /**
     * Returns the number of times the key has been stroken since last reading
     * (after last call made to isKeyDown or getKeyDownCount methods with the
     * same key code). The keycode is one of the VK_xxx codes from
     * java.awt.event.KeyEvent class.
     * 
     *@param keyCode The keycode of the key we want to query.
     *@return The number of times the key was stroken (pressed and released)
     *@since GameFrame for Java 0.9
     */
    public int getKeyDownCount( int keyCode );
    
    /**
     * Returns any typed alphanumeric keys (including space and dot) as int values that
     * correspond to that keys char value or -1 if no such keys were pressed
     * after last invokation of this method.
     * 
     *@return Any typed alphanumeric keys
     *@since GameFrame for Java 0.9
     */
    public int getTypedAlphanumericKey();
    
    /**
     * Releases all resources held by the keyboard device implementation.
     * This method may be called several times.
     * 
     *@since GameFrame for Java 0.9
     */
    public void finalize();
}
