/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.sound;

import java.io.*;
import gameframe.*;
import java.applet.Applet;
import java.net.*;

/**
 * Class for reading RIFF WAV audio files. Originally taken from my own small audio class
 * library and changed to only handle the reading as other features are not needed in the
 * GameFrame context.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class RiffWavFile
{
    /** Format tag for Microsoft Pulse Code Modulation (PCM) format (non-proprietary). */
    public static final int WAVE_FORMAT_PCM  = 0x0001;
    /** The input stream used for reading the file. */
    private DataInputStream m_inputStream;
    /** The wave buffer. */
    private short[] m_aWavBuffer;
    /** The RIFF length. */
    private int m_riffLength;
    /** The RIFF format length. */
    private int m_formatLength;
    /** The average bytes per second in the RIFF file. */
    private int m_averageBytePerSecond;
    /** The block alignment in the RIFF file. */
    private int m_blockAlignment;
    
    /** The bits per sample in the RIFF file. */
    private int m_bitsPerSample;
    /** The RIFF wave format tag. */
    private int m_wavFormatTag;
    /** The number of channels in the RIFF file. */
    private int m_numChannels;
    /** The samples per second in the RIFF file. */
    private int m_samplesPerSecond;
    /** The bytes per sample value in the RIFF file. */
    private int m_bytesPerSample;
    /** The length of the RIFF data field. */
    private int m_dataLength;
    /** The minimum sample value in the RIFF file. */
    private short m_minValue;
    /** The maximum sample value in the RIFF file. */
    private short m_maxValue;
    /** The name of the file. */
    private String m_strFileName;
    
    /**
     * Constructs a new RIFF wave file without any filename.
     */
    public RiffWavFile()
    {
        m_strFileName = null;
    }
    
    /**
     * Loads the RIFFF WAVE file from the specified file.
     *@param filename The name of the RIFF WAVE file.
     *@return True if load successfull, false if not.
     */
    public void loadWAV(String filename) throws FileNotFoundException, GameFrameException
    {
        m_strFileName = filename;
        m_inputStream = new DataInputStream ( new BufferedInputStream( new FileInputStream( filename ) ) );
        loadFromInputStream( filename );
    }
    
    /**
     * Loads the RIFFF WAVE file from the specified URL.
     *@param filename The name of the RIFF WAVE file.
     */
    public void loadWAVURL( String strFileName ) throws FileNotFoundException, GameFrameException
    {
        if ( !( GameFrame.getInstance() instanceof AppletGameFrame ) )
        {
            throw new GameFrameException( "The loadBMPURL method can only be used when running as an applet." );
        }
        
        AppletGameFrame appletGF = (AppletGameFrame) GameFrame.getInstance();
        Applet applet = appletGF.getApplet();
        URL codeBase  = applet.getCodeBase();
        
        URL url = null;
        try
        {
            url = new URL(
                codeBase.getProtocol(),
                codeBase.getHost(),
                codeBase.getPort(),
                codeBase.getFile() + strFileName.replace('\\', '/' ) );
            System.err.println( "loadWAVURL:"+url);
        }
        catch( MalformedURLException urlError )
        {
            throw new GameFrameException( "URL for WAV file was faulty:"+urlError );
        }
        
		try
		{
            m_inputStream = new DataInputStream( url.openStream() );
        }
		catch (IOException e)
		{
			throw new GameFrameException(
                "Error occured when trying to open stream to "+
                url.toString()+
                ".");
		}
        
        loadFromInputStream( strFileName );
    }
    
    /**
     * 
     */
    public void loadFromInputStream( String strFilename ) throws FileNotFoundException, GameFrameException
    {
        if (m_aWavBuffer == null)
        {
            readHeader();

            try
            {
                int maxValue = -1000000;
                int minValue = 1000000;
                int value;
                int numSamples;

                switch(m_bytesPerSample)
                {
                    case 1:
                        numSamples = m_dataLength;
                        if (m_numChannels == 2)
                        {
                            numSamples = numSamples / 2;
                        }

                        m_aWavBuffer = new short[numSamples];
                        for (int i = 0; i < numSamples; i++)
                        {
                            if (m_numChannels == 1)
                            {
                                value = (( (int) m_inputStream.readByte() ) << 8);
                            }
                            else
                            {
                                // Read left channel
                                value = (( (int) m_inputStream.readByte() ) << 8);
                                // Read and mix right channel
                                value += (( (int) m_inputStream.readByte() ) << 8);
                                // Scale back to 16-bit
                                value >>>= 1;
                            }

                            // Set min and max values...
                            if (value > maxValue)
                            {
                                maxValue = value;
                            }
                            if (value < minValue)
                            {
                                minValue = value;
                            }
                            m_aWavBuffer[i] = (short) value;
                        }

                        m_maxValue = (short) maxValue;
                        m_minValue = (short) minValue;
                        break;
                    case 2:
                        numSamples = m_dataLength / 2;
                        if (m_numChannels == 2)
                        {
                            numSamples = numSamples / 2;
                        }
                        m_aWavBuffer = new short[numSamples];
                        for (int i = 0; i < numSamples; i++)
                        {
                            if (m_numChannels == 1)
                            {
                                value = readIntelShort(m_inputStream);
                            }
                            else
                            {
                                // Read left channel
                                value = readIntelShort(m_inputStream);
                                // Read and mix right channel
                                value += readIntelShort(m_inputStream);
                                // Scale back to 16-bit
                                value >>>= 1;
                            }

                            if (value > maxValue)
                            {
                                maxValue = value;
                            }
                            if (value < minValue)
                            {
                                minValue = value;
                            }
                            m_aWavBuffer[i] = (short) value;
                        }
                        
                        m_maxValue = (short) maxValue;
                        m_minValue = (short) minValue;
                        break;
                }
            }
            catch(IOException ioe)
            {
                throw new GameFrameException("IO Exception while loading PCM data from the file "+strFilename);
            }
            finally
            {
                try
                {
                    m_inputStream.close();
                }
                catch( IOException ioe )
                {
                }
                finally
                {
                    m_inputStream = null;
                }
            }
        }
    }
    
    /**
     * Sets the sampling rate of this wav data to the given sampling rate
     *@param frequency The target sampling rate.
     */
    public void resampleTo(int targetSampleRate)
    {
        if (targetSampleRate != m_samplesPerSecond)
        {
            m_aWavBuffer = resampleTo(m_aWavBuffer, m_samplesPerSecond, targetSampleRate);
            m_samplesPerSecond = targetSampleRate;
        }
    }

    /**
     * Returns the raw linear PCM sample data in 16-bit mono format.
     * Note: Returns the reference to the internal data structure
     * of this object, modifying the data in the data structure will
     * propably make this objects state illegal.
     *@return Linear PCM sample data.
     */
    public short[] getPcmData()
    {
        return m_aWavBuffer;
    }

    /**
     * Reads and parses the RIFF WAVE header. When this method returns
     * the size of the actual sample data field has been read and
     * the m_inputStream is about to read the first actual sample
     * data.
     *@return True if header is can be parsed and format is supported, false otherwise.
     *@exception GameFrameException Thrown if the header is not correctly read.
     */
    public void readHeader() throws GameFrameException
    {
        try
        {
            // Check that the file is a RIFF file
            if (!findTag(m_inputStream, "RIFF"))
            {
                throw new GameFrameException("File "+m_strFileName+" is not a RIFF file.");
            }

            // Read the RIFF length of the file
            m_riffLength = readIntelInt(m_inputStream);

            // Check that the file is a WAVE file
            if (!findTag(m_inputStream, "WAVE"))
            {
                throw new GameFrameException("File "+m_strFileName+" is not a RIFF WAVE file.");
            }

            // Find the format chunk
            if (!findTag(m_inputStream, "fmt "))
            {
                throw new GameFrameException("File "+m_strFileName+" doesn't contain format chunk.");
            }

            // Read format chunk
            m_formatLength = readIntelInt(m_inputStream);
            m_wavFormatTag = ((int) readIntelShort(m_inputStream)) & 0xFFFF;
            if (m_wavFormatTag != WAVE_FORMAT_PCM)
            {
                throw new GameFrameException("File "+m_strFileName+" is not in Microsoft Pulse Code Modulation (PCM) format.");
            }

            m_numChannels = ((int) readIntelShort(m_inputStream)) & 0xFFFF;
            if (m_numChannels > 2)
            {
                throw new GameFrameException("File "+m_strFileName+" has more than 2 channels, max.2 channels supported.");
            }
            m_samplesPerSecond = readIntelInt(m_inputStream);
            m_averageBytePerSecond = readIntelInt(m_inputStream);
            m_blockAlignment = ((int) readIntelShort(m_inputStream)) & 0xFFFF;
            m_bitsPerSample = ((int) readIntelShort(m_inputStream)) & 0xFFFF;
            m_bytesPerSample = (m_bitsPerSample/8);
            
            // Find the data chunk
            if (!findTag(m_inputStream, "data"))
            {
                throw new GameFrameException("File "+m_strFileName+" contains no PCM data.");
            }

            // Read the length of the data chunk
            m_dataLength = readIntelInt(m_inputStream);
        }
        catch(IOException ioe)
        {
            throw new GameFrameException("IO Exception while loading RIFF WAVE header of file "+m_strFileName);
        }
    }
    
    /**
     * Resamples the given 16-bit linear PCM sample data from the given sample rate
     * to the given sample rate.
     *@param a16BitData The linear PCM sample data to be resampled.
     *@param originalSampleRate The original sample rate.
     *@param targetSampleRate The targetted sample rate.
     */
    private short[] resampleTo(short[] a16BitData, int originalSampleRate, int targetSampleRate)
    {
        if (originalSampleRate == targetSampleRate)
        {
            return a16BitData;
        }
        
        if (originalSampleRate > targetSampleRate)
        {
            float sum = 0F;
            // Increment is the number of samples to read at once
            float increment = ((float) originalSampleRate) / ((float) targetSampleRate);
            int writeLoc = 0, readLoc = 0;
            int length = a16BitData.length;
            short value = a16BitData[readLoc++];
        
            while( (writeLoc < length) && (readLoc < length) )
            {
                a16BitData[writeLoc++] = value;
                sum += increment;

                // Skip enough samples to compensate for 
                // sampling frequency difference
                while (sum > 0)
                {
                    if (readLoc < length)
                    {
                        value = a16BitData[readLoc++];
                    }
                    sum = sum - 1.0F;
                }
            }

            length = writeLoc;
            // Copy the new buffer as the actual buffer
            short[] buffer = new short[length];
            System.arraycopy(a16BitData, 0, buffer, 0, length);
            return buffer;
        }
            
        if (originalSampleRate < targetSampleRate)
        {
            System.out.println("not implemented");
            return a16BitData;
        }

        return a16BitData;
    }

    /**
     * Reads an "Intel formatted" 32-bit integer from the given data input stream.
     *@param inStream The input stream to read from.
     *@return The integer value that was read.
     */
    private int readIntelInt(DataInputStream inStream) throws IOException
    {
        int lobyte;
        int hibyte;
        int intValue = 0;
        lobyte = ((int) inStream.readByte()) & 0xFF;
        hibyte = ((int) inStream.readByte()) & 0xFF;
        intValue = (hibyte << 8) | lobyte;
        lobyte = ((int) inStream.readByte()) & 0xFF;
        hibyte = ((int) inStream.readByte()) & 0xFF;
        intValue |= (((hibyte << 8) | lobyte) << 16);
        return intValue;
    }

    /**
     * Reads an "Intel formatted" 16-bit short from the given data input stream.
     *@param inStream The input stream to read from.
     *@return The short value that was read.
     */
    private short readIntelShort(DataInputStream inStream) throws IOException
    {
        int lobyte;
        int hibyte;
        lobyte = ((int) inStream.readByte()) & 0xFF;
        hibyte = ((int) inStream.readByte()) & 0xFF;
        return ((short) ((hibyte << 8) | lobyte));
    }

    /**
     * Tries to find the specified tag from the specified inputstream. If the
     * tag is found this method will exit and the input stream will point to
     * the next value after the tag. If the tag is not found this method will
     * exit with false and the input stream will point to the end of itself.
     *@param inStream  The data input stream to read from.
     *@param tagToFind The tag we are looking for.
     *@return true if the tag was found, false if not.
     */
    private boolean findTag(DataInputStream inStream, String stringToFind) throws IOException
    {
        byte[] dataToFind = stringToFind.getBytes();
        int len = dataToFind.length;

        if (len == 0)
        {
            return true;
        }

        byte[] buffer = new byte[len];

        // First we need to fill the buffer
        for (int i = 0; i < len; i++)
        {
            buffer[i] = inStream.readByte();
        }

        int numCorrect = 0;
        for (int i = 0; i < len; i++)
        {
            if (dataToFind[i] == buffer[i])
            {
                numCorrect++;
            }
        }

        if (numCorrect == len)
        {
            return true;
        }

        while(true)
        {

            for (int i = 0; i < len-1; i++)
            {
                buffer[i] = buffer[i+1];
            }

            try
            {
                buffer[len-1] = inStream.readByte();
            }
            catch(IOException ioe)
            {
                return false;
            }

            numCorrect = 0;
            for (int i = 0; i < len; i++)
            {
                if (dataToFind[i] == buffer[i])
                {
                    numCorrect++;
                }
            }

            if (numCorrect == len)
            {
                return true;
            }
        }
    }
}
