/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.sound;

import gameframe.*;

import java.io.FileNotFoundException;

/**
 * Defines an interface to a sound engine that can be thought to be a "sound device".
 * Sample engines are used to load sample data stored in Microsoft RIFF WAV format
 * (8/16-bit, any sample rate, stereo/mono) or music files in MIDI format. The loaded
 * sample or music file is returned as an control object that is then used to play the
 * sample.
 *
 *@see gameframe.sound.Sample
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public interface SoundEngine
{
    /**
     * <p>Sets the data directory wherefrom the sounds are loaded.
     * The source directory defaults to the current directory.</p>
     * 
     * <p>Note: Remember to use the java.io.File.separator as the separator of directories.</p>
     * 
     *@see #loadSample(java.lang.String)
     *@see #loadMusic(java.lang.String)
     *@param strDirectoryName The name of the directory that contains the bitmaps.
     *@exception FileNotFoundException Thrown if the directory is not found.
     *@since GameFrame for Java 0.9
     */
    public void setDataDirectory( String strDirectoryName )
        throws FileNotFoundException;

    /**
     * Returns the data directory wherefrom the sounds are loaded.
     * The source directory defaults to the current directory.
     * 
     * Note: The returned path will have the path separator as the last character.
     * 
     *@see #loadSample(java.lang.String)
     *@see #loadMusic(java.lang.String)
     *@return The name of the directory that wherefrom the sounds are loaded.
     *@since GameFrame for Java 0.9.3
     */
    public String getDataDirectory();

    /**
     * Loads a sample in Microsoft RIFF WAV format and returns the control object
     * for the sample. Samples are loaded by default from the current directory,
     * the directory that is used to load the samples can be set with the
     * setDataDirectory() method. 
     * 
     *@see #setDataDirectory(java.lang.String)
     *@param fileName The name of the file to be loaded (without the file extension)
     *@return An unique Sample object that represents the loaded sample
     *@since GameFrame for Java 0.9
     */
    public Sample loadSample( String fileName )
        throws FileNotFoundException, GameFrameException;

    /**
     * First tries to load a music file as streamed Microsoft RIFF WAV file,
     * if no such file is found from the current data directory, tries to load
     * a music file in MIDI format. When suitable file is found returns a
     * control object for the music file if no suitable file is found returns
     * NullSound object. Music files are loaded by default from the
     * current directory, the directory that is used to load the samples
     * can be set with the setDataDirectory() method. 
     * 
     *@see #setDataDirectory(java.lang.String)
     *@param fileName The name of the file to be loaded (without the file extension)
     *@return An unique Sample object that represents the loaded sample
     *@since GameFrame for Java 0.9.2
     */
    public Music loadMusic( String fileName )
        throws FileNotFoundException, GameFrameException;

	/**
     * Returns the short name (max 8 characters) of the
     * implementation e.g. "Java11" or "DDraw".
     *@return The short name of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getName();

    /**
     * Returns the (possibly multiline) description of the
     * implementation.
     *@return A description of the graphics engine.
     *@since GameFrame for Java 0.9.3
     */
    public String getDescription();
}
