/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.sound;

/**
 * u-law, A-law and linear PCM conversions.
 *
 *@author Pasi Kernen 16. May 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9
 */
public class ULawEncoder
{
    /** Sign bit for a A-law byte. */
    public static final int SIGN_BIT    = 0x80;
    /** Quantization field mask. */
    public static final int QUANT_MASK  = 0x0f;
    /** Number of A-law segments. */
    public static final int NSEGS       = 8;
    /** Left shift for segment number. */
    public static final int SEG_SHIFT   = 4;
    /** Segment field mask. */
    public static final int SEG_MASK    = 0x70;
    /** Bias for linear code. */
    public static final int BIAS        = 0x84;

    /**
     * The u-law segment encoding look up table.
     */
    private static final int ULAW_ENCODE_SEGMENT_LUT[] = {
        0, 0, 1, 1, 2, 2, 2, 2, 3, 3, 
        3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 
        4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
        4, 4, 5, 5, 5, 5, 5, 5, 5, 5, 
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 
        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 
        5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 
        6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 
        6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 
        7, 7, 7, 7, 7, 7
    };

    /**
     * Encodes the input value of linear PCM returns the results as
     * either 8-bit u-law encoded array.
     *@param aOrigData The array containing original data, one sample per array index.
     */
    public byte[] encode( short[] aOrigData )
    {
        int     length       = aOrigData.length;
        byte[]  aEncodedData = new byte[length];

        for (int i = 0; i < length; i++)
        {
            aEncodedData[i] = linear2ulaw(aOrigData[i]);
        }
        return aEncodedData;
    }
    
    
    /**
     * Converts a linear PCM value to u-law. Method originally was implemented
     * from Sun Microsystem's free example code, but was then later on changed
     * to this (a bit more speed optimized) version.
     *
     *@param pcmValue PCM sample in 2's complement (16-bit range)
     *@return u-Law compressed sample.
     */
    byte linear2ulaw( short pcmValue )
    {
        int linearPcmValue = pcmValue >> 2;
        int aSegmentLUT[] = ULAW_ENCODE_SEGMENT_LUT;
        int segment;

        // Chekc if the value is negative
        if(linearPcmValue >= 0)
        {
            // Limit the value
            if(linearPcmValue > 32635)
            {
                linearPcmValue = 32635;
            }

            // Add the bias value
            linearPcmValue += BIAS;

            // Look the segment number from the look up table
            segment = aSegmentLUT[linearPcmValue >> 7];
            
            // Build the u-law encoded value
            linearPcmValue = (segment << 4 | ( linearPcmValue >> segment + 3 & 0xf ) ) ^ 0xff;
        }
        else
        {
            // Make the value positive
            linearPcmValue = -linearPcmValue;

            // Limit the value
            if(linearPcmValue > 32635)
            {
                linearPcmValue = 32635;
            }

            // Add the bias value
            linearPcmValue += BIAS;

            // Look the segment number from the look up table
            segment = aSegmentLUT[linearPcmValue >> 7];

            // Build the u-law encoded value
            linearPcmValue = (segment << 4 | ( linearPcmValue >> segment + 3 & 0x0f ) ) ^ 0x7f;
        }

        return (byte) linearPcmValue;
    }
}
