/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, read from the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.util;

import java.io.*;

/**
 * Output stream filter that filters all input values to big endian format
 * that is used e.g. by Intel processor equipped computers.
 * 
 *@author Pasi Kernen 18. Mar 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class BigEndianDataInputStream extends FilterInputStream
    implements DataInput
{
	/** This stream is used in case the basic methods are allright. */
	protected DataInputStream m_dataInStream;
	
	/**
	 * Constructs an big endian input filter.
	 *@param inputStream The input stream to read from.
	 */
    public BigEndianDataInputStream( InputStream inputStream )
    {
        super( inputStream );
    }

	/**
	 * Reads an integer.
	 *@param integer The integer that is read.
	 */
    public synchronized int read()
        throws IOException
    {
        return in.read();
    }

	/**
	 * Reads the series of bytes in big endian format.
	 *@parma aByte The byte array that was read.
	 */
    public synchronized int read( byte aByte[] )
        throws IOException
    {
        return in.read( aByte, 0, aByte.length );
    }

	/**
	 * Reads the series of bytes in big endian format.
	 *@parma aByte The byte array that was read.
	 *@param offset The offset into the array.
	 *@param length The length of the read array.
	 */
    public synchronized int read( byte aByte[], int offset, int length )
        throws IOException
    {
        return in.read( aByte, offset, length );
    }
	
	/**
	 * Reads some bytes from an input stream and stores them into
	 * the buffer array aBytes. The number of bytes read is equal to the
     * length of aBytes.
     *@param aBytes The buffer into which the data is read. 
	 */
	public void readFully( byte[] aBytes )
		throws IOException
	{
		readFully( aBytes, 0 , aBytes.length );
	}
	
	
	/**
	 * Reads some bytes from an input stream and stores them into
	 * the buffer array aBytes.
     *@param aBytes The buffer into which the data is read.
     *@param offset The offset into the array.
     *@param length The number of bytes to read.
	 */
    public final void readFully( byte aBytes[], int offset, int length )
		throws IOException
	{
		InputStream inStream = in;
		int idx = 0;
		while ( idx < length )
		{
			int numRead = inStream.read(aBytes, offset + idx, length - idx);
			if (numRead < 0)
			{
				throw new EOFException();
			}
			idx += numRead;
		}
    }
	
    /**
     * Skips the given number of bytes.
     * @param numBytes The number of bytes that was skipped.
     * @return The actual number of bytes skipped.
     */
    public final int skipBytes( int numBytes )
		throws IOException
	{
		InputStream inStream = this.in;
		int numSkipped = 0;
		int current = 0;

		while ( ( numSkipped < numBytes ) &&
				( ( current = (int) inStream.skip( numBytes - numSkipped ) ) > 0 ) )
		{
		    numSkipped += current;
		}

		return numSkipped;
    }
	
	private char m_achLineBuffer[] = new char[256];

	/**
	 * Reads one line of text.
	 *@return The line of text that was read.
	 */
    public final String readLine() throws IOException
	{
		char achBuffer[] = m_achLineBuffer;

		int bufferSize = achBuffer.length;
		int offset = 0;
		int reacChar;

loop:	while (true)
		{
			switch ( reacChar = in.read() )
			{
				case -1:
				case '\n':
					break loop;

				case '\r':
					int reacChar2 = in.read();
					if ( ( reacChar2 != '\n' ) && ( reacChar2 != -1 ) )
					{
						if ( !( in instanceof PushbackInputStream ) )
						{
							in = new PushbackInputStream( in );
						}
						
						( (PushbackInputStream) in ).unread( reacChar2 );
					}
					break loop;

				default:
					if ( --bufferSize < 0 )
					{
						achBuffer = new char[offset + 128];
						bufferSize = achBuffer.length - offset - 1;
						System.arraycopy( m_achLineBuffer, 0, achBuffer, 0, offset );
						m_achLineBuffer = achBuffer;
					}
					achBuffer[offset++] = (char) reacChar;
					break;
			}
		}
		
		if ( ( reacChar == -1 ) && ( offset == 0 ) )
		{
			return null;
		}
		
		return String.copyValueOf( achBuffer, 0, offset );
    }
	
	/**
	 * Reads a boolean value.
	 *@return The boolean value
	 */
    public final boolean readBoolean()
        throws IOException
    {
		int readValue = in.read();
		if ( readValue < 0 )
		{
			throw new EOFException();
		}
		
		return ( readValue != 0 );
    }

	/**
	 * Reads a byte.
	 *@param byteValue The byte that was read.
	 */
    public final byte readByte()
        throws IOException
    {
		int readValue = in.read();
		if ( readValue < 0 )
		{
			throw new EOFException();
		}
		
		return ( (byte) readValue );
    }

	/**
	 * Reads an unsigned byte.
	 *@return The read byte.
	 */
    public final int readUnsignedByte()
        throws IOException
    {
		int readValue = in.read();
		if ( readValue < 0 )
		{
			throw new EOFException();
		}
		
		return ( readValue );
    }

	/**
	 * Reads a Unicode character.
	 *@return The character that was read.
	 */
    public final char readChar()
        throws IOException
    {
		int loByte = in.read();
		int hiByte = in.read();
		if ( ( loByte | hiByte ) < 0 )
		{
			throw new EOFException();
		}
		
		return (char)( ( hiByte << 8 ) | ( loByte ) );
    }

	/**
	 * Reads a double value in big endian format.
	 *@return The double value that was read.
	 */
    public final double readDouble()
        throws IOException
    {
		return Double.longBitsToDouble( readLong() );
    }

	/**
	 * Reads a floating point value in big endian format.
	 *@return The floating point value that was read.
	 */
    public final float readFloat()
        throws IOException
    {
		return Float.intBitsToFloat( readInt() );
    }

	/**
	 * Reads a integer value in big endian format.
	 *@return The integer value that was read.
	 */
    public final int readInt()
        throws IOException
    {
		int loByte1 = in.read();
		int hiByte1 = in.read();
		int loByte2 = in.read();
		int hiByte2 = in.read();
		if ( ( loByte1 | hiByte1 |
			   loByte2 | hiByte2 ) < 0 )
		{
		     throw new EOFException();
		}
		
		int value = hiByte2;
		value <<= 8;
		value |= loByte2;
		value <<= 8;
		value |= hiByte1;
		value <<= 8;
		value |= loByte1;
		
		return ( value );
    }

	/**
	 * Reads a long value in big endian format.
	 *@return The long value that was read.
	 */
    public final long readLong()
        throws IOException
    {
		int loByte1 = in.read();
		int hiByte1 = in.read();
		int loByte2 = in.read();
		int hiByte2 = in.read();
		int loByte3 = in.read();
		int hiByte3 = in.read();
		int loByte4 = in.read();
		int hiByte4 = in.read();
		
		if ( ( loByte1 | hiByte1 |
			   loByte2 | hiByte2 |
			   loByte3 | hiByte3 |
			   loByte4 | hiByte4 ) < 0 )
		{
		     throw new EOFException();
		}
		long value = hiByte4;
		value <<= 8;
		value |= loByte4;
		value <<= 8;
		value |= hiByte3;
		value <<= 8;
		value |= loByte3;
		value <<= 8;
		value |= hiByte2;
		value <<= 8;
		value |= loByte2;
		value <<= 8;
		value |= hiByte1;
		value <<= 8;
		value |= loByte1;

		return ( value );
    }

	/**
	 * Reads a short value in big endian format.
	 *@return The short value that was read.
	 */
    public final short readShort()
        throws IOException
    {
		int loByte = in.read();
		int hiByte = in.read();
		if ( ( loByte | hiByte ) < 0 )
		{
			throw new EOFException();
		}
		
		return (short) ( ( hiByte << 8 ) | ( loByte ) );
    }

	/**
	 * Reads an unsigned short value in big endian format.
	 *@return The short value that was read.
	 */
    public final int readUnsignedShort() throws IOException
	{
		InputStream inStream = this.in;
		int loByte = inStream.read();
		int hiByte = inStream.read();
		if ( ( loByte | hiByte ) < 0 )
		{
			throw new EOFException();
		}
		
		return ( ( hiByte << 8 ) | ( loByte ) );
    }

	/**
	 * Does nothing.
	 *@return  Not used.
	 */
    public final String readUTF()
        throws IOException
    {
		return DataInputStream.readUTF( this );
    }
}
