/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.util;

import java.io.*;

/**
 * Output stream filter that filters all output values to big endian format
 * that is used e.g. by Intel processor equipped computers.
 * 
 *@author Pasi Kernen 18. Mar 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.3
 */
public class BigEndianDataOutputStream extends FilterOutputStream
    implements DataOutput
{
	/**
	 * Constructs an big endian output filter.
	 *@param outputStream The output stream to write to.
	 */
    public BigEndianDataOutputStream( OutputStream outputstream )
    {
        super( outputstream );
    }

	/**
	 * Flushes the used output stream.
	 */
    public void flush()
        throws IOException
    {
        out.flush();
    }

	/**
	 * Writes an integer to the used output stream in big endian format.
	 *@param integer The integer that is written.
	 */
    public synchronized void write( int integer )
        throws IOException
    {
        out.write( integer );
    }

	/**
	 * Writes the series of bytes in big endian format.
	 *@parma aByte The byte array to be written.
	 *@param offset The offset into the array.
	 *@param length The length of the written array.
	 */
    public synchronized void write( byte aByte[], int offset, int length )
        throws IOException
    {
        out.write( aByte, offset, length );
    }

	/**
	 * Writes the given boolean value.
	 *@param flag The boolean value
	 */
    public final void writeBoolean( boolean flag )
        throws IOException
    {
        out.write( flag ? 1 : 0 );
    }

	/**
	 * Writes the given byte.
	 *@param byteValue The byte to be written.
	 */
    public final void writeByte( int byteValue )
        throws IOException
    {
        out.write( byteValue );
    }

	/**
	 * Writes the given string as an array of bytes.
	 *@param strText The text to be written.
	 */
    public final void writeBytes( String strText )
        throws IOException
    {
        OutputStream outputstream = out;
        int length = strText.length();
        for( int i = 0; i < length; i++)
            outputstream.write( (byte) strText.charAt(i) );
    }

	/**
	 * Writes the given Unicode character.
	 *@param character The character to be written.
	 */
    public final void writeChar( int character )
        throws IOException
    {
        OutputStream outputstream = out;
        outputstream.write( character & 0xff );
        outputstream.write( character >>> 8 & 0xff );
    }

	/**
	 * Writes the given string as an array of Unicode characters.
	 *@param strText the text to be written.
	 */
    public final void writeChars( String strText )
        throws IOException
    {
        OutputStream outputstream = out;
        int length = strText.length();
        for( int i = 0; i < length; i++ )
        {
            char character = strText.charAt( i );
            outputstream.write( character >>> 8 & 0xff );
            outputstream.write( character & 0xff );
        }
    }

	/**
	 * Writes the given double value in big endian format.
	 *@parma doubleValue The double value to be written.
	 */
    public final void writeDouble( double doubleValue )
        throws IOException
    {
        writeLong( Double.doubleToLongBits( doubleValue ) );
    }

	/**
	 * Writes the given floating point value in big endian format.
	 *@param floatValue The floating point value to be written.
	 */
    public final void writeFloat( float flValue )
        throws IOException
    {
        writeInt( Float.floatToIntBits( flValue ) );
    }

	/**
	 * Writes the given integer value in big endian format.
	 *@param intValue The integer value to be written.
	 */
    public final void writeInt( int intValue )
        throws IOException
    {
        OutputStream outputstream = out;
        outputstream.write( intValue & 0xff );
		intValue >>>= 8;
        outputstream.write( intValue & 0xff );
		intValue >>>= 8;
        outputstream.write( intValue & 0xff );
		intValue >>>= 8;
        outputstream.write( intValue & 0xff );
    }

	/**
	 * Writes the given long value in big endian format.
	 *@param longValue The long value to be written.
	 */
    public final void writeLong( long longValue )
        throws IOException
    {
        OutputStream outputstream = out;
        outputstream.write( (int) longValue & 0xff);
		longValue >>>= 8;
        outputstream.write( (int) longValue & 0xff);
		longValue >>>= 8;
        outputstream.write( (int) longValue & 0xff);
		longValue >>>= 8;
        outputstream.write( (int) longValue & 0xff);
		longValue >>>= 8;
        outputstream.write( (int) longValue & 0xff);
		longValue >>>= 8;
        outputstream.write( (int) longValue & 0xff);
		longValue >>>= 8;
        outputstream.write( (int) longValue & 0xff);
		longValue >>>= 8;
        outputstream.write( (int) longValue & 0xff);
    }

	/**
	 * Writes the given short value in big endian format.
	 *@param shortValue The short value to be written.
	 */
    public final void writeShort( int shortValue )
        throws IOException
    {
        OutputStream outputstream = out;
        outputstream.write( shortValue & 0xff );
        outputstream.write( shortValue >>> 8 & 0xff );
    }

	/**
	 * Does nothing.
	 *@param strText Not used.
	 */
    public final void writeUTF( String strText )
        throws IOException
    {
    }
}
