/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.util;

import gameframe.*;
import gameframe.graphics.*;

/**
 * A counter that can display almost any number of digits. The counter uses
 * a rolling counter by default. In order to update the shown value you must
 * <code>tick()</code> this counter. This causes the counter to adjust its
 * value towards the final value in steps (makes a nice rolling effect).
 * Methods addImmediately and decImmediately adjust the counter immediately.
 * The maximum number of digits this counter can show is the maximum number
 * of digits that can fit to an integer.
 * 
 *@author Pasi Kernen 15. Jul 2000
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.4
 */
public class Counter
{
    /** The real value of this counter that it is rolling towards. */
    private int m_destCount;
    /** The currently displayed value of this counter. */
    private int m_currentCount;
    /** The initial count value, used when the counter is reset. */
    private int m_initialCount;

    /** Count for each of the 10 exp n values (n starts from 0 ). */
    private int[] m_aCounts;
    /** The x coordinate for the count at index n. */
    private int[] m_aX;
    /** Base x-coordinate of the counter. */
    private int m_x;
    /** Base y-coordinate of the counter. */
    private int m_y;
    /** The width of one font image. */
    private int m_fontWidth;
    /** The height of one font image. */
    private int m_fontHeight;
    /** The images that represent numbers. */
    private DrawableBitmap[] m_aNumbers;
    /** The last time player was ticked. */
    private long m_lastTime;
    /** The number of digits (between 1 to 6) that are displayed. */
    private int m_numDigits;

    /**
     * Constructs a counter that uses the currently set font bitmap for numbers,
     * has the given initial value, is located at the given screen
     * coordinates and uses the given number of digits.
     *@param aNumbers The bitmaps that represent numbers.
     *@param initialCount The initial counter value.
     *@param x The base x-coordinate of the counter.
     *@param y The base y-coordinate of the counter.
     *@param numDigits The number of digits (between 1 to 6) that are displayed.
     */
    public Counter( int initialCount, int x, int y, int numDigits )
        throws GameFrameException
    {
        m_x = x;
        m_y = y;

        // The get the bitmaps used in the counter
        GraphicsEngine grfxEngine = GameFrame.getGraphicsEngine();
        m_aNumbers = new DrawableBitmap[10];
        for ( int i = 0; i < 10; i++ )
        {
            m_aNumbers[i] = grfxEngine.getAsBitmap( "" + i );
        }

        m_fontWidth    = grfxEngine.getFontWidth();
        m_fontHeight   = grfxEngine.getFontHeight();
        m_currentCount = initialCount;
        m_initialCount = initialCount;
        m_destCount    = initialCount;
        m_numDigits    = numDigits;
        m_aCounts      = new int[numDigits];
        m_aX           = new int[numDigits];

        updateCount();
        setX(x);
    }

    /**
     * Calcultes the value of this counter every time
     * called.
     */
    public void tick()
    {
        if ( m_currentCount < m_destCount )
        {
            m_currentCount += ( ( m_destCount - m_currentCount ) / 3 ) + 1;
            updateCount();
        }
        else if ( m_currentCount > m_destCount )
        {
            m_currentCount -= ( ( m_currentCount - m_destCount ) / 3 ) + 1;
            updateCount();
        }
    }

    /**
     * Resets this counter to its inital value.
     */
    public void reset()
    {
        m_currentCount = m_initialCount;
        m_destCount    = m_initialCount;
        updateCount();
    }

    /**
     * Paints this counter to the backbuffer.
     */
    public void paint()
    {
        for ( int i = 0; i < m_numDigits; i++ )
        {
            m_aNumbers[m_aCounts[i]].blitTo( m_aX[i], m_y );
        }
    }

    /**
     * Returns the current (real) counter value.
     *@return The current counter value.
     */
    public int getCount()
    {
        return m_destCount;
    }
    
    /**
     * Adds the given number to the counter.
     *@param count The number to add to this counter.
     */
    public void add( int count )
    {
        m_destCount += count;
        updateCount();
    }

    /**
     * Adds the given number to the counter immediately.
     *@param count The number to add to this counter.
     */
    public void addImmediately(int count)
    {
        m_destCount    += count;
        m_currentCount = m_destCount;
        updateCount();
    }

    /**
     * Decrements the given number from the counter.
     *@param count The number to decrement from this counter.
     */
    public void dec(int count)
    {
        m_destCount -= count;
        if (m_destCount < 0)
        {
            m_destCount = 0;
        }
        updateCount();
    }

    /**
     * Decrements the given number from the counter immediately.
     *@param count The number to decrement from this counter.
     */
    public void decImmediately(int count)
    {
        m_destCount -= count;
        if (m_destCount < 0)
        {
            m_destCount = 0;
        }
        m_currentCount = m_destCount;
        updateCount();
    }

    /**
     * Sets the count of this counter to the given number.
     *@param count The counters new value.
     */
    public void setCount(int count)
    {
        m_destCount = count;
        updateCount();
    }

    /**
     * Sets the given counter value immediately without
     * the fancy scrolling effect.
     */
    public void setImmediately( int count )
    {
        m_destCount    = count;
        m_currentCount = m_destCount;
        updateCount();
    }

    /**
     * Updates the number display values from the
     * actual counter value.
     */
    private final void updateCount()
    {
        int count = m_currentCount;

        for ( int i = m_numDigits - 1; i >= 0; i-- )
        {
            m_aCounts[i] = count % 10;
            count       /= 10;
        }
    }

    /**
     * Returns the base x-coordinate of the counter based upon the
     * number of digits it needs to display. The base coordinate
     * is the leftmost numbers coordinate.
     *@return The base x-coordinate.
     */
    public int getX()
    {
        return m_aX[m_numDigits - 1];
    }

    /**
     * Sets the upper left coordinate.
     *@param x The x-coordinate this counter will use.
     */
    public void setX( int x )
    {
        m_x = x;
        for ( int i = 0; i < m_numDigits; i++ )
        {
            m_aX[i] = x;
            x      += m_fontWidth;
        }
    }

    /**
     * Returns the base y-coordinate. The base coordinate is
     * the topmost y-coordinate of the counter.
     *@return Base y-coordinate.
     */
    public int getY()
    {
        return m_y;
    }

    /**
     * Sets the upper left coordinate.
     *@param y The y-coordinate this counter will use.
     */
    public void setY(int y)
    {
        m_y = y;
    }
    
    /**
     * Returns the width of this counter.
     *@return Width of this counter. 
     */
    public int getWidth()
    {
        return m_fontWidth*m_numDigits;
    }
    
    /**
     * Returns the height of this counter.
     *@return Height of this counter. 
     */
    public int getHeight()
    {
        return m_fontHeight;
    }
}
