/*
GameFrame for Java. Makes your games run better in some Java platforms.
Copyright (C) 1999-2000  Pasi Kernen

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The author can be contacted at eMail address: javanerd@geocities.com
*/
package gameframe.util;

import java.util.*;

/**
 * This class has methods to ensure that an object is never garbage
 * collected. It is very handy when using singletons (classes that
 * have one static reference to itself) as these might be garbage
 * collected in some Java runtimes.
 * 
 *@author Pasi Kernen 20. Dec 1999
 *@version GameFrame for Java 0.9.4
 *@since GameFrame for Java 0.9.2
 */
public class ObjectPreserver implements Runnable
{
    /**
     * This keeps this class and everything it references from being
     * garbage collected by having an active thread and containing
     * all the objects being preserved in a member varialbe.
     *@since GameFrame for Java 0.9.2
     */
    private static ObjectPreserver mStatic_singletonInstance = new ObjectPreserver();

    /**
     * As this class isn't garbage collected, neither is this
     * vector of or the objects the vector references.
     *@since GameFrame for Java 0.9.2
     */
    private static Vector mStatic_protectedSet = new Vector( 10, 10 );

    /**
     * Constructs the object preserver and starts up the thread
     * that keeps this object alive.
     */
    private ObjectPreserver()
    {
        new Thread(this).start();
    }

    /**
     * The threads run method that does nothing else than keeps this
     * object alive.
     *@since GameFrame for Java 0.9.2
     */
    public void run()
    {
        Thread.currentThread().setName("PreventGCThread");

        synchronized( this )
        {
            try
            {
                // We wait indefinitely here (until
                // someone says "notify" to us).
                wait();
            }
            catch (InterruptedException e)
            {
            }
        }
    }

    /**
    * Stops the garbage collector preventing thread.
     *@since GameFrame for Java 0.9.2
    */
    public void stop()
    {
        // Causes the wait to exit
        notify();
    }

    /**
     * The objects passed to this method will be protected from garbage collection
     * until they are released by the unpreserveObject method.
     *@param object The object to be added to the preserved set of objects.
     *@since GameFrame for Java 0.9.2
     */
    public static void preserveObject( Object object )
    {
        mStatic_protectedSet.addElement( object );
    }

    /**
     * The objects passed to this method will no longer be protected from garbage
     * collection.
     *@param object The object to be removed from the preserved set of objects.
     *@since GameFrame for Java 0.9.2
     */
    public static void unpreserveObject( Object object )
    {
        mStatic_protectedSet.removeElement( object );
    }
}

